/****************************************************************************/
/** alloc.cpp                                                              **/
/** ---------------------------------------------------------------------- **/
/** Demonstrates usage of TAllocationAware class                           **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** July 30, 1997 by Oleg Kagan                                            **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>  // recommended for better compilation time
#pragma hdrstop

// Standard includes
//============================================================================
#include <iostream.h>

// Natlib Basics includes
//============================================================================
#include <nlalloc.h>
#include <nlautopt.h>

// Natlib Win32 includes
//============================================================================
#include <nwconsol.h>
#include <nwlock.h>

//============================================================================
USING_NATLIB_NAMESPACE

// TAllocationAwareTest
//============================================================================
class TAllocationAwareTest :
	public virtual TNatlibObject, public virtual TAllocationAware
{
public:
   	//------------------------------------------------------------------------
	TStdString sValue;

	//------------------------------------------------------------------------
    TAllocationAwareTest() : TAllocationAware(this) {}

	//------------------------------------------------------------------------
    TAllocationAwareTest(const TAllocationAwareTest& c_Source) :
    	TAllocationAware(this), sValue(c_Source.sValue)
    {} /*TAllocationAwareTest(const TAllocationAware&)*/

	//------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE(TAllocationAwareTest);
} /*TAllocationAwareTest*/;

// TAllocationAwareTest implementation
//============================================================================
NLOBJECT_IMPLEMENT_CLONE_DEFAULT(TAllocationAwareTest)

//============================================================================
NLALLOC_DECLSPEC(TAllocationAwareTest, NATLIB_NOTHING);

//============================================================================
void AllocationAwareExample()
{
	{
		cout << "Automatic (stack) allocation" << endl;
	    TAllocationAwareTest StackTest;
        cout << StackTest.IsDynamic() << endl;
        TWin32Console::Pause(); cout << endl;
    }

    {
		cout << "Dynamic (heap) allocation" << endl;
	    TAutoPointerToClass<TAllocationAwareTest> hDynamicTest(null, false);
        NLEXCEPT_NEW(TAllocationAwareTest, hDynamicTest);
        cout << "IsArray == " << hDynamicTest.IsArray() << endl;

        cout << hDynamicTest->IsDynamic() << endl;
        TWin32Console::Pause(); cout << endl;
    }

    {
    	cout << "Array on stack allocation" << endl;
        TAllocationAwareTest a_ArrayOnStackTest[3];
        for (size_t i = 0; i < 3; ++i)
	        cout << a_ArrayOnStackTest[i].IsDynamic() << endl;
        TWin32Console::Pause(); cout << endl;
    }

    {
    	cout << "Array on heap allocation" << endl;
  	    TAutoPointerTo<TAllocationAwareTest> pDynamicArrayTest(null, true);
       	NLEXCEPT_NEW(TAllocationAwareTest[3], pDynamicArrayTest);

        cout << "IsArray == " << pDynamicArrayTest.IsArray() << endl;

        for (size_t i = 0; i < 3; ++i) {
	        cout << ((TAllocationAwareTest*)pDynamicArrayTest)[i].IsDynamic();
            cout << endl;
        } /*for (size_t i)*/
        TWin32Console::Pause(); cout << endl;
    }
} /*AllocationAwareExample()*/

