/****************************************************************************/
/** bintre.cpp                                                             **/
/** ---------------------------------------------------------------------- **/
/** Demonstrates usage of TNatlibBinaryTreeOf<Data> template               **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** October 21, 1996 by Oleg Kagan                                         **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>  // recommended for better compilation time
#pragma hdrstop

// Standard includes
//============================================================================
#include <iostream.h>

// Natlib Basics includes
//============================================================================
#include <nlstring.h>

//#define BINTRE_TEST_BINTREE
#ifdef BINTRE_TEST_BINTREE
	#include <nlbintre.h>
	#define TTestingTreeOf TBinaryTreeOf
#else
	#define TTestingTreeOf TAvlTreeOf
	#include <nlavltre.h>
#endif /*BINTRE_TEST_BINTREE*/

#include <nlprntre.h>

// Natlib Win32 includes
//============================================================================
#include <nwconsol.h>

//============================================================================
//#ifdef BINTRE_TEST_BINTREE
//NLBINTRE_DECLSPEC(int, NATLIB_NOTHING);
//#else
//NLAVLTRE_DECLSPEC(int, NATLIB_NOTHING);
//#endif /*BINTRE_TEST_BINTREE*/

//============================================================================
USING_NATLIB_NAMESPACE

//============================================================================
template <class T> class TBinaryTreePrinterOf {
public:
	//------------------------------------------------------------------------
	void operator()(const TBinaryTreeOf<T>& c_Tree)
	{
        TStdString sDump(c_Tree.Dump());
        cout << sDump.c_str() << endl;

        c_Tree.AssertValid();

		Fast(c_Tree);
//		Slow(c_Tree);
//		SlowX(c_Tree);
	} /*operator()(const TBinaryTreeOf<T>&)*/

	//------------------------------------------------------------------------
    void Fast(const TBinaryTreeOf<T>&);
	//------------------------------------------------------------------------
    void Slow(const TBinaryTreeOf<T>&);
	//------------------------------------------------------------------------
    void SlowX(const TBinaryTreeOf<T>&);
} /*TBinaryTreePrinterOf<T>*/;

// TBinaryTreePrinterOf template implementation
//============================================================================
template <class T> void TBinaryTreePrinterOf<T>::Fast(

	const TBinaryTreeOf<T>& c_Tree

){
    const T* pc_Data = c_Tree.Search(TAnyOf<T>());

    if (pc_Data) 
		PrintBinaryTreeStructure(c_Tree, dynamic_cast<ostream&>(cout));
} /*TBinaryTreePrinterOf<T>::Fast(const TBinaryTreeOf<T>&)*/

//----------------------------------------------------------------------------
template <class T> void TBinaryTreePrinterOf<T>::Slow(

	const TBinaryTreeOf<T>& c_Tree

){
    const T* pc_Data = c_Tree.SearchFirst();
    while (pc_Data) {
        cout << *pc_Data << endl;
        pc_Data = c_Tree.SearchNext(*pc_Data);
    } /*while*/
} /*TBinaryTreePrinterOf<T>::Slow(const TBinaryTreeOf<T>&)*/

//----------------------------------------------------------------------------
template <class T> void TBinaryTreePrinterOf<T>::SlowX(

	const TBinaryTreeOf<T>& c_Tree

){
    try {
        T Data = c_Tree.FindLast();
        for (;;) {
            cout << Data << endl;
            Data = c_Tree.FindPrevious(Data);
        } /*for*/
    } /*try*/
    catch (TBinaryTreeOf<T>::TXNotFound&) {}
} /*TBinaryTreePrinterOf<T>::Slow(const TBinaryTreeOf<T>&)*/

// TOddNumber template
//============================================================================
template <class T>
class TOddNumber : public TConditionOf<T> {
public:
	//------------------------------------------------------------------------
    typedef TConditionOf<T> TParent;

   	//------------------------------------------------------------------------
    bool operator()(const T& c_i) const {return c_i % 2;}

protected:
    //------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED(TOddNumber)
} /*TOddNumber<T>*/;

// TAddOne template
//============================================================================
template <class T> class TAddOne : public TOperationOn<T> {
public:
	//------------------------------------------------------------------------
    void operator()(T& r_i) {++r_i;}

protected:
    //------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED1(TAddOne, T)
} /*TAddOne<T>*/;

//============================================================================
void BinaryTreeExample()
{
	cout << "Testing " << typeid(TTestingTreeOf<int>).name() << endl;
	TWin32Console::Pause(); cout << endl;
	// Prepare printer
	//-----------------
	TBinaryTreePrinterOf<int> Print;

	TDefaultComparatorOf<int> DefaultComparatorOfInt;
	TTestingTreeOf<int> Tree(DefaultComparatorOfInt);

	// Init tree
	//-----------
	{
	    const size_t c_nSize = 21;
	    const int c_iInit[c_nSize] = {
        	 8,  9, 11, 15, 19, 20, 21,  7,  3,  2,  1,
             5,  6,  4, 13, 14, 10, 12, 17, 16, 18
        } /*c_iInit*/;
	    for (size_t i = 0; i < c_nSize; ++i) {
        	cout << "Insert " << c_iInit[i] << endl;
        	Tree.Insert(c_iInit[i]);
        } /*for*/
    } /*Init tree*/

	Print.Slow(Tree);
	TWin32Console::Pause(); cout << endl;

	Print(Tree);
	TWin32Console::Pause(); cout << endl;

	//------------------------------------------------------------------------
	TTestingTreeOf<int> Tree1(DefaultComparatorOfInt);

	Tree1 = Tree;

	Print(Tree);
	TWin32Console::Pause(); cout << endl;

	//------------------------------------------------------------------------
	cout << "Testing Search by condition... result must be 5..." << endl;

    TOddNumber<int> OddNumberCondition;
	const int* pc_Odd = Tree1.Search(OddNumberCondition, 3);

    cout << "Result is: ";
    if (pc_Odd) cout << *pc_Odd << endl;
    else cout << "none" << endl;

	TWin32Console::Pause(); cout << endl;

    //------------------------------------------------------------------------
	cout << "Adding 1 to each node..." << endl;

    TAddOne<int> AddOne;
    Tree1.ForEach(AddOne);

    Print(Tree1);
	TWin32Console::Pause(); cout << endl;

    //------------------------------------------------------------------------
	cout << "Combining trees..." << endl;

	TTestingTreeOf<int> Tree2(DefaultComparatorOfInt);

    (Tree2 += Tree) += Tree1;

	Print(Tree2);
	TWin32Console::Pause(); cout << endl;

	//------------------------------------------------------------------------
    cout << "Difference between trees..." << endl;

	Tree2 -= (Tree -= 5);

    Print(Tree2);
	TWin32Console::Pause(); cout << endl;

    //------------------------------------------------------------------------
    cout << "Slow iteration..." << endl;

    Print.SlowX(Tree);
	TWin32Console::Pause(); cout << endl;

	//------------------------------------------------------------------------
	cout << "Additional test" << endl;
    TStdStringCaseSensitiveComparator StrComparator;
	TTestingTreeOf<TStdString> StrTree(StrComparator);

	for (int i = 0; i < 100; ++i)
		StrTree.Insert(Format("%02d", i));

	const TStdString* pc_sIterator = StrTree.SearchFirst();
	while (pc_sIterator) {
		cout << pc_sIterator->c_str() << endl;
		pc_sIterator = StrTree.SearchNext(*pc_sIterator);
	} /*while*/

	TWin32Console::Pause(); cout << endl;
} /*BinaryTreeExample()*/
