/****************************************************************************/
/** list.cpp                                                               **/
/** ---------------------------------------------------------------------- **/
/** Demonstrates usage of TNatlibDListOf<Data> template                    **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** March 27, 1997 by Ilya Marchevsky                                      **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>  // recommended for better compilation time
#pragma hdrstop

// Standard includes
//============================================================================
#include <math.h>
#include <iostream.h>
//#include <algorithm>

// Natlib Basics includes
//============================================================================
#include <nllist.h>

// Natlib Win32 includes
//============================================================================
#include <nwconsol.h>

//============================================================================
USING_NATLIB_NAMESPACE

// TEqualsTo template
//============================================================================
template <class T> class TEqualsTo : public TConditionOf<T> {
public:
	//------------------------------------------------------------------------
    typedef TConditionOf<T> TParent;
    //------------------------------------------------------------------------
    TEqualsTo(const T& c_Value) : Value(c_Value) {}
    //------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED1(TEqualsTo, T)
   	//------------------------------------------------------------------------
    bool operator()(const T& c_Data) const {return c_Data == Value;}
protected:
    //------------------------------------------------------------------------
    T Value;
} /*TEqualsTo<T>*/;

// TAddOf template
//============================================================================
template <class T> class TAddOf : public TOperationOn<T> {
public:
	//------------------------------------------------------------------------
	TAddOf(const T& c_Addition) : Addition(c_Addition) {}
	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED1(TAddOf, T)
	//------------------------------------------------------------------------
    void operator()(T& r_Data) {r_Data += Addition;}
protected:
	T Addition;
} /*TAddOf<T>*/;

// TPrintValue template
//============================================================================
template <class T> class TPrintValueOf : 
	public TConstantOperationOn<T> 
{
public:
	//------------------------------------------------------------------------
	TPrintValueOf(const TStdString& c_sPrefix, const TStdString& c_sSuffix) :
		sPrefix(c_sPrefix), sSuffix(c_sSuffix)
	{} /*TPrintValueOf(const TStdString&, const TStdString&)*/
	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED1(TPrintValueOf, T)
	//------------------------------------------------------------------------
    void operator()(const T& c_Data) 
	{
		cout << sPrefix.c_str() << c_Data << sSuffix.c_str();
	} /*operator()(const T&)*/
protected:
	//------------------------------------------------------------------------
	TStdString sPrefix, sSuffix;
} /*TPrintValueOf<T>*/;

//============================================================================
template <class T> class TPrintContainerOf : 
	public TConstantOperationOn< TContainerOf<T> >
{
public:
	//------------------------------------------------------------------------
	typedef TConstantOperationOn< TContainerOf<T> > TParent;
	//------------------------------------------------------------------------
	TPrintContainerOf(
		const TStdString& c_sPrefix, const TStdString& c_sSuffix
	):
		PrintValue(c_sPrefix, c_sSuffix)
	{} /*TPrintContainerOf(const TStdString&, const TStdString&)*/
	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED1(TPrintContainerOf, T);
	//------------------------------------------------------------------------
	void operator()(const TContainerOf<T>& c_Container)
	{
		cout << c_Container.Dump().c_str() << endl;
		c_Container.ForEach(PrintValue);
		cout << endl;
	} /*operator()(const TContainerOf<T>&)*/
protected:
	//------------------------------------------------------------------------
	TPrintValueOf<T> PrintValue;
} /*TPrintContainerOf<T>*/;

// TValueOf template
//============================================================================
template <class T> class TValueOf {
public:
	//------------------------------------------------------------------------
	TValueOf(const T& c_Value) : Data(c_Value) {}
	//------------------------------------------------------------------------
	TValueOf(const TValueOf<T>& c_Source) : Data(c_Source.Data) {}

	//------------------------------------------------------------------------
	T& Value() {return Data;}
	//------------------------------------------------------------------------
	const T& ValueConst() const {return Data;}

	//------------------------------------------------------------------------
	operator T&() {return Data;}
	//------------------------------------------------------------------------
	operator const T&() const {return Data;}

	//------------------------------------------------------------------------
	TValueOf<T>& operator=(const TValueOf<T>& c_Source)
	{
		Data = c_Source.Data; return *this;
	} /*operator=(const TValueOf<T>&)*/

	//------------------------------------------------------------------------
	TValueOf<T>& operator+=(const TValueOf<T>& c_Add)
	{
		Data += c_Add.Data; return *this;
	} /*operator+=(const TValueOf<T>&)*/

	//------------------------------------------------------------------------
	bool operator==(const TValueOf<T>& c_Value) const 
	{
		return c_Value.Data == Data;
	} /*operator==(const TValueOf<T>&) const*/
	//------------------------------------------------------------------------
	bool operator!=(const TValueOf<T>& c_Value) const 
	{
		return c_Value.Data != Data;
	} /*operator!=(const TValueOf<T>&) const*/

	//------------------------------------------------------------------------
	ostream& operator<<(ostream& r_Out) {r_Out << Data; return r_Out;}

protected:
	//------------------------------------------------------------------------
	T Data;
} /*TValueOf<T>*/;

//============================================================================
template <class T> void std_print(

	const T& r
	
){
	cout << r << endl;
} /*std_print<T>(const T&)*/

//----------------------------------------------------------------------------
void std_print_int(const int& i) {std_print(i);}

//============================================================================
void ListExample()
{
//#ifdef NEVER
	// Test list of integers
	//-----------------------
	{
		TDefaultComparatorOf<int> Comparator;
		TPrintContainerOf<int> Print("", " ");
		TListOf<int> List;

		List.Insert(12);
		List.Insert(7);
		List.Insert(10);
		List.Insert(9);
		List.Insert(8);
		List.Insert(3);
		List.Insert(2);
		List.Insert(1);
		List.Insert(1);
		List.Insert(5);

		List.Sort(Comparator);
		Print(List);

		TWin32Console::Pause(); cout << endl;

		cout << "Test cursor" << endl;
		for (TListCursorOf<int> Cursor(List); Cursor; ++Cursor)
			cout << *Cursor << " ";
		cout << endl;
		TWin32Console::Pause(); cout << endl;

//		cout << "STL compatible" << endl;
//
//		std::for_each(
//			TListConstantCursorOf<int>::Begin(List),
//			TListConstantCursorOf<int>::End(List),
//			std_print_int
//		) /*std::for_each*/;
//
//		TWin32Console::Pause(); cout << endl;
	} /*Test list of integers*/
//#endif NEVER

//#ifdef NEVER
	// Test list of objects
	//----------------------
	{
		typedef TValueOf<int> TValue;
		TPrintContainerOf<TValue> Print("", " ");
		typedef TListOf<TValue> TTestList;

		TTestList List;

		List.Insert(TValue(3));
		List.Clear();
		List.Insert(TValue(3));
		List.Insert(TValue(5));
		List.Insert(TValue(7));

		const TValue* pc_Data = List.Search(TEqualsTo<TValue>(TValue(6)));
		if (pc_Data) cout << "Data value is " << pc_Data->ValueConst() << endl;
	    else cout << "Data not found" << endl;

		TTestList ListNew(List);
		ListNew = List;

		TListConstantCursorOf<TValue> ConstCursor = TListConstantCursorOf<TValue>(ListNew, 1);
		cout << "Data value is " << (*ConstCursor).ValueConst() << endl;

		ListNew.ForEach(TAddOf<TValue>(TValue(1)));
		Print(ListNew);

		cout << "Data value is " << (*(ConstCursor++)).ValueConst() << endl;

		TListCursorOf<TValue> Cursor(ListNew, 2);
		TListCursorOf<TValue> Cursor1(ListNew, 2);

		ListNew += List;
		Cursor.InsertAfter(TValue(5));
		Cursor1.Delete();
		ListNew.Delete(TEqualsTo<TValue>(TValue(8)), TTestList::indexFirst);

		Print(ListNew);

		cout << "the expression string will cause an exception" << endl;
		cout << "Data value is" << (*Cursor).Value() << endl;
	} /*Test list of objects*/
//#endif NEVER
} /*ListExample()*/
