/****************************************************************************/
/** mtlock.cpp                                                             **/
/** ---------------------------------------------------------------------- **/
/** Demonstrates usage of TWin32Lock and TWin32Thread classes              **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** July 02, 1997 by Ilya Marchevsky                                       **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>
#pragma hdrstop

// Standard includes
//============================================================================
#include <iostream.h>

// Natlib Basics includes
//============================================================================
#include <nlcmptex.h>
#include <nlalloc.h>

// Natlib Basics includes
//============================================================================
#include <nwlock.h>
#include <nwthread.h>
#include <nwconsol.h>

//============================================================================
USING_NATLIB_NAMESPACE

// TTestThread
//============================================================================
class TTestThread : public TWin32Thread {
public:
	//------------------------------------------------------------------------
    NLEXCEPT_DECLARE(TException, TWin32Thread::TException, NATLIB_NOTHING);
    NLEXCEPT_DECLARE_OUT_OF_RANGE(TXOutOfRange, TException, NATLIB_NOTHING);

	//------------------------------------------------------------------------
    TTestThread(const TStdString& c_sText, TLock& r_ConsoleLock) :
    	sText(c_sText), pConsoleLock(&r_ConsoleLock)
    {} /*TTestThread(const TStdString&, TLock&)*/

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED(TTestThread)

protected:
	//------------------------------------------------------------------------
    TStdString sText;
    TLock* pConsoleLock;

private:
    //------------------------------------------------------------------------
	int Run();
} /*TTestThread*/;

// TTestThread implementation
//============================================================================
int TTestThread::Run()
{
    try {
		while (!ShouldTerminate()) {
			// Comment the next line to see garbage on the console
			//-----------------------------------------------------
			TAutoLocker LockConsole(*pConsoleLock, TLock::c_uWaitInfinite);

            TStdString sMessage;
	        sMessage += sText;
	        sMessage += " - transaction - ";
		    sMessage += sText;
            for (int i = 0; i < sMessage.length(); ++i) cout << sMessage[i];
			cout << endl;
	    } /*while*/
    } /*try*/
	catch (TNatlibException& r_Exception) {
    	MessageBox(
        	null, r_Exception.Why().c_str(), "Exception", MB_OK | MB_ICONSTOP
        ) /*MessageBox*/;
	} /*catch (TNatlibException&)*/
    catch (...) {
    	MessageBox(
        	null, "Unknown", "Exception", MB_OK | MB_ICONSTOP
        ) /*MessageBox*/;
    } /*catch (...)*/

    return 0;
} /*TTestThread::Run()*/

//============================================================================
void MultithreadLockExample()
{
	TWin32Lock ConsoleLock;
    TTestThread Thread1("Thread1", ConsoleLock);
	TTestThread Thread2("Thread2", ConsoleLock);

    Thread1.Start();
	Thread2.Start();

    Sleep(10000);

    Thread1.Terminate();
	Thread2.Terminate();

    Thread1.WaitForExit();
	Thread2.WaitForExit();
} /*MultithreadLockExample()*/

