/****************************************************************************/
/** safeh.cpp                                                              **/
/** ---------------------------------------------------------------------- **/
/** Demonstrates usage of Safe Handles                                     **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** March 21, 1997 by Oleg Kagan                                           **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>  // recommended for better compilation time
#pragma hdrstop

// Standard includes
//============================================================================
#include <iostream.h>

// Natlib Basics includes
//============================================================================
#include <nlsafeh.h>

// Natlib Win32 includes
//============================================================================
#include <nwconsol.h>

//============================================================================
USING_NATLIB_NAMESPACE

// TInteger
//============================================================================
class TInteger : public virtual TSafeHandleAllowed {
public:
	//------------------------------------------------------------------------
    TInteger(int nInitValue = 0) : nValue(nInitValue) {}
    //------------------------------------------------------------------------
    TInteger(const TInteger& c_Init) : nValue(c_Init.nValue) {}

	//------------------------------------------------------------------------
    void EvHandleCreated() const;
	//------------------------------------------------------------------------
    void EvHandleDeleted() const;
    //------------------------------------------------------------------------
    operator int() const {return nValue;}
    //------------------------------------------------------------------------
    operator int&() {return nValue;}
    //------------------------------------------------------------------------
    operator const int&() const {return nValue;}

protected:
	//------------------------------------------------------------------------
    int nValue;
} /*TInteger*/;

// TInteger implementation
//============================================================================
void TInteger::EvHandleCreated() const
{
	cout << "Handle created" << endl;
} /*TInteger::EvHandleCreated() const*/

//----------------------------------------------------------------------------
void TInteger::EvHandleDeleted() const
{
	cout << "Handle deleted" << endl;
} /*TInteger::EvHandleDeleted() const*/


//============================================================================
//NLSAFEH_DECLSPEC(TInteger, NATLIB_NOTHING);

//============================================================================
void SafeHandleExample()
{
    TSafeHandleOf<TInteger> hInt;
	cout << "Testing " << typeid(hInt).name() << endl;
	TWin32Console::Pause(); TWin32Console::WriteLn();

	{
    	TInteger I(123);
    	TSafeHandleOf<TInteger> hInt1(I);
		hInt.Assign(I);
        int i = *hInt;
        cout << i << "\n";
//        I.InvalidateHandles();
        cout << int(*hInt) << "\n";
    }
	cout << "The next line shold cause an exception..." << endl;
    cout << (const int&)*hInt << "\n";
} /*SafeHandleExample()*/