/****************************************************************************/
/** stack.cpp                                                              **/
/** ---------------------------------------------------------------------- **/
/** Demonstrates usage of TStackOf template                                **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** March 27, 1997 by Ilya Marchevsky                                      **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>  // recommended for better compilation time
#pragma hdrstop

// Standard includes
//============================================================================
#include <iostream.h>

// Natlib Basics includes
//============================================================================
#include <nlstack.h>
#include <nlqueue.h>
#include <nldeque.h>

// Natlib Win32 includes
//============================================================================
#include <nwconsol.h>

//============================================================================
USING_NATLIB_NAMESPACE

// TBufferPrinterOf template 
//============================================================================
template <class T> class TBufferPrinterOf {
public:
	// TPrintElement
	//------------------------------------------------------------------------
	typedef class TPrintElement : public TConstantOperationOn<T> {
	public:
		//--------------------------------------------------------------------
		typedef TConstantOperationOn<T> TParent;

		//--------------------------------------------------------------------
		NLOBJECT_CLONE_IS_NOT_ALLOWED(TPrintElement)

		//--------------------------------------------------------------------
		void operator()(const T& c_Data) {cout << c_Data << " ";}
	} typedef_TPrintElement;

	//------------------------------------------------------------------------
	void operator()(const TContainerOf<T>& c_Buffer) 
	{
		cout << "===================================================" << endl;
		cout << c_Buffer.Dump().c_str() << endl;
		c_Buffer.ForEach(TPrintElement());
		cout << endl;
	} /*operator()(const TBufferOf<T>&)*/
} /*TBufferPrinterOf<T>*/;

//============================================================================
void StackExample()
{
	TBufferPrinterOf<int> Print;

//#ifdef NEVER
	TStackOf<int> Stack;
	Print(Stack);
	TWin32Console::Pause(); cout << endl;

	{for (int i = 0; i < 9; ++i) Stack.Insert(i);}
	Print(Stack);
//#endif NEVER

//#ifdef NEVER
	TQueueOf<int> Queue;
	Print((TBufferOf<int>&)Queue);
	TWin32Console::Pause(); cout << endl;

	{for (int i = 0; i < 9; ++i) Queue.Insert(i);}
	Print((TBufferOf<int>&)Queue);
//#endif NEVER

//#ifdef NEVER
	TDequeueOf<int> Dequeue;
	Print(Dequeue);
	TWin32Console::Pause(); cout << endl;

	{for (int i = 0; i < 9; ++i) Dequeue.Insert(i);}
	Print(Dequeue); 
//#endif NEVER
} /*StackExample()*/

