/****************************************************************************/
/** storage.cpp                                                            **/
/** ---------------------------------------------------------------------- **/
/** Demonstrates usage of TNatlibStorageOf<Data, Key> template             **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** February 21, 1997 by Oleg Kagan                                        **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>  // recommended for better compilation time
#pragma hdrstop

// Standard includes
//============================================================================
#include <iostream.h>

// Natlib Basics incldues
//============================================================================
#include <nlavlst.h>
#include <nlstring.h>

// Natlib Win32 incldues
//============================================================================
#include <nwconsol.h>

//============================================================================
//NLAVLST_DECLSPEC(int, TNatlibStdString, NATLIB_NOTHING);

//============================================================================
USING_NATLIB_NAMESPACE

//============================================================================
ostream& operator<<(

	ostream& r_Output, 
	const TStdString& c_s
	
){
	return r_Output << c_s.c_str();
} /*operator<<(ostream&, const TStdString&)*/

// TPrintStoragePairOf template
//============================================================================
template <class TData, class TKey> class TPrintStoragePairOf :
	public TStorageConstantOperationOn<TData, TKey>
{
public:
	//------------------------------------------------------------------------
	typedef TStorageConstantOperationOn<TData, TKey> TParent;

	//------------------------------------------------------------------------
    TPrintStoragePairOf() {}
	//------------------------------------------------------------------------
    TPrintStoragePairOf(const TStdString& c_sPrefix) :
    	sPrefix(c_sPrefix)
    {} /*TPrintStoragePairOf(const TStdString& = TStdString())*/

    //------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED2(TPrintStoragePairOf, TData, TKey)

	//------------------------------------------------------------------------
    void SetPrefix(const TStdString& c_sPrefix) {sPrefix = c_sPrefix;}

	//------------------------------------------------------------------------
	void operator()(const TData& c_Data, const TKey& c_Key)
    {
    	cout << sPrefix.c_str() << "[";
        ::operator<<(cout, c_Key);
        cout << "] = " << c_Data << "\n";
    } /*operator()(const TData&, const TKey&)*/

protected:
	//------------------------------------------------------------------------
    TStdString sPrefix;

private:
    //------------------------------------------------------------------------
    void operator()(TData& r_Data, const TKey& c_Key)
    {
    	operator()((const TData&)r_Data, c_Key);
    } /*void operator()(TData&, const TKey&)*/
} /*TPrintStoragePairOf<TData, TKey>*/;

//============================================================================
template <class TData, class TKey> class TPrintStorageOf {
public:
	//------------------------------------------------------------------------
  	void operator()(
		const TStorageOf<TData, TKey>& c_Storage, const TStdString& c_sPrefix
	){
    	// Dump
        //------
		{
    		TStdString sDump(c_Storage.Dump());
			cout << sDump.c_str() << endl;
		} /*Dump*/

        c_Storage.AssertValid();

        PrintPair.SetPrefix(c_sPrefix);

		Fast(c_Storage);
//		Slow(c_Storage);
//		SlowX(c_Storage);
	} /*operator(const TStorageOf<TData, TKey>&, TStdString&)*/

    //------------------------------------------------------------------------
    void Fast(const TStorageOf<TData, TKey>&);
    //------------------------------------------------------------------------
    void Slow(const TStorageOf<TData, TKey>&);
    //------------------------------------------------------------------------
    void SlowX(const TStorageOf<TData, TKey>&);

protected:
	//------------------------------------------------------------------------
    TPrintStoragePairOf<TData, TKey> PrintPair;
} /*TPrintStorageOf<TData, TKey>*/;

// TPrintStorageOf template implementation
//============================================================================
template <class TData, class TKey> void TPrintStorageOf<TData, TKey>::Fast(

	const TStorageOf<TData, TKey>& c_Storage

){
   	c_Storage.Inorder(PrintPair);
} /*TPrintStorageOf<TData, TKey>::Fast(const TStorageOf<TData, TKey>&, ...)*/

//----------------------------------------------------------------------------
template <class TData, class TKey> void TPrintStorageOf<TData, TKey>::Slow(

	const TStorageOf<TData, TKey>& c_Storage

){
	try {
        for (
        	TKey Key = c_Storage.FirstKey();
            true;
            Key = c_Storage.NextKey(Key)
        ){
			PrintPair(c_Storage[Key], Key);
        } /*for (Key)*/
    }
    catch (TStorageOf<TData, TKey>::TXNotFound&) {}
} /*TPrintStorageOf<TData, TKey>::Slow(const TStorageOf<TData, TKey>&, ...)*/

//----------------------------------------------------------------------------
template <class TData, class TKey> void TPrintStorageOf<TData, TKey>::SlowX(

	const TStorageOf<TData, TKey>& c_Storage

){
	try {
        for (
        	TKey Key = c_Storage.LastKey();
            true;
            Key = c_Storage.PreviousKey(Key)
        ){
			PrintPair(c_Storage[Key], Key);
        } /*for (Key)*/
    }
    catch (TStorageOf<TData, TKey>::TXNotFound&) {}
} /*TPrintStorageOf<TData, TKey>::SlowX(const TStorageOf<TData, TKey>&, ...)*/

// TOddOf template
//============================================================================
template <class TData> class TOddOf : public TConditionOf<TData> {
public:
	//------------------------------------------------------------------------
	typedef TConditionOf<TData> TParent;

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED1(TOddOf, TData)

	//------------------------------------------------------------------------
    bool operator()(const TData& c_Data) const {return c_Data % 2;}
} /*TOddOf<TData>*/;

// TNotConditionOf template
//============================================================================
template <class TData> class TNotConditionOf : public TConditionOf<TData> {
public:
	//------------------------------------------------------------------------
	typedef TConditionOf<TData> TParent;

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED1(TNotConditionOf, TData);

	//------------------------------------------------------------------------
    TNotConditionOf(const TConditionOf<TData>& c_BaseCondition) :
    	pc_Condition(&c_BaseCondition)
    {} /*TNotConditionOf(const TConditionOf<TData>&)*/

	//------------------------------------------------------------------------
    bool operator()(const TData& c_Data) const
    {
    	return !pc_Condition->operator()(c_Data);
    } /*operator()(const TData&) const*/

protected:
	//------------------------------------------------------------------------
    const TConditionOf<TData>* pc_Condition;
} /*TNotConditionOf<TData>*/;

//============================================================================
void StorageExample()
{
	TPrintStorageOf<int, TStdString> PrintStorage;
	TDefaultComparatorOf<TStdString> StringComparator;

	TAvlStorageOf<int, TStdString> Storage(StringComparator);

    {
    	cout << "Fill storage..." << endl;
		for (int i = 0; i < 10; ++i) {
        	char szValue[1024]; sprintf(szValue, "_%02d_", i);
            Storage.Assign(i, szValue);
            if (Storage[szValue] != i)
	           	throw TNatlibException(THIS_PLACE);
			Storage.AssertValid();
        } /*for (i)*/

		PrintStorage(Storage, TStdString("Storage"));
        TWin32Console::Pause(); cout << "\n";
    }

	{
    	const char* c_szToUnassign = "_01_";
        cout << "Unassign " << c_szToUnassign << endl;
    	Storage.Unassign(c_szToUnassign);
		PrintStorage(Storage, TStdString("Storage"));
        TWin32Console::Pause(); cout << "\n";
    }

    {
    	cout << "Delete records with odd integers" << endl;
    	TOddOf<int> OddIntegers;
        Storage.Delete(OddIntegers, TStorageOf<int, TStdString>::indexAll);
		PrintStorage(Storage, "Storage");
        TWin32Console::Pause(); cout << "\n";
    }

    {
    	cout << "Delete records with even integers" << endl;
    	TOddOf<int> OddIntegers;
        TNotConditionOf<int> EvenIntegers(OddIntegers);
        Storage.Delete(EvenIntegers, TStorageOf<int, TStdString>::indexAll);
		PrintStorage(Storage, "Storage");
		TWin32Console::Pause(); cout << "\n";
    }
} /*StorageExample()*/
