/****************************************************************************/
/** vector.cpp                                                             **/
/** ---------------------------------------------------------------------- **/
/** Demonstrates usage of TVectorOf template                               **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** July 30, 1997 by Oleg Kagan                                            **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>  // recommended for better compilation time
#pragma hdrstop

// Standard includes
//============================================================================
#include <iostream.h>
#include <stdlib.h>

// Natlib Basics includes
//============================================================================
#include <nlvector.h>

// Natlib Win32 includes
//============================================================================
#include <nwconsol.h>

//============================================================================
USING_NATLIB_NAMESPACE

//============================================================================
template <class T> void PrintVector(

	const TVectorOf<T>& c_Vector

){
	cout << c_Vector.Dump() << endl << endl;
	for (size_t i = 0; i < c_Vector.Size(); ++i)
    	cout << "[" << i << "] = " << c_Vector[i] << endl;
} /*PrintVector<T>(const TVectorOf<T>&)*/

//============================================================================
template <class T>
class TPrintVectorOf : public TStorageConstantOperationOn<T, size_t> {
public:
	//------------------------------------------------------------------------
	typedef TStorageConstantOperationOn<T, size_t> TParent;

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED1(TPrintVectorOf, T);

	//------------------------------------------------------------------------
    void operator()(const T& c_Data, const size_t& c_uIndex)
    {
	    cout << "[" << c_uIndex << "] = " << c_Data << endl;
    } /*operator()(const T&, const size_t&)*/
} /*TPrintVectorOf<T>*/;

//----------------------------------------------------------------------------
template <class T> void PrintVector1(

	const TVectorOf<T>& c_Vector

){
	cout << c_Vector.Dump() << endl << endl;
    c_Vector.Inorder(TPrintVectorOf<T>());
} /*PrintVector1<T>(const TVectorOf<T>&)*/

//----------------------------------------------------------------------------
template <class T> void PrintVector2(

	const TVectorOf<T>& c_Vector

){
	cout << c_Vector.Dump() << endl << endl;
    c_Vector.Postorder(TPrintVectorOf<T>());
} /*PrintVector2<T>(const TVectorOf<T>&)*/

//----------------------------------------------------------------------------
template <class T> void PrintVector3(

	const TVectorOf<T>& c_Vector

){
	try {
		for (
	    	size_t i = c_Vector.FirstKey();
	        true;
	        i = c_Vector.NextKey(i)
	    ){
			TPrintVectorOf<T>().operator()(c_Vector[i], i);
	    } /*for (i)*/
    } /*try*/
    catch (TVectorOf<T>::TXOutOfRange&) {}
} /*PrintVector3<T>(const TVectorOf<T>&)*/

//----------------------------------------------------------------------------
template <class T> void PrintVector4(

	const TVectorOf<T>& c_Vector

){
	try {
		for (
	    	size_t i = c_Vector.LastKey();
	        true;
	        i = c_Vector.PreviousKey(i)
	    ){
			TPrintVectorOf<T>().operator()(c_Vector[i], i);
	    } /*for (i)*/
    } /*try*/
    catch (TVectorOf<T>::TXOutOfRange&) {}
} /*PrintVector4<T>(const TVectorOf<T>&)*/

//============================================================================
#define VECTOR_PRINT_VECTOR(Vector) PrintVector4(Vector);

//============================================================================
class TVectorTester {
public:
	//------------------------------------------------------------------------
    typedef class TComparator :
    	public TPointerSavingEtalonComparatorOf<TVectorTester>
    {
    public:
    	//--------------------------------------------------------------------
        basic_TComparator::TResult operator()(
			const TVectorTester& c_Tester
		) const {
        	if (Etalon().NumericValue() < c_Tester.NumericValue())
            	return compLess;
			if (Etalon().NumericValue() > c_Tester.NumericValue())
            	return compGreater;

//			if (Etalon().StringValue() < c_Tester.StringValue())
//            	return compLess;
//			if (Etalon().StringValue() > c_Tester.StringValue())
//            	return compGreater;

            return compEqual;
        } /*operator()(const TVectorTester&) const*/
        //--------------------------------------------------------------------
		NLOBJECT_DECLARE_CLONE(TComparator);
    } typedef_TComparator;

	//------------------------------------------------------------------------
    static bool bLogging;
	//------------------------------------------------------------------------
    long uValue;
    TStdString sValue;
    TStdString sValue1;
    void* pVoid; 

	//------------------------------------------------------------------------
    TVectorTester(long uInitValue = 0) : uValue(uInitValue)
    {
       	char szValue[256]; sprintf(szValue, "%04ld", uInitValue);
        sValue = szValue;
        sValue1 = szValue;
    } /*TVectorTester(long = 0)*/
	//------------------------------------------------------------------------
    TVectorTester(const TVectorTester& c_Source) :
    	uValue(c_Source.uValue),
        sValue(c_Source.sValue),
        sValue1(c_Source.sValue1),
        pVoid (c_Source.pVoid)
    {
    	if (bLogging) {
	    	cout << typeid(*this).name() << "(" << uValue << ")" << endl;
        } /*if*/
    } /*TVectorTester(const TVectorTester&)*/
	//------------------------------------------------------------------------
    ~TVectorTester()
    {
    	if (bLogging) {
	    	cout << "~" << typeid(*this).name() << "(" << uValue << ")";
            cout << endl;
        } /*if*/
    } /*~TVectorTester()*/

	//------------------------------------------------------------------------
    long NumericValue() const {return uValue;}
    //------------------------------------------------------------------------
    const TStdString& StringValue() const {return sValue;}

    //------------------------------------------------------------------------
    TVectorTester& operator=(const TVectorTester& c_Source)
    {
    	if (bLogging) {
	        cout << typeid(*this).name() << "(" << uValue << ")";
        } /*if*/
    	uValue = c_Source.uValue;
        sValue = c_Source.sValue;
        if (bLogging) {
	    	cout << " = " << typeid(*this).name() << "(" << uValue << ")";
            cout << endl;
        } /*if*/
        return *this;
    } /*operator=(const TVectorTester& c_Source)*/
} /*TVectorTester*/;

// TVectorTester::TComparator implementation
//============================================================================
NLOBJECT_CLONE_RESULT(TVectorTester::TComparator)
TVectorTester::TComparator::Clone() const
{
	TComparator* pCopy;
    NLEXCEPT_CLASS_NEW(TComparator, pCopy);
    return pCopy;
} /*TVectorTester::TComparator::Clone() const*/

// TVectorTester implementation
//============================================================================
bool TVectorTester::bLogging = true;

//============================================================================
ostream& operator<<(ostream& r_Output, const TVectorTester& c_Tester)
{
   	r_Output << c_Tester.NumericValue();
    return r_Output;
} /*operator<<(ostream&, const TVectorTester&)*/

//============================================================================
NLVECTOR_DECLSPEC(TVectorTester, NATLIB_NOTHING);

//============================================================================
void CheckOrder(

	const TVectorOf<TVectorTester>& c_Vector,
    TEtalonComparatorOf<TVectorTester>& r_SortComparator

){
	if (!c_Vector.Size()) return;
	for (size_t i = 0; i < (c_Vector.Size() - 1); ++i) {
    	r_SortComparator.SetEtalon(c_Vector[i]);
        if (r_SortComparator(c_Vector[i + 1]) == TComparator::compGreater)
        	throw TNatlibException(THIS_PLACE);
    } /*for (i)*/
} /*CheckOrder(	const TVectorOf<TVectorTester>&, ...)*/

//============================================================================
long nSwaps = 0;

//----------------------------------------------------------------------------
void QuickSortComparison()
{
	srand(1234);
	TVectorTester::bLogging = false;
    TVectorTester::TComparator SortComparator;

	const size_t c_nTestPasses = 100;
    const size_t c_nTestSize = 5000;
	long double fDirectTime = 0;
    long double fDirectSwaps = 0;
    long nDirectSwapsMax = 0;
    long nDirectSwapsMin = c_nTestSize;
    long double fVirtualTime = 0;
    long double fVirtualSwaps = 0;
    long nVirtualSwapsMax = 0;
    long nVirtualSwapsMin = c_nTestSize;

    for (size_t nPass = 1; nPass <= c_nTestPasses; ++nPass) {
    	cout << "----- Pass " << nPass << " -----" << endl;
    	TVectorOf<TVectorTester> Vector(TVectorTester(), c_nTestSize);
        for (size_t i = 0; i < Vector.Size(); ++i) {
			TVectorTester Random(
				(long double)rand() * (c_nTestSize << 1) / RAND_MAX
			) /*Random*/;
        	Vector[i] = Random;
        } /*for (i)*/

        // Test direct sorting
        //---------------------
        {
        	cout << "Direct sorting....";
        	TVectorOf<TVectorTester> VectorToSort(Vector);
//			VECTOR_PRINT_VECTOR(VectorToSort); 
//			TWin32Console::Pause(); TWin32Console::WriteLn();
			nSwaps = 0;
            long double fStartTick = GetTickCount();
            VectorToSort.QuickSortDirect(
            	SortComparator, 0, VectorToSort.Size() - 1
            ) /*QuickSortDirect*/;
            long double fFinishTick = GetTickCount();
            fDirectTime += (fFinishTick - fStartTick);
            fDirectSwaps += nSwaps;
		    if (nDirectSwapsMax < nSwaps) nDirectSwapsMax = nSwaps;
		    if (nDirectSwapsMin > nSwaps) nDirectSwapsMin = nSwaps;

//			VECTOR_PRINT_VECTOR(VectorToSort); 
//			TWin32Console::Pause(); TWin32Console::WriteLn();

			CheckOrder(VectorToSort, SortComparator);
            cout << "OK (";
            cout << (fFinishTick - fStartTick) / 1000;
            cout << " sec)" << endl;
		} /*Test direct sorting*/

        // Test virtual sorting
        //---------------------
        {
        	cout << "Virtual sorting...";
        	TVectorOf<TVectorTester> VectorToSort(Vector);

//			VECTOR_PRINT_VECTOR(VectorToSort); 
//			TWin32Console::Pause(); TWin32Console::WriteLn();

			nSwaps = 0;
            long double fStartTick = GetTickCount();
            VectorToSort.QuickSortVirtual(
				SortComparator, 0, VectorToSort.Size() - 1
            ) /*QuickSortVirtual*/;
            long double fFinishTick = GetTickCount();
            fVirtualTime += (fFinishTick - fStartTick);
            fVirtualSwaps += nSwaps;
		    if (nVirtualSwapsMax < nSwaps) nVirtualSwapsMax = nSwaps;
		    if (nVirtualSwapsMin > nSwaps) nVirtualSwapsMin = nSwaps;

//			VECTOR_PRINT_VECTOR(VectorToSort); 
//			TWin32Console::Pause(); TWin32Console::WriteLn();

			CheckOrder(VectorToSort, SortComparator);
            cout << "OK (";
            cout << (fFinishTick - fStartTick) / 1000;
            cout << " sec)" << endl;
		} /*Test virtual sorting*/
    } /*for (nPass)*/

	cout << "--------------------------" << endl;
   	cout << "Size    : " << c_nTestSize << endl;
	cout << "Direct  : " << fDirectTime / c_nTestPasses / 1000;
	cout << " Swaps : " << fDirectSwaps / c_nTestPasses;
	cout << " Min : " << nDirectSwapsMin << " Max : " << nDirectSwapsMax;
	cout << endl;
	cout << "Virtual : " << fVirtualTime / c_nTestPasses / 1000;
	cout << " Swaps : " << fVirtualSwaps / c_nTestPasses;
	cout << " Min : " << nVirtualSwapsMin << " Max : " << nVirtualSwapsMax;
	cout << endl;
    cout << fDirectTime / fVirtualTime << endl;
} /*QuickSortComparison()*/

//============================================================================
void VectorExample()
{
//#ifdef NEVER
	QuickSortComparison();
    return;
//#endif /*NEVER*/

	TVectorTester::bLogging = true;

	TVectorOf<TVectorTester> Vector(999, 3);
//	TWin32Console::Pause(); TWin32Console::WriteLn();

    VECTOR_PRINT_VECTOR(Vector);
//	TWin32Console::Pause(); TWin32Console::WriteLn();

    // Fill vector
    //-------------
    {for (size_t i = 0; i < Vector.Size(); ++i) Vector[i] = long(i);}
//	TWin32Console::Pause(); TWin32Console::WriteLn();

    VECTOR_PRINT_VECTOR(Vector);
//	TWin32Console::Pause(); TWin32Console::WriteLn();

    TVectorOf<TVectorTester> Vector1(111, 2);
//	TWin32Console::Pause(); TWin32Console::WriteLn();

    Vector1 = Vector;
//	TWin32Console::Pause(); TWin32Console::WriteLn();

    VECTOR_PRINT_VECTOR(Vector1);
//	TWin32Console::Pause(); TWin32Console::WriteLn();

    Vector1.Resize(3, 6, 999, true);
    VECTOR_PRINT_VECTOR(Vector1);
//	TWin32Console::Pause(); TWin32Console::WriteLn();

    Vector1.Resize(4, 999);
    VECTOR_PRINT_VECTOR(Vector1);
//	TWin32Console::Pause(); TWin32Console::WriteLn();

    Vector1.Resize(2, 999);
    VECTOR_PRINT_VECTOR(Vector1);
//	TWin32Console::Pause(); TWin32Console::WriteLn();

    Vector1.Resize(1, 999);
    VECTOR_PRINT_VECTOR(Vector1);
//	TWin32Console::Pause(); TWin32Console::WriteLn();

    Vector1.Clear();
//	TWin32Console::Pause(); TWin32Console::WriteLn();

	Vector1.Resize(5, 5, 12, true);
    Vector1[0] = 4;
    Vector1[1] = 3;
    Vector1[2] = 0;
    Vector1[3] = 1;
    Vector1[4] = 2;
    Vector1.QuickSort(TVectorTester::TComparator());
    VECTOR_PRINT_VECTOR(Vector1);
//	TWin32Console::Pause(); TWin32Console::WriteLn();

    cout << "Testing Insert/Delete capability" << endl;
    TWin32Console::Pause(); cout << endl;

    Vector.Resize(0, 0, 0, true);
    {
		for (size_t i = 0; i < 20; ++i) {
    		Vector.Insert(i);
			cout << "Size: " << Vector.Size();
			cout << "\t Capacity: " << Vector.Capacity() << endl;
			TWin32Console::Pause(); cout << endl;
		} /*for (i)*/
	}
	VECTOR_PRINT_VECTOR(Vector);
    TWin32Console::Pause(); cout << endl;

    while (Vector.Size()) {
    	Vector.Delete(
        	TAnyOf<TVectorTester>(), TContainerOf<TVectorTester>::indexFirst
        ) /*Delete*/;
        cout << "Size: " << Vector.Size();
        cout << "\t Capacity: " << Vector.Capacity() << endl;
		TWin32Console::Pause(); cout << endl;
    } /*while*/
	VECTOR_PRINT_VECTOR(Vector);

    Vector.Insert(10);
//	TWin32Console::Pause(); TWin32Console::WriteLn();
} /*VectorExample()*/



