/****************************************************************************/
/** nltask.cpp                                                             **/
/** ---------------------------------------------------------------------- **/
/** Demonstrates usage Natlib Tasks                                        **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** August 19, 1997 by Oleg Kagan                                          **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>  // recommended for better compilation time
#pragma hdrstop

// Standard includes
//============================================================================
#include <iostream.h>
#include <conio.h>

// Natlib Basics includes
//============================================================================
#include <nlobject.h>
#include <nlstring.h>

// Natlib Tasks includes
//============================================================================
#include <ntasks.h>
#include <ntsched.h>

// Natlib Win32 includes
//============================================================================
#include <nwconsol.h>

//============================================================================
USING_NATLIB_NAMESPACE

// TMyScheduler
//============================================================================
class TMyScheduler : public TScheduler {
public:
	//------------------------------------------------------------------------
	TMyScheduler() : TAllocationAware(this) {}
	//------------------------------------------------------------------------
	void ShowTaskException(
		TTask& r_Task, TTask::TPhase Phase, 
		const TNatlibException* pc_Exception = null
	){
		cout << "Exception at task " << r_Task.Name().c_str() << endl;
		cout << "\tPhase     : " << Phase << endl;
		if (pc_Exception) 
			cout << "\tException : " << pc_Exception->Why().c_str() << endl;
	} /*ShowTaskException(TTask&, TTask::TPhase, const TNatlibException*)*/
} /*TMyScheduler*/;

// TReadKeys
//============================================================================
class TReadKeys : public TTask {
public:
	//------------------------------------------------------------------------
    typedef TTask TParent;

	//------------------------------------------------------------------------
	typedef class TKeySignal : public TSignal {
	public:
		//--------------------------------------------------------------------
		typedef TSignal TParent;
		//--------------------------------------------------------------------
		char cKey;
		//--------------------------------------------------------------------
		TKeySignal(char cInitKey) : TAllocationAware(this), cKey(cInitKey) {}
	} typedef_TKeySignal;
	
	//------------------------------------------------------------------------
	TReadKeys(const TStdString& c_sName, const THandle& c_hKeyProcessor):
		TAllocationAware(this), TParent(c_sName), 
		a_hKeyProcessors(c_hKeyProcessor)
	{} /*TReadKeys(const TStdString&)*/

    //------------------------------------------------------------------------
    void EvTransaction(const TSignal*)
    {
		if (kbhit()) {
			char cKey = char(getch());
			Send(*new TKeySignal(cKey), a_hKeyProcessors);
		} /*if*/
    } /*EvTransaction(const TSignal*)*/

	//------------------------------------------------------------------------
	TVectorOf<THandle> a_hKeyProcessors;
} /*TReadKeys*/;

// TTerminateByAnyKey
//============================================================================
class TTerminateByKey : public TTask {
public:
	//------------------------------------------------------------------------
    typedef TTask TParent;

    //------------------------------------------------------------------------
    TTerminateByKey(
		const TStdString& c_sName, 
		const TStdString& c_sKeys, bool bInitStartKeyReader = true
	):
    	TAllocationAware(this), TParent(c_sName), 
		sKeys(c_sKeys), bStartKeyReader(bInitStartKeyReader)
    {} /*TTerminateByKey(const TStdString&, const TStdString&)*/

    //------------------------------------------------------------------------
	void EvStartup()
	{
		TParent::EvStartup();

		if (bStartKeyReader) {
			TReadKeys* pReadKeysTask;

			NLEXCEPT_CLASS_NEW(
				TReadKeys("ReadKeys", THandle(*this)), pReadKeysTask
			) /*NLEXCEPT_CLASS_NEW*/;
			Start(*pReadKeysTask);

			for (size_t i = 0; i < 5; ++i) {
				TStdString sNewName(Format("%s%02d", Name().c_str(), i));
				TTask* pTask;
				NLEXCEPT_CLASS_NEW(
					TTerminateByKey(sNewName, sKeys, false), pTask
				) /*NLEXCEPT_CLASS_NEW*/; 
				Start(*pTask);
				pReadKeysTask->a_hKeyProcessors += THandle(*pTask);
			} /*for */
		} /*if*/

		Wait();
	} /*EvStartup()*/

	//------------------------------------------------------------------------
    void EvTransaction(const TSignal*);

	//------------------------------------------------------------------------
	void EvException(
		TPhase Phase, const TNatlibException* pException
	){
		Scheduler().ShowTaskException(*this, Phase, pException);
	} /*EvExceptionv(TPhase, const TNatlibException*)*/;

protected:
	//------------------------------------------------------------------------
	TStdString sKeys;
	bool bStartKeyReader;
} /*TTerminateByKey*/;

// TTerminateByKey implementation
//============================================================================
void TTerminateByKey::EvTransaction(

	const TSignal* pc_Signal
	
){
	if (pc_Signal) {
		union {
			const TReadKeys::TKeySignal* pc_KeySignal;
		} /*union*/;

		if (
			(pc_KeySignal =
				dynamic_cast<const TReadKeys::TKeySignal*>(pc_Signal)
			) /*pc_KeySignal*/ != null
		){
			if (Name().find("03") != npos)
				throw TNatlibException(THIS_CLASS, "He-he-he");
			char cKey = pc_KeySignal->cKey;
			cout << Name().c_str() << "\t";
			cout << "Key Processing transaction " << "<" << cKey << ">";
			cout << endl;
			if (
				!sKeys.length() || (sKeys.find(cKey) != npos)
			){
				cout << Name().c_str() << "\t";
				cout << "<" << cKey << "> have been pressed to terminate";
				cout << endl;
				GlobalTerminate();
			} /*if*/
		}
		else {
			cout << Name().c_str() << "\t";
			cout << "Unknown signal transaction" << endl;
		} /*iff*/
	}
	else {
		cout << Name().c_str() << "\t";
		cout << "Background transaction" << endl;
	} /*iff*/
	Wait();
} /*TTerminateByKey::EvTransaction(const TSignal*)*/

//============================================================================
void TasksExample()
{
	TMyScheduler Scheduler;
	TTerminateByKey TerminateByKey("TerminateByKey", "q");
	Scheduler.Start(TerminateByKey);
	while (!Scheduler.IsFinished())
		Scheduler.EvTransaction();
} /*TasksExample()*/
