#ifndef __NATLIB_NLATOM_H__
#define __NATLIB_NLATOM_H__
/****************************************************************************/
/** nlatom.h                                                               **/
/** ---------------------------------------------------------------------- **/
/** Atom mechanism implementation                                          **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** January 25, 1996 by Oleg Kagan                                         **/
/****************************************************************************/

//============================================================================
#include <nlsafeh.h>
#include <nlavltre.h>
#include <nlautopt.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

//============================================================================
#define NLATOM_ALLOCATOR_DEFAULT NLAVLTRE_ALLOCATOR_DEFAULT

// TAtomMasterRecordOf template
//============================================================================
template <class T> class _NATLIB_TEMPLATE_DECLSPEC_ TAtomMasterRecordOf {
public:
	//------------------------------------------------------------------------
	T Value;
	nl_ulong_t uReferences;

	//------------------------------------------------------------------------
	TAtomMasterRecordOf(const T& c_Value) : 
		Value(c_Value), uReferences(0)
	{} /*TAtomMasterRecordOf(const T&)*/
	//------------------------------------------------------------------------
	TAtomMasterRecordOf(const TAtomMasterRecordOf<T>& c_Source) : 
		Value(c_Source.Value), uReferences(c_Source.uReferences)
	{} /*TAtomMasterRecordOf(const TAtomMasterRecordOf<T>&)*/
} /*TAtomMasterRecordOf<T>*/;

// TAtomRecordSortingComparatorOf template
//============================================================================
template <class T>
class _NATLIB_TEMPLATE_DECLSPEC_ TAtomRecordSortingComparatorOf :
	public TPointerSavingEtalonComparatorOf< TAtomMasterRecordOf<T> >
{
public:
	//------------------------------------------------------------------------
	typedef TPointerSavingEtalonComparatorOf< TAtomMasterRecordOf<T> > 
		TParent;
	//------------------------------------------------------------------------
	typedef TEtalonComparatorOf<T> TValueComparator;

	//------------------------------------------------------------------------
	TAtomRecordSortingComparatorOf(const TValueComparator&);
	//------------------------------------------------------------------------
	TAtomRecordSortingComparatorOf(const TAtomRecordSortingComparatorOf&);
	//------------------------------------------------------------------------
	~TAtomRecordSortingComparatorOf();

	//------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE1(TAtomRecordSortingComparatorOf, T);

	//------------------------------------------------------------------------
	inline void SetEtalon(const TAtomMasterRecordOf<T>&);
	//------------------------------------------------------------------------
	inline void ClearEtalon();

	//------------------------------------------------------------------------
	const TValueComparator& ValueComparatorConst() const 
	{
		return *pValueComparator;
	} /*ValueComparatorConst() const*/

	//------------------------------------------------------------------------
	inline 
	TComparator::TResult operator()(const TAtomMasterRecordOf<T>&) const;

protected:
	//------------------------------------------------------------------------
	TValueComparator* pValueComparator;
} /*TAtomRecordSortingComparatorOf<T>*/;

// TAtomRecordSearchComparatorOf template
//============================================================================
template <class T>
class _NATLIB_TEMPLATE_DECLSPEC_ TAtomRecordSearchComparatorOf :
	public TComparatorOf< TAtomMasterRecordOf<T> >
{
public:
	//------------------------------------------------------------------------
	typedef TPointerSavingEtalonComparatorOf< TAtomMasterRecordOf<T> > 
		TParent;
	//------------------------------------------------------------------------
	typedef TComparatorOf<T> TValueComparator;

	//------------------------------------------------------------------------
	TAtomRecordSearchComparatorOf(const TValueComparator&);
	//------------------------------------------------------------------------
	~TAtomRecordSearchComparatorOf();

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED1(TAtomRecordSearchComparatorOf, T);

	//------------------------------------------------------------------------
	inline 
	TComparator::TResult operator()(const TAtomMasterRecordOf<T>&) const;

protected:
	//------------------------------------------------------------------------
	const TValueComparator* pc_ValueComparator;
} /*TAtomRecordSearchComparatorOf<T>*/;

// TAtomRecordPerformDataOperationOf template 
//============================================================================
template <class T> 
class _NATLIB_TEMPLATE_DECLSPEC_ TAtomRecordPerformDataOperationOf :
	public TOperationOn< TAtomMasterRecordOf<T> >
{
public:
	//------------------------------------------------------------------------
	typedef TOperationOn< TAtomMasterRecordOf<T> > TParent;

	//------------------------------------------------------------------------
	TAtomRecordPerformDataOperationOf(TOperationOn<T>& r_Operation) :
		pOperation(&r_Operation)
	{} /*TAtomRecordPerformDataOperationOf(TOperationOf<T>&)*/

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED1(TAtomRecordPerformDataOperationOf, T)

	//------------------------------------------------------------------------
	void operator()(TAtomMasterRecordOf<T>& c_Record)
	{
		pOperation->operator()(c_Record.Value);
	} /*operator()(const TAtomMasterRecordOf<T>&)*/

protected:
	//------------------------------------------------------------------------
	TOperationOn<T>* pOperation;
} /*TAtomRecordPerformDataOperationOf<T>*/;

// TAtomRecordPerformDataOperationOf template 
//============================================================================
template <class T> 
class _NATLIB_TEMPLATE_DECLSPEC_ TAtomRecordPerformDataConstantOperationOf :
	public TConstantOperationOn< TAtomMasterRecordOf<T> >
{
public:
	//------------------------------------------------------------------------
	typedef TConstantOperationOn< TAtomMasterRecordOf<T> > TParent;

	//------------------------------------------------------------------------
	TAtomRecordPerformDataConstantOperationOf(
		TConstantOperationOn<T>& r_Operation
	):
		pOperation(&r_Operation)
	{} /*TAtomRecordPerformDataConstantOperationOf(TConstantOperationOf&)*/

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED1(
		TAtomRecordPerformDataConstantOperationOf, T
	) /*NLOBJECT_CLONE_IS_NOT_ALLOWED1*/

	//------------------------------------------------------------------------
	void operator()(const TAtomMasterRecordOf<T>& c_Record)
	{
		pOperation->operator()(c_Record.Value);
	} /*operator()(const TAtomMasterRecordOf<T>&)*/

protected:
	//------------------------------------------------------------------------
	TConstantOperationOn<T>* pOperation;
} /*TAtomRecordPerformDataConstantOperationOf<T>*/;

// TAtomOf template
//============================================================================
template <class T> class _NATLIB_TEMPLATE_DECLSPEC_ TAtomOf : 
	public TNatlibObject
{
public:
	//------------------------------------------------------------------------
	typedef TNatlibObject TParent;

	//------------------------------------------------------------------------
	NLEXCEPT_DECLARE(
		TException, TNatlibObject::TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
	NLEXCEPT_DECLARE(
		TXInvalidReferenceCounter, TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
	NLEXCEPT_DECLARE(
		TXInvalidRecordPointer, TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
	NLEXCEPT_DECLARE(
		TXNotAssigned, TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
	NLEXCEPT_DECLARE_LOW_MEMORY(
		TXLowMemory, TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE_LOW_MEMORY*/;

	//------------------------------------------------------------------------
	TAtomOf();
	//------------------------------------------------------------------------
	TAtomOf(const TAtomOf<T>&);
	//------------------------------------------------------------------------
	~TAtomOf();

	//------------------------------------------------------------------------
	void AssertValid() const;
	//------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE1(TAtomOf, T);

	//------------------------------------------------------------------------
	void Clear();

	//------------------------------------------------------------------------
	const T& Value(nl_ulong_t& r_uReferences) const;
	//------------------------------------------------------------------------
	const T& Value() const;
	//------------------------------------------------------------------------
	nl_ulong_t References() const;
	
	//------------------------------------------------------------------------
	void Let(const TAtomOf<T>&);
	// Can be used for assigned atoms only 
	//------------------------------------------------------------------------
	void Let(const T&);

	//------------------------------------------------------------------------
	TComparator::TResult Compare(const TAtomOf<T>& c_Atom) const
	{
		if (nl_ulong_t(pc_Record) > nl_ulong_t(c_Atom.pc_Record))
			return TComparator::compGreater;
		if (nl_ulong_t(pc_Record) < nl_ulong_t(c_Atom.pc_Record))
			return TComparator::compLess;
		return TComparator::compEqual;
	} /*Compare(const TAtomOf<T>&) const*/

	//------------------------------------------------------------------------
	TAtomOf<T>& operator=(const TAtomOf<T>& c_Source)
	{
		Let(c_Source); return *this;
	} /*operator=(const TAtomOf<T>&)*/
	//------------------------------------------------------------------------
	TAtomOf<T>& operator=(const T& c_Value)
	{
		Let(c_Value); return *this;
	} /*operator=(const T&)*/

	//------------------------------------------------------------------------
	const T& operator*() const {return Value();}
	//------------------------------------------------------------------------
	operator const T&() {return Value();}

	//------------------------------------------------------------------------
	bool operator==(const TAtomOf<T>& c_Atom) const
	{
		return pc_Record == c_Atom.pc_Record;
	} /*operator==(const TAtomOf<T>&) const*/
	//------------------------------------------------------------------------
	bool operator!=(const TAtomOf<T>& c_Atom) const
	{
		return pc_Record != c_Atom.pc_Record;
	} /*operator!=(const TAtomOf<T>&) const*/
	//------------------------------------------------------------------------
	bool operator>(const TAtomOf<T>& c_Atom) const
	{
		return nl_ulong_t(pc_Record) > nl_ulong_t(c_Atom.pc_Record);
	} /*operator>(const TAtomOf<T>&) const*/
	//------------------------------------------------------------------------
	bool operator<(const TAtomOf<T>& c_Atom) const
	{
		return nl_ulong_t(pc_Record) < nl_ulong_t(c_Atom.pc_Record);
	} /*operator<(const TAtomOf<T>&) const*/
	//------------------------------------------------------------------------
	bool operator>=(const TAtomOf<T>& c_Atom) const
	{
		return nl_ulong_t(pc_Record) >= nl_ulong_t(c_Atom.pc_Record);
	} /*operator>=(const TAtomOf<T>&) const*/
	//------------------------------------------------------------------------
	bool operator<=(const TAtomOf<T>& c_Atom) const
	{
		return nl_ulong_t(pc_Record) <= nl_ulong_t(c_Atom.pc_Record);
	} /*operator<=(const TAtomOf<T>&) const*/

protected:
	//------------------------------------------------------------------------
	TSafeHandleBase hMaster;
	const TAtomMasterRecordOf<T>* pc_Record;

public:
	// TFriend
	//------------------------------------------------------------------------
	typedef class _NATLIB_TEMPLATE_DECLSPEC_ TFriend {
	protected:
		//--------------------------------------------------------------------
		static TSafeHandleBase& MasterHandle(TAtomOf<T>& r_Atom)
		{
			return r_Atom.hMaster;
		} /*MasterHandle(TAtomOf<T>&)*/
		//--------------------------------------------------------------------
		static const TSafeHandleBase& MasterHandleConst(
			const TAtomOf<T>& c_Atom
		){
			return c_Atom.hMaster;
		} /*MasterHandleConst(const TAtomOf<T>&)*/
		//--------------------------------------------------------------------
		static const TAtomMasterRecordOf<T>*& RecordPtr(
			TAtomOf<T>& r_Atom
		){
			return r_Atom.pc_Record;
		} /*RecordPtr(TAtomOf<T>&)*/
		//--------------------------------------------------------------------
		static const TAtomMasterRecordOf<T>* RecordPtrConst(
			const TAtomOf<T>& c_Atom
		){
			return c_Atom.pc_Record;
		} /*RecordPtrConst(const TAtomOf<T>&)*/
	} typedef_TFriend;
	friend TFriend;
} /*TAtomOf<T>*/;

// TAtomDefaultComparatorOf template
//============================================================================
template <class T> class _NATLIB_TEMPLATE_DECLSPEC_ TAtomDefaultComparatorOf :
	public TPointerSavingEtalonComparatorOf< TAtomOf<T> >,
	protected TAtomOf<T>::TFriend
{
	//------------------------------------------------------------------------
	typedef TPointerSavingEtalonComparatorOf< TAtomOf<T> > TParent;

	//------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE1(TAtomDefaultComparatorOf, T);

	//------------------------------------------------------------------------
	inline TComparator::TResult operator()(const TAtomOf<T>&) const;
} /*TAtomDefaultComparatorOf<T>*/;

// TAtomMasterOf template
//============================================================================
template <class T> class _NATLIB_TEMPLATE_DECLSPEC_ TAtomMasterOf : 
	public TSortingContainerOf<T>,
	public TSafeHandleAllowed,
	protected TAtomOf<T>::TFriend
{
public:
	//------------------------------------------------------------------------
	typedef TSortingContainerOf<T> TParent;

	//------------------------------------------------------------------------
	typedef TAtomOf<T>::TException TException;

	//------------------------------------------------------------------------
	TAtomMasterOf(
		const TEtalonComparatorOf<T>&,
		bool bInitDeleteUnusedRecords = true,
		const TAllocator& = NLATOM_ALLOCATOR_DEFAULT()
	) /*TAtomMasterOf(const TEtalonComparatorOf<T>&, const TAllocator&)*/;
	//------------------------------------------------------------------------
	TAtomMasterOf(const TAtomMasterOf<T>&);
	//------------------------------------------------------------------------
	~TAtomMasterOf();

	//------------------------------------------------------------------------
	void AssertValid() const;
	//------------------------------------------------------------------------
	TNatlibStdString Dump() const;
	//------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE1(TAtomMasterOf, T);

	//------------------------------------------------------------------------
	TAtomOf<T> CreateAtom(const T&);
	//------------------------------------------------------------------------
	TAtomOf<T>& Assign(TAtomOf<T>&, const T&);

	//------------------------------------------------------------------------
	TSize UnreferencedRecords() const {return nUnreferencedRecords;}

	// Deletes specified number of unused records
	// (returns number of really deleted records)
	//------------------------------------------------------------------------
	TSize CollectGarbage(TSize nToCollect);

	// TContainerOf specific operations
	//------------------------------------------------------------------------
	TSize Size() const {return Tree.Size();}

	//------------------------------------------------------------------------
	void ForEach(TOperation&);
	//------------------------------------------------------------------------
	void ForEach(TConstantOperation&) const;

	// TSortingContainerOf specific operations
	//------------------------------------------------------------------------
	const T* FastSearch(
		const TComparatorOf<T>& , TIndex = indexAny
	) const /*FastSearch(const TComparatorOf<T>&, TIndex)*/;
  	//------------------------------------------------------------------------
    const T* FastSearch(const T& c_Value) const
    {
    	return TParent::FastSearch(c_Value);
    } /*FastSearch(const T&) const*/
	//------------------------------------------------------------------------
	TSize FastExist(const TComparatorOf<T>& c_Comparator) const
	{
		return FastSearch(c_Comparator) != null;
	} /*FastExist(const TComparatorOf<T>&) const*/
	//------------------------------------------------------------------------
	TSize FastExist(const T& c_Data) const
	{
		return FastSearch(c_Data) != null;
	} /*FastExist(const T&) const*/

	//------------------------------------------------------------------------
	const T* SearchFirst() const;
	//------------------------------------------------------------------------
	const T* SearchLast() const;
	//------------------------------------------------------------------------
	const T* SearchNext(const T&) const;
	//------------------------------------------------------------------------
	const T* SearchPrevious(const T&) const;
	//------------------------------------------------------------------------
	void Inorder(TOperationOn<T>&);
	//------------------------------------------------------------------------
	void Inorder(TConstantOperationOn<T>&) const;

protected:
	//------------------------------------------------------------------------
	friend TAtomOf<T>;

	//------------------------------------------------------------------------
	static TAtomMasterOf<T>* GetMasterPtr(TAtomOf<T>& r_Atom)
	{
		TSafeHandleBase& r_hMaster = MasterHandle(r_Atom);
		return (
			r_hMaster.IsTargetValid() ?
				dynamic_cast<TAtomMasterOf<T>*>(&r_hMaster.Dereference()):
				null
		) /*return*/;
	} /*TAtomMasterOf<T>* GetMasterPtr(TAtomOf<T>&)*/

	//------------------------------------------------------------------------
	static const TAtomMasterOf<T>* GetMasterPtrConst(const TAtomOf<T>& r_Atom)
	{
		return GetMasterPtr(const_cast<TAtomOf<T>&>(r_Atom));
	} /*TAtomMasterOf<T>* GetMasterPtr(TAtomOf<T>&)*/

	//------------------------------------------------------------------------
	TAvlTreeOf< TAtomMasterRecordOf<T> > Tree;
	bool bDeleteUnusedRecords;
	TAvlTreeOf< TAtomMasterRecordOf<T> >::TSize nUnreferencedRecords;

	//------------------------------------------------------------------------
	const TAtomMasterRecordOf<T>* AddReference(const TAtomMasterRecordOf<T>*);
	//------------------------------------------------------------------------
	const TAtomMasterRecordOf<T>* Release(const TAtomMasterRecordOf<T>*);

	//------------------------------------------------------------------------
	const TAtomMasterRecordOf<T>* SearchRecordByValue(const T&) const;

	//------------------------------------------------------------------------
	const T& Insert(const T&);
    //------------------------------------------------------------------------
    void Insert(const TContainerOf<T>&);
	//------------------------------------------------------------------------
	TSize Delete(const TConditionOf<T>&, TIndex = indexAll);
    //------------------------------------------------------------------------
    TSize Delete(const TComparatorOf<T>&, TIndex = indexAll);
    //------------------------------------------------------------------------
    TSize Delete(const T&);
   	//------------------------------------------------------------------------
    TSize Delete(const TContainer&);
	//------------------------------------------------------------------------
	TSize FastDelete(const TComparatorOf<T>&, TIndex = indexAll);
    //------------------------------------------------------------------------
	void Clear();
} /*TAtomMasterOf<T>*/;

// TAtomMasterCollectUnusedRecordsOf template
//============================================================================
template <class T>
class _NATLIB_TEMPLATE_DECLSPEC_ TAtomMasterCollectUnusedRecordsOf :
	public TConstantOperationOn< TAtomMasterRecordOf<T> >
{
protected:
	//------------------------------------------------------------------------
	friend TAtomMasterOf<T>;

	//------------------------------------------------------------------------
	typedef TConstantOperationOn< TAtomMasterRecordOf<T> > TParent;

	//------------------------------------------------------------------------
	const TAtomMasterRecordOf<T>** apc_Records;
	TAtomMasterOf<T>::TSize nToCollect, nCollected;

	//------------------------------------------------------------------------
	TAtomMasterCollectUnusedRecordsOf(
		const TAtomMasterRecordOf<T>** a_pUnusedRecords,
		TAtomMasterOf<T>::TSize nRecordsToCollect
	):
		apc_Records(a_pUnusedRecords), nToCollect(nRecordsToCollect),
		nCollected(0)
	{} /*TAtomMasterCollectUnusedRecordsOf(TAtomMasterRecordOf<T>**, TSize)*/

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED1(TAtomMasterCollectUnusedRecordsOf, T)

	//------------------------------------------------------------------------
	void operator()(const TAtomMasterRecordOf<T>&);
} /*TAtomMasterCollectUnusedRecordsOf<T>*/;

// TAtomRecordSortingComparatorOf template implementation
//============================================================================
template <class T> 
TAtomRecordSortingComparatorOf<T>::TAtomRecordSortingComparatorOf(

	const TValueComparator& c_ValueComparator

):
	pValueComparator(null)
{
	pValueComparator = CloneObject(c_ValueComparator);
} /*TAtomRecordSortingComparatorOf(const TValueComparator&)*/

//----------------------------------------------------------------------------
template <class T> 
TAtomRecordSortingComparatorOf<T>::TAtomRecordSortingComparatorOf(

	const TAtomRecordSortingComparatorOf<T>& c_Source

):
	pValueComparator(null)
{
	pValueComparator = CloneObject(*c_Source.pValueComparator);
} /*TAtomRecordSortingComparatorOf(const TValueComparator&)*/

//----------------------------------------------------------------------------
template <class T> 
TAtomRecordSortingComparatorOf<T>::~TAtomRecordSortingComparatorOf()
{
	if (pValueComparator) {
		delete pValueComparator; pValueComparator = null;
	} /*if*/
} /*~TAtomRecordSortingComparatorOf<T>()*/

//----------------------------------------------------------------------------
template <class T>
NLOBJECT_IMPLEMENT_CLONE_DEFAULT1(TAtomRecordSortingComparatorOf, T)

//----------------------------------------------------------------------------
template <class T> void TAtomRecordSortingComparatorOf<T>::SetEtalon(

	const TAtomMasterRecordOf<T>& c_Record

){
	pValueComparator->SetEtalon(c_Record.Value);
	TParent::SetEtalon(c_Record);
} /*TAtomRecordSortingComparatorOf<T>::SetEtalon(const TAtomMasterRecordOf&)*/

//----------------------------------------------------------------------------
template <class T> void TAtomRecordSortingComparatorOf<T>::ClearEtalon()
{
	TParent::ClearEtalon();
	pValueComparator->ClearEtalon();
} /*TAtomRecordSortingComparatorOf<T>::ClearEtalon()*/

//----------------------------------------------------------------------------
template <class T> 
TComparator::TResult TAtomRecordSortingComparatorOf<T>::operator()(

	const TAtomMasterRecordOf<T>& c_Record

) const {
	Etalon();
	return pValueComparator->operator()(c_Record.Value);
} /*TAtomRecordSortingComparatorOf<T>::operator()(...)*/

// TAtomRecordSearchComparatorOf template implementation
//============================================================================
template <class T>
TAtomRecordSearchComparatorOf<T>::TAtomRecordSearchComparatorOf(

	const TValueComparator& c_ValueComparator

):
	pc_ValueComparator(&c_ValueComparator)
{} /*TAtomRecordSearchComparatorOf(const TValueComparator&)*/

//----------------------------------------------------------------------------
template <class T> 
TAtomRecordSearchComparatorOf<T>::~TAtomRecordSearchComparatorOf()
{
	pc_ValueComparator = null;
} /*~TAtomRecordSearchComparatorOf<T>()*/

//----------------------------------------------------------------------------
template <class T> 
TComparator::TResult TAtomRecordSearchComparatorOf<T>::operator()(

	const TAtomMasterRecordOf<T>& c_Record

) const {
	return pc_ValueComparator->operator()(c_Record.Value);
} /*TAtomRecordSearchComparatorOf<T>::operator()(const TRecord&)*/

// TAtomOf template implementation
//============================================================================
template <class T> TAtomOf<T>::TAtomOf()
{} /*TAtomOf<T>()*/

//----------------------------------------------------------------------------
template <class T> TAtomOf<T>::TAtomOf(

	const TAtomOf<T>& c_Source
	
):
	hMaster(),
	pc_Record(null)
{
	Let(c_Source);
} /*TAtomOf<T>(const TAtomOf<T>&)*/

//----------------------------------------------------------------------------
template <class T> TAtomOf<T>::~TAtomOf()
{
	Clear();
} /*~TAtomOf<T>()*/

//----------------------------------------------------------------------------
template <class T> void TAtomOf<T>::AssertValid() const
{
	TParent::AssertValid();
	if (TAtomMasterOf<T>::GetMasterPtrConst(*this) && !pc_Record) 
		throw TXInvalidRecordPointer(THIS_CLASS);
} /*TAtomOf<T>::AssertValid() const*/

//----------------------------------------------------------------------------
template <class T> NLOBJECT_IMPLEMENT_CLONE_DEFAULT1(TAtomOf, T);

//----------------------------------------------------------------------------
template <class T> void TAtomOf<T>::Clear()
{
	TAtomMasterOf<T>* pMaster = TAtomMasterOf<T>::GetMasterPtr(*this);
	if (pMaster) pMaster->Release(pc_Record);
	hMaster.Unassign(); pc_Record = null;
} /*TAtomOf<T>::Clear()*/

//----------------------------------------------------------------------------
template <class T> const T& TAtomOf<T>::Value(

	nl_ulong_t& r_uReferences

) const {
	if (!TAtomMasterOf<T>::GetMasterPtrConst(*this) || !pc_Record) 
		throw TXNotAssigned(THIS_CLASS);
	r_uReferences = pc_Record->uReferences;
	return pc_Record->Value;
} /*TAtomOf<T>::Value(nl_ulong_t&) const*/

//----------------------------------------------------------------------------
template <class T> const T& TAtomOf<T>::Value() const 
{
	if (!TAtomMasterOf<T>::GetMasterPtrConst(*this) || !pc_Record) 
		throw TXNotAssigned(THIS_CLASS);
	return pc_Record->Value;
} /*TAtomOf<T>::Value() const*/

//----------------------------------------------------------------------------
template <class T> nl_ulong_t TAtomOf<T>::References() const 
{
	if (!TAtomMasterOf<T>::GetMasterPtrConst(*this) || !pc_Record) 
		throw TXNotAssigned(THIS_CLASS);
	return pc_Record->uReferences;
} /*TAtomOf<T>::References() const*/

//----------------------------------------------------------------------------
template <class T> void TAtomOf<T>::Let(

	const TAtomOf<T>& c_Source

){
	hMaster.Unassign(); pc_Record = null;

	TAtomMasterOf<T>* pMaster = (
		TAtomMasterOf<T>::GetMasterPtr(const_cast<TAtomOf<T>&>(c_Source))
	) /*pMaster*/;

	if (pMaster) {
		hMaster = c_Source.hMaster; 
		try {
			pc_Record = pMaster->AddReference(c_Source.pc_Record);
		} /*try*/
		catch (...) {
			hMaster.Unassign(); pc_Record = null; 
			throw;
		} /*catch (...)*/ 
	} /*if*/
} /*void TAtomOf<T>::Let(const TAtomof<T>&)*/

//----------------------------------------------------------------------------
template <class T> void TAtomOf<T>::Let(

	const T& c_Value

){
	TAtomMasterOf<T>* pMaster = TAtomMasterOf<T>::GetMasterPtr(*this);
	if (!pMaster) 
		throw TXNotAssigned(THIS_CLASS);
	pMaster->Assign(*this, c_Value);
} /*void TAtomOf<T>::Let(const T&)*/

// TAtomDefaultComparatorOf template implementation
//============================================================================
template <class T>
NLOBJECT_IMPLEMENT_CLONE_DEFAULT1(TAtomDefaultComparatorOf, T);

//----------------------------------------------------------------------------
template <class T>
TComparator::TResult TAtomDefaultComparatorOf<T>::operator()(

	const TAtomOf<T>& c_Atom
	
) const {
	const TAtomOf<T>& c_Etalon = Etalon();
	if (c_Etalon < c_Atom) return compLess;
	if (c_Etalon > c_Atom) return compGreater;
	return compEqual;
} /*TAtomDefaultComparatorOf<T>::operator()(const TAtomOf<T>&)*/

// TAtomMasterOf template implementation
//============================================================================
template <class T> TAtomMasterOf<T>::TAtomMasterOf(

	const TEtalonComparatorOf<T>& r_Comparator, 
	bool bInitDeleteUnusedRecords /*= true*/,
	const TAllocator& r_Allocator /*= NLATOM_ALLOCATOR_DEFAULT()*/

):
	TParent(r_Comparator),
	Tree(TAtomRecordSortingComparatorOf<T>(r_Comparator), r_Allocator),
	bDeleteUnusedRecords(bInitDeleteUnusedRecords), nUnreferencedRecords(0)
{} /*TAtomMasterOf<T>()*/

//----------------------------------------------------------------------------
template <class T> TAtomMasterOf<T>::TAtomMasterOf(

	const TAtomMasterOf<T>& r_AtomMaster
	
):
	TParent(r_AtomMaster.ComparatorConst()),
	Tree(
		r_AtomMaster.Tree.ComparatorConst(), 
		r_AtomMaster.Tree.AllocatorConst()
	) /*Tree*/,
	bDeleteUnusedRecords(r_AtomMaster.bDeleteUnusedRecords), 
	nUnreferencedRecords(0)
{} /*TAtomMasterOf<T>(const TAtomMasterOf<T>&)*/

//----------------------------------------------------------------------------
template <class T> TAtomMasterOf<T>::~TAtomMasterOf()
{} /*TAtomMasterOf<T>::~TAtomMasterOf()*/

//----------------------------------------------------------------------------
template <class T> void TAtomMasterOf<T>::AssertValid() const
{
	TParent::AssertValid();
	Tree.AssertValid();
} /*TAtomMasterOf<T>::AssertValid() const*/

//----------------------------------------------------------------------------
template <class T> TNatlibStdString TAtomMasterOf<T>::Dump() const
{
	TStdString sDump(TParent::Dump());

	// Add unreferenced
	//------------------
	{
		char szDump[256];
		sprintf(
			szDump, " unreferenced: %lu", (unsigned long)UnreferencedRecords()
		) /*sprintf*/;
		sDump += szDump;
	} /*Add unreferenced*/

	return sDump;
} /*TAtomMasterOf<T>::Dump() const*/

//----------------------------------------------------------------------------
template <class T> NLOBJECT_IMPLEMENT_CLONE_DEFAULT1(TAtomMasterOf, T)

//----------------------------------------------------------------------------
template <class T> TAtomOf<T> TAtomMasterOf<T>::CreateAtom(

	const T& c_Value
	
){
	TAtomOf<T> NewAtom;
	return Assign(NewAtom, c_Value);
} /*TAtomMasterOf<T>::CreateAtom(const T&)*/

//----------------------------------------------------------------------------
template <class T> TAtomOf<T>& TAtomMasterOf<T>::Assign(

	TAtomOf<T>& r_Atom,
	const T& c_Value

){
	MasterHandle(r_Atom) = TSafeHandleBase(*this);
	RecordPtr(r_Atom) = null;

	const TAtomMasterRecordOf<T>* pc_Record = SearchRecordByValue(c_Value);
	if (!pc_Record) {
		TAtomMasterRecordOf<T> NewRecord(c_Value);
		pc_Record = &Tree.Insert(NewRecord); ++nUnreferencedRecords;
	} /*if*/
	RecordPtr(r_Atom) = AddReference(pc_Record);

	return r_Atom;
} /*TAtomMasterOf<T>::Assign(TAtomOf<T>&, const T&)*/

//----------------------------------------------------------------------------
template <class T> TAtomMasterOf<T>::TSize TAtomMasterOf<T>::CollectGarbage(

	TSize nToCollect

){
	TSize nDeleted = 0;

	if (nToCollect > nUnreferencedRecords) nToCollect = nUnreferencedRecords;

	if (nToCollect) {
		try {
			typedef const TAtomMasterRecordOf<T>* TRecordConstPtr;
			TAutoPointerTo<TRecordConstPtr> ha_pUnusedRecords(null, true);

			NLEXCEPT_CLASS_NEW(
				TRecordConstPtr[nToCollect], ha_pUnusedRecords
			) /*NLEXCEPT_CLASS_NEW*/;

			TAtomMasterCollectUnusedRecordsOf<T> CollectUnusedRecords(
				ha_pUnusedRecords, nToCollect
			) /*CollectUnusedRecords*/;
			Tree.ForEachConst(CollectUnusedRecords);

			for (TSize i = 0; i < CollectUnusedRecords.nCollected; ++i)
				nDeleted += Tree.Delete(
                	*(((TRecordConstPtr*)ha_pUnusedRecords)[i])
                ) /*Tree.Delete*/;
			nUnreferencedRecords -= nDeleted;
		} /*try*/
		catch (...) {
			TAtomMasterCollectUnusedRecordsOf<T> CollectUnusedRecords(
				null, nToCollect
			) /*CollectUnusedRecords*/;
			Tree.ForEachConst(CollectUnusedRecords);
			nUnreferencedRecords = CollectUnusedRecords.nCollected;
			throw;
		} /*catch (...)*/
	} /*if*/

	return nDeleted;
} /*TAtomMasterOf<T>::CollectGarbage(TSize)*/

//----------------------------------------------------------------------------
template <class T> void TAtomMasterOf<T>::ForEach(

	TOperationOn<T>& r_Operation
	
){
	TAtomRecordPerformDataOperationOf<T> PerformDataOperation(
		r_Operation
	) /*PerformDataOperation*/;
	Tree.ForEach(PerformDataOperation);
} /*TAtomMasterOf<T>::ForEach(TOperation&)*/

//----------------------------------------------------------------------------
template <class T> void TAtomMasterOf<T>::ForEach(

	TConstantOperationOn<T>& r_Operation
	
) const {
	TAtomRecordPerformDataConstantOperationOf<T> PerformDataOperation(
		r_Operation
	) /*PerformDataOperation*/;
	Tree.ForEach(PerformDataOperation);
} /*TAtomMasterOf<T>::ForEach(TConstantOperation&)*/

//----------------------------------------------------------------------------
template <class T> const T* TAtomMasterOf<T>::FastSearch(
	
	const TComparatorOf<T>& c_Comparator, 
	TIndex nIndex /*= indexAny*/

) const {
	TAtomRecordSearchComparatorOf<T> SearchComparator(c_Comparator);
	const TAtomMasterRecordOf<T>* pc_Found = Tree.FastSearch(
    	SearchComparator, nIndex
    ) /*Tree.FastSearch*/;
	return pc_Found ? &(pc_Found->Value) : null;
} /*TAtomMasterOf<T>::FastSearch(const TComparatorOf<T>&, TIndex)*/;

//----------------------------------------------------------------------------
template <class T> const T* TAtomMasterOf<T>::SearchFirst() const
{
	const TAtomMasterRecordOf<T>* pc_Found = Tree.SearchFirst();
	return pc_Found ? &(pc_Found->Value) : null;
} /*TAtomMasterOf<T>::SearchFirst() const*/

//----------------------------------------------------------------------------
template <class T> const T* TAtomMasterOf<T>::SearchLast() const
{
	const TAtomMasterRecordOf<T>* pc_Found = Tree.SearchLast();
	return pc_Found ? &(pc_Found->Value) : null;
} /*TAtomMasterOf<T>::SearchLast() const*/

//----------------------------------------------------------------------------
template <class T> const T* TAtomMasterOf<T>::SearchNext(

	const T& c_Data
	
) const {
	TAtomMasterRecordOf<T> ToSearch(c_Data);
	const TAtomMasterRecordOf<T>* pc_Found = Tree.SearchNext(c_Data);
	return pc_Found ? &(pc_Found->Value) : null;
} /*TAtomMasterOf<T>::SearchNext(const T&)*/

//----------------------------------------------------------------------------
template <class T> const T* TAtomMasterOf<T>::SearchPrevious(

	const T& c_Data
	
) const {
	TAtomMasterRecordOf<T> ToSearch(c_Data);
	const TAtomMasterRecordOf<T>* pc_Found = Tree.SearchPrevious(c_Data);
	return pc_Found ? &(pc_Found->Value) : null;
} /*TAtomMasterOf<T>::SearchPrevious(const T&)*/

//----------------------------------------------------------------------------
template <class T> void TAtomMasterOf<T>::Inorder(

	TOperationOn<T>& r_Operation
	
){
	TAtomRecordPerformDataOperationOf<T> PerformOperation(r_Operation);
	Tree.Inorder(PerformOperation);
} /*TAtomMasterOf<T>::Inorder(TOperationOn<T>&)*/

//----------------------------------------------------------------------------
template <class T> void TAtomMasterOf<T>::Inorder(

	TConstantOperationOn<T>& r_Operation
	
) const {
	TAtomRecordPerformDataConstantOperationOf<T> PerformOperation(
		r_Operation
	) /*PerformOperation*/;
	Tree.Inorder(PerformOperation);
} /*TAtomMasterOf<T>::Inorder(TConstantOperationOn<T>&) const*/

//----------------------------------------------------------------------------
template <class T> 
const TAtomMasterRecordOf<T>* TAtomMasterOf<T>::AddReference(

	const TAtomMasterRecordOf<T>* pc_Record
	
){
	if (pc_Record) {
		TAtomMasterRecordOf<T>* pRecord = (
        	const_cast<TAtomMasterRecordOf<T>*>(pc_Record)
        ) /*pRecord*/;
		if (!pc_Record->uReferences) --nUnreferencedRecords;
		if (!++(pRecord->uReferences)) 
			throw TAtomOf<T>::TXInvalidReferenceCounter(THIS_CLASS);
	} /*pc_Record*/
	return pc_Record;
} /*TAtomMasterOf<T>::AddReference(const TAtomMasterRecordOf<T>*)*/

//----------------------------------------------------------------------------
template <class T> const TAtomMasterRecordOf<T>* TAtomMasterOf<T>::Release(

	const TAtomMasterRecordOf<T>* pc_Record

){
	if (pc_Record) {
		TAtomMasterRecordOf<T>* pRecord = (
        	const_cast<TAtomMasterRecordOf<T>*>(pc_Record)
        ) /*pRecord*/;
		if (!pRecord->uReferences)
			throw TAtomOf<T>::TXInvalidReferenceCounter(THIS_CLASS);

		if (!--(pRecord->uReferences)) {
			if (bDeleteUnusedRecords) {
				Tree.Delete(*pc_Record); 
				pc_Record = null;
			}
			else {
				++nUnreferencedRecords;
			} /*iff*/
		} /*if*/
	} /*if*/
	return pc_Record;
} /*TAtomMasterOf<T>::Release(const TAtomMasterRecordOf<T>*)*/

//----------------------------------------------------------------------------
template <class T>
const TAtomMasterRecordOf<T>* TAtomMasterOf<T>::SearchRecordByValue(

	const T& c_Value

) const {
	TEtalonComparatorOf<T>* pValueComparator = &(
		const_cast<TEtalonComparatorOf<T>&>(
			dynamic_cast<const TAtomRecordSortingComparatorOf<T>&>(
				Tree.ComparatorConst()
			)/*dynamic_cast*/.ValueComparatorConst()
		) /*const_cast<TEtalonComparatorOf<T>*>*/
	) /*pValueComparator*/;

	TAutoRestoreEtalonComparatorHandleOf<T> hSaveEtalon(
		*pValueComparator
	) /*hSaveEtalon*/;

	pValueComparator->SetEtalon(c_Value);
	TAtomRecordSearchComparatorOf<T> SearchComparator(*pValueComparator);
	return Tree.FastSearch(SearchComparator);
} /*TAtomMasterOf<T>::SearchRecordByValue(const T&) const*/

//----------------------------------------------------------------------------
template <class T> const T& TAtomMasterOf<T>::Insert(

	const T& c_Data

){
    throw TXOperationIsNotAllowed(THIS_CLASS); return c_Data;
} /*TAtomMasterOf<T>::Insert(const T&)*/

//----------------------------------------------------------------------------
template <class T> void TAtomMasterOf<T>::Insert(

	const TContainerOf<T>&

){
    throw TXOperationIsNotAllowed(THIS_CLASS);
} /*TAtomMasterOf<T>::Insert(const TContainerOf<T>&)*/

//----------------------------------------------------------------------------
template <class T> TAtomMasterOf<T>::TSize TAtomMasterOf<T>::Delete(

	const TConditionOf<T>&,
    TIndex /*= indexAll*/
){
    throw TXOperationIsNotAllowed(THIS_CLASS); return TSize(0);
} /*TAtomMasterOf<T>::Delete(const TConditionOf<T>&, TIndex)*/

//----------------------------------------------------------------------------
template <class T> TAtomMasterOf<T>::TSize TAtomMasterOf<T>::Delete(

	const TComparatorOf<T>&,
    TIndex /*= indexAll*/

){
    throw TXOperationIsNotAllowed(THIS_CLASS); return TSize(0);
} /*TAtomMasterOf<T>::Delete(const TComparatorOf<T>&, TIndex)*/

//----------------------------------------------------------------------------
template <class T> TAtomMasterOf<T>::TSize TAtomMasterOf<T>::Delete(

	const T&

){
    throw TXOperationIsNotAllowed(THIS_CLASS); return TSize(0);
} /*TAtomMasterOf<T>::Delete(const T&)*/

//----------------------------------------------------------------------------
template <class T> TAtomMasterOf<T>::TSize TAtomMasterOf<T>::Delete(

	const TContainer&

){
    throw TXOperationIsNotAllowed(THIS_CLASS); return TSize(0);
} /*TAtomMasterOf<T>::Delete(const TContainer&)*/

//----------------------------------------------------------------------------
template <class T> TAtomMasterOf<T>::TSize TAtomMasterOf<T>::FastDelete(

	const TComparatorOf<T>&,
    TIndex /*= indexAll*/

){
    throw TXOperationIsNotAllowed(THIS_CLASS); return TSize(0);
} /*TAtomMasterOf<T>::FastDelete(const TComparatorOf<T>&, TIndex)*/

//------------------------------------------------------------------------
template <class T> void TAtomMasterOf<T>::Clear()
{
	throw TXOperationIsNotAllowed(THIS_CLASS);
} /*TAtomMasterOf<T>::Clear()*/

// TAtomMasterCollectUnusedRecordsOf template implementation
//============================================================================
template <class T> void TAtomMasterCollectUnusedRecordsOf<T>::operator()(

	const TAtomMasterRecordOf<T>& c_Record

){
	if (!c_Record.uReferences) {
		if (apc_Records) apc_Records[nCollected] = &c_Record;
		++nCollected;
	} /*if*/
	if (nCollected >= nToCollect) Abort();
} /*TAtomMasterCollectUnusedRecordsOf<T>::operator()(...)*/

//============================================================================
NATLIB_NAMESPACE_END

//============================================================================
#define NLATOM_DECLSPEC(T, Declspec) \
	NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TAtomMasterRecordOf), T, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/; \
	NLCOMPAR_DECLSPEC( \
		NATLIB_NAMESPACE_MEMBER(TAtomMasterRecordOf< T >), Declspec \
	) /*NLCOMPAR_DECLSPEC*/; \
	NLOPERAT_DECLSPEC( \
		NATLIB_NAMESPACE_MEMBER(TAtomMasterRecordOf< T >), Declspec \
	) /*NLOPERAT_DECLSPEC*/; \
	NLAVLTRE_DECLSPEC( \
		NATLIB_NAMESPACE_MEMBER(TAtomMasterRecordOf< T >), Declspec \
	) /*NLAVLTRE_DECLSPEC*/; \
	NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TAtomRecordSortingComparatorOf), T, \
		Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/; \
	NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TAtomRecordSearchComparatorOf), T, \
		Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/; \
	NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TAtomRecordPerformDataOperationOf), T, \
		Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/; \
	NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TAtomRecordPerformDataConstantOperationOf), \
		T, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/; \
	NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TAtomOf), T, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/; \
	NLCOMPAR_DECLSPEC( \
		NATLIB_NAMESPACE_MEMBER(TAtomOf< T >), Declspec \
	) /*NLCOMPAR_DECLSPEC*/; \
	NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TAtomDefaultComparatorOf), T, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/; \
	NLSCONT_DECLSPEC(T, Declspec); \
	NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TAtomMasterOf), T, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/; \
	NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TAtomMasterCollectUnusedRecordsOf), T, \
		Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/

#endif /*!defined(__NATLIB_NLSTATOM_H__)*/