#ifndef __NATLIB_NLAVLST_H__
#define __NATLIB_NLAVLST_H__
/****************************************************************************/
/** nlavlst.h                                                              **/
/** ---------------------------------------------------------------------- **/
/** Natlib Storage based on AVL binary tree                                **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** February 19, 1997 by Oleg Kagan                                        **/
/****************************************************************************/

//============================================================================
#include <nlavltre.h>
#include <nlstorag.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

//============================================================================
#define NLAVLST_ALLOCATOR_DEFAULT NLAVLTRE_ALLOCATOR_DEFAULT
#define NLAVLST_DELETE_BUFFER_SIZE 1024

// TAvlStorageRecordOf template
//============================================================================
template <class TData, class TKey>
class _NATLIB_TEMPLATE_DECLSPEC_ TAvlStorageRecordOf {
public:
	//------------------------------------------------------------------------
    TAvlStorageRecordOf(const TData& c_Data, const TKey& c_Key) :
    	Data(c_Data), Key(c_Key)
    {} /*TAvlStorageRecordOf(const TKey&, const TData&)*/

    //------------------------------------------------------------------------
    TAvlStorageRecordOf(const TAvlStorageRecordOf& c_Record) :
    	Data(c_Record.Data), Key(c_Record.Key)
    {} /*TAvlStorageRecordOf(const TAvlStorageRecordOf&)*/

	//------------------------------------------------------------------------
    TData Data;
    TKey Key;
} /*TAvlStorageRecordOf<TData, TKey>*/;

// TAvlStorageSearchComparatorOf template
//============================================================================
template <class TData, class TKey>
class _NATLIB_TEMPLATE_DECLSPEC_ TAvlStorageSearchComparatorOf :
	public TComparatorOf< TAvlStorageRecordOf<TData, TKey> >
{
public:
	//------------------------------------------------------------------------
    typedef TComparatorOf< TAvlStorageRecordOf<TData, TKey> > TParent;
	typedef TAvlStorageRecordOf<TData, TKey> TRecord;

    //------------------------------------------------------------------------
    TAvlStorageSearchComparatorOf(const TComparatorOf<TKey>&);

    //------------------------------------------------------------------------
    TComparator::TResult operator()(const TRecord& c_Record) const 
	{
   		return pc_KeyComparator->operator()(c_Record.Key);
	} /*operator()(const TRecord&) const*/

protected:
	//------------------------------------------------------------------------
    const TComparatorOf<TKey>* pc_KeyComparator;

private:
    //------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED2(
		TAvlStorageSearchComparatorOf, TData, TKey
	) /*NLOBJECT_CLONE_IS_NOT_ALLOWED2*/
} /*TAvlStorageSearchComparatorOf<TData, TKey>*/;

// TAvlStorageSortingComparatorOf template
//============================================================================
template <class TData, class TKey>
class _NATLIB_TEMPLATE_DECLSPEC_ TAvlStorageSortingComparatorOf :
	public TPointerSavingEtalonComparatorOf<TAvlStorageRecordOf<TData, TKey> >
{
public:
	//------------------------------------------------------------------------
    typedef TPointerSavingEtalonComparatorOf<
    	TAvlStorageRecordOf<TData, TKey>
    > /*TPointerSavingEtalonComparatorOf*/ TParent;

    //------------------------------------------------------------------------
    typedef TAvlStorageRecordOf<TData, TKey> TStorageRecord;
    typedef TEtalonComparatorOf<TKey> TKeyComparator;

	//------------------------------------------------------------------------
	NLEXCEPT_DECLARE_BASE(TException, _NATLIB_TEMPLATE_DECLSPEC_);	
	NLEXCEPT_DECLARE(
		TXUnableToCloneComparator, TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
	
    //------------------------------------------------------------------------
    TAvlStorageSortingComparatorOf(const TKeyComparator&);
    //------------------------------------------------------------------------
    ~TAvlStorageSortingComparatorOf();

    //------------------------------------------------------------------------
	const TEtalonComparatorOf<TKey>* KeyComparatorPtrConst() const
    {
    	return pKeyComparator;
    } /*KeyComparatorPtrConst() const*/

    //------------------------------------------------------------------------
    void SetEtalon(const TAvlStorageRecordOf<TData, TKey>& c_Record)
    {
    	TParent::SetEtalon(c_Record);
        pKeyComparator->SetEtalon(c_Record.Key);
    } /*SetEtalon(const TAvlStorageRecordOf<TData, TKey>&)*/
    //------------------------------------------------------------------------
    void ClearEtalon()
    {
    	pKeyComparator->ClearEtalon();
        TParent::ClearEtalon();
    } /*ClearEtalon()*/

    //------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE2(TAvlStorageSortingComparatorOf, TData, TKey);

    //------------------------------------------------------------------------
    TComparator::TResult operator()(
		const TAvlStorageRecordOf<TData, TKey>& c_Record
	) const {
    	return pKeyComparator->operator()(c_Record.Key);
    } /*operator()(const TAvlStorageRecordOf<TData, TKey>&) const*/

protected:
	//------------------------------------------------------------------------
    TEtalonComparatorOf<TKey>* pKeyComparator;
} /*TAvlStorageSortingComparatorOf<TData, TKey>*/;

// TAvlStorageDataConditionOf template
//============================================================================
template <class TData, class TKey>
class _NATLIB_TEMPLATE_DECLSPEC_ TAvlStorageDataConditionOf :
	public TConditionOf< TAvlStorageRecordOf<TData, TKey> >
{
public:
	//------------------------------------------------------------------------
	typedef TConditionOf< TAvlStorageRecordOf<TData, TKey> > TParent;

    //------------------------------------------------------------------------
    TAvlStorageDataConditionOf(const TConditionOf<TData>& c_Condition) :
    	pc_Condition(&c_Condition)
    {} /*TAvlStorageDataConditionOf(const TConditionOf<TData>&)*/

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED2(TAvlStorageDataConditionOf, TData, TKey)

    //------------------------------------------------------------------------
    bool operator()(const TAvlStorageRecordOf<TData, TKey>& c_Record) const
    {
    	return pc_Condition->operator()(c_Record.Data);
    } /*operator()(const TAvlStorageRecordOf<TData>&) const*/

protected:
	//------------------------------------------------------------------------
    const TConditionOf<TData>* pc_Condition;
} /*TAvlStorageDataConditionOf<TData, TKey>*/;

// TAvlStorageApplyOperationOn template
//============================================================================
template <class TData, class TKey>
class _NATLIB_TEMPLATE_DECLSPEC_ TAvlStorageApplyOperationOn :
	public TOperationOn< TAvlStorageRecordOf<TData, TKey> >
{
public:
	//------------------------------------------------------------------------
    typedef TOperationOn< TAvlStorageRecordOf<TData, TKey> > TParent;

    //------------------------------------------------------------------------
    typedef TStorageOperationOn<TData, TKey> TStorageOperation;
    typedef TAvlStorageRecordOf<TData, TKey> TStorageRecord;

	//------------------------------------------------------------------------
    TAvlStorageApplyOperationOn(TStorageOperation& r_StorageOperation) :
    	pStorageOperation(&r_StorageOperation)
    {} /*TAvlStorageApplyOperationOn(TStorageOperation&)*/

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED2(
		TAvlStorageApplyOperationOn, TData, TKey
	) /*NLOBJECT_CLONE_IS_NOT_ALLOWED1*/

    //------------------------------------------------------------------------
    void operator()(TStorageRecord& r_Record)
    {
    	pStorageOperation->operator()(r_Record.Data, r_Record.Key);
    } /*operator()(TStorageRecord&)*/

protected:
	//------------------------------------------------------------------------
    TStorageOperation* pStorageOperation;
} /*TAvlStorageApplyOperationOn<TData, TKey>*/;

// TAvlStorageExecuteConstantOperation template
//============================================================================
template <class TData, class TKey>
class _NATLIB_TEMPLATE_DECLSPEC_ TAvlStorageApplyConstantOperationOn :
	public TConstantOperationOn< TAvlStorageRecordOf<TData, TKey> >
{
public:
	//------------------------------------------------------------------------
    typedef TConstantOperationOn< TAvlStorageRecordOf<TData, TKey> > TParent;

    //------------------------------------------------------------------------
    typedef TStorageConstantOperationOn<TData, TKey>
    	TStorageConstantOperation;
    typedef TAvlStorageRecordOf<TData, TKey> TStorageRecord;

	//------------------------------------------------------------------------
    TAvlStorageApplyConstantOperationOn(
    	TStorageConstantOperation& r_StorageOperation
    ) :
    	pStorageOperation(&r_StorageOperation)
    {} /*TAvlStorageApplyConstantOperationOn(TStorageConstantOperation&)*/

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED2(
		TAvlStorageApplyConstantOperationOn, TData, TKey
	) /*NLOBJECT_CLONE_IS_NOT_ALLOWED2*/

    //------------------------------------------------------------------------
    void operator()(const TStorageRecord& c_Record)
    {
    	pStorageOperation->operator()(c_Record.Data, c_Record.Key);
    } /*operator()(const TStorageRecord&)*/

protected:
	//------------------------------------------------------------------------
    TStorageConstantOperation* pStorageOperation;

private:
    //------------------------------------------------------------------------
    void operator()(TStorageRecord& r_Record)
    {
    	operator()((const TStorageRecord&)r_Record);
    } /*operator()(TStorageRecord&)*/
} /*TAvlStorageApplyConstantOperationOn<TData, TKey>*/;

// TAvlStorageOf template
//============================================================================
template <class TData, class TKey>
class _NATLIB_TEMPLATE_DECLSPEC_ TAvlStorageOf :
	public TStorageOf<TData, TKey>
{
public:
	//------------------------------------------------------------------------
    typedef TStorageOf<TData, TKey> TParent;
    typedef TAvlStorageOf<TData, TKey> TAvlStorage;

    //------------------------------------------------------------------------
	typedef TAvlStorageRecordOf<TData, TKey> TAvlStorageRecord;
    typedef TAvlStorageSortingComparatorOf<TData, TKey>
		TAvlStorageSortingComparator;

	//------------------------------------------------------------------------
	typedef TStorageOf<TData, TKey> TParentStorage;
	NLEXCEPT_DECLARE(
		TException, TParentStorage::TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
	NLEXCEPT_DECLARE(
		TXInvalidComparator, TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;

	//------------------------------------------------------------------------
    TAvlStorageOf(
		const TEtalonComparatorOf<TKey>&,
		const TAllocator& = NLAVLST_ALLOCATOR_DEFAULT()
	) /*TAvlStorageOf(const TEtalonComparatorOf<TKey>&,const TAllocator&)*/;
	//------------------------------------------------------------------------
    TAvlStorageOf(const TAvlStorage&);
    //------------------------------------------------------------------------
    TAvlStorageOf(const TAvlStorage&, const TAllocator&);
	
    //------------------------------------------------------------------------
    void AssertValid() const {Tree.AssertValid();}

    //------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE2(TAvlStorageOf, TData, TKey);

	//------------------------------------------------------------------------
    TSize Size() const {return Tree.Size();}
    //------------------------------------------------------------------------
    void Clear() {Tree.Clear();}

	//------------------------------------------------------------------------
    const TData* SearchByKeyConst(const TKey& c_Key) const;
	//------------------------------------------------------------------------
    TData* SearchByKey(const TKey& c_Key);

   	//------------------------------------------------------------------------
    TKey FirstKey() const;
    //------------------------------------------------------------------------
    TKey LastKey() const;
    //------------------------------------------------------------------------
    TKey NextKey(const TKey&) const;
    //------------------------------------------------------------------------
    TKey PreviousKey(const TKey&) const;

    //------------------------------------------------------------------------
    TData& Assign(const TData& c_Data, const TKey& c_Key);
    //------------------------------------------------------------------------
    bool Unassign(const TKey& c_Key);

    //------------------------------------------------------------------------
    TSize Delete(const TConditionOf<TData>&, TIndex = indexAll);
    //------------------------------------------------------------------------
    TSize Delete(
		const TComparatorOf<TData>& c_Comparator, TIndex nIndex = indexAll
	){
    	return TParent::Delete(c_Comparator, nIndex);
    } /*Delete(const TComparatorOf<TData>&, TIndex = indexAll)*/

    //------------------------------------------------------------------------
    void Inorder(TStorageOperation& c_Operation);
    //------------------------------------------------------------------------
    void Postorder(TStorageOperation& c_Operation);
    //------------------------------------------------------------------------
    void Inorder(TStorageConstantOperation& c_Operation) const;
    //------------------------------------------------------------------------
    void Postorder(TStorageConstantOperation& c_Operation) const;

    //------------------------------------------------------------------------
    virtual void Preorder(TStorageOperation& c_Operation);
    //------------------------------------------------------------------------
    virtual void Preorder(TStorageConstantOperation& c_Operation) const;

    //------------------------------------------------------------------------
    TAvlStorage& operator=(const TAvlStorage& c_Source)
    {
    	Tree.Let(c_Source.Tree); return *this;
    } /*operator=(const TAvlStorage&)*/
    //------------------------------------------------------------------------
    TAvlStorage& operator=(const TStorage& c_Source);

protected:
	//------------------------------------------------------------------------
    TAvlTreeOf< TAvlStorageRecordOf<TData, TKey> > Tree;

    //------------------------------------------------------------------------
    const TEtalonComparatorOf<TKey>& GetSortingKeyComparatorConst() const;

    //------------------------------------------------------------------------
    void Insert(const TStorageOf<TData, TKey>& c_Storage)
    {
    	TParent::Insert(c_Storage);
    } /*Insert(const TStorageOf<TData, TKey>&)*/

protected:
	//------------------------------------------------------------------------
    const TData& Insert(const TData&)
    {
    	throw TXOperationIsNotAllowed(THIS_CLASS); return *(const TData*)null;
    } /*Insert(const TData&)*/
    //------------------------------------------------------------------------
    void Insert(const TContainer&) 
	{
		throw TXOperationIsNotAllowed(THIS_CLASS);
    } /*Insert(const TContainer&)*/
} /*TAvlStorageOf<TData, TKey>*/;

// TAvlStorageSearchComparatorOf template implementation
//============================================================================
template <class TData, class TKey> 
TAvlStorageSearchComparatorOf<TData, TKey>::TAvlStorageSearchComparatorOf(

   	const TComparatorOf<TKey>& c_KeyComparator

):
	pc_KeyComparator(&c_KeyComparator)
{} /*TAvlStorageSearchComparatorOf<TData, TKey>(const TComparatorOf<TKey>&)*/

// TAvlStorageSortingComparatorOf template implementation
//============================================================================
template <class TData, class TKey>
TAvlStorageSortingComparatorOf<TData, TKey>::TAvlStorageSortingComparatorOf(

	const TKeyComparator& c_KeyComparator
	
):
	pKeyComparator(null)
{
	pKeyComparator = CloneObject(c_KeyComparator);
    if (!pKeyComparator) 
		throw TXUnableToCloneComparator(THIS_CLASS);
} /*TAvlStorageSortingComparatorOf<TData, TKey>(const TKeyComparator&)*/

//----------------------------------------------------------------------------
template <class TData, class TKey>
TAvlStorageSortingComparatorOf<TData, TKey>::~TAvlStorageSortingComparatorOf()
{
	delete pKeyComparator; pKeyComparator = null;
} /*~TAvlStorageSortingComparatorOf<TData, TKey>()*/

//----------------------------------------------------------------------------
template <class TData, class TKey>
NLOBJECT_CLONE_RESULT2(TAvlStorageSortingComparatorOf, TData, TKey)
TAvlStorageSortingComparatorOf<TData, TKey>::Clone() const
{
	typedef TAvlStorageSortingComparatorOf<TData, TKey> TAvlSortingComparator;
	TAvlSortingComparator* pCopy;
    NLEXCEPT_CLASS_NEW(TAvlSortingComparator(*pKeyComparator), pCopy);
    return pCopy;
} /*TAvlStorageSortingComparatorOf<TData, TKey>::Clone() const*/

// TAvlStorageOf template implementation
//============================================================================
template <class TData, class TKey> TAvlStorageOf<TData, TKey>::TAvlStorageOf(

	const TEtalonComparatorOf<TKey>& c_SortingKeyComparator,
	const TAllocator& c_Allocator /*= NLAVLST_ALLOCATOR_DEFAULT()*/

):
   	Tree(TAvlStorageSortingComparator(c_SortingKeyComparator), c_Allocator)
{} /*TAvlStorageOf(const TKeySortingComparator&, const TAllocator&)*/

//----------------------------------------------------------------------------
template <class TData, class TKey> TAvlStorageOf<TData, TKey>::TAvlStorageOf(

	const TAvlStorage& c_Storage
	
): 
	Tree(c_Storage.Tree) 
{} /*TAvlStorageOf(const TAvlStorageOf<TData, TKey>&)*/

//----------------------------------------------------------------------------
template <class TData, class TKey> TAvlStorageOf<TData, TKey>::TAvlStorageOf(

	const TAvlStorage& c_Storage, 
	const TAllocator& c_Allocator

):
   	Tree(c_Storage.Tree.ComparatorConst(), c_Allocator)
{
   	Tree.Let(c_Storage.Tree);
} /*TAvlStorageOf(const TAvlStorageOf<TData, TKey>&, const TAllocator&)*/

//----------------------------------------------------------------------------
template <class TData, class TKey>
NLOBJECT_IMPLEMENT_CLONE_DEFAULT2(TAvlStorageOf, TData, TKey)

//----------------------------------------------------------------------------
template <class TData, class TKey>
const TData* TAvlStorageOf<TData, TKey>::SearchByKeyConst(

	const TKey& c_Key

) const {
	TAutoPointerTo< TEtalonComparatorOf<TKey> > hSearchKeyComparator;
    hSearchKeyComparator.Assign(
		CloneObject(GetSortingKeyComparatorConst()), false
    ) /*Assign*/;
	(*hSearchKeyComparator).SetEtalon(c_Key);
	TAvlStorageSearchComparatorOf<TData, TKey> SearchComparator(
    	*hSearchKeyComparator
    ) /*SearchComparator*/;
    const TAvlStorageRecord* pc_FoundRecord = (
    	Tree.FastSearch(SearchComparator)
    ) /*pc_FoundRecord*/;
	return pc_FoundRecord ? &(pc_FoundRecord->Data) : null;
} /*TAvlStorageOf<TData, TKey>::SearchByKeyConst(const TKey&) const*/

//----------------------------------------------------------------------------
template <class TData, class TKey>
TData* TAvlStorageOf<TData, TKey>::SearchByKey(

	const TKey& c_Key

){
	const TData* pc_Found = SearchByKeyConst(c_Key);
    return pc_Found ? const_cast<TData*>(pc_Found) : null;
} /*TAvlStorageOf<TData, TKey>::SearchByKey(const TKey&)*/

//----------------------------------------------------------------------------
template <class TData, class TKey>
TKey TAvlStorageOf<TData, TKey>::FirstKey() const
{
	const TAvlStorageRecordOf<TData, TKey>* pc_Found = Tree.SearchFirst();
    if (!pc_Found) 
		throw TXNotFound(THIS_CLASS);
    return pc_Found->Key;
} /*TAvlStorageOf<TData, TKey>::FirstKey() const*/

//----------------------------------------------------------------------------
template <class TData, class TKey>
TKey TAvlStorageOf<TData, TKey>::LastKey() const
{
	const TAvlStorageRecordOf<TData, TKey>* pc_Found = Tree.SearchLast();
    if (!pc_Found) 
		throw TXNotFound(THIS_CLASS);
    return pc_Found->Key;
} /*TAvlStorageOf<TData, TKey>::LastKey() const*/

//----------------------------------------------------------------------------
template <class TData, class TKey> TKey TAvlStorageOf<TData, TKey>::NextKey(

	const TKey& c_Key

) const {
	TAutoPointerTo< TEtalonComparatorOf<TKey> > hSearchKeyComparator;
    hSearchKeyComparator.Assign(
		CloneObject(GetSortingKeyComparatorConst()), false
    ) /*Assign*/;
	(*hSearchKeyComparator).SetEtalon(c_Key);
	TAvlStorageSearchComparatorOf<TData, TKey> SearchComparator(
    	*hSearchKeyComparator
    ) /*SearchComparator*/;
    const TAvlStorageRecord* pc_Found = Tree.SearchNext(SearchComparator);
    if (!pc_Found) 
		throw TXOutOfRange(THIS_CLASS);
	return pc_Found->Key;
} /*TAvlStorageOf<TData, TKey>::NextKey(const TKey&)*/

//----------------------------------------------------------------------------
template <class TData, class TKey>
TKey TAvlStorageOf<TData, TKey>::PreviousKey(

	const TKey& c_Key

) const {
	TAutoPointerTo< TEtalonComparatorOf<TKey> > hSearchKeyComparator;
    hSearchKeyComparator.Assign(
		CloneObject(GetSortingKeyComparatorConst()), false
    ) /*Assign*/;
	(*hSearchKeyComparator).SetEtalon(c_Key);
	TAvlStorageSearchComparatorOf<TData, TKey> SearchComparator(
    	*hSearchKeyComparator
    ) /*SearchComparator*/;
    const TAvlStorageRecord* pc_Found = Tree.SearchPrevious(SearchComparator);
    if (!pc_Found) 
		throw TXOutOfRange(THIS_CLASS);
	return pc_Found->Key;
} /*TAvlStorageOf<TData, TKey>::PreviousKey(const TKey&)*/

//----------------------------------------------------------------------------
template <class TData, class TKey> TData& TAvlStorageOf<TData, TKey>::Assign(

	const TData& c_Data,
    const TKey& c_Key

){
	TAvlStorageRecord ToInsert(c_Data, c_Key);
	return const_cast<TAvlStorageRecord&>(Tree.Insert(ToInsert)).Data;
} /*TAvlStorageOf<TData, TKey>::Assign(const TData&, const TKey&)*/

//----------------------------------------------------------------------------
template <class TData, class TKey> bool TAvlStorageOf<TData, TKey>::Unassign(

	const TKey& c_Key

){
	TAutoPointerTo< TEtalonComparatorOf<TKey> > hDeleteKeyComparator;
    hDeleteKeyComparator.Assign(
		CloneObject(GetSortingKeyComparatorConst()), false
    ) /*Assign*/;
	(*hDeleteKeyComparator).SetEtalon(c_Key);
	TAvlStorageSearchComparatorOf<TData, TKey> DeleteComparator(
    	*hDeleteKeyComparator
    ) /*DeleteComparator*/;
    return Tree.FastDelete(DeleteComparator, indexFirst);
} /*TAvlStorageOf<TData, TKey>::Unassign(const TKey&)*/

//----------------------------------------------------------------------------
template <class TData, class TKey>
nl_ulong_t TAvlStorageOf<TData, TKey>::Delete(

	const TConditionOf<TData>& c_Condition,
    nl_ulong_t nIndex /*= indexAll*/

){
	TAvlStorageDataConditionOf<TData, TKey> DeleteCondition(c_Condition);
    return Tree.Delete(DeleteCondition, nIndex);
} /*TAvlStorageOf<TData, TKey>::Delete(const TConditionOf<TData>&, TIndex*/

//----------------------------------------------------------------------------
template <class TData, class TKey> void TAvlStorageOf<TData, TKey>::Inorder(

	TStorageOperationOn<TData, TKey>& c_Operation

){
	TAvlStorageApplyOperationOn<TData, TKey> Apply(c_Operation);
    Tree.Inorder(Apply);
} /*TAvlStorageOf<TData, TKey>::Inorder(TStorageOperation&)*/

//----------------------------------------------------------------------------
template <class TData, class TKey> void TAvlStorageOf<TData, TKey>::Postorder(

	TStorageOperationOn<TData, TKey>& c_Operation

){
	TAvlStorageApplyOperationOn<TData, TKey> Apply(c_Operation);
    Tree.Postorder(Apply);
} /*TAvlStorageOf<TData, TKey>::Postorder(TStorageOperation&)*/

//----------------------------------------------------------------------------
template <class TData, class TKey> void TAvlStorageOf<TData, TKey>::Inorder(

	TStorageConstantOperationOn<TData, TKey>& c_Operation

) const {
	TAvlStorageApplyConstantOperationOn<TData, TKey> Apply(c_Operation);
    Tree.Inorder(Apply);
} /*TAvlStorageOf<TData, TKey>::Inorder(TStorageOperation&)*/

//----------------------------------------------------------------------------
template <class TData, class TKey> void TAvlStorageOf<TData, TKey>::Postorder(

	TStorageConstantOperationOn<TData, TKey>& c_Operation

) const {
	TAvlStorageApplyConstantOperationOn<TData, TKey> Apply(c_Operation);
    Tree.Postorder(Apply);
} /*TAvlStorageOf<TData, TKey>::Postorder(TStorageOperation&)*/

//----------------------------------------------------------------------------
template <class TData, class TKey> void TAvlStorageOf<TData, TKey>::Preorder(

	TStorageOperationOn<TData, TKey>& c_Operation

){
	TAvlStorageApplyOperationOn<TData, TKey> Apply(c_Operation);
    Tree.Preorder(Apply);
} /*TAvlStorageOf<TData, TKey>::Preorder(TStorageOperation&)*/

//----------------------------------------------------------------------------
template <class TData, class TKey> void TAvlStorageOf<TData, TKey>::Preorder(

	TStorageConstantOperationOn<TData, TKey>& c_Operation

) const {
	TAvlStorageApplyConstantOperationOn<TData, TKey> Apply(c_Operation);
    Tree.Preorder(Apply);
} /*TAvlStorageOf<TData, TKey>::Preorder(TStorageOperation&)*/

//----------------------------------------------------------------------------
template <class TData, class TKey> 
TAvlStorageOf<TData, TKey>& TAvlStorageOf<TData, TKey>::operator=(

	const TStorage& c_Source
	
){
	const TAvlStorage* pc_AvlSource = (
		dynamic_cast<const TAvlStorage*>(&c_Source)
	) /*pc_AvlSource*/;

	if (pc_AvlSource) Tree.Let(pc_AvlSource->Tree);
	else Let(c_Source);

	return *this;
} /*TAvlStorageOf<TData, TKey>::operator=(const TAvlStorage&)*/

//----------------------------------------------------------------------------
template <class TData, class TKey> 
const TEtalonComparatorOf<TKey>& 
TAvlStorageOf<TData, TKey>::GetSortingKeyComparatorConst() const
{
	const TAvlStorageSortingComparator* pc_SortingComparator = (
		dynamic_cast<const TAvlStorageSortingComparator*>(
			&Tree.ComparatorConst()
		) /*dynamic_cast<const TAvlStorageSortingComparator*>*/
	) /*pc_SortingComparator*/;
	if (!pc_SortingComparator)
		throw TXInvalidComparator(THIS_CLASS);

	const TEtalonComparatorOf<TKey>* pc_KeyComparator = (
		pc_SortingComparator->KeyComparatorPtrConst()
	) /*pc_KeyComparator*/;
	if (!pc_KeyComparator)
		throw TXInvalidComparator(THIS_CLASS);

	return *pc_KeyComparator;
} /*TAvlStorageOf<TData, TKey>::GetSortingKeyComparatorConst() const*/

//============================================================================
NATLIB_NAMESPACE_END

//============================================================================
#define NLAVLST_DECLSPEC(TData, TKey, Declspec) \
	NATLIB_EXPAND_TEMPLATE2( \
		NATLIB_NAMESPACE_MEMBER(TAvlStorageRecordOf), TData, TKey, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE2*/; \
	typedef NATLIB_NAMESPACE_MEMBER2(TAvlStorageRecordOf, TData, TKey) \
		NLAVLST_TAvlStorageRecordOf_##TData##_##TKey; \
	NLAVLTRE_DECLSPEC( \
		NLAVLST_TAvlStorageRecordOf_##TData##_##TKey, Declspec \
	) /*NLAVLTRE_DECLSPEC*/; \
	NLSTORAG_DECLSPEC(TData, TKey, Declspec); \
	NATLIB_EXPAND_TEMPLATE2( \
		NATLIB_NAMESPACE_MEMBER(TAvlStorageSearchComparatorOf), \
		TData, TKey, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE2*/; \
	NATLIB_EXPAND_TEMPLATE2( \
		NATLIB_NAMESPACE_MEMBER(TAvlStorageSortingComparatorOf), \
		TData, TKey, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE2*/; \
	NATLIB_EXPAND_TEMPLATE2( \
		NATLIB_NAMESPACE_MEMBER(TAvlStorageDataConditionOf), \
		TData, TKey, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE2*/; \
	NATLIB_EXPAND_TEMPLATE2( \
		NATLIB_NAMESPACE_MEMBER(TAvlStorageApplyOperationOn), \
		TData, TKey, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE2*/; \
	NATLIB_EXPAND_TEMPLATE2( \
		NATLIB_NAMESPACE_MEMBER(TAvlStorageApplyConstantOperationOn), \
		TData, TKey, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE2*/; \
	NATLIB_EXPAND_TEMPLATE2( \
		NATLIB_NAMESPACE_MEMBER(TAvlStorageOf), TData, TKey, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE2*/

#endif /*!defined(__NATLIB_NLAVLST_H__)*/
