#ifndef __NATLIB_NLBUFFER_H__
#define __NATLIB_NLBUFFER_H__
/****************************************************************************/
/** nlbuffer.h                                                             **/
/** ---------------------------------------------------------------------- **/
/** Abstract base of the data buffer (e.g. FIFO, LIFO, etc.)               **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** November 26, 1997 by Oleg Kagan                                        **/
/****************************************************************************/

//============================================================================
#include <nlcont.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TBuffer template
//============================================================================
template <class T> class _NATLIB_TEMPLATE_DECLSPEC_ TBufferOf : 
	public TContainerOf<T> 
{
public:
	//------------------------------------------------------------------------
	typedef TContainerOf<T> TParent;
	typedef TBufferOf<T> TBuffer;

	//------------------------------------------------------------------------
	NLEXCEPT_DECLARE(
		TException, TContainerOf<T>::TException, 
		_NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
	NLEXCEPT_DECLARE_OUT_OF_RANGE(
		TXBufferOverflow, TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE_OUT_OF_RANGE*/;
	NLEXCEPT_DECLARE_OUT_OF_RANGE(
		TXBufferIsEmpty, TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE_OUT_OF_RANGE*/;

	//------------------------------------------------------------------------
	virtual NLOBJECT_DECLARE_CLONE1(TBufferOf, T) = 0;

	// Insert data according to the buffering policy (In) 
	// (see TContainerOf template)
	//------------------------------------------------------------------------
	virtual const T& Insert(const T&) = 0;
	//------------------------------------------------------------------------
	void Insert(const TContainer& c_Container) {TParent::Insert(c_Container);}

	// Returns data according to the buffering policy (Out)
	//------------------------------------------------------------------------
	virtual const T* CurrentPtr() const = 0;
	//------------------------------------------------------------------------
	const T& Current() const;
	
	// Destroys the data at the Head of the buffer and reordering the buffer 
	// according to the buffering policy
	//------------------------------------------------------------------------
	virtual void Remove(TIndex nToRemove = 1) = 0;

	//------------------------------------------------------------------------
	const T& operator*() const {return Current();}

	//------------------------------------------------------------------------
	inline TBuffer& operator=(const TBuffer& c_Source)
    {
    	Let(c_Source); return *this;
    } /*operator=(const TContainer& c_Source)*/
	//------------------------------------------------------------------------
	inline TBuffer& operator=(const TContainer& c_Source)
    {
    	Let(c_Source); return *this;
    } /*operator=(const TContainer& c_Source)*/

	//------------------------------------------------------------------------
	inline TBuffer& operator+=(const T& c_Data)
    {
    	Insert(c_Data); return *this;
    } /*operator+=(const T&)*/
	//------------------------------------------------------------------------
	inline TBuffer& operator+=(const TContainer& c_Source)
    {
    	Insert(c_Source); return *this;
    } /*operator+=(const TContainer&)*/
	//------------------------------------------------------------------------
    inline TBuffer& operator-=(const TConditionOf<T>& c_Condition)
    {
    	Delete(c_Condition); return *this;
    } /*operator-=(const TConditionOf<T>&)*/
    //------------------------------------------------------------------------
    inline TBuffer& operator-=(const TComparatorOf<T>& c_Comparator)
    {
    	Delete(c_Comparator); return *this;
    } /*operator-=(const TComparatorOf<T>&)*/
} /*TBufferOf<T>*/;

// TBufferOf template implementation
//============================================================================
template <class T> const T& TBufferOf<T>::Current() const 
{
	const T* pc_Current = CurrentPtr();
	if (!pc_Current)
		throw TXBufferIsEmpty(THIS_CLASS);
	return *pc_Current;
} /*TBufferOf<T>::Current() const*/

//============================================================================
NATLIB_NAMESPACE_END

//============================================================================
#define NLBUFFER_DECLSPEC(T, Declspec) \
	NLCONT_DECLSPEC(T, Declspec); \
	NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TBufferOf), T, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/; 

#endif /*!defined(__NATLIB_NLBUFFER_H__)*/
