#ifndef __NATLIB_NLCOMPAR_H__
#define __NATLIB_NLCOMPAR_H__
/****************************************************************************/
/** nlcompar.h                                                             **/
/** ---------------------------------------------------------------------- **/
/** Abstract comparators                                                   **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** June 14, 1996 by Oleg Kagan                                            **/
/****************************************************************************/

//============================================================================
#include <nlcondit.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

//============================================================================
#define NLCOMPAR_ETALON_ASSIGNMENT_GUARD(Target) \
	{if (!Target.EtalonPtr()) throw TXNoEtalonDefined(THIS_CLASS);}

// TComparator
//============================================================================
typedef class _NATLIB_DECLSPEC_ TComparator : public virtual TNatlibObject {
public:
	//------------------------------------------------------------------------
	typedef TNatlibObject TParent;

	// Result of any comparison
	//------------------------------------------------------------------------
	enum TResult {
		compLess = -1, compEqual, compIn = compEqual, compGreater,
		compUndefined
	} /*TResult*/;

    //------------------------------------------------------------------------
#ifndef _NATLIB_NO_EXTENDED_OVERLOADING_
    TComparator* Clone() const = 0;
#endif /*_NATLIB_NO_EXTENDED_OVERLOADING_*/
} basic_TComparator;

// TComparatorOf template
// Comparator for specified object type
//============================================================================
template <class T> class _NATLIB_TEMPLATE_DECLSPEC_ TComparatorOf :
	public TComparator 
{
public:
	//------------------------------------------------------------------------
	typedef TComparator TParent;

	//------------------------------------------------------------------------
	virtual TComparator::TResult operator()(const T&) const = 0;

	//------------------------------------------------------------------------
#ifndef _NATLIB_NO_EXTENDED_OVERLOADING_
    TComparatorOf<T>* Clone() const = 0;
#endif /*_NATLIB_NO_EXTENDED_OVERLOADING_*/
} /*TComparatorOf<T>*/;

// Explicit redeclaration of TNatibComparatorOf template for scalar types
//============================================================================
#ifndef _NATLIB_NO_EXTENDED_OVERLOADING_
#define NLCOMPAR_NONRTTI_DECL(T, Declspec) \
	_NATLIB_EXPLICIT_TEMPLATE_ \
    class Declspec TComparatorOf<T> : public TComparator { \
	public: \
		typedef TComparator TParent; \
		virtual TComparator::TResult operator()(const T&) const = 0; \
		TComparatorOf<T>* Clone() const = 0; \
	} /*TComparatorOf*/
#else
#define NLCOMPAR_NONRTTI_DECL(T, Declspec) \
	_NATLIB_EXPLICIT_TEMPLATE_ \
	class Declspec TComparatorOf<T> : public TComparator { \
	public: \
		typedef TComparator TParent; \
		virtual TComparator::TResult operator()(const T&) const = 0; \
	} /*TComparatorOf*/
#endif /*_NATLIB_NO_EXTENDED_OVERLOADING_*/

//----------------------------------------------------------------------------
#ifndef _NATLIB_NO_EXPLICIT_TEMPLATE_DECLSPEC_
	#define _NLCOMPAR_NONRTTI_DECLSPEC_ _NATLIB_DECLSPEC_
#else
	#define _NLCOMPAR_NONRTTI_DECLSPEC_ _NATLIB_TEMPLATE_DECLSPEC_
#endif /*_NATLIB_EXPLICIT_TEMPLATE_DECLSPEC_*/

//----------------------------------------------------------------------------
NLCOMPAR_NONRTTI_DECL(char, _NLCOMPAR_NONRTTI_DECLSPEC_);
NLCOMPAR_NONRTTI_DECL(unsigned char, _NLCOMPAR_NONRTTI_DECLSPEC_);
NLCOMPAR_NONRTTI_DECL(int, _NLCOMPAR_NONRTTI_DECLSPEC_);
NLCOMPAR_NONRTTI_DECL(unsigned, _NLCOMPAR_NONRTTI_DECLSPEC_);
NLCOMPAR_NONRTTI_DECL(long, _NLCOMPAR_NONRTTI_DECLSPEC_);
NLCOMPAR_NONRTTI_DECL(unsigned long, _NLCOMPAR_NONRTTI_DECLSPEC_);
NLCOMPAR_NONRTTI_DECL(float, _NLCOMPAR_NONRTTI_DECLSPEC_);
NLCOMPAR_NONRTTI_DECL(double, _NLCOMPAR_NONRTTI_DECLSPEC_);
NLCOMPAR_NONRTTI_DECL(long double, _NLCOMPAR_NONRTTI_DECLSPEC_);
NLCOMPAR_NONRTTI_DECL(nl_pvoid_t, _NLCOMPAR_NONRTTI_DECLSPEC_);
#ifndef _NATLIB_NO_BOOL_
NLCOMPAR_NONRTTI_DECL(bool, _NLCOMPAR_NONRTTI_DECLSPEC_);
#endif /*_NATLIB_NO_BOOL_*/

// TEtalonComparatorOf template
// Comprison with predefined etalon
// Use SetEtalon to define etalon for comparison and ClearEtalon to undefine
// etalon (if etalon is not defined then operator(const T&) should
// return TComparator::compUndefined)
//============================================================================
template <class T> class _NATLIB_TEMPLATE_DECLSPEC_ TEtalonComparatorOf :
	public TComparatorOf<T>
{
public:
	//------------------------------------------------------------------------
    typedef TComparatorOf<T> TParent;
    //------------------------------------------------------------------------
    typedef TEtalonComparatorOf<T> TEtalonComparator;

	//------------------------------------------------------------------------
    NLEXCEPT_DECLARE(
		TException, TComparatorOf<T>::TException, 
		_NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
	//------------------------------------------------------------------------
    NLEXCEPT_DECLARE(
		TXNoEtalonDefined, TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;

	//------------------------------------------------------------------------
	virtual void SetEtalon(const T&) = 0;
	//------------------------------------------------------------------------
	virtual void ClearEtalon() = 0;

    //------------------------------------------------------------------------
    virtual const T& Etalon() const;

    //------------------------------------------------------------------------
    virtual const T* EtalonPtr() const = 0;

	//------------------------------------------------------------------------
#ifndef _NATLIB_NO_EXTENDED_OVERLOADING_
	TEtalonComparatorOf<T>* Clone() const = 0;
#endif /*_NATLIB_NO_EXTENDED_OVERLOADING_*/
} /*TEtalonComparatorOf<T>*/;

// TAutoRestoreEtalonComparatorHandleOf template
// Allows safely set and clear etalon in the scope
//============================================================================
template <class T> 
class _NATLIB_TEMPLATE_DECLSPEC_ TAutoRestoreEtalonComparatorHandleOf {
public:
    //------------------------------------------------------------------------
	NLEXCEPT_DECLARE_BASE(TException, _NATLIB_TEMPLATE_DECLSPEC_);
    NLEXCEPT_DECLARE(
		TXNoComparatorAssigned, TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;

	//------------------------------------------------------------------------
	explicit TAutoRestoreEtalonComparatorHandleOf(
		TEtalonComparatorOf<T>& r_Comparator
	):
    	pComparator(null), pc_PreviousEtalon(null)
	{
    	Assign(r_Comparator);
	} /*TAutoRestoreEtalonComparatorHandleOf(TEtalonComparatorOf<T>&)*/

	//------------------------------------------------------------------------
	TAutoRestoreEtalonComparatorHandleOf(
		const TAutoRestoreEtalonComparatorHandleOf<T>&
	):
    	pComparator(null), pc_PreviousEtalon(null)
	{
		throw TXOperationIsNotAllowed(THIS_CLASS);
	} /*TAutoRestoreEtalonComparatorHandleOf(TEtalonComparatorOf<T>&)*/

    //------------------------------------------------------------------------
    ~TAutoRestoreEtalonComparatorHandleOf() {Unassign();}

    //------------------------------------------------------------------------
    void Assign(TEtalonComparatorOf<T>& r_Comparator)
	{
    	Unassign();
		pComparator = &r_Comparator;
    	pc_PreviousEtalon = pComparator->EtalonPtr();
	} /*Assign(TEtalonComparatorOf<T>&)*/
    //------------------------------------------------------------------------
    void Unassign()
	{
        if (pComparator) {
            pComparator->ClearEtalon();
            if (pc_PreviousEtalon) {
            	pComparator->SetEtalon(*pc_PreviousEtalon);
                pc_PreviousEtalon = null;
            } /*if*/
            pComparator = null;
        } /*if*/
	} /*Unassign()*/
	//------------------------------------------------------------------------
    TEtalonComparatorOf<T>& Comparator()
    {
    	if (!pComparator) 
			throw TXNoComparatorAssigned(THIS_CLASS);
        return *pComparator;
    } /*Comparator()*/

    //------------------------------------------------------------------------
    TEtalonComparatorOf<T>& operator*() {return Comparator();}
    //------------------------------------------------------------------------
    TEtalonComparatorOf<T>* operator->() {return &Comparator();}

protected:
    //------------------------------------------------------------------------
    TEtalonComparatorOf<T>* pComparator;
    const T* pc_PreviousEtalon;
} /*TAutoRestoreEtalonComparatorHandleOf<T>*/;

// TPointerSavingEtalonComparatorOf template
// Useful implementation of Etalon keeping scheme...
//============================================================================
template <class T>
class _NATLIB_TEMPLATE_DECLSPEC_ TPointerSavingEtalonComparatorOf :
	public TEtalonComparatorOf<T>
{
public:
	//------------------------------------------------------------------------
	typedef TEtalonComparatorOf<T> TParent;

	//------------------------------------------------------------------------
	TPointerSavingEtalonComparatorOf() : pc_Etalon(null) {}

    //------------------------------------------------------------------------
    TPointerSavingEtalonComparatorOf(
    	const TPointerSavingEtalonComparatorOf<T>& c_Source
    ) :
    	pc_Etalon(c_Source.pc_Etalon)
    {} /*TPointerSavingEtalonComparatorOf(...)*/

	//------------------------------------------------------------------------
	void SetEtalon(const T& c_Object) {pc_Etalon = &c_Object;}
	//------------------------------------------------------------------------
	void ClearEtalon() {pc_Etalon = null;}

    //------------------------------------------------------------------------
    const T* EtalonPtr() const {return pc_Etalon;}

    //------------------------------------------------------------------------
#ifndef _NATLIB_NO_EXTENDED_OVERLOADING_
	TPointerSavingEtalonComparatorOf<T>* Clone() const = 0;
#endif /*_NATLIB_NO_EXTENDED_OVERLOADING_*/

protected:
	//------------------------------------------------------------------------
	const T* pc_Etalon;
} /*TPointerSavingEtalonComparatorOf<T>*/;

// TSafeDefaultComparatorOf template
// Default comparison for any object implementation
// Note: operator> operator< and operator== must be defined for T
//============================================================================
template <class T> class _NATLIB_TEMPLATE_DECLSPEC_ TSafeDefaultComparatorOf :
	public TPointerSavingEtalonComparatorOf<T>
{
public:
	//------------------------------------------------------------------------
	typedef TPointerSavingEtalonComparatorOf<T> TParent;

    //------------------------------------------------------------------------
    TSafeDefaultComparatorOf() {}
    //------------------------------------------------------------------------
    TSafeDefaultComparatorOf(const TSafeDefaultComparatorOf<T>& c_Source):
    	TParent(c_Source)
    {} /*TSafeDefaultComparatorOf(const TSafeDefaultComparatorOf&)*/
	//------------------------------------------------------------------------
    TSafeDefaultComparatorOf(
		const TPointerSavingEtalonComparatorOf<T>& c_Source
    ):
    	TParent(c_Source)
    {} /*TSafeDefaultComparatorOf(const TPointerSavingEtalonComparatorOf&)*/

	//------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE1(TSafeDefaultComparatorOf, T);

	//------------------------------------------------------------------------
	TComparator::TResult operator()(const T&) const;
} /*TSafeDefaultComparatorOf<T>*/;

// TDefaultComparatorOf template
// Fast default comparison for any object implementation
// Note: operator> operator< must be defined for T
//============================================================================
template <class T> class _NATLIB_TEMPLATE_DECLSPEC_ TDefaultComparatorOf :
	public TPointerSavingEtalonComparatorOf<T>
{
public:
	//------------------------------------------------------------------------
	typedef TPointerSavingEtalonComparatorOf<T> TParent;

    //------------------------------------------------------------------------
    TDefaultComparatorOf() {}
    //------------------------------------------------------------------------
    TDefaultComparatorOf(const TDefaultComparatorOf<T>& c_Source) :
		TParent(c_Source)
	{} /*TDefaultComparatorOf(const TDefaultComparatorOf<T>&)*/
    //------------------------------------------------------------------------
    TDefaultComparatorOf(const TPointerSavingEtalonComparatorOf<T>& c_Source):
    	TParent(c_Source)
    {} /*TDefaultComparatorOf(const TPointerSavingEtalonComparatorOf<T>&)*/

	//------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE1(TDefaultComparatorOf, T);

	//------------------------------------------------------------------------
	TComparator::TResult operator()(const T&) const;
} /*TDefaultComparatorOf<T>*/;

// TIsEqualByComparator template
//============================================================================
template <class T> class _NATLIB_TEMPLATE_DECLSPEC_ TIsEqualByComparatorOf :
	public TConditionOf<T>
{
public:
	//------------------------------------------------------------------------
	typedef TConditionOf<T> TParent;

    //------------------------------------------------------------------------
    NLEXCEPT_DECLARE(
    	TXNoComparatorSpecified, TConditionOf<T>::TException, 
		_NATLIB_TEMPLATE_DECLSPEC_
    ) /*NLEXCEPT_DECLARE*/;

	//------------------------------------------------------------------------
    TIsEqualByComparatorOf(const TIsEqualByComparatorOf<T>& c_Condition) :
    	pComparator(null)
	{
    	if (c_Condition.pComparator) SetComparator(c_Condition.Comparator());
    } /*TIsEqualByComparatorOf(const TIsEqualByComparatorOf<T>&)*/

	//------------------------------------------------------------------------
    TIsEqualByComparatorOf(const TComparatorOf<T>& c_Comparator) :
    	pComparator(null)
    {
		SetComparator(c_Comparator);
    } /*TIsEqualByComparatorOf(const TComparatorOf<T>&)*/

	//------------------------------------------------------------------------
    ~TIsEqualByComparatorOf() {delete pComparator;}

	//------------------------------------------------------------------------
    void SetComparator(const TComparatorOf<T>& c_Comparator)
    {
    	if (pComparator) {
			delete pComparator;
            pComparator = null;
        } /*if*/
        pComparator = CloneObject(c_Comparator);
    } /*SetComparator(const TComparatorOf<T>&)*/

	//------------------------------------------------------------------------
    const TComparatorOf<T>& Comparator() const
    {
    	if (!pComparator) 
			throw TXNoComparatorSpecified(THIS_CLASS);
        return *pComparator;
    } /*Comparator() const*/

    //------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE1(TIsEqualByComparatorOf, T);

	//------------------------------------------------------------------------
    bool operator()(const T& c_Data) const
    {
    	return Comparator()(c_Data) == TComparatorOf<T>::compEqual;
    } /*operator()(const T&)*/

protected:
	//------------------------------------------------------------------------
    TComparatorOf<T>* pComparator;
} /*TIsEqualByComparatorOf<T>*/;

// TEtalonComparatorOf template implementation
//============================================================================
template <class T> const T& TEtalonComparatorOf<T>::Etalon() const
{
	const T* pc_Etalon = EtalonPtr();
	if (!pc_Etalon) 
		throw TXNoEtalonDefined(THIS_CLASS);
	return *pc_Etalon;
} /*TEtalonComparatorOf<T>::Etalon() const*/

// TSafeDefaultComparatorOf template implementation
//============================================================================
template <class T>
NLOBJECT_IMPLEMENT_CLONE_DEFAULT1(TSafeDefaultComparatorOf, T)

//----------------------------------------------------------------------------
template <class T>
TComparator::TResult TSafeDefaultComparatorOf<T>::operator()(

	const T& c_Object

) const {
	NLCOMPAR_ETALON_ASSIGNMENT_GUARD((*this));

	if (*pc_Etalon < c_Object) return compLess;
	if (*pc_Etalon > c_Object) return compGreater;
	if (*pc_Etalon == c_Object) return compEqual;
	return compUndefined;
} /*TSafeDefaultComparatorOf<T>::operator()(const T&) const*/;

// TDefaultComparatorOf template implementation
//============================================================================
template <class T> NLOBJECT_IMPLEMENT_CLONE_DEFAULT1(TDefaultComparatorOf, T)

//----------------------------------------------------------------------------
template <class T> TComparator::TResult TDefaultComparatorOf<T>::operator()(

	const T& c_Object

) const {
	NLCOMPAR_ETALON_ASSIGNMENT_GUARD((*this));

	if (*pc_Etalon < c_Object) return compLess;
	if (*pc_Etalon > c_Object) return compGreater;
	return compEqual;
} /*TDefaultComparatorOf<T>::operator()(const T&) const*/;

// TIsEqualByComparatorOf template implementation
//============================================================================
template <class T>
NLOBJECT_IMPLEMENT_CLONE_DEFAULT1(TIsEqualByComparatorOf, T)

//============================================================================
NATLIB_NAMESPACE_END

//============================================================================
#define NLCOMPAR_DECLSPEC(T, Declspec) \
	NLCONDIT_DECLSPEC(T, Declspec); \
    NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TComparatorOf), T, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/; \
    NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TEtalonComparatorOf), \
		T, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/; \
    NATLIB_EXPAND_TEMPLATE1( \
    	NATLIB_NAMESPACE_MEMBER(TAutoRestoreEtalonComparatorHandleOf), \
		T, Declspec \
    ) /*NATLIB_EXPAND_TEMPLATE1*/; \
    NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TPointerSavingEtalonComparatorOf), \
		T, Declspec \
    ) /*NATLIB_EXPAND_TEMPLATE1*/; \
	NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TIsEqualByComparatorOf), T, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/

#endif /*!defined(__NATLIB_NLCOMPAR_H__)*/
