#ifndef __NATLIB_NLCOMPAT_H__
#define __NATLIB_NLCOMPAT_H__
/****************************************************************************/
/** nlcompat.h                                                             **/
/** ---------------------------------------------------------------------- **/
/** Compatibility tools main header                                        **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** June 14, 1996 by Oleg Kagan                                            **/
/****************************************************************************/

// C++ only
//============================================================================
#ifndef __cplusplus
	#error Natlib Class Library can be compiled with C++ only
#endif /*__cplusplus*/

//============================================================================
#define NATLIB_NOTHING /**/

// C++ compilers
// Use next declarations if Your compiler doesn't support C++ aproppriate
// C++ futures
//
//   #define bool int 
//   #define true 1
//   #define false 0
//   #define typename class
//   #define explicit
//   #define mutable
//
// Use next declaration if Your compiler doesn't support C++ namespaces
//
//   #define _NATLIB_NO_NAMESPACE_
//
// Use next definition to export or import Natlib classes and templates
//
//   #define _NATLIB_DECLSPEC_ ...
//   #define _NATLIB_TEMPLATE_DECLSPEC_ ...
//
// Use next definition to export or import Natlib Tasks classes and templates
//   #define _NATLIB_TASKS_DECLSPEC_ ...
//   #define _NATLIB_TASKS_TEMPLATE_DECLSPEC_ ...
//
// Use next definition if You compiler doesn't support explicit specificators
// in form "template class Declspec TTemplate<T>", where Declspec can be
// _export, _import, e.t.c. TTemplate is template name and T is  predefined
// type, for example "template class _export TNatlibBinaryTree<int>;"
//
//   #define _NATLIB_NO_EXPLICIT_TEMPLATE_DECLSPEC_
// 
// Use next templates for throw specificator (functions) "void f() throw()
//
//   #define _NATLIB_THROW_NONE_ throw()
//   #define _NATLIB_THROW2_(Ex1, Ex2) throw(Ex1, Ex2)
//   #define _NATLIB_THROW2_(Ex1, Ex2, Ex3) throw(Ex1, Ex2, Ex3)
//   ...
//
// Use next definition if Your compiler doesn't support extended virtual
// member overloading
//
//   #define _NATLIB_NO_EXTENDED_OVERLOADING_
//
// Use next definition to expand template with specified modificator
//
//	#define NATLIB_EXPAND_TEMPLATE(TemplateDecl, Declspec)
//
// Use next definition to precede explicit template declaration
//
//	#define _NATLIB_EXPLICIT_TEMPLATE_ template <>
//
// Use the next definition to disable Wide (Unicode) string declaration
//
//  #define _NATLIB_NO_WIDE_STRINGS_
//
// Use the next definition to specify way of adding specifiers to functions
// and variables declarations
//
//  #define NATLIB_FUNC_DECL(Declspec, ReturnType)
//============================================================================
#if defined(__BCPLUSPLUS__)  // Borland compilers
	#include <nlbc.h>
#elif defined(_MSC_VER) // Microsoft compilers
	#include <nlmsc.h>
#elif defined(__GNUG__) // GNU GPP compilers
	#include <nlgpp.h>
#else // Unknown compiler
	#error Compatibility tools have not been defined for Your compiler yet
#endif /*compilers compatibility*/

// Declaration specificator for all Natlib objects
// May be used to compile Natlib for different models and/or
// export or import Natlib classes
//============================================================================
#ifndef _NATLIB_DECLSPEC_
	#pragma message("_NATLIB_DECLSPEC_ undefined and ignored")
	#define _NATLIB_DECLSPEC_ NATLIB_NOTHING
#endif /*_NATLIB_DECLSPEC_*/

//----------------------------------------------------------------------------
#ifndef _NATLIB_TEMPLATE_DECLSPEC_
	#pragma message("_NATLIB_TEMPLATE_DECLSPEC_ undefined and ignored")
	#define _NATLIB_TEMPLATE_DECLSPEC_ NATLIB_NOTHING
#endif /*_NATLIB_TEMPLATE_DECLSPEC_*/

//----------------------------------------------------------------------------
#ifndef _NATLIB_TASKS_DECLSPEC_
	#pragma message("_NATLIB_TASKS_DECLSPEC_ undefined and assumed equal to _NATLIB_DECLSPEC_")
	#define _NATLIB_TASKS_DECLSPEC_ _NATLIB_DECLSPEC_
#endif /*_NATLIB_TASKS_DECLSPEC_*/

//----------------------------------------------------------------------------
#ifndef _NATLIB_TASKS_TEMPLATE_DECLSPEC_
	#pragma message("_NATLIB_TASKS_TEMPLATE_DECLSPEC_ undefined and assumed equal to _NATLIB_DECLSPEC_")
	#define _NATLIB_TASKS_TEMPLATE_DECLSPEC_ _NATLIB_TEMPLATE_DECLSPEC_
#endif /*_NATLIB_TASKS_TEMPLATE_DECLSPEC_*/

// Natlib namespace
//============================================================================
#ifndef _NATLIB_NO_NAMESPACE_
	#ifndef _NATLIB_NAMESPACE_
    	#define _NATLIB_NAMESPACE_ Natlib
    #endif /*_NATLIB_NAMESPACE_*/
#else
	#ifdef _NATLIB_NAMESPACE_
    	#pragma message("_NATLIB_NAMESPACE_ have been undefined")
    	#undef _NATLIB_NAMESPACE_
    #endif /*_NATLIB_NAMESPACE_*/
#endif /*_NATLIB_NO_NAMESPACE_*/

//----------------------------------------------------------------------------
#ifdef _NATLIB_NAMESPACE_
	#define NATLIB_NAMESPACE_BEGIN namespace _NATLIB_NAMESPACE_ {
    #define NATLIB_NAMESPACE_END }
    #define USING_NATLIB_NAMESPACE using namespace ::_NATLIB_NAMESPACE_;
	#define NATLIB_NAMESPACE_MEMBER(Member) _NATLIB_NAMESPACE_::##Member
	#define NATLIB_NAMESPACE_MEMBER1(Member, T1) \
		_NATLIB_NAMESPACE_::##Member< T1 >
	#define NATLIB_NAMESPACE_MEMBER2(Member, T1, T2) \
		_NATLIB_NAMESPACE_::##Member< T1, T2 >
	#define NATLIB_NAMESPACE_MEMBER3(Member, T1, T2, T3) \
		_NATLIB_NAMESPACE_::##Member< T1, T2, T3 >
#else
	#define NATLIB_NAMESPACE_BEGIN NATLIB_NOTHING
    #define NATLIB_NAMESPACE_END NATLIB_NOTHING
    #define USING_NATLIB_NAMESPACE NATLIB_NOTHING
    #define NATLIB_NAMESPACE_MEMBER(Member) Member
	#define NATLIB_NAMESPACE_MEMBER1(Member, T1) Member< T1 >
	#define NATLIB_NAMESPACE_MEMBER2(Member, T1, T2) Member< T1, T2 >
	#define NATLIB_NAMESPACE_MEMBER3(Member, T1, T2, T3) Member< T1, T2, T3 >
#endif /*_NATLIB_NAMESPACE_*/

// NULL pointer
//============================================================================
#ifndef _NATLIB_NO_NULL_
	#define null 0
#endif /*_NATLIB_NO_NULL_*/

// void pointer
//============================================================================
typedef void* nl_pvoid_t;
typedef const void* nl_pcvoid_t;

// NATLIB_INIT
// Used in TObject::TInitializer to initialize natlibs internals
//============================================================================
#ifndef NATLIB_INIT
	#define NATLIB_INIT NATLIB_NOTHING
#endif /*NATLIB_INIT*/

// Insure basic includes
//============================================================================
#include <stdio.h>
#include <limits.h>

// NATLIB_GET_COMPATIBILITY_CORE
//============================================================================
#ifndef NATLIB_GET_COMPATIBILITY_CORE
	#define NATLIB_GET_COMPATIBILITY_CORE NLCMPTEX_StdGetCompatibilityCore
#endif /*NATLIB_GET_COMPATIBILITY_CORE*/

#endif /*!defined(__NATLIB_COMPAT_H__)*/
