#ifndef __NATLIB_NLCONDIT_H__
#define __NATLIB_NLCONDIT_H__
/****************************************************************************/
/** nlcondit.h                                                             **/
/** ---------------------------------------------------------------------- **/
/** Abstract condition base                                                **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** October 31, 1996 by Oleg Kagan                                         **/
/****************************************************************************/

//============================================================================
#include <nlobject.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TCondition
//============================================================================
typedef class _NATLIB_DECLSPEC_ TCondition : public virtual TNatlibObject {
public:
	//------------------------------------------------------------------------
	typedef TNatlibObject TParent;
	//------------------------------------------------------------------------
	NLEXCEPT_DECLARE(
		TException, TNatlibObject::TException, _NATLIB_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
   	//------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE(TCondition);
} basic_TCondition;

// TConditionOf template
// condition for specific object type
//============================================================================
template <class T> class _NATLIB_TEMPLATE_DECLSPEC_ TConditionOf : 
	public TCondition 
{
public:
	//------------------------------------------------------------------------
	typedef basic_TCondition TParent;
	//------------------------------------------------------------------------
	NLEXCEPT_DECLARE(
		TException, TCondition::TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
	//------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE1(TConditionOf, T) = 0;
	//------------------------------------------------------------------------
	virtual bool operator()(const T&) const = 0;
} /*TConditionOf<T>*/;

// TAnyOf template
// Condition that true for all values of specified type
//============================================================================
template <class T> class _NATLIB_TEMPLATE_DECLSPEC_ TAnyOf :
	public TConditionOf<T>
{
public:
	//------------------------------------------------------------------------
    typedef TConditionOf<T> TParent;
	//------------------------------------------------------------------------
    TAnyOf() {}
	//------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE1(TAnyOf, T);
    //------------------------------------------------------------------------
	bool operator()(const T&) const {return true;}
} /*TAnyOf*/;

// TAnyOf template implementation
//============================================================================
template <class T> NLOBJECT_IMPLEMENT_CLONE_DEFAULT1(TAnyOf, T)

//============================================================================
NATLIB_NAMESPACE_END

//============================================================================
#define NLCONDIT_DECLSPEC(T, Declspec) \
	NLOBJECT_DECLSPEC(T, Declspec); \
    NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TConditionOf), T, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/; \
	NATLIB_EXPAND_TEMPLATE1(NATLIB_NAMESPACE_MEMBER(TAnyOf), T, Declspec)

#endif /*!defined(__NATLIB_NLCONDIT_H__)*/
