#ifndef __NATLIB_NLDEQUE_H__
#define __NATLIB_NLDEQUE_H__
/****************************************************************************/
/** nldeque.h                                                              **/
/** ---------------------------------------------------------------------- **/
/** Contents declarations of the double ended queue (deque).               **/
/** This implementantion is based on double-linked list.                   **/
/** Note: Copy contructor for template argument must be available.         **/
/** Neither support for priority exists nor it can be easily added, if you **/
/** are interesting in please look for implementation based on tree        **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** April 28, 1996 by Ilya Marchevsky                                      **/
/****************************************************************************/

//============================================================================
#include <nlqueue.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

//============================================================================
#define NLDEQUE_ALLOCATOR_DEFAULT NLQUEUE_ALLOCATOR_DEFAULT

// TDequeueOf template
//============================================================================
template <class T> class TDequeueOf : public TQueueOf<T> {
public:
	//------------------------------------------------------------------------
	typedef TQueueOf<T> TParent;
	typedef TDequeueOf<T> TDequeue;

public:
	//------------------------------------------------------------------------
    TDequeueOf(const TAllocator& = NLDEQUE_ALLOCATOR_DEFAULT());
	//------------------------------------------------------------------------
    TDequeueOf(const TDequeue&);
    //------------------------------------------------------------------------
    TDequeueOf(
		const TContainer&, const TAllocator& = NLDEQUE_ALLOCATOR_DEFAULT()
	) /*TDequeueOf(const TContainer&, const TAllocator&)*/;

    //------------------------------------------------------------------------
    void AssertValid() const;

    //------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE1(TDequeueOf, T);

    //------------------------------------------------------------------------
    const T& Insert(const T& c_Data) {return TParent::Insert(c_Data);}
	//------------------------------------------------------------------------
	void Insert(const TContainer& c_Container) {TParent::Insert(c_Container);}
	//------------------------------------------------------------------------
    const T* CurrentPtr() const {return TParent::CurrentPtr();}
    //------------------------------------------------------------------------
    void Remove(TIndex nToRemove = 1) {TParent::Remove(nToRemove);}

    //------------------------------------------------------------------------
    const T& AppendHead(const T& c_Data);
    //------------------------------------------------------------------------
    const T& AppendTail(const T& c_Data) {return Inqueue(c_Data);}
    //------------------------------------------------------------------------
    void RemoveHead(TIndex nToRemove = 1) {Dequeue(nToRemove);}
    //------------------------------------------------------------------------
    void RemoveTail(TIndex = 1);

	//------------------------------------------------------------------------
    const T& Head() const {return TParent::Current();}
	//------------------------------------------------------------------------
    const T& Tail() const;

	//------------------------------------------------------------------------
	inline TDequeue& operator=(const TDequeue& c_Source)
    {
    	TParent::Let(c_Source); return *this;
    } /*TDequeue& operator=(const TDequeue&)*/
	//------------------------------------------------------------------------
	inline TDequeue& operator=(const TContainer& c_Source)
    {
    	TParent::Let(c_Source); return *this;
    } /*operator=(const TContainer&)*/

	//------------------------------------------------------------------------
	inline TDequeue& operator+=(const T& c_Data)
    {
    	Insert(c_Data); return *this;
    } /*operator+=(const T&)*/
	//------------------------------------------------------------------------
	inline TDequeue& operator+=(const TContainer& c_Source)
    {
    	Insert(c_Source); return *this;
    } /*operator+=(const TContainer&)*/
	//------------------------------------------------------------------------
	inline TDequeue& operator-=(const TConditionOf<T>& c_Condition)
    {
    	Delete(c_Condition); return *this;
    } /*operator-=(const TConditionOf<T>&)*/
    //------------------------------------------------------------------------
	inline TDequeue& operator-=(const TComparatorOf<T>& c_Comparator)
    {
    	Delete(c_Comparator); return *this;
    } /*operator-=(const TComparatorOf<T>&)*/
} /*TDequeueOf<T>*/;

// TDequeueOf template implementation
//============================================================================
template <class T> TDequeueOf<T>::TDequeueOf(

	const TAllocator& c_Allocator /*= NLDEQUE_ALLOCATOR_DEFAULT()*/

):
	TParent(c_Allocator)
{} /*TDequeueOf<T>(const TAllocator&)*/

//----------------------------------------------------------------------------
template <class T> TDequeueOf<T>::TDequeueOf(

	const TDequeue& c_Source
	
):
	TParent(c_Source)
{} /*TDequeueOf<T>(const TDequeue&)*/

//----------------------------------------------------------------------------
template <class T> TDequeueOf<T>::TDequeueOf(

	const TContainer& c_Source, 
	const TAllocator& c_Allocator /*= NLDEQUE_ALLOCATOR_DEFAULT()*/

):
	TParent(c_Source, c_Allocator)
{} /*TDequeueOf(const TContainer&, const TAllocator&)*/;

//----------------------------------------------------------------------------
template <class T> void TDequeueOf<T>::AssertValid() const
{
	TParent::AssertValid();
} /*TDequeueOf<T>::AssertValid() const*/

//----------------------------------------------------------------------------
template <class T> NLOBJECT_IMPLEMENT_CLONE_DEFAULT1(TDequeueOf, T)

//----------------------------------------------------------------------------
template <class T> const T& TDequeueOf<T>::Tail() const
{
	if (IsEmpty())
		throw TXBufferIsEmpty(THIS_CLASS);
	TListConstantCursorOf<T> Cursor; Cursor.ToLast();
	return *Cursor;
} /*TDequeueOf<T>::Tail() const*/

//----------------------------------------------------------------------------
template <class T> const T& TDequeueOf<T>::AppendHead(

	const T& c_Data

){
	if (IsEmpty()) return List.Insert(c_Data);
   	TListCursorOf<T> Cursor; Cursor.ToLast();
    return Cursor.InsertAfter(c_Data);
} /*TDequeueOf<T>::AppendHead(const T&)*/

//----------------------------------------------------------------------------
template <class T> void TDequeueOf<T>::RemoveTail(

	TIndex nToDelete /*= 1*/
	
){
	while (nToDelete) {
		if (IsEmpty()) 
			throw TXBufferIsEmpty(THIS_CLASS);
		TListCursorOf<T> Cursor(List); Cursor.ToLast(); Cursor.Delete();
	} /*while (nToDelete)*/
} /*TDequeueOf<T>::RemoveTail(TIndex)*/

//============================================================================
NATLIB_NAMESPACE_END

//============================================================================
#define NLDEQUE_DECLSPEC(T, Declspec) \
	NLLIST_DECLSPEC(T, Declspec); \
    NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TDequeueOf), T, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/

#endif /*!defined(__NATLIB_NLDEQUE_H__)*/
