#ifndef __NATLIB_NLEXCEPT_H__
#define __NATLIB_NLEXCEPT_H__
/****************************************************************************/
/** nlexcept.h                                                             **/
/** ---------------------------------------------------------------------- **/
/** Definition of base classes for Errors Handling                         **/
/** and Emergency Information Passing                                      **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** June 16, 1996 by Oleg Kagan                                            **/
/****************************************************************************/

//============================================================================
#include <nlsrcloc.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TNatlibException
// The base for all exceptions that will represent errors of the system
//============================================================================
typedef class _NATLIB_DECLSPEC_ TNatlibException : public TSourceLocation {
public:
	//------------------------------------------------------------------------
	static const char* const c_szWhyWhereSeparator;
	static const char* const c_szStartDelimiter;
	static const char* const c_szEndDelimiter;
    static const char* const c_szSeparator;

	//------------------------------------------------------------------------
	TNatlibException(
		const TSourceLocation&,
		const TStdString& c_sExplanation = TStdString()
	) _NATLIB_THROW_NONE_ /*TNatlibException(const TSourceLocation&, ...)*/;
	// Copy constructor
	//------------------------------------------------------------------------
	TNatlibException(const TNatlibException&) _NATLIB_THROW_NONE_;

	//------------------------------------------------------------------------
	virtual void Rethrow() _NATLIB_THROW1_(TNatlibException);

	// Rethrow exception with adding location information
	//------------------------------------------------------------------------
	virtual void Rethrow(const TSourceLocation&)
		_NATLIB_THROW1_(TNatlibException)
	/*Rethrow(const TSourceLocation&) const*/;

	//------------------------------------------------------------------------
	virtual TStdString Explanation() const _NATLIB_THROW_NONE_;

	//------------------------------------------------------------------------
	virtual TStdString Why() const _NATLIB_THROW_NONE_;

	//------------------------------------------------------------------------
	virtual TStdString OptimizedWhere() const _NATLIB_THROW_NONE_;

	//------------------------------------------------------------------------
	TNatlibException& operator=(const TNatlibException&) _NATLIB_THROW_NONE_;

	//------------------------------------------------------------------------
	operator TStdString() const _NATLIB_THROW_NONE_;

protected:
	//------------------------------------------------------------------------
	TStdString sExplanation;
} basic_TNatlibException;

//============================================================================
#define NLEXCEPT_DECLARE_BASE(TExceptionBase, Declspec) \
	typedef class Declspec TExceptionBase : \
    	public virtual basic_TNatlibException \
    { \
	public: \
		/*typedef basic_TNatlibException TParent;*/ \
		TExceptionBase( \
			const TSourceLocation& c_Location, \
			const TStdString& c_sExplanation = TStdString() \
		) : \
			basic_TNatlibException(c_Location, c_sExplanation) \
		{} /*TExceptionBase(const TSourceLocation&)*/ \
		TExceptionBase(const TExceptionBase& c_Exception) : \
			basic_TNatlibException(c_Exception) \
		{} /*TExceptionBase(const TNatlibException&)*/ \
	} typedef_base_declspec##TExceptionBase

//============================================================================
#define NLEXCEPT_DECLARE(TXSimpleError, TExceptionBase, Declspec) \
	typedef class Declspec TXSimpleError : public TExceptionBase { \
	public: \
		/*typedef TExceptionBase TParent;*/ \
		TXSimpleError( \
			const TSourceLocation& c_Location, \
			const TStdString& c_sExplanation = TStdString() \
		) : \
        	basic_TNatlibException(c_Location, c_sExplanation), \
			TExceptionBase(c_Location, c_sExplanation) \
		{} /*TXSimpleError(const TSourceLocation&)*/ \
		TXSimpleError(const TXSimpleError& c_Exception) : \
			basic_TNatlibException(c_Exception), TExceptionBase(c_Exception) \
		{} /*TXSimpleError(const TXSimpleError&)*/ \
	} typedef_##TXSimpleError

//============================================================================
#define NLEXCEPT_DECLARE_DATA_HOLDER( \
	TXDataHolder, T, TExceptionBase, Declspec \
) \
	typedef class Declspec TXDataHolder : public TExceptionBase { \
	public: \
    	/*typedef TExceptionBase TParent*/ \
        TXDataHolder( \
        	const T& c_XData, \
            const TSourceLocation& c_Location, \
            const TStdString& c_sExplanation = TStdString() \
        ) : \
        	basic_TNatlibException(c_Location, c_sExplanation), \
            TExceptionBase(c_Location, c_sExplanation), \
            XData(c_XData) \
        {} /*TXDataHolder(cobst T&, const TSourceLocation&, ...)*/ \
        TXDataHolder(const TXDataHolder& c_XDataHolder) : \
        	basic_TNatlibException(c_XDataHolder), \
            TExceptionBase(c_XDataHolder), \
            XData(c_XDataHolder.XData) \
        {} /*TXDataHolder(const TXDataHolder&)*/ \
        const T& Value() const {return XData;} \
    protected: \
    	T XData; \
    } typedef_##TXDataHolder

// TXBusy
// The base for all syncronization exceptions
//============================================================================
NLEXCEPT_DECLARE_BASE(basic_TXBusy, _NATLIB_DECLSPEC_);
typedef basic_TXBusy TXBusy;

//============================================================================
#define NLEXCEPT_DECLARE_BUSY(TLocalXBusy, TExceptionBase, Declspec) \
	typedef class Declspec TLocalXBusy : \
    	public TExceptionBase, public basic_TXBusy \
    { \
	public: \
		/*typedef TExceptionBase TParent;*/ \
		TLocalXBusy( \
			const TSourceLocation& c_Location, \
			const TStdString& c_sExplanation = TStdString() \
		): \
			basic_TNatlibException(c_Location, c_sExplanation), \
			TExceptionBase(c_Location, c_sExplanation), \
			basic_TXBusy(c_Location, c_sExplanation) \
		{} /*TLocalXBusy(const TSourceLocation&)*/ \
		TLocalXBusy(const TLocalXBusy& c_XBusy) : \
			basic_TNatlibException(c_XBusy), \
            TExceptionBase(c_XBusy), \
			basic_TXBusy(c_XBusy) \
		{} /*TLocalXBusy(const TLocalXBusy&)*/ \
	} typedef_##TLocalXBusy

// TXAbort
// The base class for abortion of the loops, processes, operation e.t.c
//============================================================================
NLEXCEPT_DECLARE_BASE(basic_TXAbort, _NATLIB_DECLSPEC_);
typedef basic_TXAbort TXAbort;

//============================================================================
#define NLEXCEPT_DECLARE_ABORT(TLocalXAbort, TExceptionBase, Declspec) \
	typedef class Declspec TLocalXAbort : \
    	public TExceptionBase, public basic_TXAbort \
    { \
	public: \
		/*typedef TExceptionBase TParent;*/ \
		TLocalXAbort( \
			const TSourceLocation& c_Location, \
			const TStdString& c_sExplanation = TStdString() \
		): \
			basic_TNatlibException(c_Location, c_sExplanation), \
			TExceptionBase(c_Location, c_sExplanation), \
			basic_TXAbort(c_Location, c_sExplanation) \
		{} /*TLocalXAbort(const TSourceLocation&)*/ \
		TLocalXAbort(const TLocalXAbort& c_XAbort) : \
			basic_TNatlibException(c_XAbort), \
            TExceptionBase(c_XAbort), \
			basic_TXAbort(c_XAbort) \
		{} /*TLocalXAbort(const TLocalXAbort&)*/ \
	} typedef_##TLocalXAbort

// TXInvalidParameter
// The base for invalid paramaeter exceptions classes
//============================================================================
NLEXCEPT_DECLARE_BASE(TXInvalidParameter, _NATLIB_DECLSPEC_);
typedef TXInvalidParameter basic_TXInvalidParameter;

//============================================================================
#define NLEXCEPT_DECLARE_INVALID_PARAMETER( \
	TLocalXBadParam, TExceptionBase, Declspec \
) \
	typedef class Declspec TLocalXBadParam : \
    	public TExceptionBase, public basic_TXInvalidParameter \
    { \
	public: \
		/*typedef TExceptionBase TParent;*/ \
		TLocalXBadParam( \
			const TSourceLocation& c_Location, \
			const TStdString& c_sExplanation = TStdString() \
		): \
			basic_TNatlibException(c_Location, c_sExplanation), \
			TExceptionBase(c_Location, c_sExplanation), \
			basic_TXInvalidParameter(c_Location, c_sExplanation) \
		{} /*TLocalXBadParam(const TSourceLocation&)*/ \
		TLocalXBadParam(const TLocalXBadParam& c_XBadParam) : \
			basic_TNatlibException(c_XBadParam), \
            TExceptionBase(c_XBadParam), \
			basic_TXInvalidParameter(c_XBadParam) \
		{} /*TLocalXBadParam(const TLocalXBadParam&)*/ \
	} typedef_##TLocalXBadParam

// TXLowMemory
// Low Memory Exception
//============================================================================
typedef class _NATLIB_DECLSPEC_ TXLowMemory: virtual public TNatlibException {
public: 
	//------------------------------------------------------------------------
//	typedef TNatlibException TParent;

	//------------------------------------------------------------------------
	TXLowMemory(
    	const TSourceLocation&,
		const size_t c_nRequested = 0,
		const TStdString& c_sExplanation = TStdString()
    ) _NATLIB_THROW_NONE_ /*TXLowMemory(const TSourceLocation&, const s...)*/;
	//------------------------------------------------------------------------
	TXLowMemory(const TXLowMemory&) _NATLIB_THROW_NONE_;

	//------------------------------------------------------------------------
	TStdString Explanation() const _NATLIB_THROW_NONE_;

	//------------------------------------------------------------------------
	virtual size_t Requested() const _NATLIB_THROW_NONE_;

protected:
	//------------------------------------------------------------------------
	size_t nRequested;
} basic_TXLowMemory;

//============================================================================
#define NLEXCEPT_DECLARE_LOW_MEMORY( \
	TLocalXLowMemory, TExceptionBase, Declspec \
) \
	typedef class Declspec TLocalXLowMemory : \
		public TExceptionBase, public basic_TXLowMemory \
	{ \
	public: \
		/*typedef TExceptionBase TParent;*/ \
		TLocalXLowMemory( \
			const TSourceLocation& c_Location, \
			const size_t c_nRequested = 0, \
			const TStdString& c_sExplanation = TStdString() \
		): \
        	basic_TNatlibException(c_Location, c_sExplanation), \
	        basic_TXLowMemory(c_Location, c_nRequested, c_sExplanation), \
    		TExceptionBase(c_Location, c_sExplanation) \
		{} /*TLocalXLowMemory(const TSourceLocation, const size_t = 0)*/ \
		TLocalXLowMemory(const TLocalXLowMemory& c_XLowMemory) : \
			basic_TNatlibException(c_XLowMemory), \
			TExceptionBase(c_XLowMemory), \
            basic_TXLowMemory(c_XLowMemory) \
		{} /*TLocalXLowMemory(cons TLocalXLowMemory&)*/ \
	} typedef_##TLocalXLowMemory

// TXOutOfRange
// The basic "out of range" exception
//============================================================================
NLEXCEPT_DECLARE_BASE(TXOutOfRange, _NATLIB_DECLSPEC_);
typedef TXOutOfRange basic_TXOutOfRange;

//============================================================================
#define NLEXCEPT_DECLARE_OUT_OF_RANGE( \
	TLocalXOutOfRange, TExceptionBase, Declspec \
) \
	typedef class Declspec TLocalXOutOfRange : \
    	public TExceptionBase, public basic_TXOutOfRange \
    { \
	public: \
		/*typedef TExceptionBase TParent;*/ \
		TLocalXOutOfRange( \
			const TSourceLocation& c_Location, \
			const TStdString& c_sExplanation = TStdString() \
		): \
			basic_TNatlibException(c_Location, c_sExplanation), \
			TExceptionBase(c_Location, c_sExplanation), \
			basic_TXOutOfRange(c_Location, c_sExplanation) \
		{} /*TLocalXIndexOutOfrange(const TSourceLocation&)*/ \
		TLocalXOutOfRange(const TLocalXOutOfRange& c_XOutOfRange) : \
			basic_TNatlibException(c_XOutOfRange), \
            TExceptionBase(c_XOutOfRange), \
			basic_TXOutOfRange(c_XOutOfRange) \
		{} /*TLocalXOutOfRange(const TLocalXOutOfRange&)*/ \
	} typedef_##TLocalXOutOfRange

// TXBadCast
// Error of type casting (different from standard bad_cast because contains 
// pointer to type_info of source and target)
//============================================================================
typedef class _NATLIB_DECLSPEC_ TXBadCast : virtual TNatlibException {
public: 
	//------------------------------------------------------------------------
//	typedef TNatlibException TParent;

	//------------------------------------------------------------------------
    TXBadCast(
    	const TSourceLocation&,
        const type_info& c_Source, const type_info& c_Target,
		const TStdString& c_sExplanation = TStdString()
    ) _NATLIB_THROW_NONE_ /*TXBadCast(const TSourceLocation&, ...)*/;
    //------------------------------------------------------------------------
    TXBadCast(const TXBadCast&) _NATLIB_THROW_NONE_;

	//------------------------------------------------------------------------
    TStdString Explanation() const _NATLIB_THROW_NONE_;

    //------------------------------------------------------------------------
    virtual const type_info& SourceTypeId() const _NATLIB_THROW_NONE_;
    //------------------------------------------------------------------------
    virtual const type_info& TargetTypeId() const _NATLIB_THROW_NONE_;

protected:
	//------------------------------------------------------------------------
    const type_info* pc_Source;
    const type_info* pc_Target;
} basic_TXBadCast;

//============================================================================
#define NLEXCEPT_DECLARE_BAD_CAST(TLocalXBadCast, TExceptionBase, Declspec) \
	typedef class Declspec TLocalXBadCast : \
		public TExceptionBase, public basic_TXBadCast \
	{ \
	public: \
		/*typedef TExceptionBase TParent;*/ \
		TLocalXBadCast( \
			const TSourceLocation& c_Location, \
			const type_info& c_Source, \
            const type_info& c_Target, \
			const TStdString& c_sExplanation = TStdString() \
		) : \
			basic_TNatlibException(c_Location, c_sExplanation), \
			TExceptionBase(c_Location, c_sExplanation), \
			basic_TXBadCast(c_Location, c_Source, c_Target, c_sExplanation) \
		{} /*TLocalXBadCast(const TSourceLocation, const type_info&...)*/ \
		TLocalXBadCast(const TLocalXBadCast& c_XBadCast) : \
			basic_TNatlibException(c_XBadCast), \
			basic_TXBadCast(c_XBadCast), TExceptionBase(c_XBadCast) \
		{} /*TLocalXBadCast(const TLocalXBadCast&)*/ \
	} typedef_##TLocalXBadCast

// TOperationIsNotAllowed
// Can be thrown to indicate that call to specific virtual function doesnt 
// have any meaning for the object or invalid in the current context
//============================================================================
NLEXCEPT_DECLARE_BASE(TXOperationIsNotAllowed, _NATLIB_DECLSPEC_);
typedef TXOperationIsNotAllowed basic_TXOperationIsNotAllowed;

//============================================================================
#define NLEXCEPT_DECLARE_OPERATION_IS_NOT_ALLOWED( \
	TLocalXOperationIsNotAllowed, TExceptionBase, Declspec \
) \
	typedef class Declspec TLocalXOperationIsNotAllowed : \
    	public TExceptionBase, public basic_TXOperationIsNotAllowed \
    { \
	public: \
		/*typedef TExceptionBase TParent;*/ \
		TLocalXOperationIsNotAllowed( \
			const TSourceLocation& c_Location, \
			const TStdString& c_sExplanation = TStdString() \
		): \
			basic_TNatlibException(c_Location, c_sExplanation), \
			TExceptionBase(c_Location, c_sExplanation), \
			basic_TXOperationIsNotAllowed(c_Location, c_sExplanation) \
		{} /*TLocalXOperationIsNotAllowed(const TSourceLocation&, ...)*/ \
		TLocalXOperationIsNotAllowed( \
        	const TLocalXOperationIsNotAllowed& c_XOperationIsNotAllowed \
        ) : \
			basic_TNatlibException(c_XOperationIsNotAllowed), \
            TExceptionBase(c_XOperationIsNotAllowed), \
			basic_TXOperationIsNotAllowed(c_XOperationIsNotAllowed) \
		{} /*TLocalXOperationIsNotAllowed(...)*/ \
	} typedef_##TLocalXOperationIsNotAllowed

// Constructs object according to ConstructionExpression and
// puts pointer to it into pObject
// throws Natlib Exceptions (e.g. TXLowMemory) instead of standard
//============================================================================
#define NLEXCEPT_CLASS_NEW(ConstructionExpression, pObject) { \
   	try { \
		pObject = new ConstructionExpression; \
	} \
    catch (const NATLIB_XALLOC& c_XAlloc) { \
		pObject = null; \
        throw TXLowMemory(THIS_CLASS, NATLIB_XALLOC_REQUESTED(c_XAlloc)); \
    } /*catch (const NATLIB_XALLOC& c_XAlloc)*/ \
    if (!pObject) throw TXLowMemory(THIS_CLASS); \
} /*NLEXCEPT_CLASS_NEW(ConstructionExpression, pObject)*/

//============================================================================
#define NLEXCEPT_NEW(ConstructionExpression, pObject) { \
   	try { \
		pObject = new ConstructionExpression; \
	} \
    catch (const NATLIB_XALLOC& c_XAlloc) { \
		pObject = null; \
        throw TXLowMemory(THIS_PLACE, NATLIB_XALLOC_REQUESTED(c_XAlloc)); \
    } /*catch (const NATLIB_XALLOC& c_XAlloc)*/ \
    if (!pObject) throw TXLowMemory(THIS_PLACE); \
} /*NLEXCEPT_NEW(ConstructionExpression, pObject)*/

// Mapping to standard exceptions
// Note: Standard exceptions header must be included before using of it
//============================================================================
#define MAP_TO_STDEXCEPT \
	catch (const basic_TXBadCast& c_XBadCast) { \
    	throw invalid_argument(c_XBadCast.Why()); \
    } /*catch (const basic_TXBadCast&)*/ \
    catch (const basic_TXOutOfRange& c_XOutOfRange) { \
    	throw range_error(c_XOutOfRange.Why()); \
    } /*catch (const basic_TXOutOfRange&)*/ \
    catch (const basic_TXLowMemory& c_XLowMemory) { \
		throw overflow_error(c_XLowMemory.Why()); \
    } /*catch (const basic_TXLowMemory&)*/ \
    catch (const basic_TNatlibException& c_Exception) { \
		throw exception(c_Exception.Why()); \
    } /*catch (const basic_TNatlibException&)*/ 

// Mapping to standard xmsg exception
// Note: <except.h> header must be included before using of it
//============================================================================
#define MAP_TO_XMSG \
    catch (const basic_TNatlibException& c_Exception) { \
		throw xmsg(c_Exception.Why()); \
    } /*catch (const basic_TNatlibException&)*/ \

//============================================================================
NATLIB_NAMESPACE_END

#endif /*!defined(__NATLIB_NLEXCEPT_H__)*/
