#ifndef __NATLIB_NLLIST_H__
#define __NATLIB_NLLIST_H__
/****************************************************************************/
/** nllist.h                                                               **/
/** ---------------------------------------------------------------------- **/
/** Contents declarations of the double linked list and helper classes     **/
/** Note: Copy contructor for template argument must be available          **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** February 06, 1996 by Oleg Kagan & Ilya Marchevsky                      **/
/****************************************************************************/

//============================================================================
#include <nlcont.h>
#include <nlalloc.h>
#include <nlsafeh.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

//============================================================================
#define NLLIST_ALLOCATOR_DEFAULT TStdFixedAllocator

//============================================================================
#define NLLIST_CURSOR_LIST_ASSIGNMENT_GUARD(Cursor) \
{ \
	if (!(Cursor).hList.IsTargetValid()) \
		throw TXListIsNotAssigned(THIS_CLASS); \
} /*NLLIST_CURSOR_LIST_ASSIGNMENT_GUARD(Cursor)*/

//----------------------------------------------------------------------------
#define NLLIST_CURSOR_NODE_VALIDITY_GUARD(Cursor) \
{ \
	NLLIST_CURSOR_LIST_ASSIGNMENT_GUARD(Cursor) \
	if (!(Cursor).IsNodeValid()) throw TXInvalidNode(THIS_CLASS); \
} /*NLLIST_CURSOR_NODE_VALIDITY_GUARD(Cursor)*/

// TListReferenceCounter
//============================================================================
class _NATLIB_DECLSPEC_ TListReferenceCounter {
public:
    //------------------------------------------------------------------------
    typedef nl_ulong_t TCounter;

	//------------------------------------------------------------------------
	NLEXCEPT_DECLARE_BASE(TException, _NATLIB_DECLSPEC_);
	NLEXCEPT_DECLARE(
		TXInvalidReferenceCounter, TException, _NATLIB_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
    NLEXCEPT_DECLARE(
		TXDestructionOfReferencedObject, TException, _NATLIB_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;

	//------------------------------------------------------------------------
	TListReferenceCounter() : nReferences(0) {}
	//------------------------------------------------------------------------
	~TListReferenceCounter();

	//------------------------------------------------------------------------
	void RegisterReference();
	//------------------------------------------------------------------------
	void UnregisterReference();

    //------------------------------------------------------------------------
    TCounter References() const {return nReferences;}

protected:
	//------------------------------------------------------------------------
	TCounter nReferences;
} /*TListReferenceCounter*/;

// TListNodeOf template
//============================================================================
template <class T> class _NATLIB_TEMPLATE_DECLSPEC_ TListNodeOf :
    public TListReferenceCounter
{
public:
    //------------------------------------------------------------------------
    typedef TListReferenceCounter TParent;

	//------------------------------------------------------------------------
	typedef TListNodeOf<T> TNode;
	typedef TAllocationEnvelopeOf<T> TDataEnvelope;

	//------------------------------------------------------------------------
    NLEXCEPT_DECLARE_BASE(TException, _NATLIB_TEMPLATE_DECLSPEC_);
    NLEXCEPT_DECLARE(
		TXInvalidPointer, TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
    NLEXCEPT_DECLARE(
		TXDestructionOfChainedNode, TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
    NLEXCEPT_DECLARE(
		TXAccessToDelistedNode, TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;

	//------------------------------------------------------------------------
	TNode* pNext;
	TNode* pPrevious;

	//------------------------------------------------------------------------
	char a_cStorage[1];

	//------------------------------------------------------------------------
	TListNodeOf(const T& c_Data) : pNext(null), pPrevious(null)
	{
    	new (a_cStorage) TDataEnvelope(c_Data);
    } /*TListNodeOf(const T&)*/
	//------------------------------------------------------------------------
	~TListNodeOf()
	{
		if (IsChained())
			throw TXDestructionOfChainedNode(THIS_CLASS);
		pNext = pPrevious = null;
        typedef T _TData;
		reinterpret_cast<_TData*>(a_cStorage)->~_TData();
	} /*~TListNodeOf*/

    //------------------------------------------------------------------------
    void AssertValid() const;

	//------------------------------------------------------------------------
    T* DataPtr()
    {
    	return IsChained() ? reinterpret_cast<T*>(a_cStorage) : null;
    } /*DataPtr()*/
    //------------------------------------------------------------------------
    T& Data();
	//------------------------------------------------------------------------
    const T* DataPtrConst() const
    {
    	return (
			IsChained() ? reinterpret_cast<const T*>(a_cStorage) : null
		) /*return*/;
    } /*DataPtrConst() const*/
    //------------------------------------------------------------------------
    const T& DataConst() const;
    //------------------------------------------------------------------------
	T& RawData() {return *reinterpret_cast<T*>(a_cStorage);}
    //------------------------------------------------------------------------
    const T& RawDataConst() const
    {
    	return *reinterpret_cast<const T*>(a_cStorage);
    } /*RawDataConst() const*/

    //------------------------------------------------------------------------
    bool IsChained() const {return pNext != null;}

	//------------------------------------------------------------------------
    void* operator new(size_t, void* pPlacement) {return pPlacement;}
} /*TListNodeOf<T>*/;

//----------------------------------------------------------------------------
#define NLLIST_NONRTTI_NODE_DECL(T, Declspec) \
    _NATLIB_EXPLICIT_TEMPLATE_ class Declspec TListNodeOf<T> : \
        public TListReferenceCounter \
    { \
    public: \
        typedef TListReferenceCounter TParent; \
		typedef TListNodeOf<T> TNode; \
        typedef TAllocationEnvelopeOf<T> TDataEnvelope; \
		NLEXCEPT_DECLARE( \
            TException, TListReferenceCounter::TException, Declspec \
        ) /*NLEXCEPT_DECLARE*/; \
        NLEXCEPT_DECLARE(TXInvalidPointer, TException, Declspec); \
        NLEXCEPT_DECLARE(TXDestructionOfChainedNode, TException, Declspec); \
        NLEXCEPT_DECLARE(TXAccessToDelistedNode, TException, Declspec); \
		TNode* pNext; \
		TNode* pPrevious; \
		T Storage; \
        TListNodeOf(const T& c_Data) : \
        	pNext(null), pPrevious(null), Storage(c_Data) \
        {} /*TListNodeOf(const T& c_Data)*/ \
        ~TListNodeOf() \
        { \
            if (IsChained()) \
                throw TXDestructionOfChainedNode(THIS_CLASS); \
            pNext = pPrevious = null; \
		} /*~TListNodeOf*/ \
        void AssertValid() const \
        { \
        	if (!pNext != !pPrevious) \
				throw TXInvalidPointer(THIS_CLASS); \
        } /*AssertValid() const*/ \
        T* DataPtr() {return IsChained() ? &Storage : null;} \
        T& Data() {return const_cast<T&>(DataConst());} \
        const T* DataPtrConst() const \
        { \
        	return IsChained() ? &Storage : null; \
        } /*DataPtrConst() const*/ \
        const T& DataConst() const \
        { \
            const T* pc_Data = DataPtrConst(); \
            if (!pc_Data) \
                throw TXAccessToDelistedNode(THIS_CLASS); \
            return *pc_Data; \
        } \
        T& RawData() {return Storage;} \
        const T& RawDataConst() const {return Storage;} \
		bool IsChained() const {return pNext != null;} \
        void* operator new(size_t, void* pPlacement) {return pPlacement;} \
    } /*TListNodeOf<T>*/

// TListOf template
// This is a double list with each node having pointer to the next and
// previous elements. pTail is pHead->pPrev, list looks like a ring
//============================================================================
template <class T> class _NATLIB_TEMPLATE_DECLSPEC_ TListOf :
	public TContainerOf<T>, public virtual TSafeHandleAllowed
{
public:
    //------------------------------------------------------------------------
	typedef TContainerOf<T> TParent;
    typedef TComparatorOf<T> TComparator;
    typedef TConditionOf<T> TCondition;

	//------------------------------------------------------------------------
	typedef TListNodeOf<T> TNode;
    typedef TListOf<T> TList;

	//------------------------------------------------------------------------
    NLEXCEPT_DECLARE(
		TException, TContainerOf<T>::TException,
		_NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
	NLEXCEPT_DECLARE(
		TXUnableToConstructNode, TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
	NLEXCEPT_DECLARE(TXInvalidChain, TException, _NATLIB_TEMPLATE_DECLSPEC_);
	NLEXCEPT_DECLARE(TXInvalidSize, TException, _NATLIB_TEMPLATE_DECLSPEC_);
	NLEXCEPT_DECLARE(
		TXAllocatorIsNotDefined, TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
	
	//------------------------------------------------------------------------
    TListOf(const TAllocator& = NLLIST_ALLOCATOR_DEFAULT());
	//------------------------------------------------------------------------
    TListOf(const TList&);
    //------------------------------------------------------------------------
    TListOf(
		const TContainer&, const TAllocator& = NLLIST_ALLOCATOR_DEFAULT()
	) /*TListOf(const TContainer&, const TAllocator&)*/;
	//------------------------------------------------------------------------
    ~TListOf();

    //------------------------------------------------------------------------
    void AssertValid() const;
    //------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE1(TListOf, T);

	// TContainer specific overloads
	//------------------------------------------------------------------------
    TSize Size() const {return nSize;}

	//------------------------------------------------------------------------
	const T& Insert(const T& c_Data)
	{
		return InsertData(c_Data, pTail);
    } /*Insert(const T&)*/
    //------------------------------------------------------------------------
    void Insert(const TContainer& c_Source) {TParent::Insert(c_Source);}

	//------------------------------------------------------------------------
    TSize Delete(const TConditionOf<T>&, TIndex = TContainer::indexAll);
    //------------------------------------------------------------------------
    TSize Delete(
        const TComparator& c_Comparator, TIndex nIndex = TContainer::indexAll
    ){
        return TParent::Delete(c_Comparator, nIndex);
    } /*Delete(const TComparator&, TIndex = TContainer::indexAll*/

    //------------------------------------------------------------------------
    void Clear();

    //------------------------------------------------------------------------
	void ForEach(TOperationOn<T>&);
    //------------------------------------------------------------------------
	void ForEach(TConstantOperationOn<T>&) const;

    //------------------------------------------------------------------------
	void Sort(TEtalonComparatorOf<T>&);

	//------------------------------------------------------------------------
	const TAllocator& AllocatorConst() const {return *pAllocator;}

	//------------------------------------------------------------------------
	inline TList& operator=(const TList& c_Source)
    {
    	Let(c_Source); return *this;
    } /*operator=(const TList&)*/
	//------------------------------------------------------------------------
	inline TList& operator=(const TContainer& c_Source)
    {
    	Let(c_Source); return *this;
    } /*operator=(const TContainer& c_Source)*/

	//------------------------------------------------------------------------
	inline TList& operator+=(const T& c_Data) {Insert(c_Data); return *this;}
	//------------------------------------------------------------------------
	inline TList& operator+=(const TContainer& c_Container)
    {
    	Insert(c_Container); return *this;
    } /*operator+=(const TContainer&)*/
	//------------------------------------------------------------------------
	inline TList& operator-=(const TCondition& c_Condition)
    {
    	Delete(c_Condition, indexAll); return *this;
    } /*operator-=(const TCondition&)*/
    //------------------------------------------------------------------------
	inline TList& operator-=(const TComparator& c_Comparator)
    {
    	Delete(c_Comparator, indexAll); return *this;
    } /*operator-=(const TComparator&)*/

protected:
	//------------------------------------------------------------------------
	static size_t AllocationSize()
	{
		return sizeof(TNode) + sizeof(TAllocationEnvelopeOf<T>) - 1;
	} /*AllocationSize()*/

	//------------------------------------------------------------------------
	TNode* pTail;  // pointer to the tail of the list
	TSize nSize;  // number of valid nodes
	TAllocator* pAllocator;

	//------------------------------------------------------------------------
	TNode* HeadPtr() const {return pTail ? pTail->pNext : null;}
	//------------------------------------------------------------------------
	TNode* TailPtr() const {return pTail;}

	//------------------------------------------------------------------------
	TNode* NextNodePtr(TNode* pNode) const
    {
    	return (!pNode || pNode == pTail) ? null : pNode->pNext;
    } /*NextNodePtr(TNode*)*/
   	//------------------------------------------------------------------------
	TNode* NextNodePtr(TNode*, TIndex uOffset) const;

	//------------------------------------------------------------------------
	TNode* PreviousNodePtr(TNode* pNode) const
    {
    	return (pNode && pNode != HeadPtr()) ? pNode->pPrevious : null;
    } /*PreviousNodePtr(TNode*)*/
	//------------------------------------------------------------------------
	TNode* PreviousNodePtr(TNode*, TIndex uOffset) const;

	//------------------------------------------------------------------------
	TNode* SeekNodePtr(TNode* pNode, nl_long_t nOffset) const
    {
    	return (
        	nOffset >= 0 ?
            	NextNodePtr(pNode, nOffset) : PreviousNodePtr(pNode, -nOffset)
        ) /*return*/;
    } /*SeekNodePtr(TNode*, nl_long_t) const*/
	//------------------------------------------------------------------------
	TNode* NodeByIndex(TIndex uIndex) const
    {
    	return NextNodePtr(HeadPtr(), uIndex);
    } /*NodeByIndex(TIndex) const*/

    //------------------------------------------------------------------------
    TNode* NextByCondition(TNode*, const TCondition&, TIndex = 0) const;
    //------------------------------------------------------------------------
    TNode* PreviousByCondition(TNode*, const TCondition&, TIndex = 0) const;

	//------------------------------------------------------------------------
	virtual TNode* AllocateNode(const T&) const;
   	//------------------------------------------------------------------------
	virtual void DeallocateNode(TNode*);

   	//------------------------------------------------------------------------
	virtual void DelistNode(TNode&);
	//------------------------------------------------------------------------
	virtual void InsertNodeAfter(TNode&, TNode* pPosition = null);
	//------------------------------------------------------------------------
	virtual void InsertNodeBefore(TNode& r_Node, TNode* pPosition)
    {
    	InsertNodeAfter( r_Node, PreviousNodePtr(pPosition));
    } /*InsertNodeBefore(TNode&, TNode*)*/
	//------------------------------------------------------------------------
    void AppendNode(TNode* pNode);

	//------------------------------------------------------------------------
	virtual const T& InsertData(const T&, TNode* pPosition);

public:
	// TFriend
	//------------------------------------------------------------------------
	typedef class _NATLIB_TEMPLATE_DECLSPEC_ TFriend {
	public:
		//--------------------------------------------------------------------
		static TNode* HeadPtr(const TList& c_List) {return c_List.HeadPtr();}
		//--------------------------------------------------------------------
		static TNode* TailPtr(const TList& c_List) {return c_List.pTail;}
		//--------------------------------------------------------------------
		static TNode* NextNodePtr(const TList& c_List, TNode* pNode)
		{
			return c_List.NextNodePtr(pNode);
		} /*NextNodePtr(const TList&, TNode* pNode)*/
   		//--------------------------------------------------------------------
		static TNode* NextNodePtr(
			const TList& c_List, TNode* pNode, TIndex uOffset)
		{
			return c_List.NextNodePtr(pNode, uOffset);
		} /*NextNodePtr(const TList&, TNode*, TIndex)*/
		//--------------------------------------------------------------------
		static TNode* PreviousNodePtr(const TList& c_List, TNode* pNode)
		{
    		return c_List.PreviousNodePtr(pNode);
		} /*PreviousNodePtr(const TList&, TNode*)*/
		//--------------------------------------------------------------------
		static TNode* PreviousNodePtr(
			const TList& c_List, TNode* pNode, TIndex uOffset
		){
			return c_List.PreviousNodePtr(pNode, uOffset);
		} /*PreviousNodePtr(const TList&, TNode*, TIndex)*/
		//--------------------------------------------------------------------
		static TNode* SeekNodePtr(
			const TList& c_List, TNode* pNode, nl_long_t nOffset
		){
			return c_List.SeekNodePtr(pNode, nOffset);
		} /*SeekNodePtr(const TList&, TNode*, nl_long_t)*/
		//--------------------------------------------------------------------
		static TNode* NodeByIndex(const TList& c_List, TIndex uIndex)
		{
			return c_List.NodeByIndex(uIndex);
		} /*NodeByIndex(const TList&, TIndex)*/
	    //--------------------------------------------------------------------
		static TNode* NextByCondition(
			const TList& c_List, TNode* pNode, 
			const TConditionOf<T>& c_Condition, TIndex uIndex = 0
		){
			return c_List.NextByCondition(pNode, c_Condition, uIndex);
		} /*NextByCondition(const TList&, TNode*, const TCondition&, ...)*/
		//--------------------------------------------------------------------
		static TNode* PreviousByCondition(
			const TList& c_List, TNode* pNode, 
			const TConditionOf<T>& c_Condition, TIndex uIndex = 0
		){
			return c_List.PreviousByCondition(pNode, c_Condition, uIndex);
		} /*PreviousByCondition(const TList&, TNode*, const TCondition&,...)*/

		//--------------------------------------------------------------------
		static void DeallocateNode(TList& r_List, TNode* pNode)
		{
			r_List.DeallocateNode(pNode);
		} /*DeallocateNode(TList&, TNode*)*/
		//--------------------------------------------------------------------
		static void DelistNode(TList& r_List, TNode& r_Node)
		{
			r_List.DelistNode(r_Node);
		} /*DelistNode(TList&, TNode&)*/
		//--------------------------------------------------------------------
		static const T& InsertData(
			TList& r_List, const T& c_Data, TNode* pPosition
		){
			return r_List.InsertData(c_Data, pPosition);
		} /*InsertData(TList&, const T&, TNode*)*/
	} typedef_TFriend;
	friend TFriend;
} /*TListOf<T>*/;

// TListConstantCursorOf template
//============================================================================
template <class T> 
class _NATLIB_TEMPLATE_DECLSPEC_ TListConstantCursorOf :
	public virtual TNatlibObject, protected TListOf<T>::TFriend
{
public:
	//------------------------------------------------------------------------
	typedef TNatlibObject TParent;

	//------------------------------------------------------------------------
	typedef TListConstantCursorOf<T> TListConstantCursor;
	typedef TListOf<T> TList;
	typedef TList::TIndex TIndex;
    typedef TListNodeOf<T> TNode;
	typedef TConditionOf<T> TCondition;
	typedef TComparatorOf<T> TComparator;

	//------------------------------------------------------------------------
    NLEXCEPT_DECLARE_BASE(TException, _NATLIB_TEMPLATE_DECLSPEC_);
    NLEXCEPT_DECLARE(
		TXInvalidListAssignment, TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
    NLEXCEPT_DECLARE(
		TXInvalidNodeAssignment, TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
    NLEXCEPT_DECLARE(
		TXListIsNotAssigned, TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
    NLEXCEPT_DECLARE(
		TXNodeIsNotAssigned, TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
    NLEXCEPT_DECLARE(TXInvalidList, TException, _NATLIB_TEMPLATE_DECLSPEC_);
    NLEXCEPT_DECLARE(TXInvalidNode, TException, _NATLIB_TEMPLATE_DECLSPEC_);
    NLEXCEPT_DECLARE(TXEndOfList, TException, _NATLIB_TEMPLATE_DECLSPEC_);

	//------------------------------------------------------------------------
	static TListConstantCursorOf<T> Begin(const TList&);
	//------------------------------------------------------------------------
	static TListConstantCursorOf<T> End(const TList&);

	//------------------------------------------------------------------------
	TListConstantCursorOf();
	//------------------------------------------------------------------------
	TListConstantCursorOf(const TList&, TIndex uPosition = 0);
    //------------------------------------------------------------------------
	TListConstantCursorOf(const TListConstantCursor&);
   	//------------------------------------------------------------------------
	TListConstantCursorOf(const TListConstantCursor&, nl_long_t nOffset);
   	//------------------------------------------------------------------------
    ~TListConstantCursorOf();

    //------------------------------------------------------------------------
	void AssertValid() const;

	//------------------------------------------------------------------------
    NLOBJECT_DECLARE_CLONE1(TListConstantCursorOf, T);

    //------------------------------------------------------------------------
	void Assign(const TList&, TIndex uPosition = 0);
    //------------------------------------------------------------------------
	void Assign(const TListConstantCursor&, nl_long_t nOffset = 0);

    //------------------------------------------------------------------------
	bool IsNodeValid() const 
	{
		return hList.IsTargetValid() && pNode && pNode->IsChained();
	} /*IsNodeValid() const*/

    //------------------------------------------------------------------------
	bool IsFirst() const;
    //------------------------------------------------------------------------
	bool IsLast() const;
    //------------------------------------------------------------------------
	bool IsEndOfList() const;

    //------------------------------------------------------------------------
	TListConstantCursorOf<T>& ToFirst(TIndex uPosition = 0);
    //------------------------------------------------------------------------
	TListConstantCursorOf<T>& ToLast(TIndex uOffset = 0);
	//------------------------------------------------------------------------
	TListConstantCursorOf<T>& ToEnd(TIndex uOffset = 0);
	//------------------------------------------------------------------------
	TListConstantCursorOf<T>& ToNext();
	//------------------------------------------------------------------------
	TListConstantCursorOf<T>& ToNext(TIndex uOffset);
	//------------------------------------------------------------------------
	TListConstantCursorOf<T>& ToPrevious();
	//------------------------------------------------------------------------
	TListConstantCursorOf<T>& ToPrevious(TIndex uOffset);
    //------------------------------------------------------------------------
    TListConstantCursorOf<T>& Seek(nl_long_t nOffset);
	//------------------------------------------------------------------------
	TListConstantCursorOf<T>& ToNextByCondition(
		const TCondition&, TIndex uIndex = 0
	) /*ToNextByCondition(const TCondition&, TIndex = 0)*/;
	//------------------------------------------------------------------------
	TListConstantCursorOf<T>& ToPreviousByCondition(
		const TCondition&, TIndex uIndex = 0
	) /*ToPreviousByCondition(const TCondition&, TIndex = 0);*/;
	//------------------------------------------------------------------------
	TListConstantCursorOf<T>& ToNextByComparator(
		const TComparator&, TIndex = 0
    ) /*ToNextByComparator(const TComparator&, TIndex = 0)*/;
	//------------------------------------------------------------------------
	TListConstantCursorOf<T>& ToPreviousByComparator(
		const TComparator&, TIndex = 0
    ) /*ToPreviousByComparator(const TComparator&, TIndex = 0)*/;

    //------------------------------------------------------------------------
	virtual void _ToFirst(TIndex uPosition = 0);
    //------------------------------------------------------------------------
	virtual void _ToLast(TIndex uOffset = 0);
	//------------------------------------------------------------------------
	virtual void _ToEnd(TIndex uOffset = 0);
	//------------------------------------------------------------------------
	virtual void _ToNext();
	//------------------------------------------------------------------------
	virtual void _ToNext(TIndex uOffset);
	//------------------------------------------------------------------------
	virtual void _ToPrevious();
	//------------------------------------------------------------------------
	virtual void _ToPrevious(TIndex uOffset);
    //------------------------------------------------------------------------
    virtual void _Seek(nl_long_t nOffset);
	//------------------------------------------------------------------------
	virtual void _ToNextByCondition(const TCondition&, TIndex uIndex = 0);
	//------------------------------------------------------------------------
	virtual void _ToPreviousByCondition(const TCondition&, TIndex uIndex = 0);
	//------------------------------------------------------------------------
	virtual void _ToNextByComparator(const TComparator&, TIndex = 0);
	//------------------------------------------------------------------------
	virtual void _ToPreviousByComparator(const TComparator&, TIndex = 0);

    //------------------------------------------------------------------------
    void Unassign();

    //------------------------------------------------------------------------
    bool IsEqual(const TListConstantCursor& c_Cursor) const
    {
        return pNode == c_Cursor.pNode;
    } /*IsEqual(const TListConstantCursor&) const*/

    //------------------------------------------------------------------------
	const T* DereferencePtrConst() const;
    //------------------------------------------------------------------------
	const T& DereferenceConst() const;

    //------------------------------------------------------------------------
	TListConstantCursor& operator=(const TListConstantCursor&);

    //------------------------------------------------------------------------
	bool operator==(const TListConstantCursor& c_Cursor) const
    {
        return IsEqual(c_Cursor);
    } /*operator==(const TListConstantCursor&)*/

    //------------------------------------------------------------------------
	TListConstantCursor operator++(int);
    //------------------------------------------------------------------------
	inline TListConstantCursor& operator++() {ToNext(); return *this;}
    //------------------------------------------------------------------------
	inline TListConstantCursor& operator+=(nl_long_t nOffset)
    {
    	Seek(nOffset); return *this;
    } /*operator+=(nl_long_t)*/
    //------------------------------------------------------------------------
	TListConstantCursor operator+(nl_long_t nOffset) const;

    //------------------------------------------------------------------------
	TListConstantCursor operator--(int);
    //------------------------------------------------------------------------
  	inline TListConstantCursor& operator--() {ToPrevious(); return *this;}
    //------------------------------------------------------------------------
	inline TListConstantCursor& operator-=(nl_long_t nOffset)
    {
    	Seek(-nOffset); return *this;
    } /*operator-=(nl_long_t)*/
    //------------------------------------------------------------------------
	TListConstantCursor operator-(nl_long_t nOffset) const;

    //------------------------------------------------------------------------
	inline operator bool() const {return IsNodeValid();}

    //------------------------------------------------------------------------
	inline const T& operator*() const {return DereferenceConst();}
	//------------------------------------------------------------------------
//	inline const T* operator->() const {return &DereferenceConst();}

protected:
    //------------------------------------------------------------------------
	TNode* pNode;
    TSafeHandleOf<TList> hList;

	//------------------------------------------------------------------------
    void AssignNode(TNode*);
	//------------------------------------------------------------------------
	void Assign(TList&, TIndex uPosition = 0);
	//------------------------------------------------------------------------
	void Assign(TList&, TNode*, nl_long_t nOffset = 0);
	//------------------------------------------------------------------------
	const T& InsertBefore(const T&) const;
	//------------------------------------------------------------------------
	const T& InsertAfter(const T&) const;
	//------------------------------------------------------------------------
	void Delete();
} /*TListConstantCursorOf<T>*/;

// TListCursorOf template
//============================================================================
template <class T> class _NATLIB_TEMPLATE_DECLSPEC_ TListCursorOf :
    public TListConstantCursorOf<T>
{
public:
	//------------------------------------------------------------------------
	typedef TListConstantCursorOf<T> TParent;
	typedef TListCursorOf<T> TCursor;

	//------------------------------------------------------------------------
	static TListCursorOf<T> Begin(TList&);
	//------------------------------------------------------------------------
	static TListCursorOf<T> End(TList&);
	
	//------------------------------------------------------------------------
	TListCursorOf();
	//------------------------------------------------------------------------
	TListCursorOf(TList&, TIndex uPosition = 0);
   	//------------------------------------------------------------------------
	TListCursorOf(const TCursor&);
   	//------------------------------------------------------------------------
	TListCursorOf(const TCursor&, nl_long_t nOffset);

    //------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE1(TListCursorOf, T);

    //------------------------------------------------------------------------
	void Assign(TList& r_List, TIndex uPosition = 0) 
	{
		TParent::Assign(r_List, uPosition);
	} /*Assign(TList&, TIndex = 0) */
    //------------------------------------------------------------------------
	void Assign(const TCursor& c_Source, nl_long_t nOffset = 0)
	{
		TParent::Assign(*c_Source.hList, c_Source.pNode, nOffset);
	} /*Assign(const TCursor&, nl_long_t = 0)*/

	//------------------------------------------------------------------------
	TListCursorOf<T>& ToFirst(TIndex uPosition = 0);
    //------------------------------------------------------------------------
	TListCursorOf<T>& ToLast(TIndex uOffset = 0);
	//------------------------------------------------------------------------
	TListCursorOf<T>& ToEnd(TIndex uOffset = 0);
	//------------------------------------------------------------------------
	TListCursorOf<T>& ToNext();
	//------------------------------------------------------------------------
	TListCursorOf<T>& ToNext(TIndex uOffset);
	//------------------------------------------------------------------------
	TListCursorOf<T>& ToPrevious();
	//------------------------------------------------------------------------
	TListCursorOf<T>& ToPrevious(TIndex uOffset );
    //------------------------------------------------------------------------
    TListCursorOf<T>& Seek(nl_long_t nOffset);
	//------------------------------------------------------------------------
	TListCursorOf<T>& ToNextByCondition(const TCondition&, TIndex = 0);
	//------------------------------------------------------------------------
	TListCursorOf<T>& ToPreviousByCondition(const TCondition&, TIndex = 0);
	//------------------------------------------------------------------------
	TListCursorOf<T>& ToNextByComparator(const TComparator&, TIndex = 0);
	//------------------------------------------------------------------------
	TListCursorOf<T>& ToPreviousByComparator(const TComparator&, TIndex = 0);

    //------------------------------------------------------------------------
    T* DereferencePtr() const
	{
		return const_cast<T*>(DereferencePtrConst());
	} /*DereferencePtr() const*/
    //------------------------------------------------------------------------
    T& Dereference() const {return const_cast<T&>(DereferenceConst());}

	//------------------------------------------------------------------------
    const T& InsertBefore(const T& c_Data) const
	{
		return TParent::InsertBefore(c_Data);
	} /*InsertBefore(const T&) const*/
	//------------------------------------------------------------------------
    const T& InsertAfter(const T& c_Data) const
	{
		return TParent::InsertAfter(c_Data);
	} /*InsertAfter(const T&) const*/

	//------------------------------------------------------------------------
    void Delete() {TParent::Delete();}

    //------------------------------------------------------------------------
	TCursor& operator=(const TCursor&);

    //------------------------------------------------------------------------
	bool operator==(const TCursor& c_Cursor) const
    {
        return IsEqual(c_Cursor);
    } /*operator==(const TCursor&)*/

    //------------------------------------------------------------------------
	TCursor operator++(int);
    //------------------------------------------------------------------------
	inline TCursor& operator++() {ToNext(); return *this;}
    //------------------------------------------------------------------------
	inline TCursor& operator+=(nl_long_t nOffset) 
	{
		Seek(nOffset); return *this;
	} /*operator+=(nl_long_t)*/
    //------------------------------------------------------------------------
	TCursor operator+(nl_long_t nOffset) const;

    //------------------------------------------------------------------------
	TCursor operator--(int);
    //------------------------------------------------------------------------
  	inline TCursor& operator--() {ToPrevious(); return *this;}
    //------------------------------------------------------------------------
	inline TCursor& operator-=(nl_long_t nOffset) 
	{
		Seek(-nOffset); return *this;
	} /*operator-=(nl_long_t)*/
    //------------------------------------------------------------------------
	TCursor operator-(nl_long_t nOffset) const;
    //------------------------------------------------------------------------
	inline T& operator*() const {return Dereference();}
    //------------------------------------------------------------------------
//	inline T* operator->() const {return &Dereference();}
} /*TListCursorOf<T>*/;

// TSortListOf template
//============================================================================
template <class T> class TSortListOf {
public:
	//------------------------------------------------------------------------
    typedef TListNodeOf<T> TNode;
    typedef TEtalonComparatorOf<T> TEtalonComparator;

	//------------------------------------------------------------------------
    TSortListOf(TEtalonComparator& r_Comparator):
    	pComparator(&r_Comparator)
    {} /*TSortListOf(TEtalonComparator&)*/

    //------------------------------------------------------------------------
    void operator()(TNode*& r_pTail);

protected:
	// TSequence
	//------------------------------------------------------------------------
    typedef class _NATLIB_TEMPLATE_DECLSPEC_ TSequence {
    public:
    	//--------------------------------------------------------------------
        TNode* pHead;
    	TNode* pTail;

		//--------------------------------------------------------------------
		void Init() {pTail = pHead = null;}
		//--------------------------------------------------------------------
        void MoveFrom(TSequence& r_Source)
        {
        	if (pHead) pTail = (pTail->pNext = r_Source.pHead);
            else pHead = pTail = r_Source.pHead;

            r_Source.pHead = pTail->pNext; pTail->pNext = null;
            if (!r_Source.pHead) r_Source.pTail = null;
        } /*MoveFrom(TSequence&)*/
    } typedef_TSequence;

	//------------------------------------------------------------------------
	void Swap(TSequence& r_Sequence1, TSequence& r_Sequence2)
	{
		TSequence TempSequence = r_Sequence1;
		r_Sequence1 = r_Sequence2;
		r_Sequence2 = TempSequence;
	} /*Swap(TSequence&, TSequence&)*/

	//------------------------------------------------------------------------
    TEtalonComparator* pComparator;
    TComparator::TResult SortOrder;
    TSequence InpLeft, InpRight, OutLeft, OutRight;

	// Checks for the end of the segment according to current settings of
    // the comparator
    //------------------------------------------------------------------------
    bool IsEndOfSegment(const TSequence& r_Sequence)
    {
    	return (
        	!r_Sequence.pHead ||
            (
            	(*pComparator)(r_Sequence.pHead->RawDataConst()) ==
            	TComparator::compGreater
            )
        ) /*return*/;
    } /*IsEndOfSegment(TSequence)*/
	//------------------------------------------------------------------------
    void MoveSegment(TSequence& r_Inp, TSequence& r_Out);
    //------------------------------------------------------------------------
    void SplitBySegments(TSequence& r_Inp);
    //------------------------------------------------------------------------
    void MergeSegment(TSequence& r_Out);
	//------------------------------------------------------------------------
	void AppendToLeftOutput(TSequence&);
	//------------------------------------------------------------------------
	void RestoreDoubleConnectivity(TNode*& r_pTail);
} /*TSortListOf<T>*/;

// TListNodeOf template implementation
//============================================================================
template <class T> void TListNodeOf<T>::AssertValid() const
{
	if (!pNext != !pPrevious) 
		throw TXInvalidPointer(THIS_CLASS);
} /*TListNodeOf<T>::AssertValid() const*/

//----------------------------------------------------------------------------
template <class T> T& TListNodeOf<T>::Data() 
{
	T* pData = DataPtr();
	if (!pData)
		throw TXAccessToDelistedNode(THIS_CLASS);
	return *pData;
} /*T& TListNodeOf<T>::Data()*/

//----------------------------------------------------------------------------
template <class T> const T& TListNodeOf<T>::DataConst() const 
{
	const T* pc_Data = DataPtrConst();
	if (!pc_Data) 
		throw TXAccessToDelistedNode(THIS_CLASS);
	return *pc_Data;
} /*TListNodeOf<T>::DataConst() const*/

//----------------------------------------------------------------------------
#ifndef _NATLIB_NO_EXPLICIT_TEMPLATE_DECLSPEC_
	#define _NLLIST_NONRTTI_NODE_DECLSPEC_ _NATLIB_DECLSPEC_
#else
	#define _NLLIST_NONRTTI_NODE_DECLSPEC_ _NATLIB_TEMPLATE_DECLSPEC_
#endif /*_NATLIB_EXPLICIT_TEMPLATE_DECLSPEC_*/

//----------------------------------------------------------------------------
NLLIST_NONRTTI_NODE_DECL(char, _NLLIST_NONRTTI_NODE_DECLSPEC_);
NLLIST_NONRTTI_NODE_DECL(unsigned char, _NLLIST_NONRTTI_NODE_DECLSPEC_);
NLLIST_NONRTTI_NODE_DECL(int, _NLLIST_NONRTTI_NODE_DECLSPEC_);
NLLIST_NONRTTI_NODE_DECL(unsigned, _NLLIST_NONRTTI_NODE_DECLSPEC_);
NLLIST_NONRTTI_NODE_DECL(long, _NLLIST_NONRTTI_NODE_DECLSPEC_);
NLLIST_NONRTTI_NODE_DECL(unsigned long, _NLLIST_NONRTTI_NODE_DECLSPEC_);
NLLIST_NONRTTI_NODE_DECL(float, _NLLIST_NONRTTI_NODE_DECLSPEC_);
NLLIST_NONRTTI_NODE_DECL(double, _NLLIST_NONRTTI_NODE_DECLSPEC_);
NLLIST_NONRTTI_NODE_DECL(long double, _NLLIST_NONRTTI_NODE_DECLSPEC_);
NLLIST_NONRTTI_NODE_DECL(nl_pvoid_t, _NLLIST_NONRTTI_NODE_DECLSPEC_);
#ifndef _NATLIB_NO_BOOL_
NLLIST_NONRTTI_NODE_DECL(bool, _NLLIST_NONRTTI_NODE_DECLSPEC_);
#endif /*_NATLIB_NO_BOOL_*/

// TListOf template implementation
//============================================================================
template <class T> TListOf<T>::TListOf(

	const TAllocator& c_Allocator /*= NLLIST_ALLOCATOR_DEFAULT()*/

): 
	pTail(null), nSize(0), pAllocator(null)
{
	pAllocator = CloneObject(c_Allocator);
	pAllocator->SetAllocationSize(AllocationSize());
} /*TListOf<T>(const TAllocator&)*/

//----------------------------------------------------------------------------
template <class T> TListOf<T>::TListOf(

	const TList& c_Source
	
): 
	pTail(null), nSize(0), pAllocator(null)
{
	pAllocator = CloneObject(c_Source.AllocatorConst());
	pAllocator->SetAllocationSize(AllocationSize());

	Insert(c_Source);
} /*TListOf<T>(const TList&)*/

//----------------------------------------------------------------------------
template <class T> TListOf<T>::TListOf(

	const TContainer& c_Source,
	const TAllocator& c_Allocator /*= NLLIST_ALLOCATOR_DEFAULT()*/
	
): 
	pTail(null), nSize(0), pAllocator(null)
{
	pAllocator = CloneObject(c_Allocator);
	pAllocator->SetAllocationSize(AllocationSize());

	Insert(c_Source);
} /*TListOf(const TContainer&, const TAllocator&)*/

//----------------------------------------------------------------------------
template <class T> TListOf<T>::~TListOf() 
{
	Clear();
	delete pAllocator; pAllocator = null;
} /*~TListOf<T>()*/

// TListConstantCursorOf template implementation
//============================================================================
template <class T> TListConstantCursorOf<T> TListConstantCursorOf<T>::Begin(

	const TList& c_List
	
){
	TListConstantCursorOf<T> BeginCur(c_List);
	return BeginCur;
} /*TListConstantCursorOf<T>::Begin(const TList&)*/

//----------------------------------------------------------------------------
template <class T> TListConstantCursorOf<T> TListConstantCursorOf<T>::End(

	const TList& c_List
	
){
	TListConstantCursorOf<T> EndCur(c_List);
	return EndCur.ToEnd();
} /*TListConstantCursorOf<T>::End(const TList&)*/

//----------------------------------------------------------------------------
template <class T> TListConstantCursorOf<T>::TListConstantCursorOf():
	pNode(null)
{} /*TListConstantCursorOf<T>()*/

//----------------------------------------------------------------------------
template <class T> TListConstantCursorOf<T>::TListConstantCursorOf(

	const TListOf<T>& c_List, 
	nl_ulong_t uPosition
	
):
	pNode(null)
{
	Assign(c_List, uPosition);
} /*TListConstantCursorOf<T>(const TList&, TIndex = 0)*/

//----------------------------------------------------------------------------
template <class T> TListConstantCursorOf<T>::TListConstantCursorOf(

	const TListConstantCursorOf<T>& c_Source
	
): 
	pNode(null)
{
	Assign(c_Source);
} /*TListConstantCursorOf<T>(const TListConstantCursor&)*/

//----------------------------------------------------------------------------
template <class T> TListConstantCursorOf<T>::TListConstantCursorOf(

	const TListConstantCursorOf<T>& c_Source, 
	nl_long_t nOffset

):
	pNode(null)
{
	Assign(c_Source, nOffset);
} /*TListConstantCursorOf<T>(const TListConstantCursor&, TIndex)*/

//----------------------------------------------------------------------------
template <class T> TListConstantCursorOf<T>::~TListConstantCursorOf() 
{
	Unassign();
} /*~TListConstantCursorOf<T>()*/

//------------------------------------------------------------------------
template <class T> 
void TListConstantCursorOf<T>::AssertValid() const
{
    TParent::AssertValid();
    if (!hList.IsValid()) 
		throw TXInvalidListAssignment(THIS_CLASS);

	if (pNode && !(hList.IsTargetValid() && pNode->IsChained())) 
		throw TXInvalidNodeAssignment(THIS_CLASS);
} /*TListConstantCursorOf<T>::AssertValid() const*/

//----------------------------------------------------------------------------
template <class T>
NLOBJECT_IMPLEMENT_CLONE_DEFAULT1(TListConstantCursorOf, T)

//----------------------------------------------------------------------------
template <class T> void TListConstantCursorOf<T>::Assign(

	const TListOf<T>& c_List, 
	nl_ulong_t uPosition /*= 0*/
	
){
	hList.Assign(const_cast<TList&>(c_List));
    AssignNode(NodeByIndex(c_List, uPosition));
} /*TListConstantCursorOf<T>::Assign(const TList&, TIndex)*/

//----------------------------------------------------------------------------
template <class T> void TListConstantCursorOf<T>::Assign(

	const TListConstantCursorOf<T>& c_Cursor, 
	nl_long_t nOffset /*= 0*/
	
){
    hList.Assign(*(c_Cursor.hList));
    AssignNode(SeekNodePtr(*hList, c_Cursor.pNode, nOffset));
} /*TListConstantCursorOf<T>::Assign(const TListConstantCursor&, TIndex)*/

//----------------------------------------------------------------------------
template <class T> bool TListConstantCursorOf<T>::IsFirst() const
{
    NLLIST_CURSOR_LIST_ASSIGNMENT_GUARD(*this);
    return pNode == HeadPtr(*hList);
} /*TListConstantCursorOf<T>::IsFirst() const*/

//----------------------------------------------------------------------------
template <class T> bool TListConstantCursorOf<T>::IsLast() const
{
    NLLIST_CURSOR_LIST_ASSIGNMENT_GUARD(*this);
    return pNode == TailPtr(*hList);
} /*TListConstantCursorOf<T>::IsLast() const*/

//----------------------------------------------------------------------------
template <class T> bool TListConstantCursorOf<T>::IsEndOfList() const
{
    NLLIST_CURSOR_LIST_ASSIGNMENT_GUARD(*this);
    return !pNode;
} /*TListConstantCursorOf<T>::IsEndOfList() const*/

//----------------------------------------------------------------------------
template <class T> inline
TListConstantCursorOf<T>& TListConstantCursorOf<T>::ToFirst(

	nl_ulong_t uPosition /*= 0*/
	
){
	_ToFirst(uPosition);
	return *this;
} /*TListConstantCursorOf<T>::ToFirst(TIndex = 0)*/

//----------------------------------------------------------------------------
template <class T> inline
TListConstantCursorOf<T>& TListConstantCursorOf<T>::ToLast(

	nl_ulong_t uOffset /*= 0*/
	
){
	_ToLast(uOffset);
	return *this;
} /*TListConstantCursorOf<T>::ToLast(TIndex = 0)*/

//----------------------------------------------------------------------------
template <class T> inline 
TListConstantCursorOf<T>& TListConstantCursorOf<T>::ToEnd(

	nl_ulong_t uOffset /*= 0*/
	
){
	_ToEnd(uOffset);
	return *this;
} /*TListConstantCursorOf<T>::ToEnd(TIndex = 0)*/

//----------------------------------------------------------------------------
template <class T> inline
TListConstantCursorOf<T>& TListConstantCursorOf<T>::ToNext()
{
	_ToNext();
	return *this;
} /*TListConstantCursorOf<T>::ToNext()*/

//----------------------------------------------------------------------------
template <class T> inline 
TListConstantCursorOf<T>& TListConstantCursorOf<T>::ToNext(

	nl_ulong_t uOffset
	
){
	_ToNext(uOffset);
	return *this;
} /*TListConstantCursorOf<T>::ToNext(TIndex)*/

//----------------------------------------------------------------------------
template <class T> inline 
TListConstantCursorOf<T>& TListConstantCursorOf<T>::ToPrevious()
{
	_ToPrevious();
	return *this;
} /*TListConstantCursorOf<T>::ToPrevious()*/

//----------------------------------------------------------------------------
template <class T> inline 
TListConstantCursorOf<T>& TListConstantCursorOf<T>::ToPrevious(

	nl_ulong_t uOffset
	
){
	_ToPrevious(uOffset);
	return *this;
} /*TListConstantCursorOf<T>::ToPrevious(TIndex)*/

//----------------------------------------------------------------------------
template <class T> inline 
TListConstantCursorOf<T>& TListConstantCursorOf<T>::Seek(

	nl_long_t nOffset
	
){
	_Seek(nOffset);
	return *this;
} /*TListConstantCursorOf<T>::Seek(nl_long_t)*/

//----------------------------------------------------------------------------
template <class T> inline 
TListConstantCursorOf<T>& TListConstantCursorOf<T>::ToNextByCondition(

    const TConditionOf<T>& c_Condition, 
	nl_ulong_t uIndex /*= 0*/

){
	_ToNextByCondition(c_Condition, uIndex);
	return *this;
} /*TListConstantCursorOf<T>::ToNextByCondition(const TCondition&, ...)*/

//----------------------------------------------------------------------------
template <class T> inline 
TListConstantCursorOf<T>& TListConstantCursorOf<T>::ToPreviousByCondition(

    const TConditionOf<T>& c_Condition, 
	nl_ulong_t uIndex /*= 0*/

){
	_ToPreviousByCondition(c_Condition, uIndex);
	return *this;
} /*TListConstantCursorOf<T>::ToPreviousByCondition(...)*/

//----------------------------------------------------------------------------
template <class T> inline 
TListConstantCursorOf<T>& TListConstantCursorOf<T>::ToNextByComparator(

	const TComparatorOf<T>& c_Comparator, 
	nl_ulong_t uIndex /*= 0*/

){
	_ToNextByComparator(c_Comparator, uIndex);
	return *this;
} /*TListConstantCursorOf<T>::ToNextByComparator(...)*/

//----------------------------------------------------------------------------
template <class T> inline 
TListConstantCursorOf<T>& TListConstantCursorOf<T>::ToPreviousByComparator(

	const TComparatorOf<T>& c_Comparator, 
	nl_ulong_t uIndex /*= 0*/

){
	_ToPreviousByComparator(c_Comparator, uIndex);
	return *this;
} /*TListConstantCursorOf<T>::ToPreviousByComparator(...)*/

//----------------------------------------------------------------------------
template <class T> void TListConstantCursorOf<T>::_ToFirst(

	nl_ulong_t uPosition /*= 0*/
	
){
    NLLIST_CURSOR_LIST_ASSIGNMENT_GUARD(*this);
    AssignNode(NodeByIndex(*hList, uPosition));
} /*TListConstantCursorOf<T>::_ToFirst(TIndex = 0)*/

//----------------------------------------------------------------------------
template <class T> void TListConstantCursorOf<T>::_ToLast(

	nl_ulong_t uOffset /*= 0*/
	
){
    NLLIST_CURSOR_LIST_ASSIGNMENT_GUARD(*this);
	TList& r_List = *hList;
    AssignNode(PreviousNodePtr(r_List, TailPtr(r_List), uOffset));
} /*TListConstantCursorOf<T>::_ToLast(TIndex = 0)*/

//----------------------------------------------------------------------------
template <class T> void TListConstantCursorOf<T>::_ToEnd(

	nl_ulong_t uOffset /*= 0*/
	
){
    NLLIST_CURSOR_LIST_ASSIGNMENT_GUARD(*this);
	if (uOffset) ToLast(uOffset - 1);
	else pNode = null;
} /*TListConstantCursorOf<T>::_ToEnd(TIndex = 0)*/

//----------------------------------------------------------------------------
template <class T> void TListConstantCursorOf<T>::_ToNext()
{
    NLLIST_CURSOR_NODE_VALIDITY_GUARD(*this);
    AssignNode(NextNodePtr(*hList, pNode));
} /*TListConstantCursorOf<T>::_ToNext()*/

//----------------------------------------------------------------------------
template <class T> void TListConstantCursorOf<T>::_ToNext(

	nl_ulong_t uOffset
	
){
    NLLIST_CURSOR_NODE_VALIDITY_GUARD(*this);
    AssignNode(NextNodePtr(*hList, pNode, uOffset));
} /*TListConstantCursorOf<T>::_ToNext(TIndex)*/

//----------------------------------------------------------------------------
template <class T> void TListConstantCursorOf<T>::_ToPrevious()
{
    NLLIST_CURSOR_NODE_VALIDITY_GUARD(*this);
    AssignNode(PreviousNodePtr(*hList, pNode));
} /*TListConstantCursorOf<T>::_ToPrevious()*/

//----------------------------------------------------------------------------
template <class T> void TListConstantCursorOf<T>::_ToPrevious(

	nl_ulong_t uOffset
	
){
    NLLIST_CURSOR_NODE_VALIDITY_GUARD(*this);
    AssignNode(PreviousNodePtr(*hList, pNode, uOffset));
} /*TListConstantCursorOf<T>::_ToPrevious(TIndex)*/

//----------------------------------------------------------------------------
template <class T> void TListConstantCursorOf<T>::_Seek(

	nl_long_t nOffset
	
){
    NLLIST_CURSOR_NODE_VALIDITY_GUARD(*this);
    AssignNode(SeekNodePtr(*hList, pNode, nOffset));
} /*TListConstantCursorOf<T>::_Seek(nl_long_t)*/

//----------------------------------------------------------------------------
template <class T> void TListConstantCursorOf<T>::_ToNextByCondition(

    const TConditionOf<T>& c_Condition, 
	nl_ulong_t uIndex /*= 0*/

){
    NLLIST_CURSOR_NODE_VALIDITY_GUARD(*this);
    AssignNode(NextByCondition(*hList, pNode, c_Condition, uIndex));
} /*TListConstantCursorOf<T>::_ToNextByCondition(const TCondition&, ...)*/

//----------------------------------------------------------------------------
template <class T> void TListConstantCursorOf<T>::_ToPreviousByCondition(

    const TConditionOf<T>& c_Condition, 
	nl_ulong_t uIndex /*= 0*/

){
    NLLIST_CURSOR_NODE_VALIDITY_GUARD(*this);
    AssignNode(PreviousByCondition(*hList, pNode, c_Condition, uIndex));
} /*TListConstantCursorOf<T>::_ToPreviousByCondition(const TCondition&, ...)*/

//----------------------------------------------------------------------------
template <class T> void TListConstantCursorOf<T>::_ToNextByComparator(

	const TComparatorOf<T>& c_Comparator, 
	nl_ulong_t uIndex /*= 0*/

){
	TIsEqualByComparatorOf<T> IsEqual(c_Comparator);
	ToNextByCondition(IsEqual, uIndex);
} /*TListConstantCursorOf<T>::_ToNextByComparator(const TComparator&, ...)*/

//----------------------------------------------------------------------------
template <class T> void TListConstantCursorOf<T>::_ToPreviousByComparator(

	const TComparatorOf<T>& c_Comparator,
	nl_ulong_t uIndex /*= 0*/

){
	TIsEqualByComparatorOf<T> IsEqual(c_Comparator);
	ToPreviousByCondition(IsEqual, uIndex);
} /*TListConstantCursorOf<T>::_ToPreviousByComparator(...)*/

//----------------------------------------------------------------------------
template <class T> void TListConstantCursorOf<T>::Unassign()
{
    if (pNode) {
        pNode->UnregisterReference();
        if (!pNode->IsChained() && !pNode->References())
			DeallocateNode(*hList, pNode);
        pNode = null;
    } /*if*/
} /*TListConstantCursorOf<T>::Unassign()*/

//----------------------------------------------------------------------------
template <class T>
const T* TListConstantCursorOf<T>::DereferencePtrConst() const
{
	return IsNodeValid() ? pNode->DataPtrConst() : null;
} /*TListConstantCursorOf<T>::DereferencePtrConst() const*/

//----------------------------------------------------------------------------
template <class T>
const T& TListConstantCursorOf<T>::DereferenceConst() const 
{
	if (IsEndOfList()) 
		throw TXEndOfList(THIS_CLASS);
	return pNode->Data();
} /*TListConstantCursorOf<T>::DereferenceConst() const */

//----------------------------------------------------------------------------
template <class T>
TListConstantCursorOf<T>& TListConstantCursorOf<T>::operator=(

	const TListConstantCursor& c_Source
	
){
	Assign(c_Source); return *this;
} /*TListConstantCursorOf<T>::operator=(const TListConstantCursor&)*/

//----------------------------------------------------------------------------
template <class T>
TListConstantCursorOf<T> TListConstantCursorOf<T>::operator++(int)
{
    TListConstantCursor PreviousValue(*this);
    ToNext();
    return PreviousValue;
} /*TListConstantCursorOf<T>::operator++(int)*/

//----------------------------------------------------------------------------
template <class T>
TListConstantCursorOf<T> TListConstantCursorOf<T>::operator+(

	nl_long_t nOffset

) const {
    return TListConstantCursor(*this) += nOffset;
} /*TListConstantCursorOf<T>::operator+(nl_long_t) const*/

//----------------------------------------------------------------------------
template <class T>
TListConstantCursorOf<T> TListConstantCursorOf<T>::operator--(int)
{
    TListConstantCursor PreviousValue(*this);
    ToPrevious();
    return PreviousValue;
} /*TListConstantCursorOf<T>::operator++(int)*/

//----------------------------------------------------------------------------
template <class T>
TListConstantCursorOf<T> TListConstantCursorOf<T>::operator-(

	nl_long_t nOffset

) const{
    return TListConstantCursor(*this) -= nOffset;
} /*TListConstantCursorOf<T>::operator-(nl_long_t) const*/

//----------------------------------------------------------------------------
template <class T> void TListConstantCursorOf<T>::AssignNode(

	TListNodeOf<T>* pTarget
	
){
    Unassign();
    if (pTarget) {pNode = pTarget; pNode->RegisterReference();}
} /*TListConstantCursorOf<T>::AssignNode(const TNode*)*/

//----------------------------------------------------------------------------
template <class T> void TListConstantCursorOf<T>::Assign(

	TListOf<T>& r_List, 
	nl_ulong_t uPosition /*= 0*/
	
){
	hList.Assign(r_List);
	AssignNode(NodeByIndex(r_List, uPosition));
} /*TListConstantCursorOf<T>::Assign(TList&, TIndex)*/

//----------------------------------------------------------------------------
template <class T> void TListConstantCursorOf<T>::Assign(

	TListOf<T>& c_List, 
	TListNodeOf<T>* p_Node, 
	nl_long_t nOffset /*= 0*/
	
){
	hList.Assign(c_List);
	AssignNode(SeekNodePtr(*hList, p_Node, nOffset));
} /*TListConstantCursorOf<T>::Assign(TList&, TNopde*, nl_long_t = 0)*/

//----------------------------------------------------------------------------
template <class T> 
const T& TListConstantCursorOf<T>::InsertBefore(

	const T& c_Data
	
) const {
	NLLIST_CURSOR_LIST_ASSIGNMENT_GUARD(*this);
	TList& r_List = *hList;
	return InsertData(r_List, c_Data, PreviousNodePtr(r_List, pNode));
} /*TListConstantCursorOf<T>::InsertData(const T&)*/

//----------------------------------------------------------------------------
template <class T> const T& TListConstantCursorOf<T>::InsertAfter(

	const T& c_Data
	
) const {
	NLLIST_CURSOR_NODE_VALIDITY_GUARD(*this);
	return InsertData(*hList, c_Data, pNode);
} /*TListConstantCursorOf<T>::InsertData(const T&)*/

//----------------------------------------------------------------------------
template <class T> void TListConstantCursorOf<T>::Delete()
{
	NLLIST_CURSOR_NODE_VALIDITY_GUARD(*this);
	TNode* pNodeToDelete = pNode;
	ToNext();
	DelistNode(*hList, *pNodeToDelete);
} /*TListConstantCursorOf<T>::Delete()*/

// TListCursorOf template implementation
//============================================================================
template <class T> TListCursorOf<T> TListCursorOf<T>::Begin(

	TList& r_List
	
){
	TListCursorOf<T> BeginCur(r_List);
	return BeginCur;
} /*TListCursorOf<T>::Begin(TList&)*/

//----------------------------------------------------------------------------
template <class T> TListCursorOf<T> TListCursorOf<T>::End(

	TList& r_List
	
){
	TListCursorOf<T> EndCur(r_List);
	return EndCur.ToEnd();
} /*TListCursorOf<T>::End(TList&)*/

//----------------------------------------------------------------------------
template <class T> TListCursorOf<T>::TListCursorOf()
{} /*TListCursorOf<T>()*/

//----------------------------------------------------------------------------
template <class T> TListCursorOf<T>::TListCursorOf(

	TList& r_List, 
	TIndex uPosition /*= 0*/
	
){
	Assign(r_List, uPosition);
} /*TListCursorOf<T>::(TList&, TIndex)*/

//----------------------------------------------------------------------------
template <class T> TListCursorOf<T>::TListCursorOf(

	const TCursor& c_Cursor
	
){
	Assign(c_Cursor);
} /*TListCursorOf<T>(TCursor&)*/

//----------------------------------------------------------------------------
template <class T> TListCursorOf<T>::TListCursorOf(

	const TCursor& c_Cursor, 
	nl_long_t nOffset
	
){
	Assign(c_Cursor, nOffset);
} /*TListCursorOf<T>(TCursor&, TIndex)*/

//----------------------------------------------------------------------------
template <class T>
NLOBJECT_IMPLEMENT_CLONE_DEFAULT1(TListCursorOf, T)

//----------------------------------------------------------------------------
template <class T> inline TListCursorOf<T>& TListCursorOf<T>::ToFirst(

	nl_ulong_t uPosition /*= 0*/ 

){
	_ToFirst(uPosition);
	return *this;
} /*TListCursorOf<T>::ToFirst(TIndex = 0)*/

//----------------------------------------------------------------------------
template <class T> inline TListCursorOf<T>& TListCursorOf<T>::ToLast(

	nl_ulong_t uOffset /*= 0*/

){
	_ToLast(uOffset);
	return *this;
} /*TListCursorOf<T>::ToLast(TIndex = 0)*/

//----------------------------------------------------------------------------
template <class T> inline TListCursorOf<T>& TListCursorOf<T>::ToEnd(

	nl_ulong_t uOffset /*= 0*/
	
){
	_ToEnd(uOffset);
	return *this;
} /*TListCursorOf<T>::ToEnd(TIndex = 0)*/

//----------------------------------------------------------------------------
template <class T> inline TListCursorOf<T>& TListCursorOf<T>::ToNext()
{
	_ToNext();
	return *this;
} /*TListCursorOf<T>& TListCursorOf<T>::ToNext()*/

//----------------------------------------------------------------------------
template <class T> inline TListCursorOf<T>& TListCursorOf<T>::ToNext(

	nl_ulong_t uOffset
	
){
	_ToNext(uOffset);
	return *this;
} /*TListCursorOf<T>::ToNext(TIndex)*/

//----------------------------------------------------------------------------
template <class T> inline TListCursorOf<T>& TListCursorOf<T>::ToPrevious()
{
	_ToPrevious();
	return *this;
} /*TListCursorOf<T>::ToPrevious()*/

//----------------------------------------------------------------------------
template <class T> inline TListCursorOf<T>& TListCursorOf<T>::ToPrevious(

	nl_ulong_t uOffset

){
	_ToPrevious(uOffset);
	return *this;
} /*TListCursorOf<T>::ToPrevious(TIndex)*/

//----------------------------------------------------------------------------
template <class T> inline TListCursorOf<T>& TListCursorOf<T>::Seek(

	nl_long_t nOffset

){
	_Seek(nOffset);
	return *this;
} /*TListCursorOf<T>::Seek(nl_long_t)*/

//----------------------------------------------------------------------------
template <class T> inline 
TListCursorOf<T>& TListCursorOf<T>::ToNextByCondition(

	const TConditionOf<T>& c_Condition, 
	nl_ulong_t uIndex /*= 0*/
	
){
	_ToNextByCondition(c_Condition, uIndex);
	return *this;
} /*TListCursorOf<T>::ToNextByCondition(const TCondition&, TIndex = 0)*/

//----------------------------------------------------------------------------
template <class T> inline 
TListCursorOf<T>& TListCursorOf<T>::ToPreviousByCondition(

	const TConditionOf<T>& c_Condition, 
	nl_ulong_t uIndex /*= 0*/
	
){
	_ToPreviousByCondition(c_Condition, uIndex);
	return *this;
} /*TListCursorOf<T>::ToPreviousByCondition(const TCondition&, TIndex = 0)*/

//----------------------------------------------------------------------------
template <class T> inline 
TListCursorOf<T>& TListCursorOf<T>::ToNextByComparator(

	const TComparatorOf<T>& c_Comparator, 
	nl_ulong_t uIndex /*= 0*/
	
){
	_ToNextByComparator(c_Comparator, uIndex);
	return *this;
} /*TListCursorOf<T>::ToNextByComparator(const TComparator&, TIndex = 0)*/

//----------------------------------------------------------------------------
template <class T>
inline TListCursorOf<T>& TListCursorOf<T>::ToPreviousByComparator(

	const TComparatorOf<T>& c_Comparator,
	nl_ulong_t uIndex /*= 0*/

){
	_ToPreviousByComparator(c_Comparator, uIndex);
	return *this;
} /*TListCursorOf<T>::ToPreviousByComparator(const TComparator&, TIndex = 0)*/

//----------------------------------------------------------------------------
template <class T> TListCursorOf<T>& TListCursorOf<T>::operator=(

	const TListCursorOf<T>& c_Source

){
	Assign(c_Source); return *this;
} /*TListCursorOf<T>::operator=(const TCursor&)*/

//----------------------------------------------------------------------------
template <class T> TListCursorOf<T> TListCursorOf<T>::operator++(int)
{
    TCursor PreviousValue(*this); ToNext(); return PreviousValue;
} /*TListCursorOf<T>::operator++(int)*/

//----------------------------------------------------------------------------
template <class T> TListCursorOf<T> TListCursorOf<T>::operator+(

	nl_long_t nOffset

) const {
    return TCursor(*this) += nOffset;
} /*TListCursorOf<T>::operator+(nl_long_t) const*/

//----------------------------------------------------------------------------
template <class T> TListCursorOf<T> TListCursorOf<T>::operator--(int)
{
    TCursor PreviousValue(*this); ToPrevious(); return PreviousValue;
} /*TListCursorOf<T>::operator--(int)*/

//----------------------------------------------------------------------------
template <class T> TListCursorOf<T> TListCursorOf<T>::operator-(

	nl_long_t nOffset

) const {
    return TCursor(*this) -= nOffset;
} /*TListCursorOf<T>::operator-(nl_long_t) const*/

// TSortListOf template implementation
//============================================================================
template <class T> void TSortListOf<T>::operator()(

	TListNodeOf<T>*& r_pTail

){
	if (r_pTail->pNext == r_pTail) return;

	try {
		// Initiate sorting sequences
		//----------------------------
		{
			TSequence Source; {
				Source.pHead = r_pTail->pNext; Source.pTail = r_pTail; 
				r_pTail = null;

				// Disconnect list cycle
				//-----------------------
				Source.pTail->pNext = null;
			} /*Source*/

			InpLeft.Init(); InpRight.Init(); OutLeft.Init(); OutRight.Init();
			SplitBySegments(Source);
		} /*Initiate sorting sequences*/

  	
		do {
			// Swap output and input
			//-----------------------
			InpLeft = OutLeft; InpRight = OutRight; 
			OutRight.Init(); OutLeft.Init();

			do {
    			MergeSegment(OutLeft);
				MergeSegment(OutRight);
			} while (InpLeft.pHead && InpRight.pHead);

			if (InpLeft.pHead) SplitBySegments(InpLeft);
			Swap(OutLeft, OutRight);
			if (InpRight.pHead) SplitBySegments(InpRight);
		} while (OutLeft.pHead && OutRight.pHead);
	} /*try*/
	catch (...) {RestoreDoubleConnectivity(r_pTail); throw;}
		
	RestoreDoubleConnectivity(r_pTail);
} /*TSortListOf<T>::operator()(TNode*&)*/

//----------------------------------------------------------------------------
template <class T> void TSortListOf<T>::MoveSegment(

	TSortListOf<T>::TSequence& r_Inp,
    TSortListOf<T>::TSequence& r_Out

){
    if (!r_Inp.pHead) return;
    do {
        r_Out.MoveFrom(r_Inp);
        pComparator->SetEtalon(r_Out.pTail->RawDataConst());
    } while (!IsEndOfSegment(r_Inp));
} /*TSortListOf<T>::MoveSegment(TSequence&, TSequence&&)*/

//----------------------------------------------------------------------------
template <class T> void TSortListOf<T>::SplitBySegments(

    TSortListOf<T>::TSequence& r_Inp

){
    while (r_Inp.pHead) {
        MoveSegment(r_Inp, OutLeft);
        MoveSegment(r_Inp, OutRight);
    } /*while*/
} /*TSortListOf<T>::SplitBySegments(TSequence&)*/

//----------------------------------------------------------------------------
template <class T> void TSortListOf<T>::MergeSegment(

    TSortListOf<T>::TSequence& r_Out

){
    if (!InpLeft.pHead) {MoveSegment(InpRight, r_Out); return;}
    if (!InpRight.pHead) {MoveSegment(InpLeft, r_Out); return;}

	for (;;) {
	    pComparator->SetEtalon(InpLeft.pHead->RawDataConst());
        if (
        	(*pComparator)(InpRight.pHead->RawDataConst()) !=
            TComparator::compGreater
        ){
            r_Out.MoveFrom(InpLeft);
            if (IsEndOfSegment(InpLeft)) {
                MoveSegment(InpRight, r_Out); return;
            } /*if*/
        }
        else {
        	r_Out.MoveFrom(InpRight);
            pComparator->SetEtalon(r_Out.pTail->RawDataConst());
            if (IsEndOfSegment(InpRight)) {
                MoveSegment(InpLeft, r_Out); return;
            } /*if*/
        } /*if*/
    } /*for (;;)*/
} /*TSortListOf<T>::MergeSegment(TSequence&)*/

//----------------------------------------------------------------------------
template <class T> void TSortListOf<T>::AppendToLeftOutput(

	TSortListOf<T>::TSequence& r_Source

){
	if (OutLeft.pHead) while (r_Source.pHead) OutLeft.MoveFrom(r_Source);
	else OutLeft = r_Source;
} /*void TSortListOf<T>::AppendToLeftOutput(TSequence&)*/

//----------------------------------------------------------------------------
template <class T> void TSortListOf<T>::RestoreDoubleConnectivity(

	TListNodeOf<T>*& r_pTail

){
	AppendToLeftOutput(OutRight);
	AppendToLeftOutput(InpRight);
	AppendToLeftOutput(InpLeft);

	for (
		TNode* pNode = OutLeft.pHead;
		pNode != OutLeft.pTail; 
		pNode = pNode->pNext
	) pNode->pNext->pPrevious = pNode;
	OutLeft.pHead->pPrevious = OutLeft.pTail; 
	OutLeft.pTail->pNext = OutLeft.pHead;

	r_pTail = OutLeft.pTail;

	OutLeft.Init();
	OutRight.Init();
} /*TSortListOf<T>::RestoreDoubleConnectivity(TNode*&)*/

// TListOf template implementation
//============================================================================
template <class T> void TListOf<T>::AssertValid() const
{
	TParent::AssertValid();
	
	if (IsEmpty()) {
		if (Size() || HeadPtr() || pTail)
			throw TXInvalidChain(THIS_CLASS);
	}
	else {
		TNode* pCheck = HeadPtr();
		TNode* pPrevious = pTail;
		nl_ulong_t nCheckSize = 1;

		do {
			pCheck->AssertValid();
			if (
				(pCheck->pPrevious != pPrevious) || 
				(pPrevious->pNext != pCheck)
			){
				throw TXInvalidChain(THIS_CLASS);
			} /*if*/
			pPrevious = pCheck; pCheck = pCheck->pNext;
			++nCheckSize;
		} while (pCheck != pTail);

		if (nCheckSize != Size())
			throw TXInvalidSize(THIS_CLASS);
	} /*iff*/
} /*TListOf<T>::AssertValid() const*/

//----------------------------------------------------------------------------
template <class T>
NLOBJECT_IMPLEMENT_CLONE_DEFAULT1(TListOf, T);

//----------------------------------------------------------------------------
template <class T> nl_ulong_t TListOf<T>::Delete(

	const TConditionOf<T>& c_Condition,
    nl_ulong_t uIndex

){
    TNode* pNode = HeadPtr();
    if (uIndex != indexAll) {
    	TIndex nFound = uIndex;
    	while (pNode) {
    		if (c_Condition(pNode->RawData()) && !--nFound) {
        		TNode* pNodeToDelist = pNode;
                DelistNode(*pNodeToDelist);
                return 1;
        	}
    		else pNode = NextNodePtr(pNode);
    	} /*while (pNode)*/
        return 0;
    }
    else {
    	TSize nDeleted = 0;
        while (pNode) {
      		if (c_Condition(pNode->RawData())) {
        		TNode* pNodeToDelist = pNode;
                pNode = NextNodePtr(pNode);
                DelistNode(*pNodeToDelist);
                ++nDeleted;
        	}
    		else pNode = NextNodePtr(pNode);
        } /*while (pNode)*/
        return nDeleted;
    } /*iff*/
} /*TListOf<T>::Delete(const TCondition&, TIndex = indexAll)*/;

//----------------------------------------------------------------------------
template <class T> void TListOf<T>::Clear()
{
	while (pTail) DelistNode(*pTail);
} /*TListOf<T>::Clear()*/

//----------------------------------------------------------------------------
template <class T> void TListOf<T>::ForEach(

	TOperationOn<T>& r_Operation

){
	try {
    	for (TListCursorOf<T> Cursor(*this); Cursor; ++Cursor)
        	r_Operation(*Cursor);
    } /*try*/
	catch (const TOperation::TXAbort&) {}
} /*TListOf<T>::ForEach(TOperation&)*/

//----------------------------------------------------------------------------
template <class T> void TListOf<T>::ForEach(

	TConstantOperationOn<T>& r_Operation

) const {
	try {
    	for(TListConstantCursorOf<T> Cursor(*this); Cursor; ++Cursor)
        	r_Operation(*Cursor);
    } /*try*/
	catch (const TConstantOperation::TXAbort&) {}
} /*TListOf<T>::ForEach(TConstantOperation&) const*/

//----------------------------------------------------------------------------
template <class T> void TListOf<T>::Sort(

	TEtalonComparatorOf<T>& r_SortingComparator

){
	TSortListOf<T> SortMe(r_SortingComparator);
	SortMe(pTail);
} /*TListOf<T>::Sort(TEtalonComparatorOf<T>&)*/

//----------------------------------------------------------------------------
template <class T> TListNodeOf<T>* TListOf<T>::NextNodePtr(

	TListNodeOf<T>* pNode,
    nl_ulong_t uOffset

) const {
    while (uOffset-- && pNode) pNode = NextNodePtr(pNode);
    return pNode;
} /*TListOf<T>::NextNodePtr(TNode*, nl_ulong_t)*/

//----------------------------------------------------------------------------
template<class T> TListNodeOf<T>* TListOf<T>::PreviousNodePtr(

	TListNodeOf<T>* pNode,
    nl_ulong_t uOffset

) const {
    while (uOffset-- && pNode) pNode = PreviousNodePtr(pNode);
    return pNode;
} /*TListOf<T>::PreviousNodePtr(TNode*, nl_ulong_t)*/

//----------------------------------------------------------------------------
template<class T> TListNodeOf<T>* TListOf<T>::NextByCondition(

   	TListNodeOf<T>* pNode,
	const TConditionOf<T>& c_Condition,	
	nl_ulong_t Index

) const {
	TContainerSearchByIndex<T> Search(c_Condition, Index);
    try {
    	while (pNode) {
			Search(pNode->RawDataConst()); 
			pNode = NextNodePtr(pNode);
		} /*while (pNode)*/
    } /*try*/
    catch (const TConstantOperation::TXAbort&) {}
    return pNode;
} /*TListOf<T>::NextByCondition(TListNodeOf<T>*, ...) const*/

//----------------------------------------------------------------------------
template<class T> TListNodeOf<T>* TListOf<T>::PreviousByCondition(

   	TListNodeOf<T>* pNode,
	const TConditionOf<T>& c_Condition,
	nl_ulong_t Index

) const {
	TContainerSearchByIndex<T> Search(c_Condition, Index);
    try {
    	while (pNode) {
        	Search(pNode->RawDataConst()); 
			pNode = PreviousNodePtr(pNode);
        } /*while (pNode)*/
    } /*try*/
    catch (const TConstantOperation::TXAbort&) {}
	return pNode;
} /*TListOf<T>::PreviousByCondition(TListNodeOf<T>*, ...) const*/

//----------------------------------------------------------------------------
template<class T> TListNodeOf<T>* TListOf<T>::AllocateNode(

	const T& c_Data

) const {
    return new (pAllocator->Allocate(AllocationSize())) TNode(c_Data);
} /*TListOf<T>::AllocateNode(const T&) const*/

//----------------------------------------------------------------------------
template <class T> void TListOf<T>::DeallocateNode(

	TListNodeOf<T>* pNode
	
){
	if (pNode) {
    	typedef TListNodeOf<T> _TListNode;
		pNode->~_TListNode();
		pAllocator->Deallocate(pNode);
	} /*if*/
} /*TListOf<T>::DeallocateNode(TNode*)*/

//----------------------------------------------------------------------------
template <class T> void TListOf<T>::InsertNodeAfter(

	TListNodeOf<T>& r_Node,
	TListNodeOf<T>* pPosition

){
    TNode* pNode = &r_Node;
	if (!pPosition) {
    	AppendNode(pNode);
		pTail = pNode->pPrevious;
	}
    else if (pPosition == pTail) 
    	AppendNode(pNode);
    else {
        pPosition->pNext->pPrevious = pNode;
        r_Node.pNext = pPosition->pNext;
        pPosition->pNext = pNode;
        r_Node.pPrevious = pPosition;
    	++nSize;
    } /*iff*/
} /*TListOf<T>::InsertNodeAfter(TNode&, TNode*)*/

//----------------------------------------------------------------------------
template <class T> void TListOf<T>::AppendNode(

	TListNodeOf<T>* pNode

){
	if (!pTail) {
    	pTail = pNode->pNext = pNode->pPrevious = pNode;
	}
    else {
    	pTail->pNext->pPrevious = pNode;
        pNode->pNext = pTail->pNext;
        pTail->pNext = pNode;
        pNode->pPrevious = pTail;
        pTail = pNode;
    } /*iff*/
    ++nSize;
} /*TListOf<T>::AppendNode(TListNodeOf<T>*)*/

//----------------------------------------------------------------------------
template <class T> const T& TListOf<T>::InsertData(

	const T& c_Data,
    TListNodeOf<T>* pPosition

){
	TNode* pNode = AllocateNode(c_Data);
    if (!pNode)
    	throw TXUnableToConstructNode(THIS_CLASS);
    InsertNodeAfter(*pNode, pPosition);
    return pNode->RawDataConst();
} /*TListOf<T>::InsertData(const T&, TNode*)*/

//----------------------------------------------------------------------------
template <class T> void TListOf<T>::DelistNode(

	TListNodeOf<T>& r_Node

){
    if (!r_Node.IsChained()) return;

    TNode* pNode = &r_Node;
    if (nSize == 1)	pTail = null; // remove the last node - list becomes empty
    else {
        r_Node.pPrevious->pNext = r_Node.pNext;
        r_Node.pNext->pPrevious = r_Node.pPrevious;
        if (pNode == pTail) pTail = pNode->pPrevious;
    } /*iff*/
    --nSize;
    r_Node.pNext = r_Node.pPrevious = null;
    if (!r_Node.References()) DeallocateNode(pNode);
} /*TListOf<T>::DelistNode(TNode&)*/

//============================================================================
NATLIB_NAMESPACE_END

//============================================================================
#define NLLIST_DECLSPEC(T, Declspec) \
	NLCONT_DECLSPEC(T, Declspec); \
    NLALLOC_DECLSPEC(T, Declspec); \
	NLSAFEH_DECLSPEC(TListOf<T>, Declspec); \
    NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TListNodeOf), T, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/; \
    NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TListOf), T, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/; \
    NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TListConstantCursorOf), T, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/; \
    NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TListCursorOf), T, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/; \
	NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TSortListOf), T, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/

#endif /*!defined(__NATLIB_NLLIST_H__)*/
