#ifndef __NATLIB_NLLOCK_H__
#define __NATLIB_NLLOCK_H__
/****************************************************************************/
/** nllock.h                                                               **/
/** ---------------------------------------------------------------------- **/
/** Natlib syncronization facility base                                    **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** June 14, 1996 by Oleg Kagan                                            **/
/****************************************************************************/

//============================================================================
#include <nlobject.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TLock
//============================================================================
class _NATLIB_DECLSPEC_ TLock : public virtual TNatlibObject {
public:
	//------------------------------------------------------------------------
	typedef TNatlibObject TParent;

	//------------------------------------------------------------------------
	NLEXCEPT_DECLARE_BASE(TException, _NATLIB_DECLSPEC_);
    NLEXCEPT_DECLARE_BUSY(TXBusy, TException, _NATLIB_DECLSPEC_);
    NLEXCEPT_DECLARE(TXInvalidKey, TException, _NATLIB_DECLSPEC_);

	//------------------------------------------------------------------------
    static const nl_ulong_t c_uWaitInfinite;

    //------------------------------------------------------------------------
    TLock() : pc_Key(null) {}
    //------------------------------------------------------------------------
    virtual ~TLock() {}

    //------------------------------------------------------------------------
    TNatlibStdString Dump() const;
	//------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE(TLock);

	// Tries to lock with supplied key while uWaitTimeout milliseconds
	//------------------------------------------------------------------------
    virtual void Lock(const void* pc_LockKey, nl_ulong_t uWaitTimeout = 0);
	// Tries to unlock with supplied key
    //------------------------------------------------------------------------
    virtual void Unlock(const void* pc_LockKey);
    //------------------------------------------------------------------------
	virtual bool IsLocked() const {return Key() != null;}
	// Returns the key used to lock the object
    //------------------------------------------------------------------------
    virtual const void* Key() const {return pc_Key;}

protected:
	//------------------------------------------------------------------------
	const void* pc_Key;

private:
	//------------------------------------------------------------------------
	TLock(const TLock&) : pc_Key(null) 
	{
		throw TXOperationIsNotAllowed(THIS_CLASS); 
	} /*TLock(const TLock&)*/
	//------------------------------------------------------------------------
	TLock& operator=(const TLock&) 
	{
		throw TXOperationIsNotAllowed(THIS_CLASS); return *this;
	} /*operator=(const TLock&) */
} /*TLock*/;

//============================================================================
class _NATLIB_DECLSPEC_ TAutoLocker {
public:
	//------------------------------------------------------------------------
	TAutoLocker(TLock& r_InitLock, nl_ulong_t uTimeout = 0) :
    	r_Lock(r_InitLock)
    {
    	r_Lock.Lock(this, uTimeout);
    } /*TAutoLocker(TLock&, nl_ulong_t = 0)*/
	//------------------------------------------------------------------------
	virtual ~TAutoLocker() {r_Lock.Unlock(this);}

protected:
	//------------------------------------------------------------------------
	TAutoLocker& operator=(const TAutoLocker&) 
	{
		throw TXOperationIsNotAllowed(THIS_CLASS); return *this;
	} /*operator=(const TLock&)*/

private:
	//------------------------------------------------------------------------
	TLock& r_Lock;
} /*TAutoLocker*/;

//============================================================================
NATLIB_NAMESPACE_END

#endif /*!defined(__NATLIB_NLLOCK_H__)*/
