#ifndef __NATLIB_NLLSTACK_H__
#define __NATLIB_NLLSTACK_H__
/****************************************************************************/
/** nllstack.h                                                             **/
/** ---------------------------------------------------------------------- **/
/** Natlib Lite Stack (unsafe but efficient stack implementation)          **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** June 11, 1998 by Oleg Kagan                                            **/
/****************************************************************************/

//============================================================================
#include <nlobject.h>
#include <nlalloc.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TLiteStack
//============================================================================
class _NATLIB_DECLSPEC_ TLiteStack : public TNatlibObject {
public:
	//------------------------------------------------------------------------
	typedef TNatlibObject TParent;

	//------------------------------------------------------------------------
	typedef nl_ulong_t TSize;

	// TRecord
	//------------------------------------------------------------------------
	typedef struct _NATLIB_DECLSPEC_ TRecord {
		//--------------------------------------------------------------------
		TRecord* pNext;
		char a_cBuffer[1];
	} typedef_TRecord;

	//------------------------------------------------------------------------
	NLEXCEPT_DECLARE_BASE(TException, _NATLIB_DECLSPEC_);
	NLEXCEPT_DECLARE(TXUnableToAllocateRecord, TException, _NATLIB_DECLSPEC_);
	NLEXCEPT_DECLARE(TXInvalidSize, TException, _NATLIB_DECLSPEC_);
	NLEXCEPT_DECLARE(TXEndOfStack, TException, _NATLIB_DECLSPEC_);

	//------------------------------------------------------------------------
	TLiteStack(const TAllocator& c_Allocator = TStdAllocator());
	//------------------------------------------------------------------------
	~TLiteStack();

	//------------------------------------------------------------------------
	void AssertValid() const;
	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED(TLiteStack)

	//------------------------------------------------------------------------
	void* Current() {return pHead ? pHead->a_cBuffer : null;}
	//------------------------------------------------------------------------
	const void* CurrentConst() const {return pHead ? pHead->a_cBuffer : null;}

	//------------------------------------------------------------------------
	void* Push(const void*, size_t);
	//------------------------------------------------------------------------
	void Pop();

	//------------------------------------------------------------------------
	TSize Size() const {return nSize;}
	//------------------------------------------------------------------------
	bool IsEmpty() const {return !nSize;}

protected:
	//------------------------------------------------------------------------
	TSize nSize;
	TRecord* pHead;
	TAllocator* pAllocator;
} /*TLiteStack*/;

//============================================================================
NATLIB_NAMESPACE_END

#endif /*!defined(__NATLIB_NLLSTACK_H__)*/