#ifndef __NATLIB_NLOBJECT_H__
#define __NATLIB_NLOBJECT_H__
/****************************************************************************/
/** nlobject.h                                                             **/
/** ---------------------------------------------------------------------- **/
/** The virtual base of all Natlib objects defined here                    **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** June 14, 1996 by Oleg Kagan                                            **/
/****************************************************************************/

//============================================================================
#include <nlexcept.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TNatlibInitializer
// Constructor of this class will be used to initialize Natlib
// and destructor to close it
//============================================================================
class _NATLIB_DECLSPEC_ TNatlibInitializer {
public:
	//------------------------------------------------------------------------
	TNatlibInitializer() _NATLIB_THROW_NONE_;
	//------------------------------------------------------------------------
	virtual ~TNatlibInitializer() _NATLIB_THROW_NONE_;
} /*TNatlibInitializer*/;

// _TNatlibObject
// the virtual base of most classes in the Libary
// Note: must be inherited virtually
//============================================================================
class _NATLIB_DECLSPEC_ _TNatlibObject {
public:
	//------------------------------------------------------------------------
	NLEXCEPT_DECLARE_BASE(TException, _NATLIB_DECLSPEC_);
    NLEXCEPT_DECLARE_LOW_MEMORY(TXLowMemory, TException, _NATLIB_DECLSPEC_);
    NLEXCEPT_DECLARE_OPERATION_IS_NOT_ALLOWED(
    	TXOperationIsNotAllowed, TException, _NATLIB_DECLSPEC_
    ) /*NLEXCEPT_DECLARE_OPERATION_IS_NOT_ALLOWED*/;
	NLEXCEPT_DECLARE_OPERATION_IS_NOT_ALLOWED(
		TXCanNotBeCloned, TException, _NATLIB_DECLSPEC_
	) /*NLEXCEPT_DECLARE_OPERATION_IS_NOT_ALLOWED*/;
	NLEXCEPT_DECLARE_BAD_CAST( 
		TXCanNotBeConstructed, TException, _NATLIB_DECLSPEC_
	) /*NLEXCEPT_DECLARE_BAD_CAST*/;

	//------------------------------------------------------------------------
	virtual ~_TNatlibObject();

    // Asserts validity of the objects (throws appropriate exception if
    // object is not valid
    // User defined class should call parent's AsserValid and the validate
    // its own members
    //------------------------------------------------------------------------
    virtual void AssertValid() const;

	// Adds diagnostics to specified string (no \n at the end)
    // User defined class should call parents' Dump and then print
    // text values of own members (by default prints name of the class
    // received though RTTI)
    //------------------------------------------------------------------------
    virtual TNatlibStdString Dump() const;

	// Virtual constructor
	// returns valid pointer to valid copy of the object or throws
	// TXCanNotBeCloned otherwise
	//------------------------------------------------------------------------
	virtual _TNatlibObject* Clone() const = 0;

	// Virtual typecasting
	// Construct new object from any other object if possible
	// otherwise throws TXCanNotBeConstructed (default)
	//------------------------------------------------------------------------
	virtual _TNatlibObject* ConstructFrom(const _TNatlibObject&) const;

private:
	//------------------------------------------------------------------------
	static TNatlibInitializer NatlibInitializer;

public:
	// This function is used to identify any TNatlibObject child in templates
	//------------------------------------------------------------------------
	inline void _AssertNatlibObjectChild(const _TNatlibObject&) const {}
} /*_TNatlibObject*/;

//============================================================================
template <class T> T* _NATLIB_TEMPLATE_DECLSPEC_ CloneObject(

	const T& c_Object

){
//	c_Object._AssertNatlibObjectChild(c_Object);
	T* pClone = dynamic_cast<T*>(c_Object.Clone());
	if (!pClone) 
		throw TNatlibObject::TXCanNotBeCloned(THIS_PLACE_CLASS(c_Object));
	return pClone;
} /*CloneObject<T>(const T&)*/

//============================================================================
NATLIB_NAMESPACE_END

//============================================================================
typedef NATLIB_NAMESPACE_MEMBER(_TNatlibObject) TNatlibObject;
typedef NATLIB_NAMESPACE_MEMBER(_TNatlibObject) basic_TNatlibObject;

//============================================================================
#ifdef _NATLIB_NO_EXTENDED_OVERLOADING_
	#define NLOBJECT_CLONE_RESULT(Obj) TNatlibObject*
	#define NLOBJECT_CLONE_RESULT1(Obj, P1) TNatlibObject*
	#define NLOBJECT_CLONE_RESULT2(Obj, P1, P2) TNatlibObject*
	#define NLOBJECT_CLONE_RESULT3(Obj, P1, P2, P3) TNatlibObject*
	#define NLOBJECT_CLONE_RESULT4(Obj, P1, P2, P3, P4) TNatlibObject*
	#define NLOBJECT_CLONE_RESULT5(Obj, P1, P2, P3, P4, P5) TNatlibObject*
#else 
	#define NLOBJECT_CLONE_RESULT(Obj) Obj*
	#define NLOBJECT_CLONE_RESULT1(Obj, P1) Obj< P1 >*
	#define NLOBJECT_CLONE_RESULT2(Obj, P1, P2) Obj< P1, P2 >*
	#define NLOBJECT_CLONE_RESULT3(Obj, P1, P2, P3) Obj< P1, P2, P3 >*
	#define NLOBJECT_CLONE_RESULT4(Obj, P1, P2, P3, P4) \
		Obj< P1, P2, P3, P4 >*
	#define NLOBJECT_CLONE_RESULT5(Obj, P1, P2, P3, P4, P5) \
		Obj< P1, P2, P3, P4, P5 >*
#endif /*_NATLIB_NO_EXTENDED_OVERLOADING_*/

//============================================================================
#define NLOBJECT_CLONE_IS_NOT_ALLOWED(Obj) \
	NLOBJECT_CLONE_RESULT(Obj) Clone() const \
		{throw TXCanNotBeCloned(THIS_CLASS); return null;}
//----------------------------------------------------------------------------
#define NLOBJECT_CLONE_IS_NOT_ALLOWED1(Obj, P1) \
	NLOBJECT_CLONE_RESULT1(Obj, P1) Clone() const \
		{throw TXCanNotBeCloned(THIS_CLASS); return null;}
//----------------------------------------------------------------------------
#define NLOBJECT_CLONE_IS_NOT_ALLOWED2(Obj, P1, P2) \
	NLOBJECT_CLONE_RESULT2(Obj, P1, P2) Clone() const \
		{throw TXCanNotBeCloned(THIS_CLASS); return null;}
//----------------------------------------------------------------------------
#define NLOBJECT_CLONE_IS_NOT_ALLOWED3(Obj, P1, P2, P3) \
	NLOBJECT_CLONE_RESULT3(Obj, P1, P2, P3) Clone() const \
		{throw TXCanNotBeCloned(THIS_CLASS); return null;}
//----------------------------------------------------------------------------
#define NLOBJECT_CLONE_IS_NOT_ALLOWED4(Obj, P1, P2, P3, P4) \
	NLOBJECT_CLONE_RESULT4(Obj, P1, P2, P3, P4) Clone() const \
		{throw TXCanNotBeCloned(THIS_CLASS); return null;}
//----------------------------------------------------------------------------
#define NLOBJECT_CLONE_IS_NOT_ALLOWED5(Obj, P1, P2, P3, P4, P5) \
	NLOBJECT_CLONE_RESULT5(Obj, P1, P2, P3, P4, P5) Clone() const \
	{ \
		throw TXCanNotBeCloned(THIS_CLASS, typeid(*this).name()); \
		return null; \
	} /*Clone() const*/

//============================================================================
#define NLOBJECT_DECLARE_CLONE(Object) \
	NLOBJECT_CLONE_RESULT(Object) Clone() const
//----------------------------------------------------------------------------
#define NLOBJECT_DECLARE_CLONE1(Object, P1) \
	NLOBJECT_CLONE_RESULT1(Object, P1) Clone() const
//----------------------------------------------------------------------------
#define NLOBJECT_DECLARE_CLONE2(Object, P1, P2) \
	NLOBJECT_CLONE_RESULT2(Object, P1, P2) Clone() const
//----------------------------------------------------------------------------
#define NLOBJECT_DECLARE_CLONE3(Object, P1, P2, P3) \
	NLOBJECT_CLONE_RESULT3(Object, P1, P2, P3) Clone() const
//----------------------------------------------------------------------------
#define NLOBJECT_DECLARE_CLONE4(Object, P1, P2, P3, P4) \
	NLOBJECT_CLONE_RESULT4(Object, P1, P2, P3, P4) Clone() const
//----------------------------------------------------------------------------
#define NLOBJECT_DECLARE_CLONE5(Object, P1, P2, P3, P4, P5) \
	NLOBJECT_CLONE_RESULT5(Object, P1, P2, P3, P4, P5) Clone() const

//============================================================================
#define NLOBJECT_IMPLEMENT_CLONE_DEFAULT(Object) \
	NLOBJECT_CLONE_RESULT(Object) Object::Clone() const \
	{ \
		typedef Object NLOBJECT_TCopyOf_Object; \
		NLOBJECT_TCopyOf_Object* pCopy; \
		NLEXCEPT_CLASS_NEW(NLOBJECT_TCopyOf_Object(*this), pCopy); \
		return pCopy; \
	} /*Object::Clone() const*/
//----------------------------------------------------------------------------
#define NLOBJECT_IMPLEMENT_CLONE_DEFAULT1(Object, P1) \
	NLOBJECT_CLONE_RESULT1(Object, P1) \
	Object< P1 >::Clone() const \
	{ \
		typedef Object< P1 > NLOBJECT_TCopyOf_Object; \
		NLOBJECT_TCopyOf_Object* pCopy; \
		NLEXCEPT_CLASS_NEW(NLOBJECT_TCopyOf_Object(*this), pCopy); \
		return pCopy; \
	} /*Object<P1>::Clone() const*/
//----------------------------------------------------------------------------
#define NLOBJECT_IMPLEMENT_CLONE_DEFAULT2(Object, P1, P2) \
	NLOBJECT_CLONE_RESULT2(Object, P1, P2) \
	Object< P1, P2 >::Clone() const \
	{ \
		typedef Object< P1, P2 > NLOBJECT_TCopyOf_Object; \
		NLOBJECT_TCopyOf_Object* pCopy; \
		NLEXCEPT_CLASS_NEW(NLOBJECT_TCopyOf_Object(*this), pCopy); \
		return pCopy; \
	} /*Object<P1, P2>::Clone() const*/
//----------------------------------------------------------------------------
#define NLOBJECT_IMPLEMENT_CLONE_DEFAULT3(Object, P1, P2, P3) \
	NLOBJECT_CLONE_RESULT3(Object, P1, P2, P3) \
	Object< P1, P2, P3 >::Clone() const \
	{ \
		typedef Object< P1, P2, P3 > NLOBJECT_TCopyOf_Object; \
		NLOBJECT_TCopyOf_Object* pCopy; \
		NLEXCEPT_CLASS_NEW(NLOBJECT_TCopyOf_Object(*this), pCopy); \
		return pCopy; \
	} /*Object<P1, P2, P3>::Clone() const*/
//----------------------------------------------------------------------------
#define NLOBJECT_IMPLEMENT_CLONE_DEFAULT4(Object, P1, P2, P3, P4) \
	NLOBJECT_CLONE_RESULT4(Object, P1, P2, P3, P4) \
	Object< P1, P2, P3, P4 >::Clone() const \
	{ \
		typedef Object< P1, P2, P3, P4 > NLOBJECT_TCopyOf_Object; \
		NLOBJECT_TCopyOf_Object* pCopy; \
		NLEXCEPT_CLASS_NEW(NLOBJECT_TCopyOf_Object(*this), pCopy); \
		return pCopy; \
	} /*Object<P1, P2, P3, P4>::Clone() const*/
//----------------------------------------------------------------------------
#define NLOBJECT_IMPLEMENT_CLONE_DEFAULT5(Object, P1, P2, P3, P4, P5) \
	NLOBJECT_CLONE_RESULT5(Object, P1, P2, P3, P4, P5) \
	Object< P1, P2, P3, P4, P5 >::Clone() const \
	{ \
		typedef Object< P1, P2, P3, P4, P5 > NLOBJECT_TCopyOf_Object;  \
		NLOBJECT_TCopyOf_Object* pCopy; \
		NLEXCEPT_CLASS_NEW(NLOBJECT_TCopyOf_Object(*this), pCopy); \
		return pCopy; \
	} /*Object<P1, P2, P3, P4, P5>::Clone() const*/

//============================================================================
#define NLOBJECT_INLINE_CLONE_DEFAULT(Object) \
	NLOBJECT_CLONE_RESULT(Object) Clone() const \
	{ \
		typedef Object NLOBJECT_TCopyOf_Object; \
		NLOBJECT_TCopyOf_Object* pCopy; \
		NLEXCEPT_CLASS_NEW(NLOBJECT_TCopyOf_Object(*this), pCopy); \
		return pCopy; \
	} /*Clone() const*/
//----------------------------------------------------------------------------
#define NLOBJECT_INLINE_CLONE_DEFAULT1(Object, P1) \
	NLOBJECT_CLONE_RESULT1(Object, P1) \
	Clone() const \
	{ \
		typedef Object< P1 > NLOBJECT_TCopyOf_Object; \
		NLOBJECT_TCopyOf_Object* pCopy; \
		NLEXCEPT_CLASS_NEW(NLOBJECT_TCopyOf_Object(*this), pCopy); \
		return pCopy; \
	} /*Clone() const*/
//----------------------------------------------------------------------------
#define NLOBJECT_INLINE_CLONE_DEFAULT2(Object, P1, P2) \
	NLOBJECT_CLONE_RESULT2(Object, P1, P2) \
	Clone() const \
	{ \
		typedef Object< P1, P2 > NLOBJECT_TCopyOf_Object; \
		NLOBJECT_TCopyOf_Object* pCopy; \
		NLEXCEPT_CLASS_NEW(NLOBJECT_TCopyOf_Object(*this), pCopy); \
		return pCopy; \
	} /*Clone() const*/
//----------------------------------------------------------------------------
#define NLOBJECT_INLINE_CLONE_DEFAULT3(Object, P1, P2, P3) \
	NLOBJECT_CLONE_RESULT3(Object, P1, P2, P3) \
	Clone() const \
	{ \
		typedef Object< P1, P2, P3 > NLOBJECT_TCopyOf_Object; \
		NLOBJECT_TCopyOf_Object* pCopy; \
		NLEXCEPT_CLASS_NEW(NLOBJECT_TCopyOf_Object(*this), pCopy); \
		return pCopy; \
	} /*Clone() const*/
//----------------------------------------------------------------------------
#define NLOBJECT_INLINE_CLONE_DEFAULT4(Object, P1, P2, P3, P4) \
	NLOBJECT_CLONE_RESULT4(Object, P1, P2, P3, P4) \
	Clone() const \
	{ \
		typedef Object< P1, P2, P3, P4 > NLOBJECT_TCopyOf_Object; \
		NLOBJECT_TCopyOf_Object* pCopy; \
		NLEXCEPT_CLASS_NEW(NLOBJECT_TCopyOf_Object(*this), pCopy); \
		return pCopy; \
	} /*Clone() const*/
//----------------------------------------------------------------------------
#define NLOBJECT_INLINE_CLONE_DEFAULT5(Object, P1, P2, P3, P4, P5) \
	NLOBJECT_CLONE_RESULT5(Object, P1, P2, P3, P4, P5) \
	Clone() const \
	{ \
		typedef Object< P1, P2, P3, P4, P5 > NLOBJECT_TCopyOf_Object;  \
		NLOBJECT_TCopyOf_Object* pCopy; \
		NLEXCEPT_CLASS_NEW(NLOBJECT_TCopyOf_Object(*this), pCopy); \
		return pCopy; \
	} /*Clone() const*/

//============================================================================
#define NLOBJECT_DECLSPEC(T, Declspec)

#endif /*!defined(__NATLIB_NLOBJECT_H__)*/

