#ifndef __NLPQUEUE_H__
#define __NLPQUEUE_H__
/****************************************************************************/
/** nlpqueue.h                                                             **/
/** ---------------------------------------------------------------------- **/
/** Priority queue allows you to extract from the buffer smallest data     **/
/** entered at current time                                                **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** November 29, 1997 by Oleg Kagan                                        **/
/****************************************************************************/

//============================================================================
#include <nlbuffer.h>
#include <nlvector.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

//============================================================================
#define NLPQUEUE_ALLOCATOR_DEFAULT NVECTOR_ALLOCATOR_DEFAULT
#define NLPQUEUE_CAPACITY_DEFAULT 16

// TPriorityQueueOf template
//============================================================================
template <class T> class _NATLIB_TEMPLATE_DECLSPEC_ TPriorityQueueOf :
	public TBufferOf<T>
{
public:
	//------------------------------------------------------------------------
    typedef TBufferOf<T> TParent;
    typedef TPriorityQueueOf<T> TPriorityQueue;

    //------------------------------------------------------------------------
    TPriorityQueueOf(
    	size_t nCapacity = NLPQUEUE_CAPACITY_DEFAULT,
        const TAllocator& = NLPQUEUE_ALLOCATOR_DEFAULT()
    ) /*TPriorityQueueOf(size_t, const TAllocator&)*/;
    //------------------------------------------------------------------------
	TPriorityQueueOf(const TPriorityQueue&);
    //------------------------------------------------------------------------
    TPriorityQueueOf(
    	const TContainerOf<T>&,
        const TAllocator& = NLPQUEUE_ALLOCATOR_DEFAULT()
    ) /*TPriorityQueueOf(const TContainerOf<T>&, const TAllocator&)*/;

    //------------------------------------------------------------------------
    void AssertValid() const;
    //------------------------------------------------------------------------
    NLOBJECT_DECLARE_CLONE1(TPriorityQueueOf, T);

	//------------------------------------------------------------------------
    TSize Size() const {return Buffer.Size();}
    //------------------------------------------------------------------------
    virtual TSize Capacity() const {return Buffer.Capacity();}
    
    //------------------------------------------------------------------------
	const T& Insert(const T&);
    //------------------------------------------------------------------------
	const T* CurrentPtr() const;
    //------------------------------------------------------------------------
    void Remove(TIndex nToRemove = 1);

    //------------------------------------------------------------------------
	inline TPriorityQueue& operator=(const TPriorityQueue& c_Source)
    {
    	Let(c_Source); return *this;
    } /*operator=(const TPriorityQueue& c_Source)*/
    //------------------------------------------------------------------------
	inline TPriorityQueue& operator=(const TContainer& c_Source)
    {
    	Let(c_Source); return *this;
    } /*operator=(const TContainer& c_Source)*/

	//------------------------------------------------------------------------
	inline TPriorityQueue& operator+=(const T& c_Data)
    {
    	Insert(c_Data); return *this;
    } /*operator+=(const T&)*/
	//------------------------------------------------------------------------
	inline TPriorityQueue& operator+=(const TContainer& c_Source)
    {
    	Insert(c_Source); return *this;
    } /*operator+=(const TContainer&)*/
	//------------------------------------------------------------------------
    inline TPriorityQueue& operator-=(const TConditionOf<T>& c_Condition)
    {
    	Delete(c_Condition); return *this;
    } /*operator-=(const TConditionOf<T>&)*/
    //------------------------------------------------------------------------
    inline TPriorityQueue& operator-=(const TComparatorOf<T>& c_Comparator)
    {
    	Delete(c_Comparator); return *this;
    } /*operator-=(const TComparatorOf<T>&)*/

protected:
	//------------------------------------------------------------------------
    TVectorOf<T> Buffer;
} /*TPriorityQueueOf*/;

// TPriorityQueueOf template implementation
//============================================================================
template <class T> TPriorityQueueOf<T>::TPriorityQueueOf(

    size_t nCapacity /*= NLPQUEUE_CAPACITY_DEFAULT*/,
    const TAllocator& c_Allocator /*= NLPQUEUE_ALLOCATOR_DEFAULT()*/

):
	Buffer(0, nCapacity, c_Allocator)
{} /*TPriorityQueueOf<T>(size_t, const TAllocator&)*/;

//----------------------------------------------------------------------------
template <class T> TPriorityQueueOf<T>::TPriorityQueueOf(

	const TPriorityQueue& c_Source

){
} /*TPriorityQueueOf<T>(const TPriorityQueue&)*/

//----------------------------------------------------------------------------
template <class T> TPriorityQueueOf<T>::TPriorityQueueOf(

    const TContainerOf<T>& c_Source,
    const TAllocator& c_Allocator /*= NLPQUEUE_ALLOCATOR_DEFAULT()*/

){
} /*TPriorityQueueOf(const TContainerOf<T>&, const TAllocator&)*/;

//----------------------------------------------------------------------------
template <class T> void TPriorityQueueOf<T>::AssertValid() const
{
	TParent::AssertValid();
    Buffer::AssertValid();
} /*TPriorityQueueOf<T>::AssertValid() const*/

//----------------------------------------------------------------------------
template <class T> NLOBJECT_IMPLEMENT_CLONE_DEFAULT1(TPriorityQueueOf, T)

//----------------------------------------------------------------------------
template <class T> const T& TPriorityQueueOf<T>::Insert(

	const T& c_Data

){
} /*TPriorityQueueOf<T>::Insert(const T&)*/

//----------------------------------------------------------------------------
template <class T> const T* TPriorityQueueOf<T>::CurrentPtr() const
{
	if (!Size())
    	throw TXBufferIsEmpty(THIS_CLASS);
    return Buffer.RawElement(0);
} /*TPriorityQueueOf<T>::CurrentPtr() const*/

//------------------------------------------------------------------------
template <class T> void TPriotiryQueueOf<T>::Remove(

	TIndex nToRemove /*= 1*/

){
} /*TPriotiryQueueOf<T>::Remove(TIndex)*/

//============================================================================
NATLIB_NAMESPACE_END

#endif /*!defined(__NLPQUEUE_H__)*/
