#ifndef __NATLIB_NLPRNTRE__
#define __NATLIB_NLPRNTRE__
/****************************************************************************/
/** nlprntre.h                                                             **/
/** ---------------------------------------------------------------------- **/
/** Printing of binary tree structure to stream                            **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** October 22, 1996 by Oleg Kagan                                         **/
/****************************************************************************/

// Standard includes
//============================================================================
#include <iostream.h>

// Natlib Basics includes
//============================================================================
#include <nlbintre.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

//============================================================================
#define NLPRNTRE_MAX_HEIGHT_DEFAULT	2048
#define NLPRNTRE_TAB_SIZE_DEFAULT	3

//----------------------------------------------------------------------------
#define NLPRNTRE_GRAPHICS_CHARS
#ifndef NLPRNTRE_GRAPHICS_CHARS
	#define NLPRNTRE_RIGHT_BRANCH_SZ "/--------------"
	#define NLPRNTRE_BRANCH_SZ       "|              "
	#define NLPRNTRE_LEFT_BRANCH_SZ  "\\--------------"
	#define NLPRNTRE_SPACE_SZ        "               "
#else
	#define NLPRNTRE_RIGHT_BRANCH_SZ  ""
	#define NLPRNTRE_BRANCH_SZ        "              "
	#define NLPRNTRE_LEFT_BRANCH_SZ   ""
	#define NLPRNTRE_SPACE_SZ         "               "
#endif /*NLPRNTRE_GRAPHICS_CHARS*/

// Dumps binary tree structure starting from specified specified node
//============================================================================
template <class T> class _NATLIB_TEMPLATE_DECLSPEC_ TBinaryTreePrintTreeOf : 
	private TBinaryTreeOf<T>::TConstFriend 
{
	//------------------------------------------------------------------------
	typedef TBinaryTreeNodeOf<T> TNode;

	//------------------------------------------------------------------------
	typedef enum {chdLeft, chdRight, chdParent} TChildDirection;

	//------------------------------------------------------------------------
	ostream& r_Stream;  // output stream
	TChildDirection* a_BranchMap;  // Branches map
	nl_int_t c_nTab;  // Tab size
	nl_int_t c_nHeight;  // Maximum printable tree height

public:
	//------------------------------------------------------------------------
	TBinaryTreePrintTreeOf(
		ostream& r_InitStream,
		nl_int_t = NLPRNTRE_TAB_SIZE_DEFAULT,
		nl_int_t = NLPRNTRE_MAX_HEIGHT_DEFAULT
	) /*TBinaryTreePrintTreeOf*/;

    //------------------------------------------------------------------------
    ~TBinaryTreePrintTreeOf() {if (a_BranchMap) delete[] a_BranchMap;}

	//------------------------------------------------------------------------
    void PrintSubtree(const TNode* pc_Subtree, nl_int_t nIndent);

	//------------------------------------------------------------------------
	void operator()(const TBinaryTreeOf<T>& c_Tree) 
	{
		PrintSubtree(static_cast<const TNode*>(GetRootPtrConst(c_Tree)), 0);
	} /*operator()(const TBinaryTreeOf<T>&)*/
	//------------------------------------------------------------------------
    void PrintNode(const TNode*, nl_int_t nIndent);
} /*TBinaryTreePrintTreeOf<T>*/;

// class TBinaryTreePrintTreeOf template implementation
//============================================================================
template <class T> TBinaryTreePrintTreeOf<T>::TBinaryTreePrintTreeOf(

    ostream& r_InitStream,
    nl_int_t c_nTabSize,
    nl_int_t c_nMaxHeight

):
    r_Stream(r_InitStream),
    c_nTab(
        (
			(c_nTabSize) && 
			(c_nTabSize <= nl_int_t(strlen(NLPRNTRE_SPACE_SZ)))
		) ? 
		c_nTabSize : NLPRNTRE_TAB_SIZE_DEFAULT
    ) /*c_nTab*/,
    c_nHeight((c_nMaxHeight) ? c_nMaxHeight : NLPRNTRE_MAX_HEIGHT_DEFAULT)
{
	// Allocate branches map
    //-----------------------
	NLEXCEPT_CLASS_NEW(TChildDirection[c_nHeight], a_BranchMap);

    // Init branches map
    //-------------------
    for (size_t i = 0; i < c_nHeight; i++) a_BranchMap[i] = chdParent;
} /*TBinaryTreePrintTreeOf<T>::TBinaryTreePrintTreeOf(...)*/;

//----------------------------------------------------------------------------
template <class T> void TBinaryTreePrintTreeOf<T>::PrintSubtree(

    const TBinaryTreeNodeOf<T>* pc_Subtree,
    nl_int_t nIndent

){
    if ((nIndent < c_nHeight) && (pc_Subtree)) {
        // Print right subtree
        //---------------------
        a_BranchMap[nIndent] = chdRight;
        PrintSubtree(
			static_cast<const TNode*>(pc_Subtree->pRight), nIndent + 1
		) /*PrintSubtree*/;

        // Print root of the subtree
        //---------------------------
        PrintNode(pc_Subtree, nIndent);

        // Print left subtree
        //--------------------
        a_BranchMap[nIndent] = chdLeft;
        PrintSubtree(
			static_cast<const TNode*>(pc_Subtree->pLeft), nIndent + 1
		) /*PrintSubtree*/;
    } /*if*/
} /*TBinaryTreePrintTreeOf<T>::PrintSubtree(const TNode*, nl_int_t)*/;

//----------------------------------------------------------------------------
template <class T> void TBinaryTreePrintTreeOf<T>::PrintNode(

	const TBinaryTreeNodeOf<T>* pc_Node,
	nl_int_t nIndent

){
    // Indentication
    //---------------
    if (nIndent) {
        for (nl_int_t i = 0; i < nIndent - 1; i++) {
        	TStdString sPrefix;
            if (
                (a_BranchMap[i] != chdParent) &&
                (a_BranchMap[i] != a_BranchMap[i + 1])
            ){
            	sPrefix = TStdString(NLPRNTRE_BRANCH_SZ);
            }
            else {
            	sPrefix = TStdString(NLPRNTRE_SPACE_SZ);
            } /*iff*/
            r_Stream << sPrefix.substr(0, c_nTab).c_str();
        } /*for (i)*/

		switch (a_BranchMap[nIndent - 1]) {
            case chdLeft:
                r_Stream << TStdString(
                	NLPRNTRE_LEFT_BRANCH_SZ
                )/*TStdString*/.substr(0, c_nTab).c_str();
                break;

            case chdRight:
                r_Stream << TStdString(
                	NLPRNTRE_RIGHT_BRANCH_SZ
                )/*TStdString*/.substr(0, c_nTab).c_str();
                break;

            case chdParent:
            default:
                break;
        } /*switch (Direction)*/
    } /*if*/

    // Print node
    //------------
	r_Stream << pc_Node->Data << endl;
} /*TBinaryTreePrintTreeOf<T>::PrintNode(...)*/;

// Print tree structure
//============================================================================
template <class T> inline void PrintBinaryTreeStructure(

	const TBinaryTreeOf<T>& c_Tree,
	ostream& r_Stream,
	nl_int_t nTab = 0,
	nl_int_t nMaxHeight = 0

){
	if (!nTab) nTab = NLPRNTRE_TAB_SIZE_DEFAULT;
    if (!nMaxHeight) nMaxHeight = NLPRNTRE_MAX_HEIGHT_DEFAULT;

	TBinaryTreePrintTreeOf<T> PrintTree(
    	r_Stream, nTab, nMaxHeight
    ) /*PrintTree*/;
	PrintTree(c_Tree);
} /*PrintBinaryTreeStructure<T>(const TBinaryTree&, ostream&,...)*/;

//============================================================================
NATLIB_NAMESPACE_END

//============================================================================
#define NLPRNTRE_DECLSPEC(T, Declspec) \
	NLBINTRE_DECLSPEC(T, Declspec); \
	NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TBinaryTreePrintSubtree), T, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/

#endif /*!defined(__NATLIB_NLPRNTRE__)*/
