#ifndef __NATLIB_NLQUEUE_H__
#define __NATLIB_NLQUEUE_H__
/****************************************************************************/
/** nqueue.h                                                               **/
/** ---------------------------------------------------------------------- **/
/** Contents declarations of the queue container.                          **/
/** This implementantion is based on TListOf template                      **/
/** Note: Copy contructor for template argument must be available.         **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** November 26, 1997 by Oleg Kagan                                        **/
/****************************************************************************/

//============================================================================
#include <nlbuffer.h>
#include <nllist.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

//============================================================================
#define NLQUEUE_ALLOCATOR_DEFAULT NLLIST_ALLOCATOR_DEFAULT

// TQueueOf<T>
//============================================================================
template <class T> class _NATLIB_TEMPLATE_DECLSPEC_ TQueueOf : 
	public TBufferOf<T>
{
public:
	//------------------------------------------------------------------------
	typedef TBufferOf<T> TParent;
	typedef TQueueOf<T> TQueue;
	typedef TListOf<T> TList;

public:
	//------------------------------------------------------------------------
    TQueueOf(const TAllocator& = NLQUEUE_ALLOCATOR_DEFAULT());
	//------------------------------------------------------------------------
    TQueueOf(const TQueue&);
    //------------------------------------------------------------------------
    TQueueOf(
		const TContainer&, const TAllocator& = NLQUEUE_ALLOCATOR_DEFAULT()
	) /*TQueueOf(const TContainer&, const TAllocator&)*/;

    //------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE1(TQueueOf, T);

    //------------------------------------------------------------------------
    void AssertValid() const;
	
    //------------------------------------------------------------------------
    const T& Insert(const T& c_Data) {return Inqueue(c_Data);}
	//------------------------------------------------------------------------
    const T* CurrentPtr() const;
    //------------------------------------------------------------------------
    void Remove(TIndex nToDequeue = 1) {Dequeue(nToDequeue);}

	//------------------------------------------------------------------------
	const T& Inqueue(const T& c_Data); 
    //------------------------------------------------------------------------
    void Dequeue(TIndex nToDequeue = 1);

	//------------------------------------------------------------------------
	TSize Size() const {return List.Size();}

	//------------------------------------------------------------------------
	void Insert(const TContainer& c_Container) {TParent::Insert(c_Container);}

	//------------------------------------------------------------------------
    TSize Delete(
		const TConditionOf<T>& c_Condition, TIndex nToDelete = indexAll
	){
		return List.Delete(c_Condition, nToDelete);
	} /*Delete(const TCondition&, TIndex = indexAll)*/
	//------------------------------------------------------------------------
    TSize Delete(
		const TComparatorOf<T>& c_Comparator, TIndex nToDelete = indexAll
	){
		return List.Delete(c_Comparator, nToDelete);
	} /*Delete(const TComparator&, TIndex = indexAll)*/

	//------------------------------------------------------------------------
	void Clear() {List.Clear();}

	//------------------------------------------------------------------------
	void Let(const TContainer& r_Source) {List.Let(r_Source);}
	//------------------------------------------------------------------------
	void Let(const TQueue& r_Source) {List.Let(r_Source);}

	//------------------------------------------------------------------------
	void ForEach(TOperationOn<T>& r_Operation) 
	{
		List.ForEach(r_Operation);
	} /*ForEach(TOperationOn<T>&)*/
    //------------------------------------------------------------------------
	void ForEach(TConstantOperationOn<T>& r_Operation) const
	{
		List.ForEach(r_Operation);
	} /*ForEach(TConstantOperationOn<T>&) const*/

	//------------------------------------------------------------------------
	inline TQueue& operator=(const TQueue& c_Source)
    {
    	TParent::Let(c_Source); return *this;
    } /*TDeque& operator=(const TDeque&)*/
	//------------------------------------------------------------------------
	inline TQueue& operator=(const TContainer& c_Source)
    {
    	Let(c_Source); return *this;
    } /*operator=(const TContainer&)*/

	//------------------------------------------------------------------------
	inline TQueue& operator+=(const T& c_Data) {Insert(c_Data); return *this;}
	//------------------------------------------------------------------------
	inline TQueue& operator+=(const TContainer& c_Source)
    {
		Insert(c_Source); return *this;
    } /*operator+=(const TContainer&)*/
	//------------------------------------------------------------------------
	inline TQueue& operator-=(const TConditionOf<T>& c_Condition)
    {
    	Delete(c_Condition); return *this;
    } /*operator-=(const TConditionOf<T>&)*/
    //------------------------------------------------------------------------
	inline TQueue& operator-=(const TComparatorOf<T>& c_Comparator)
    {
    	Delete(c_Comparator); return *this;
    } /*operator-=(const TComparatorOf<T>&)*/

protected:
	//------------------------------------------------------------------------
	TList List;
} /*TQueueOf<T>*/;

// TQueueOf<T> implementation
//============================================================================
template <class T> TQueueOf<T>::TQueueOf(

	const TAllocator& c_Allocator /*= NLQUEUE_ALLOCATOR_DEFAULT()*/
	
):
	List(c_Allocator)
{} /*TQueueOf<T>(const TAllocator&)*/

//----------------------------------------------------------------------------
template <class T> TQueueOf<T>::TQueueOf(

	const TQueue& c_Queue
	
):
	List(c_Queue)
{} /*TQueueOf<T>(const TQueue&)*/

//----------------------------------------------------------------------------
template <class T> TQueueOf<T>::TQueueOf(

	const TContainer& c_Source, 
	const TAllocator& c_Allocator /*= NLQUEUE_ALLOCATOR_DEFAULT()*/

): 
	List(c_Allocator)
{
	TParent::Insert(c_Source);
} /*TQueueOf<T>(const TContainer&, const TAllocator&)*/;

//----------------------------------------------------------------------------
template <class T> NLOBJECT_IMPLEMENT_CLONE_DEFAULT1(TQueueOf, T)

//----------------------------------------------------------------------------
template <class T> const T* TQueueOf<T>::CurrentPtr() const 
{
	if (List.IsEmpty()) return null;
	TListConstantCursorOf<T> First(List); First.ToFirst();
	return First.DereferencePtrConst();
} /*TQueueOf<T>::CurrentPtr() const*/

//----------------------------------------------------------------------------
template <class T> void TQueueOf<T>::AssertValid() const
{
	TParent::AssertValid(); 
	List.AssertValid();
} /*TQueueOf<T>::AssertValid() const*/

//----------------------------------------------------------------------------
template <class T> const T& TQueueOf<T>::Inqueue(

	const T& c_Data
	
){
	if (List.IsEmpty()) return List.Insert(c_Data);
	TListCursorOf<T> Cursor(List); Cursor.ToLast(); 
	return Cursor.InsertAfter(c_Data);
} /*TQueueOf<T>::Inqueue(const T&)*/

//----------------------------------------------------------------------------
template <class T> void TQueueOf<T>::Dequeue(

	TIndex nToDequeue /*= 1*/
	
){
	while (nToDequeue) {
		if (List.IsEmpty()) 
			throw TXBufferIsEmpty(THIS_CLASS);
		TListCursorOf<T> Cursor(List); Cursor.Delete();
	} /*while (nToDequeue)*/
} /*TQueueOf<T>::Dequeue(TIndex)*/

//============================================================================
NATLIB_NAMESPACE_END

//============================================================================
#define NLQUEUE_DECLSPEC(T, Declspec) \
	NLCONT_DECLSPEC(T, Declspec); \
	NLLIST_DECLSPEC(T, Declspec); \
    NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TQueueOf), T , Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/

#endif /*!defined(__NATLIB_NLQUEUE_H__)*/