#ifndef __NATLIB_NLSINGLE_H__
#define __NATLIB_NLSINGLE_H__
/****************************************************************************/
/** nlsingle.h                                                             **/
/** ---------------------------------------------------------------------- **/
/** Container that contains one and only one element                       **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** June 13, 1997 by Oleg Kagan                                            **/
/****************************************************************************/

//============================================================================
#include <nlcont.h>
#include <nlalloc.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TSingleOf template
//============================================================================
template <class T> class TSingleOf : public TContainerOf<T> {
public:
	//------------------------------------------------------------------------
    typedef TContainerOf<T> TParent;
    typedef TSingleOf<T> TSingle;

	//------------------------------------------------------------------------
	NLEXCEPT_DECLARE(
		TException, TContainerOf<T>::TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;

	//------------------------------------------------------------------------
	explicit TSingleOf(const T& c_Data) : DataEnvelope(c_Data) {}
    //------------------------------------------------------------------------
    TSingleOf(const TSingle& c_Single) :
    	DataEnvelope(c_Single.DataEnvelope)
    {} /*TSingleOf(const TSingle&)*/
    //------------------------------------------------------------------------
    ~TSingleOf() {DestructData();}

    //------------------------------------------------------------------------
    TSize Size() const {return 1;}

    //------------------------------------------------------------------------
	const T& Insert(const T& c_Data)
    {
    	ReconstructData(c_Data); return DataEnvelope.Data;
    } /*Insert(const T&)*/
    //------------------------------------------------------------------------
	void Insert(const TContainer& c_Source)
    {
    	TParent::Insert(c_Source);
    } /*Insert(const TContainerOf<T>&)*/

    //------------------------------------------------------------------------
	void ForEach(TOperation& r_Operation) {r_Operation(DataEnvelope.Data);}
    //------------------------------------------------------------------------
    void ForEach(TConstantOperation& r_Operation) const
    {
    	r_Operation(DataEnvelope.Data);
    } /*ForEach(TConstantOperation&) const*/

    //------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE1(TSingleOf, T);

    //------------------------------------------------------------------------
    inline TSingle& operator=(const TSingle& c_Source)
	{
    	Insert(c_Source.DataEnvelope.Data); return *this;
    } /*operator=(const TSingle&)*/
    //------------------------------------------------------------------------
    inline TSingle& operator=(const TContainer& c_Source)
    {
    	Let(c_Source); return *this;
    } /*operator=(const TContainer&)*/

    //------------------------------------------------------------------------
    inline TSingle& operator+=(const T& c_Data)
    {
    	Insert(c_Data); return *this;
    } /*operator+=(const T&)*/
    //------------------------------------------------------------------------
    inline TSingle& operator+=(const TContainer& c_Container)
    {
    	Insert(c_Container); return *this;
    } /*operator+=(const TContainer&)*/
    //------------------------------------------------------------------------
    inline TSingle& operator-=(const TConditionOf<T>& c_Condition)
    {
    	Delete(c_Condition); return *this;
    } /*operator-=(const TConditionOf<T>&)*/
    //------------------------------------------------------------------------
    inline TSingle& operator-=(const TComparatorOf<T>& c_Comparator)
    {
    	Delete(c_Comparator); return *this;
    } /*operator-=(const TComparatorOf<T>&)*/

protected:
	//------------------------------------------------------------------------
	TAllocationEnvelopeOf<T> DataEnvelope;

	//------------------------------------------------------------------------
    void ConstructData(const T& c_Data)
    {
    	new (&DataEnvelope) TAllocationEnvelopeOf<T>(c_Data);
    } /*ConstructData(const T& c_Data)*/
	//------------------------------------------------------------------------
    void DestructData()
    {
    	typedef TAllocationEnvelopeOf<T> _TAllocationEnvelope;
    	DataEnvelope.~_TAllocationEnvelope();
    } /*DestructData()*/

    //------------------------------------------------------------------------
    void ReconstructData(const T& c_Data)
    {
    	DestructData(); ConstructData(c_Data);
    } /*ReconstructData(const T&)*/

    //------------------------------------------------------------------------
    TSize Delete(const TConditionOf<T>&, TIndex = indexAll)
    {
    	throw TXOperationIsNotAllowed(THIS_CLASS); return TSize(0);
    } /*Delete(const TConditionOf<T>&, TIndex = indexAll)*/
    //------------------------------------------------------------------------
    TSize Delete(const TComparatorOf<T>&, TIndex = indexAll)
    {
    	throw TXOperationIsNotAllowed(THIS_CLASS); return TSize(0);
    } /*Delete(const TComparatorOf<T>&, TIndex = indexAll)*/

    //------------------------------------------------------------------------
	void Clear() {throw TXOperationIsNotAllowed(THIS_CLASS);}
} /*TSingleOf<T>*/;

// TSingleOf template implementation
//============================================================================
template <class T> NLOBJECT_IMPLEMENT_CLONE_DEFAULT1(TSingleOf, T)

//============================================================================
NATLIB_NAMESPACE_END

//============================================================================
#define NLSINGLE_DECLSPEC(T, Declspec) \
	NLCONT_DECLSPEC(T, Declspec); \
    NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TSingleOf), T, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/

#endif /*!defined(__NATLIB_NLSINGLE_H__)*/
