#ifndef __NATLIB_NLSRCLOC_H__
#define __NATLIB_NLSRCLOC_H__
/****************************************************************************/
/** nlsrcloc.h                                                             **/
/** -----------------------------------------------------------------------**/
/** Provides basic ability to represent source locations                   **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** June 14, 1996 by Oleg Kagan                                            **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

//============================================================================
#define THIS_PLACE_CLASS(Class) \
	TSourceLocation(__FILE__, __LINE__, typeid(Class).name())

//----------------------------------------------------------------------------
#define THIS_CLASS THIS_PLACE_CLASS(*this)
#define THIS_PLACE TSourceLocation(__FILE__, __LINE__)

// Declare string inside of NATLIB namespace
//============================================================================
typedef TNatlibStdString TStdString;

// Wide string definition
//============================================================================
#if !defined(_NATLIB_NO_WIDE_STRINGS_)
typedef TNatlibWideString TWideString;
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

// TSourceLocation
// Represens information about exact location in the source
//============================================================================
typedef class _NATLIB_DECLSPEC_ TSourceLocation {
public:
	//------------------------------------------------------------------------
	static const char* const c_szUnknownLocation;
	static const char* const c_szStartDelimiter;
	static const char* const c_szEndDelimiter;
	static const char* const c_szSeparator;
	static const unsigned    c_nLineCharSize;
	static const char* const c_szNoSource;
	static const unsigned    c_nNoLine;
	static const char* const c_szNoClass;

	//------------------------------------------------------------------------
    static const TSourceLocation c_None;

	//------------------------------------------------------------------------
	TStdString sSource;
	nl_uint_t nLine;
	TStdString sClass;

	//------------------------------------------------------------------------
	TSourceLocation(
		const TStdString& c_sSource = c_szNoSource,
		const unsigned c_nLine = c_nNoLine,
		const TStdString& c_sClass = c_szNoClass
	) /*TSourceLocation(const TStdString&, const unsigned, ...)*/;

	// Copy constructor
	//------------------------------------------------------------------------
	TSourceLocation(const TSourceLocation&);

    //------------------------------------------------------------------------
    virtual ~TSourceLocation() {}

	//------------------------------------------------------------------------
    static const TSourceLocation& None() {return c_None;}
     
	//------------------------------------------------------------------------
	TStdString Where() const;

	//------------------------------------------------------------------------
	TSourceLocation& operator=(const TSourceLocation&);

	//------------------------------------------------------------------------
	virtual operator TStdString() const;
} basic_TSourceLocation;

//============================================================================
NATLIB_NAMESPACE_END

#endif /*!defined(__NATLIB_NLSRCLOC_H__)*/
