#ifndef __NATLIB_NLSTACK_H__
#define __NATLIB_NLSTACK_H__
/****************************************************************************/
/** nstack.h                                                               **/
/** ---------------------------------------------------------------------- **/
/** Contents declarations of the stack container.                          **/
/** This implementantion is based on TListOf template                      **/
/** Note: Copy contructor for template argument must be available.         **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** November 26, 1997 by Oleg Kagan                                        **/
/****************************************************************************/

//============================================================================
#include <nlbuffer.h>
#include <nllist.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

//============================================================================
#define NLSTACK_ALLOCATOR_DEFAULT NLLIST_ALLOCATOR_DEFAULT

// TStackOf template
//============================================================================
template <class T> class _NATLIB_TEMPLATE_DECLSPEC_ TStackOf :
	public TBufferOf<T>
{
public:
	//------------------------------------------------------------------------
	typedef TBufferOf<T> TParent;
	typedef TStackOf<T> TStack;
    typedef TListOf<T> TList;

	//------------------------------------------------------------------------
	NLEXCEPT_DECLARE(
		TException, TBufferOf<T>::TException, _NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;

	//------------------------------------------------------------------------
    TStackOf(const TAllocator& c_Allocator = NLSTACK_ALLOCATOR_DEFAULT());
	//------------------------------------------------------------------------
    TStackOf(const TStack&);
    //------------------------------------------------------------------------
    TStackOf(
		const TContainer&, const TAllocator& = NLSTACK_ALLOCATOR_DEFAULT()
	) /*TStackOf(const TContainer&, const TAllocator&)*/;

    //------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE1(TStackOf, T);

    //------------------------------------------------------------------------
    void AssertValid() const;

    //------------------------------------------------------------------------
    const T& Insert(const T& c_Data) {return Push(c_Data);}
	//------------------------------------------------------------------------
    const T* CurrentPtr() const;
    //------------------------------------------------------------------------
    void Remove(TIndex nToPop = 1) {Pop(nToPop);}

	//------------------------------------------------------------------------
	const T& Push(const T&);
    //------------------------------------------------------------------------
    void Pop(TIndex nToPop = 1);

	//------------------------------------------------------------------------
	TSize Size() const {return List.Size();}

	//------------------------------------------------------------------------
	void Insert(const TContainer& c_Container) {TParent::Insert(c_Container);}

	//------------------------------------------------------------------------
    TSize Delete(
		const TConditionOf<T>& c_Condition,
        TIndex nToDelete = TParent::indexAll
	){
		return List.Delete(c_Condition, TList::TIndex(nToDelete));
	} /*Delete(const TCondition&, TIndex = indexAll)*/
	//------------------------------------------------------------------------
    TSize Delete(
		const TComparatorOf<T>& c_Comparator,
        TIndex nToDelete = TParent::indexAll
	){
		return List.Delete(c_Comparator, nToDelete);
	} /*Delete(const TComparator&, TIndex = indexAll)*/

	//------------------------------------------------------------------------
	void Clear() {List.Clear();}

	//------------------------------------------------------------------------
	void Let(const TContainer& c_Source) {List.Let(c_Source);}
	//------------------------------------------------------------------------
	void Let(const TStack& c_Source) {List.Let(c_Source.List);}

	//------------------------------------------------------------------------
	void ForEach(TOperationOn<T>& r_Operation)
    {
    	List.ForEach(r_Operation);
	} /*ForEach(TOperationOn<T>&)*/
    //------------------------------------------------------------------------
	void ForEach(TConstantOperationOn<T>& r_Operation) const
	{
		List.ForEach(r_Operation);
	} /*ForEach(TConstantOperationOn<T>&) const*/

	//------------------------------------------------------------------------
	inline TStack& operator=(const TStack& c_Source)
    {
    	Let(c_Source); return *this;
    } /*operator=(const TStack&)*/
	//------------------------------------------------------------------------
	inline TStack& operator=(const TContainer& c_Source)
	{
		Let(c_Source); return *this;
    } /*operator=(const TContainer&)*/

	//------------------------------------------------------------------------
	inline TStack& operator+=(const T& c_Data) {Insert(c_Data); return *this;}
	//------------------------------------------------------------------------
	inline TStack& operator+=(const TContainer& c_Source)
    {
    	Insert(c_Source); return *this;
    } /*operator+=(const TContainer&)*/
	//------------------------------------------------------------------------
	inline TStack& operator-=(const TConditionOf<T>& c_Condition)
    {
    	Delete(c_Condition); return *this;
    } /*operator-=(const TConditionOf<T>&)*/
    //------------------------------------------------------------------------
	inline TStack& operator-=(const TComparatorOf<T>& c_Comparator)
    {
    	Delete(c_Comparator); return *this;
    } /*operator-=(const TComparatorOf<T>&)*/

protected:
	//------------------------------------------------------------------------
    TList List;
} /*TStackOf<T>*/;

// TStackOf template  implementation
//============================================================================
template <class T> TStackOf<T>::TStackOf(

	const TAllocator& c_Allocator /*= NLSTACK_ALLOCATOR_DEFAULT()*/

):
	List(c_Allocator)
{} /*TStackOf<T>(const TAllocator&)*/

//----------------------------------------------------------------------------
template <class T> TStackOf<T>::TStackOf(

	const TStack& c_Stack

):
	List(c_Stack.List)
{} /*TStackOf<T>(const TStack&)*/

//----------------------------------------------------------------------------
template <class T> TStackOf<T>::TStackOf(

	const TContainer& c_Source, 
	const TAllocator& c_Allocator /*= NLSTACK_ALLOCATOR_DEFAULT()*/

): 
	List(c_Allocator)
{
	const TStack* pc_SourceStack = dynamic_cast<const TStack*>(&c_Source);
	if (pc_SourceStack) List.Insert(pc_SourceStack->List);
	else TParent::Insert(c_Source);
} /*TStackOf<T>(const TContainer&, const TAllocator&)*/;

//----------------------------------------------------------------------------
template <class T> NLOBJECT_IMPLEMENT_CLONE_DEFAULT1(TStackOf, T)

//----------------------------------------------------------------------------
template <class T> const T* TStackOf<T>::CurrentPtr() const
{
	if (List.IsEmpty()) return null;
    TListConstantCursorOf<T> HeadCur(List); HeadCur.ToFirst();
	return &*HeadCur;
} /*TStackOf<T>::CurrentPtr() const*/

//----------------------------------------------------------------------------
template <class T> void TStackOf<T>::AssertValid() const
{
	TParent::AssertValid();
	List.AssertValid();
} /*TStackOf<T>::AssertValid() const*/

//----------------------------------------------------------------------------
template <class T> const T& TStackOf<T>::Push(

	const T& c_Data

){
 	TListCursorOf<T> HeadCur(List); HeadCur.ToFirst();
    return HeadCur.InsertBefore(c_Data);
} /*Push(const T&)*/

//----------------------------------------------------------------------------
template <class T> void TStackOf<T>::Pop(

	TIndex nToPop /*= 1*/
	
){
	TListCursorOf<T> HeadCur(List);
	while (nToPop) {
		if (List.IsEmpty())
			throw TXBufferIsEmpty(THIS_CLASS);
        HeadCur.ToFirst(); HeadCur.Delete();
		--nToPop;
	} /*while (nToPop)*/
} /*TStackOf<T>::Pop(TIndex)*/

//============================================================================
NATLIB_NAMESPACE_END

//============================================================================
#define NLSTACK_DECLSPEC(T, Declspec) \
	NLCONT_DECLSPEC(T, Declspec); \
	NLLIST_DECLSPEC(T, Declspec); \
    NATLIB_EXPAND_TEMPLATE1( \
		NATLIB_NAMESPACE_MEMBER(TStackOf), T , Declspec \
	) /*NATLIB_EXPAND_TEMPLATE1*/

#endif /*!defined(__NATLIB_NLSTACK_H__)*/
