#ifndef __NATLIB_NLSTORAG_H__
#define __NATLIB_NLSTORAG_H__
/****************************************************************************/
/** nlstorag.h                                                             **/
/** ---------------------------------------------------------------------- **/
/** Natlib abstract Storage stores data indexed by specified keys          **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** January 25, 1997 by Oleg Kagan                                         **/
/****************************************************************************/

//============================================================================
#include <nlcont.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TStorageOperationOn template
//============================================================================
template <class TData, class TKey>
class _NATLIB_TEMPLATE_DECLSPEC_ TStorageOperationOn : public TOperation {
public:
    //------------------------------------------------------------------------
    typedef TOperation TParent;

	//------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE2(TStorageOperationOn, TData, TKey) = 0;

    //------------------------------------------------------------------------
    virtual void operator()(TData&, const TKey&) = 0;
} /*TStorageOperationOn<TData, TKey>*/;

// TStorageConstantOperationOn template
//============================================================================
template <class TData, class TKey>
class _NATLIB_TEMPLATE_DECLSPEC_ TStorageConstantOperationOn :
    public TStorageOperationOn<TData, TKey>
{
public:
    //------------------------------------------------------------------------
    typedef TStorageOperationOn<TData, TKey> TParent;

#ifndef _NATLIB_NO_EXTENDED_OVERLOADING_
	//------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE2(TStorageConstantOperationOn, TData, TKey) = 0;
#endif /*_NATLIB_NO_EXTENDED_OVERLOADING_*/

    //------------------------------------------------------------------------
    virtual void operator()(const TData&, const TKey&) = 0;

protected:
    //------------------------------------------------------------------------
    void operator()(TData& r_Data, const TKey& c_Key)
    {
        operator()((const TData&)r_Data, c_Key);
    } /*operator()(TData&, const TKey&)*/
} /*TStorageConstantOperationOn<TData, TKey>*/;

// TStorageOf template
//============================================================================
template <class TData, class TKey>
class _NATLIB_TEMPLATE_DECLSPEC_ TStorageOf : public TContainerOf<TData> {
public:
	//------------------------------------------------------------------------
    typedef TContainerOf<TData> TParent;
    typedef TStorageOf<TData, TKey> TStorage;

    //------------------------------------------------------------------------
    typedef TComparatorOf<TKey> TKeyComparator;
    typedef TEtalonComparatorOf<TKey> TSortingComparator;

    //------------------------------------------------------------------------
    typedef TConditionOf<TKey> TKeyCondition;
    typedef TConditionOf<TData> TDataCondition;

    //------------------------------------------------------------------------
    typedef TStorageOperationOn<TData, TKey> TStorageOperation;
    typedef TStorageConstantOperationOn<TData, TKey>
    	TStorageConstantOperation;

	//------------------------------------------------------------------------
	NLEXCEPT_DECLARE(
		TException, TContainerOf<TData>::TException,
		_NATLIB_TEMPLATE_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;

    //------------------------------------------------------------------------
#ifndef _NATLIB_NO_EXTENDED_OVERLOADING_
	NLOBJECT_DECLARE_CLONE2(TStorageOf, TData, TKey) = 0;
#endif /*_NATLIB_NO_EXTENDED_OVERLOADING_*/

	//------------------------------------------------------------------------
	virtual const TData* SearchByKeyConst(const TKey&) const = 0;
	//------------------------------------------------------------------------
	const TData& FindByKeyConst(const TKey&) const;

    //------------------------------------------------------------------------
    virtual TData* SearchByKey(const TKey&) = 0;
    //------------------------------------------------------------------------
    TData& FindByKey(const TKey&);

	// next 5 functions throw TXNotFound if appropriate key can not be found
    //------------------------------------------------------------------------
    virtual TKey FindKey(const TConditionOf<TData>&, TIndex = indexAny) const;
	//------------------------------------------------------------------------
    virtual TKey FirstKey() const = 0;
    //------------------------------------------------------------------------
    virtual TKey LastKey() const = 0;
    //------------------------------------------------------------------------
    virtual TKey NextKey(const TKey&) const = 0;
    //------------------------------------------------------------------------
    virtual TKey PreviousKey(const TKey&) const = 0;

	//------------------------------------------------------------------------
    virtual TData& Assign(const TData&, const TKey&) = 0;
	// Clears data assigned to specified key (e.g. delete)
    //------------------------------------------------------------------------
    virtual bool Unassign(const TKey&) = 0;

    // Performs operation in increasing order
    // (according to the sorting comparison)
    //------------------------------------------------------------------------
    virtual void Inorder(TStorageOperation&) = 0;
    //------------------------------------------------------------------------
    virtual void Inorder(TStorageConstantOperation&) const = 0;

	// Performs specified operation in decreasing order
    //------------------------------------------------------------------------
    virtual void Postorder(TStorageOperation&) = 0;
    //------------------------------------------------------------------------
    virtual void Postorder(TStorageConstantOperation&) const = 0;

    //------------------------------------------------------------------------
    virtual void ForEach(TStorageOperation& r_Operation)
    {
    	Inorder(r_Operation);
    } /*ForEach(TStorageOperation&)*/
    //------------------------------------------------------------------------
    virtual void ForEach(TStorageConstantOperation& r_Operation) const
    {
    	Inorder(r_Operation);
    } /*ForEach(TConstantStorageOperation&) const*/

    //------------------------------------------------------------------------
    void ForEach(TOperationOn<TData>&);
    //------------------------------------------------------------------------
    void ForEach(TConstantOperationOn<TData>&) const;

	//------------------------------------------------------------------------
    virtual void Insert(const TStorageOf<TData, TKey>& c_Source);
	//------------------------------------------------------------------------
    void Insert(const TContainerOf<TData>& c_Source)
    {
    	TParent::Insert(c_Source);
    } /*Insert(const TContainerOf<TData>&)*/
    //------------------------------------------------------------------------
    virtual void Let(const TStorageOf<TData, TKey>& c_Source)
    {
    	Clear(); Insert(c_Source);
    } /*Let(const TStorage<TData, TKey>&)*/
	//------------------------------------------------------------------------
    void Let(const TContainerOf<TData>& c_Source)
    {
    	TParent::Let(c_Source);
    } /*Let(const TContainerOf<TData>&)*/

    //------------------------------------------------------------------------
    inline TStorageOf<TData, TKey>& operator=(
    	const TStorageOf<TData, TKey>& c_Source
    ){
    	Let(c_Source); return *this;
    } /*operator=(const TStorageOf<TData, TKey>&)*/

    //------------------------------------------------------------------------
    inline TStorageOf<TData, TKey>& operator=(
    	const TContainerOf<TData>& c_Source
    ){
    	Let(c_Source); return *this;
    } /*operator=(const TContainerOf<TData>&)*/

    //------------------------------------------------------------------------
    const TData& operator[](const TKey& c_Key) const
    {
    	return FindByKeyConst(c_Key);
    } /*operator[](const TKey&) const*/
    //------------------------------------------------------------------------
    TData& operator[](const TKey& c_Key) {return FindByKey(c_Key);}
} /*TStorageOf<TData, TKey>*/;

// TStorageApplyDataOperationOn template
//============================================================================
template <class TData, class TKey>
class _NATLIB_TEMPLATE_DECLSPEC_ TStorageApplyDataOperationOn :
	public TStorageOperationOn<TData, TKey>
{
public:
	//------------------------------------------------------------------------
	TStorageApplyDataOperationOn(TOperationOn<TData>& c_DataOperation) :
    	pDataOperation(&c_DataOperation)
    {} /*TStorageApplyDataOperationOn(TOperationOn<TData>&)*/

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED2(
		TStorageApplyDataOperationOn, TData, TKey
	) /*NLOBJECT_CLONE_IS_NOT_ALLOWED2*/

    //------------------------------------------------------------------------
    void operator()(TData& r_Data, const TKey&) {(*pDataOperation)(r_Data);}

protected:
	//------------------------------------------------------------------------
    TOperationOn<TData>* pDataOperation;
} /*TStorageApplyDataOperationOn<TData, TKey>*/;

// TStorageApplyDataConstantOperationOn template
//============================================================================
template <class TData, class TKey>
class _NATLIB_TEMPLATE_DECLSPEC_ TStorageApplyDataConstantOperationOn :
	public TStorageConstantOperationOn<TData, TKey>
{
public:
	//------------------------------------------------------------------------
	TStorageApplyDataConstantOperationOn(
    	TConstantOperationOn<TData>& c_DataOperation
    ) :
    	pDataOperation(&c_DataOperation)
    {} /*TStorageApplyDataConstantOperationOn(TConstantOperationOn<TData>&)*/

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED2(
		TStorageApplyDataConstantOperationOn, TData, TKey
	) /*NLOBJECT_CLONE_IS_NOT_ALLOWED2*/

    //------------------------------------------------------------------------
    void operator()(const TData& c_Data, const TKey&)
    {
    	(*pDataOperation)(c_Data);
    } /*operator()(const TData&, const TKey&)*/

protected:
	//------------------------------------------------------------------------
    TConstantOperationOn<TData>* pDataOperation;

private:
    //------------------------------------------------------------------------
    void operator()(TData& r_Data, const TKey& c_Key)
    {
		operator()((const TData&)r_Data, c_Key);
    } /*operator()(TData&, const TKey&)*/
} /*TStorageApplyDataConstantOperationOn<TData, TKey>*/;

// TStorageSearchKeyOf template
//============================================================================
template <class TData, class TKey>
class _NATLIB_TEMPLATE_DECLSPEC_ TStorageSearchKeyOf :
	public TStorageConstantOperationOn<TData, TKey>
{
public:
	//------------------------------------------------------------------------
    typedef TStorageConstantOperationOn<TData, TKey> TParent;

    //------------------------------------------------------------------------
    typedef TStorageOf<TData, TKey>::TIndex TIndex;

    //------------------------------------------------------------------------
    TStorageSearchKeyOf(
		const TConditionOf<TData>& c_Condition, TIndex nSearchIndex
    ):
		pc_Condition(&c_Condition),
        nIndex(
        	(nSearchIndex == TStorageOf<TData, TKey>::indexAny) ?
            	TStorageOf<TData, TKey>::indexFirst : nSearchIndex
        ) /*nIndex*/,
        pKeyFound(null)
    {
    	if (!nIndex) // something strange (invalid parameters)
        	throw TException(THIS_CLASS);
    } /*TStorageSearchKeyOf(const TConditionOf<TData>&, TIndex)*/

	//------------------------------------------------------------------------
    ~TStorageSearchKeyOf() {delete pKeyFound; pKeyFound = null;}

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED2(TStorageSearchKeyOf, TData, TKey);

	//------------------------------------------------------------------------
    const TKey* KeyFoundPtr() const {return pKeyFound;}

    //------------------------------------------------------------------------
    void operator()(const TData&, const TKey&);

protected:
	//------------------------------------------------------------------------
    const TConditionOf<TData>* pc_Condition;
	TStorageOf<TData, TKey>::TIndex nIndex;
    TKey* pKeyFound;

private:
	//------------------------------------------------------------------------
    void operator()(TData& r_Data, const TKey& c_Key)
    {
    	operator()((const TData&)r_Data, c_Key);
    } /*operator()(TData&, const TKey&)*/
} /*TStorageSearchKeyOf<TData, TKey>*/;

// TStorageInsertTo template
//============================================================================
template <class TData, class TKey>
class _NATLIB_TEMPLATE_DECLSPEC_ TStorageInsertTo :
    public TStorageConstantOperationOn<TData, TKey>
{
public:
    //------------------------------------------------------------------------
    typedef TStorageConstantOperationOn<TData, TKey> TParent;
    //------------------------------------------------------------------------
    TStorageInsertTo(TStorageOf<TData, TKey>&);

    //------------------------------------------------------------------------
    NLOBJECT_DECLARE_CLONE2(TStorageInsertTo, TData, TKey);

    //------------------------------------------------------------------------
    void operator()(const TData& c_Data, const TKey& c_Key)
    {
        pTarget->Assign(c_Data, c_Key);
    } /*operator()(const TData&, const TKey&)*/

protected:
    //------------------------------------------------------------------------
    TStorageOf<TData, TKey>* pTarget;

    //--------------------------------------------------------------------
    void operator()(TData& r_Data, const TKey& c_Key)
    {
        TParent::operator()(r_Data, c_Key);
    } /*operator()(const TData&, const TKey&)*/
} /*TStorageInsertTo<TData>*/;

// TStorageSearchKeyOf template implementation
//============================================================================
template <class TData, class TKey>
void TStorageSearchKeyOf<TData, TKey>::operator()(

	const TData& c_Data,
    const TKey& c_Key

){
    if ((*pc_Condition)(c_Data) && !--nIndex) {
        if (pKeyFound) // Abort doesn't work
			throw TException(THIS_CLASS);
        NLEXCEPT_CLASS_NEW(TKey(c_Key), pKeyFound);
        Abort();
    } /*if*/
} /*operator()(const TData&, const TKey&)*/

// TStorageOf template implementation
//============================================================================
template <class TData, class TKey>
const TData& TStorageOf<TData, TKey>::FindByKeyConst(

	const TKey& c_Key

) const {
	const TData* pc_Found = SearchByKeyConst(c_Key);
    if (!pc_Found) 
		throw TXNotFound(THIS_CLASS);
    return *pc_Found;
} /*TStorageOf<TData, TKey>::FindByKeyConst(const TKey&) const*/

//----------------------------------------------------------------------------
template <class TData, class TKey> TData& TStorageOf<TData, TKey>::FindByKey(

	const TKey& c_Key

){
	TData* pFound = SearchByKey(c_Key);
    if (!pFound) 
		throw TXNotFound(THIS_CLASS);
    return *pFound;
} /*TStorageOf<TData, TKey>::FindByKey(const TKey&)*/

//----------------------------------------------------------------------------
template <class TData, class TKey> TKey TStorageOf<TData, TKey>::FindKey(

	const TConditionOf<TData>& c_Condition,
    TIndex nIndex /*= indexAny*/

) const {
	TStorageSearchKeyOf<TData, TKey> SearchKey(c_Condition, nIndex);
    ForEach(SearchKey);
    if (!SearchKey.KeyFoundPtr()) 
		throw TXNotFound(THIS_CLASS);
    return *(SearchKey.KeyFoundPtr());
} /*TStorageOf<TData, TKey>::FindKey(const TConditionOf<TData>&,...) const*/

//----------------------------------------------------------------------------
template <class TData, class TKey> void TStorageOf<TData, TKey>::ForEach(

	TOperationOn<TData>& r_Operation

){
	TStorageApplyDataOperationOn<TData, TKey> ApplyDataOperation(r_Operation);
	ForEach(ApplyDataOperation);
} /*TStorageOf<TData, TKey>::ForEach(TParent::TOperation&)*/

//----------------------------------------------------------------------------
template <class TData, class TKey> void TStorageOf<TData, TKey>::ForEach(

	TConstantOperationOn<TData>& r_Operation

) const {
	TStorageApplyDataConstantOperationOn<TData, TKey> ApplyDataOperation(
    	r_Operation
    ) /*ApplyDataOperation*/;
	ForEach(ApplyDataOperation);
} /*TStorageOf<TData, TKey>::ForEach(TParent::TConstantOperation&) const*/

//----------------------------------------------------------------------------
template <class TData, class TKey> void TStorageOf<TData, TKey>::Insert(

	const TStorageOf<TData, TKey>& c_Source

){
	TStorageInsertTo<TData, TKey> InsertToThis(*this);
    c_Source.ForEach(InsertToThis);
} /*TStorageOf<TData, TKey>::Insert(const TStorageOf<TData, TKey>&)*/

// TStorageInsertTo template implementation
//============================================================================
template <class TData, class TKey>
TStorageInsertTo<TData, TKey>::TStorageInsertTo(

	TStorageOf<TData, TKey>& r_Target

):
	pTarget(&r_Target)
{} /*TStorageInsertTo<TData, TKey>(TStorageOf<TData, TKey>&)*/

//----------------------------------------------------------------------------
template <class TData, class TKey>
NLOBJECT_IMPLEMENT_CLONE_DEFAULT2(TStorageInsertTo, TData, TKey)

//============================================================================
NATLIB_NAMESPACE_END

//============================================================================
#define NLSTORAG_DECLSPEC(TData, TKey, Declspec) \
	NLCONT_DECLSPEC(TData, Declspec); \
	NATLIB_EXPAND_TEMPLATE2( \
		NATLIB_NAMESPACE_MEMBER(TStorageOperationOn), \
		TData, TKey, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE2*/; \
	NATLIB_EXPAND_TEMPLATE2( \
		NATLIB_NAMESPACE_MEMBER(TStorageConstantOperationOn), \
		TData, TKey, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE2*/; \
	NATLIB_EXPAND_TEMPLATE2( \
    	NATLIB_NAMESPACE_MEMBER(TStorageApplyDataOperationOn), \
		TData, TKey, Declspec \
    ) /*NATLIB_EXPAND_TEMPLATE2*/; \
	NATLIB_EXPAND_TEMPLATE2( \
    	NATLIB_NAMESPACE_MEMBER(TStorageApplyDataConstantOperationOn), \
		TData, TKey, Declspec \
    ) /*NATLIB_EXPAND_TEMPLATE2*/; \
	NATLIB_EXPAND_TEMPLATE2( \
		NATLIB_NAMESPACE_MEMBER(TStorageSearchKeyOf), \
		TData, TKey, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE2*/; \
	NATLIB_EXPAND_TEMPLATE2( \
		NATLIB_NAMESPACE_MEMBER(TStorageOf), TData, TKey, Declspec \
	) /*NATLIB_EXPAND_TEMPLATE2*/

#endif /*!defined(__NATLIB_NLSTORAG_H__)*/
