#ifndef __NATLIB_NLSTRING_H__
#define __NATLIB_NLSTRING_H__
/****************************************************************************/
/** nlstring.h                                                             **/
/** -----------------------------------------------------------------------**/
/** Some utilities for string management                                   **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** August 20, 1997 by Oleg Kagan                                          **/
/****************************************************************************/

//============================================================================
#include <nlcompar.h>

// String tools
//============================================================================
NATLIB_NAMESPACE_BEGIN

//============================================================================
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, TStdString) Format(const char*, ...);
//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, TStdString) Format(
	size_t, const char*, ...
) /*Format(size_t, const char*, ...)*/;

// Sets the size of the buffer used to perform ssprintf operations
// returns previous size of the buffer
//============================================================================
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_,size_t) SetFormatBufferSize(size_t);
// Returns size of the buffer used for ssprintf operations
//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, size_t) SetFormatGetBufferSize();

// TStdStringCaseSensitiveComparator
//============================================================================
class TStdStringCaseSensitiveComparator :
	public TPointerSavingEtalonComparatorOf<TStdString>
{
public:
	//------------------------------------------------------------------------
    typedef TPointerSavingEtalonComparatorOf<TStdString> TParent;

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_RESULT(TStdStringCaseSensitiveComparator) Clone() const
	{
		TStdStringCaseSensitiveComparator* pCopy;
		NLEXCEPT_CLASS_NEW(TStdStringCaseSensitiveComparator, pCopy);
		return pCopy;
	} /*Clone() const*/;

    //------------------------------------------------------------------------
    TComparator::TResult operator()(const TStdString& c_sToCompare) const
	{
		const TStdString& c_sEtalon = Etalon();
//		if (c_sEtalon.data() == c_sToCompare.data()) return compEqual;
		size_t nEtalonLength = c_sEtalon.length();
		size_t nLength = c_sToCompare.length();
		// Borland's ClassLib string class doesn't support some standard 
		// features of STL string like data() method
		//---------------------------------------------------------------
#if defined(__BCPLUSPLUS__) && !defined(_NATLIB_USE_STL_STRING_CLASS_)
		int iResult = strncmp(
			c_sEtalon.c_str(), c_sToCompare.c_str(), 
				min(nEtalonLength, nLength)
		) /*strncmp*/;
#else
		int iResult = strncmp(
			c_sEtalon.data(), c_sToCompare.data(), min(nEtalonLength, nLength)
		) /*strncmp*/;
#endif /*defined(__BCPLUSPLUS__) && !defined(_NATLIB_USE_STL_STRING_CLASS_)*/
		if (iResult < 0) return compLess;
		if (iResult > 0) return compGreater;
		if (nEtalonLength < nLength) return compLess;
		if (nEtalonLength > nLength) return compGreater;
		return compEqual;
	} /*operator()(const TStdString&) const*/
} /*TStdStringCaseSensitiveComparator*/;

// TStdStringCaseInsensitiveComparator
//============================================================================
class TStdStringCaseInsensitiveComparator :
	public TPointerSavingEtalonComparatorOf<TStdString>
{
public:
	//------------------------------------------------------------------------
    typedef TPointerSavingEtalonComparatorOf<TStdString> TParent;

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_RESULT(TStdStringCaseInsensitiveComparator) Clone() const
	{
		TStdStringCaseInsensitiveComparator* pCopy;
		NLEXCEPT_CLASS_NEW(TStdStringCaseInsensitiveComparator, pCopy);
		return pCopy;
	} /*Clone() const*/;

    //------------------------------------------------------------------------
    TComparator::TResult operator()(const TStdString& c_sToCompare) const
	{
	   	const TStdString& c_sEtalon = Etalon();
//		if (c_sEtalon.data() == c_sToCompare.data()) return compEqual;
		size_t nEtalonLength = c_sEtalon.length();
		size_t nLength = c_sToCompare.length();
		// Borland's ClassLib string class doesn't support some standard 
		// features of STL string like data() method
		//---------------------------------------------------------------
#if defined(__BCPLUSPLUS__) && !defined(_NATLIB_USE_STL_STRING_CLASS_)
		int iResult = strnicmp(
			c_sEtalon.c_str(), c_sToCompare.c_str(), 
			min(nEtalonLength, nLength)
		) /*strnicmp*/;
#else
		int iResult = strnicmp(
			c_sEtalon.data(), c_sToCompare.data(), min(nEtalonLength, nLength)
		) /*strnicmp*/;
#endif /*defined(__BCPLUSPLUS__) && !defined(_NATLIB_USE_STL_STRING_CLASS_)*/
		if (iResult < 0) return compLess;
		if (iResult > 0) return compGreater;
		if (nEtalonLength < nLength) return compLess;
		if (nEtalonLength > nLength) return compGreater;
		return compEqual;
	} /*operator()(const TStdString&) const*/
} /*TStdStringCaseInsensitiveComparator*/;

//============================================================================
extern NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, const char) const c_szNull[];
extern NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, const wchar_t) const c_wszNull[];

//----------------------------------------------------------------------------
extern NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, const TStdString) c_sNull;

//============================================================================
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, TStdString) ToStdString(
	const char*, size_t = 0, bool bMultibyte = true
) /*ToStdString(const char*, size_t = 0, bool = true)*/;
//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, TStdString) ToStdString(
	const wchar_t*, size_t = 0, bool bMultibyte = true
) /*ToStdString(const wchar_t*, size_t = 0, bool = true)*/;
//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, TStdString) ToStdString(
	const TStdString&, bool bMultibyte = true
) /*ToStdString(const TStdString&, bool = true)*/;

// Wide string tools
//============================================================================
#if !defined(_NATLIB_NO_WIDE_STRINGS_)

//============================================================================
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, TWideString) Format(const wchar_t*, ...);
//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, TWideString) Format(
	size_t, const wchar_t*, ...
) /*Format(size_t, const wchar_t*, ...)*/;

// TWideStringCaseSensitiveComparator
//============================================================================
class TWideStringCaseSensitiveComparator :
	public TPointerSavingEtalonComparatorOf<TWideString>
{
public:
	//------------------------------------------------------------------------
	typedef TPointerSavingEtalonComparatorOf<TWideString> TParent;

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_RESULT(TWideStringCaseSensitiveComparator) Clone() const
	{
		TWideStringCaseSensitiveComparator* pCopy;
		NLEXCEPT_CLASS_NEW(TWideStringCaseSensitiveComparator, pCopy);
		return pCopy;
	} /*Clone() const*/

	//------------------------------------------------------------------------
	TComparator::TResult operator()(const TWideString& c_wsToCompare) const 
	{
		const TWideString& c_wsEtalon = Etalon();
//		if (c_wsEtalon.data() == c_wsToCompare.data()) return compEqual;
		size_t nEtalonLength = c_wsEtalon.length();
		size_t nLength = c_wsToCompare.length();
		nl_int_t iResult = memcmp(
			c_wsEtalon.data(), c_wsToCompare.data(), 
			min(nEtalonLength, nLength) * sizeof(wchar_t)
		) /*memcmp*/;
		if (iResult < 0) return compLess;
		if (iResult > 0) return compGreater;
		if (nEtalonLength < nLength) return compLess;
		if (nEtalonLength > nLength) return compGreater;
		return compEqual;
	} /*operator()(const TWideString&) const*/
} /*TWideStringCaseSensitiveComparator*/;

// TWideStringCaseInsensitiveComparator
//============================================================================
class TWideStringCaseInsensitiveComparator :
	public TPointerSavingEtalonComparatorOf<TWideString>
{
public:
	//------------------------------------------------------------------------
	typedef TPointerSavingEtalonComparatorOf<TWideString> TParent;

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_RESULT(TWideStringCaseInsensitiveComparator) Clone() const
	{
		TWideStringCaseInsensitiveComparator* pCopy;
		NLEXCEPT_CLASS_NEW(TWideStringCaseInsensitiveComparator, pCopy);
		return pCopy;
	} /*Clone() const*/

	//------------------------------------------------------------------------
	TComparator::TResult operator()(const TWideString& c_wsToCompare) const 
	{
   		const TWideString& c_wsEtalon = Etalon();
	//	if (c_wsEtalon.data() == c_wsToCompare.data()) return compEqual;
		size_t nEtalonLength = c_wsEtalon.length();
		size_t nLength = c_wsToCompare.length();
		const wchar_t* pc_cEtalon = (const wchar_t*)(c_wsEtalon.data());
		const wchar_t* pc_cToCompare = (const wchar_t*)(c_wsToCompare.data());
		size_t nCount = min(nEtalonLength, nLength);
		while (
			--nCount && 
			(
				towlower((unsigned)*pc_cEtalon) == 
					towlower((unsigned)*pc_cToCompare)
			) 
		){
			pc_cEtalon += 1; pc_cToCompare += 1;
		} /*while*/
		if (
			towlower((unsigned)*pc_cEtalon) < 
				towlower((unsigned)*pc_cToCompare)
		) return compLess;
		if (
			towlower((unsigned)*pc_cEtalon) > 
				towlower((unsigned)*pc_cToCompare)
		) return compGreater;
		if (nEtalonLength < nLength) return compLess;
		if (nEtalonLength > nLength) return compGreater;
		return compEqual;
	} /*operator()(const TWideString&) const*/
} /*TWideStringCaseInsensitiveComparator*/;

//============================================================================
extern NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, TWideString) c_wsNull;

// Used to convert TWideString<-->TStdString
//============================================================================
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, TStdString) ToStdString(
	const TWideString&, bool bMultibyte = true
) /*ToStdString(const TWideString&, bool = true)*/;
//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, TWideString) ToWideString(
	const wchar_t*, size_t = 0, bool bMultibyte = true
) /*ToWideString(const wchar_t*, size_t = 0, bool = true)*/;
//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, TWideString) ToWideString(
	const char*, size_t = 0, bool bMultibyte = true
) /*ToWideString(const char*, size_t = 0, bool = true)*/;
//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, TWideString) ToWideString(
	const TStdString&, bool bMultibyte = true
) /*ToWideString(const TStdString&, bool = true)*/;
//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, TWideString) ToWideString(
	const TWideString&, bool bMultibyte = true
) /*ToWideString(const TWideString&, bool = true)*/;

//============================================================================
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

// Some compatible wide string routines
//============================================================================
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, wchar_t*) SzToWide(
	const char*, wchar_t* = null, bool bMultibyte = true
) /*SzToWide(const char*, wchar_t* = null, bool = true)*/;
//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, wchar_t*) SzToWide(
	const wchar_t*, wchar_t* = null, bool bMultibyte = true
) /*SzToWide(const wchar_t*, wchar_t* = null, bool = true)*/;

//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, char*) SzToStd(
	const wchar_t*, char* = null, bool bMultibyte = true
) /*SzToStd(const wchar_t*, char* = null, bool = true)*/;
//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, char*) SzToStd(
	const char*, char* = null, bool bMultibyte = true
) /*SzToStd(const char*, char* = null, bool = true)*/;

//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, size_t) SzLength(const wchar_t*);
//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, size_t) SzLength(const char*);

//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, int) SzCmp(const char*, const char*);
//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_,int) SzCmp(const wchar_t*, const wchar_t*);
//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, int) SzCmp(const char*, const wchar_t*);
//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, int) SzCmp(const wchar_t*, const char*);

//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, int) SzICmp(const char*, const char*);
//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_,int) SzICmp(const wchar_t*,const wchar_t*);
//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, int) SzICmp(const char*, const wchar_t*);
//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, int) SzICmp(const wchar_t*, const char*);

//============================================================================
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, int) MemCmp(
	const void*, const void*, size_t
) /*MemCmp(const void*, const void*, size_t)*/;
//----------------------------------------------------------------------------
NATLIB_FUNC_DECL(_NATLIB_DECLSPEC_, int) MemICmp(
	const void*, const void*, size_t
) /*MemICmp(const void*, const void*, size_t)*/;

//============================================================================
NATLIB_NAMESPACE_END

#endif /*!defined(__NATLIB_NLSTRING_H__)*/
