#ifndef __NATLIB_TASKS_NTSCHED_H__
#define __NATLIB_TASKS_NTSCHED_H__
/****************************************************************************/
/** ntsched.h                                                              **/
/** ---------------------------------------------------------------------- **/
/** Natlib Tasks non-preemptive scheduler                                  **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** September 01, 1997 by Oleg Kagan                                       **/
/****************************************************************************/

//============================================================================
#include <ntasks.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TScheduler
//============================================================================
class _NATLIB_TASKS_DECLSPEC_ TScheduler : public TSchedulerInterface {
public:
	//------------------------------------------------------------------------
    typedef TSchedulerInterface TParent;

    //------------------------------------------------------------------------
    NLEXCEPT_DECLARE(
    	TException, TSchedulerInterface::TException, _NATLIB_TASKS_DECLSPEC_
    ) /*NLEXCEPT_DECLARE*/;
	NLEXCEPT_DECLARE(
    	TXUnknownServiceRequest, TException, _NATLIB_TASKS_DECLSPEC_
    ) /*NLEXCEPT_DECLARE*/;
	NLEXCEPT_DECLARE(
    	TXTaskAlreadyStarted, TException, _NATLIB_TASKS_DECLSPEC_
    ) /*NLEXCEPT_DECLARE*/;

	// TTaskByNameComparator
    //------------------------------------------------------------------------
    typedef class _NATLIB_TASKS_DECLSPEC_ TTaskByNameComparator :
    	public TPointerSavingEtalonComparatorOf<TTask::THandle>
    {
    public:
    	//--------------------------------------------------------------------
        typedef TPointerSavingEtalonComparatorOf<TTask::THandle> TParent;
    	//--------------------------------------------------------------------
        TComparator::TResult operator()(const TTask::THandle&) const;
        //--------------------------------------------------------------------
		NLOBJECT_CLONE_IS_NOT_ALLOWED(TTaskByNameComparator);
    } typedef_TTaskByNameComparator;

	//------------------------------------------------------------------------
    TScheduler();
    //------------------------------------------------------------------------
    virtual ~TScheduler();

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED(TScheduler)

    //------------------------------------------------------------------------
    virtual void EvTransaction();
    //------------------------------------------------------------------------
    virtual bool IsFinished();

	//------------------------------------------------------------------------
    void Start(TTask&, bool bAutodelete = true);
    //------------------------------------------------------------------------
    void Terminate(const TTask::THandle&, bool bUnconditional = false);
	//------------------------------------------------------------------------
	void TerminateAll();
    //------------------------------------------------------------------------
    void Send(TSignal&, const TTask::THandle& c_hTarget);
    //------------------------------------------------------------------------
    void Send(TSignal&, const TContainerOf<TTask::THandle>& c_Targets);

    //------------------------------------------------------------------------
    void ShowTaskException(
    	TTask&, TTask::TPhase, const TNatlibException* = null
    ) /*ShowTaskException(TTask&, TPhase, const TNatlibException* = null)*/;

protected:
	//------------------------------------------------------------------------
    void InterfaceStart(
		TSchedulerInterface&, TTask&, bool bAutodelete = true
	) /*InterfaceStart(TSchedulerInterface&, TTask&, bool = true)*/;
    //------------------------------------------------------------------------
    void InterfaceSend(
		TSchedulerInterface&, TSignal&, const TTask::THandle& c_hTarget
	) /*InterfaceSend(TSchedulerInterface&, TSignal&, ...)*/;
    //------------------------------------------------------------------------
    void InterfaceSend(
		TSchedulerInterface&, TSignal&, const TContainerOf<TTask::THandle>&
	) /*InterfaceSend(TSchedulerInterface&, TSignal&, ...)*/;
	//------------------------------------------------------------------------
    void InterfaceShowTaskException(
    	TSchedulerInterface&, 
		TTask&, TTask::TPhase, const TNatlibException* = null
    ) /*InterfaceShowTaskException(TSchedulerInterface&, TTask&, ...)*/;

protected:
	//------------------------------------------------------------------------
    TSafeBuffer StartupQueue, ActiveQueue, IdleQueue, TerminationQueue;
    TSafeBuffer SignalQueue;

    //------------------------------------------------------------------------
	bool IsTaskStarted(TTask&) const;

	//------------------------------------------------------------------------
	virtual void PerformTasksStartups();
    //------------------------------------------------------------------------
    virtual void DispatchTasksSignals();
	//------------------------------------------------------------------------
    virtual void PerformTasksTransactions();
    //------------------------------------------------------------------------
    virtual void PerformTasksTermination();

    //------------------------------------------------------------------------
    virtual bool CallTaskCanTerminate(TTask&);
    //------------------------------------------------------------------------
	virtual void CallTaskException(TTask&, TTask::TPhase, TNatlibException*);

    //------------------------------------------------------------------------
    virtual
    void ProcessXServiceRequest(TTask&, TTask::TPhase, TXServiceRequest&);
} /*TScheduler*/;

//============================================================================
NATLIB_NAMESPACE_END

#endif /*!defined(__NATLIB_TASKS_NTSCHED_H__)*/
