#ifndef __NATLIB_NWCONSOL_H__
#define __NATLIB_NWCONSOL_H__
/****************************************************************************/
/** nwconsol.h                                                             **/
/** ---------------------------------------------------------------------- **/
/** C++ envelope for Win32 Console API                                     **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** January 18, 1998 by Oleg Kagan                                         **/
/****************************************************************************/

//============================================================================
#ifndef WIN32
	#error nwconsol.h can be used only for Microsoft Win32 applications
#endif /*!defined(WIN23)*/

// Standard includes
//============================================================================
#include <windows.h>

// Natlib Basics includes
//============================================================================
#include <nlobject.h>
#include <nlstring.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TWin32Console
//============================================================================
class _NATLIB_DECLSPEC_ TWin32Console {
public:
	// TColor
	//------------------------------------------------------------------------
	typedef enum {
		colorFirst = 0,	
		colorBlack = colorFirst, colorBlue, colorGreen, 
		colorCyan, colorRed, colorMagenta, 
		colorYellow, colorWhite,
		colorLightBlack, colorLightBlue, colorLightGreen, 
		colorLightCyan, colorLightRed, colorLightMagenta, 
		colorLightYellow, colorLightWhite,
		colorAll
	} TColor;

	// TColorStruct
	//------------------------------------------------------------------------
	typedef union TColorStruct {
		//--------------------------------------------------------------------
		TColor Color;
		//--------------------------------------------------------------------
		unsigned uColor;
		//--------------------------------------------------------------------
		struct {
			unsigned short uBlue  : 1;
			unsigned short uGreen : 1;
			unsigned short uRed   : 1;
			unsigned short uLight : 1;
		} Rgb;

		//--------------------------------------------------------------------
		TColorStruct() {uColor = 0;}
		//--------------------------------------------------------------------
		TColorStruct(TColor InitColor) {uColor = 0; Color = InitColor;}
		//--------------------------------------------------------------------
		TColorStruct(bool bRed, bool bGreen, bool bBlue, bool bLight = false)
		{
			uColor = 0;
			Rgb.uRed = BYTE(bRed);
            Rgb.uGreen = BYTE(bGreen);
            Rgb.uBlue = BYTE(bBlue);
			Rgb.uLight = BYTE(bLight);
		} /*TColorStruct(bool, bool, bool)*/
	} typedef_TColorStruct;

	//------------------------------------------------------------------------
	typedef enum {
		noteFirst = 0,
		noteC = noteFirst,
		noteCd, noteDb = noteCd,
		noteD,
		noteDd, noteEb = noteDd,
		noteE,	
		noteF,	
		noteFd, noteGb = noteFd,
		noteG,	
		noteGd, noteAb = noteGd,
		noteA,	
		noteAd, noteBb = noteAd,
		noteB,
		noteAll
	} TNote;

	//------------------------------------------------------------------------
	NLEXCEPT_DECLARE(
    	TException, TNatlibObject::TException, _NATLIB_DECLSPEC_
    ) /*NLEXCEPT_DECLARE*/;
	NLEXCEPT_DECLARE(TXInvalidHandle, TException, _NATLIB_DECLSPEC_);
	NLEXCEPT_DECLARE(TXOperationFailed, TException, _NATLIB_DECLSPEC_);

	//------------------------------------------------------------------------
	static const char* const c_szNewLine;
	static const char* const c_szPauseDefaultPrompt;

	//------------------------------------------------------------------------
	static const unsigned long const cac_ulOctave[5][noteAll];

	//------------------------------------------------------------------------
	static TColor GetNextColor(TColor);
	//------------------------------------------------------------------------
	static TColor GetPreviousColor(TColor);

	//------------------------------------------------------------------------
	static void Initialize();
	//------------------------------------------------------------------------
	static void Close();

	//------------------------------------------------------------------------
   	static void SetTitle(const char* c_szTitle);
	//------------------------------------------------------------------------
   	static void SetTitle(const wchar_t* c_wszTitle);
    //------------------------------------------------------------------------
	static void SetTitle(const TStdString& c_sTitle);
#ifndef _NATLIB_NO_WIDE_STRINGS_
    //------------------------------------------------------------------------
	static void SetTitle(const TWideString& c_wsTitle);
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

	//------------------------------------------------------------------------
	static TStdString GetTitleA();
#ifndef _NATLIB_NO_WIDE_STRINGS_
    //------------------------------------------------------------------------
	static TWideString GetTitleW();
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

#if (defined(_UNICODE) && !defined(_NATLIB_NO_WIDE_STRINGS_))
	//------------------------------------------------------------------------
	static TWideString GetTitle() {return GetTitleW();}
#else
	//------------------------------------------------------------------------
	static TStdString GetTitle() {return GetTitleA();}
#endif /*(defined(_UNICODE) && !defined(_NATLIB_NO_WIDE_STRINGS_))*/

	//------------------------------------------------------------------------
	static void Fill(char cFiller);
	//------------------------------------------------------------------------
	static void Fill(wchar_t wcFiller);
	//------------------------------------------------------------------------
	static void Clear() {Fill(wchar_t(' '));}

	//------------------------------------------------------------------------
	static bool ShowCursor(bool);
	//------------------------------------------------------------------------
	static bool IsCursorVisible();
	//------------------------------------------------------------------------
	static short SetCursorSize(short);
	//------------------------------------------------------------------------
	static short GetCursorSize();

	//------------------------------------------------------------------------
	static void GotoXY(short nX, short nY);
	//------------------------------------------------------------------------
	static short WhereX();
	//------------------------------------------------------------------------
	static short WhereY();
	//------------------------------------------------------------------------
	static void WhereXY(short& r_nX, short& r_nY);
	//------------------------------------------------------------------------
	static short GetWidth();
	//------------------------------------------------------------------------
	static short GetHeight();

	//------------------------------------------------------------------------
    static size_t Write(const char*);
	//------------------------------------------------------------------------
    static size_t Write(const wchar_t*);
    //------------------------------------------------------------------------
	static size_t Write(const TStdString&);
#ifndef _NATLIB_NO_WIDE_STRINGS_
    //------------------------------------------------------------------------
	static size_t Write(const TWideString&);
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

	//------------------------------------------------------------------------
	static size_t WriteLn(const char* = null);
	//------------------------------------------------------------------------
	static size_t WriteLn(const wchar_t*);
	//------------------------------------------------------------------------
	static size_t WriteLn(const TStdString&);
#ifndef _NATLIB_NO_WIDE_STRINGS_
	//------------------------------------------------------------------------
	static size_t WriteLn(const TWideString&);
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

	//------------------------------------------------------------------------
	static size_t WriteError(const char*);
	//------------------------------------------------------------------------
	static size_t WriteError(const wchar_t*);
	//------------------------------------------------------------------------
	static size_t WriteError(const TStdString&);
#ifndef _NATLIB_NO_WIDE_STRINGS_
	//------------------------------------------------------------------------
	static size_t WriteError(const TWideString&);
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

	//------------------------------------------------------------------------
	static size_t WriteErrorLn(const char* = null);
	//------------------------------------------------------------------------
	static size_t WriteErrorLn(const wchar_t*);
	//------------------------------------------------------------------------
	static size_t WriteErrorLn(const TStdString&);
#ifndef _NATLIB_NO_WIDE_STRINGS_
	//------------------------------------------------------------------------
	static size_t WriteErrorLn(const TWideString&);
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

	//------------------------------------------------------------------------
	static void Pause(const char* c_szPrompt = c_szPauseDefaultPrompt);
	//------------------------------------------------------------------------
	static void Pause(const wchar_t* c_wszPrompt);
	//------------------------------------------------------------------------
	static void Pause(const TStdString&);
#ifndef _NATLIB_NO_WIDE_STRINGS_
	//------------------------------------------------------------------------
	static void Pause(const TWideString&);
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

	//------------------------------------------------------------------------
	static TColor SetColor(TColor);
	//------------------------------------------------------------------------
	static TColor GetColor();
	//------------------------------------------------------------------------
	static bool SetHighlight(bool);
	//------------------------------------------------------------------------
	static bool GetHighlight();

	//------------------------------------------------------------------------
	static TColor SetBackgroundColor(TColor);
	//------------------------------------------------------------------------
	static TColor GetBackgroundColor();
	//------------------------------------------------------------------------
	static bool SetBackgroundHighlight(bool);
	//------------------------------------------------------------------------
	static bool GetBackgroundHighlight();

	//------------------------------------------------------------------------
	static void Delay(DWORD ulDuration);

	//------------------------------------------------------------------------
	static unsigned long Octave(unsigned long ulOctave, TNote);
	//------------------------------------------------------------------------
	static void Sound(unsigned long ulFrequency, unsigned long ulDuration);
	//------------------------------------------------------------------------
	static void Sound(
		unsigned long ulOctave, TNote, unsigned long ulDuration
	) /*Sound(unsigned long, TNote, unsigned long)*/;

	//------------------------------------------------------------------------
	static void SaveState();
	//------------------------------------------------------------------------
	static void RestoreState();

	//------------------------------------------------------------------------
	static char ReadChar(bool bEcho = true);
	//------------------------------------------------------------------------
	static bool KeyboardHit();

protected:
	// TInitializer
	//------------------------------------------------------------------------
	typedef class _NATLIB_DECLSPEC_ TInitializer {
	public:
		//--------------------------------------------------------------------
		TInitializer();
		//--------------------------------------------------------------------
		~TInitializer();
	} typedef_TInitializer;
	friend TInitializer;

	//------------------------------------------------------------------------
	static TInitializer Initializer;

	//------------------------------------------------------------------------
	static HANDLE hInput; // Standard input handle
	static HANDLE hOutput; // Standard output handle
	static HANDLE hError; // Standard error handle

	//------------------------------------------------------------------------
	static size_t Write(HANDLE hDestination, const char*);
	//------------------------------------------------------------------------
	static size_t Write(HANDLE hDestination, const wchar_t*);
	//------------------------------------------------------------------------
	static size_t Write(HANDLE hDestination, const TStdString&);
#ifndef _NATLIB_NO_WIDE_STRINGS_
	//------------------------------------------------------------------------
	static size_t Write(HANDLE hDestination, const TWideString&);
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

	//------------------------------------------------------------------------
	static WORD GetAttributes();
	//------------------------------------------------------------------------
	static void SetAttributes(WORD);
	//------------------------------------------------------------------------
	static void SetCursorInfo(const CONSOLE_CURSOR_INFO&);
	//------------------------------------------------------------------------
	static void ChangeAttributeFlag(WORD& r_wAttributes, WORD wMask, bool);
	//------------------------------------------------------------------------
	static void GetConsoleInfo(CONSOLE_SCREEN_BUFFER_INFO&);
	//------------------------------------------------------------------------
	static void GetCursorInfo(CONSOLE_CURSOR_INFO&);

private:
	//------------------------------------------------------------------------
	TWin32Console();
	//------------------------------------------------------------------------
	~TWin32Console();
} /*TWin32Console*/;

//============================================================================
NATLIB_NAMESPACE_END

#endif /*!defined(__NATLIB_NWCONSOL_H__)*/