#ifndef __NATLIB_NWEVLOG_H__
#define __NATLIB_NWEVLOG_H__
/****************************************************************************/
/** nwevlog.h                                                              **/
/** ---------------------------------------------------------------------- **/
/** Event logging utilities for Win32 (Windows NT)                         **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** January 18, 1998 by Oleg Kagan                                         **/
/****************************************************************************/

// Natlib Basics includes
//============================================================================
#include <nlobject.h>
#include <nlstring.h>

// Natlib Win32 includes
//============================================================================
#include <nwerror.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

//============================================================================
#ifdef _NATLIB_NO_WIDE_STRINGS_
	#define NWEVLOG_STRING_TYPE TStdString
	#define NWEVLOG_STRING_TYPE_CAST(String) ToStdString(String)
    #define NWEVLOG_CHAR_TYPE char
    #define NWEVLOG_STRING_NULL c_sNull
#else
	#define NWEVLOG_STRING_TYPE TWideString
	#define NWEVLOG_STRING_TYPE_CAST(String) ToWideString(String)
    #define NWEVLOG_CHAR_TYPE wchar_t
    #define NWEVLOG_STRING_NULL c_wsNull
#endif /*_NATLIB_NO_WIDE_STRINGS_*/

// TWin32EventLog
//============================================================================
class _NATLIB_DECLSPEC_ TWin32EventLog : public TNatlibObject {
public:
	//------------------------------------------------------------------------
	typedef TNatlibObject TParent;

	// TEventType
	//------------------------------------------------------------------------
	typedef enum {
		eventFirst,
		eventError = eventFirst, eventWarning, eventInformation,
		eventAuditSuccess, eventAuditFailure,
		eventAll
	} TEventType;

	//------------------------------------------------------------------------
	NLEXCEPT_DECLARE(TException, TNatlibObject::TException,_NATLIB_DECLSPEC_);
	NLEXCEPT_DECLARE(TXInvalidOSVersion, TException, _NATLIB_DECLSPEC_);
	NLEXCEPT_DECLARE(TXInvalidEventType, TException, _NATLIB_DECLSPEC_);
	NLEXCEPT_DECLARE(
    	TXUnableToCreateRegistryKey, TException, _NATLIB_DECLSPEC_
    ) /*NLEXCEPT_DECLARE*/;
	NLEXCEPT_DECLARE(
    	TXUnableToSetRegistryValue, TException, _NATLIB_DECLSPEC_
    ) /*NLEXCEPT_DECLARE*/;
	NLEXCEPT_DECLARE(
    	TXUnableToRegisterEventSource, TException, _NATLIB_DECLSPEC_
    ) /*NLEXCEPT_DECLARE*/;

	//------------------------------------------------------------------------
	static const char* const c_szRegistryPath;
	//------------------------------------------------------------------------
	static const char* const c_szEventMessageFileValue;
	static const char* const c_szTypesSupportedValue;

	//------------------------------------------------------------------------
	static WORD ToStdEventType(TEventType);
	//------------------------------------------------------------------------
	static TEventType FromStdEventType(WORD);

	//------------------------------------------------------------------------
	TWin32EventLog(
		const TStdString& c_sApplication,
		const TStdString& c_sMessagesDll
	) /*TWin32EventLog(const TStdString&, const TStdString&)*/;

#ifndef _NATLIB_NO_WIDE_STRINGS_
	//------------------------------------------------------------------------
	TWin32EventLog(
		const TWideString& c_wsApplication,
		const TWideString& c_wsMessagesDll
	) /*TWin32EventLog(const TWideString&, const TWideString&)*/;
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/
	//------------------------------------------------------------------------
	~TWin32EventLog();

	//------------------------------------------------------------------------
	NLOBJECT_CLONE_IS_NOT_ALLOWED(TWin32EventLog)

	//------------------------------------------------------------------------
	DWORD GetLastError() const {return uLastError;}
	//------------------------------------------------------------------------
	DWORD ClearLastError()
	{
		DWORD uError = uLastError; uLastError = 0; return uError;
	} /*ClearLastError()*/

	//------------------------------------------------------------------------
	bool ReportEvent(
		TEventType, WORD uCategory, DWORD uEventId,
		const TStdString& c_sParam0,
		const TStdString& c_sParam1 = c_sNull,
		const TStdString& c_sParam2 = c_sNull,
		const TStdString& c_sParam3 = c_sNull,
		const TStdString& c_sParam4 = c_sNull,
		const TStdString& c_sParam5 = c_sNull,
		const TStdString& c_sParam6 = c_sNull,
		const TStdString& c_sParam7 = c_sNull,
		const TStdString& c_sParam8 = c_sNull
	) /*ReportEvent(TWin32EventLog::TEventType, WORD, DWORD, ...)*/;

#ifndef _NATLIB_NO_WIDE_STRINGS_
	//------------------------------------------------------------------------
	bool ReportEvent(
		TEventType, WORD uCategory, DWORD uEventId,
		const TWideString& c_wsParam0,
		const TWideString& c_wsParam1 = c_wsNull,
		const TWideString& c_wsParam2 = c_wsNull,
		const TWideString& c_wsParam3 = c_wsNull,
		const TWideString& c_wsParam4 = c_wsNull,
		const TWideString& c_wsParam5 = c_wsNull,
		const TWideString& c_wsParam6 = c_wsNull,
		const TWideString& c_wsParam7 = c_wsNull,
		const TWideString& c_wsParam8 = c_wsNull
	) /*ReportEvent(TWin32EventLog::TEventType, WORD, DWORD, ...)*/;
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

	//------------------------------------------------------------------------
	bool ReportEvent(
		DWORD uIdEvent, 
		size_t nParams, const wchar_t* a_wszParams[],
		WORD uType = EVENTLOG_INFORMATION_TYPE, WORD uCategory = 0
	) /*ReportEvent(DWORD, size_t, const wchar_t*, WORD, WORD)*/;
	//------------------------------------------------------------------------
	bool ReportEvent(
		DWORD uIdEvent, 
		size_t nParams, const char* a_szParams[],
		WORD uType = EVENTLOG_INFORMATION_TYPE, WORD uCategory = 0
	) /*ReportEvent(DWORD, size_t, const char*, WORD, WORD)*/;

	//------------------------------------------------------------------------
	const NWEVLOG_STRING_TYPE& Application() const {return wsApplication;}
	//------------------------------------------------------------------------
	TStdString ApplicationA() const {return ToStdString(wsApplication);}
#ifndef _NATLIB_NO_WIDE_STRINGS_
	//------------------------------------------------------------------------
	const TWideString& ApplicationW() const {return wsApplication;}
#endif /*!defined (NATLIB_NO_WIDE_STRINGS_)*/

	//------------------------------------------------------------------------
	const NWEVLOG_STRING_TYPE& MessagesDll() const {return wsMessagesDll;}
	//------------------------------------------------------------------------
	TStdString MessagesDllA() const {return ToStdString(wsMessagesDll);}
#ifndef _NATLIB_NO_WIDE_STRINGS_
	//------------------------------------------------------------------------
	const TWideString& MessagesDllW() const {return wsMessagesDll;}
#endif /*!defined (NATLIB_NO_WIDE_STRINGS_)*/

protected:
	//------------------------------------------------------------------------
	NWEVLOG_STRING_TYPE wsApplication;
	NWEVLOG_STRING_TYPE wsMessagesDll;
	HANDLE hEventSource;

	//------------------------------------------------------------------------
	mutable DWORD uLastError;

	//------------------------------------------------------------------------
	void Init();
} /*TWin32EventLog*/;

//============================================================================
NATLIB_NAMESPACE_END

#endif /*!defined(__NATLIB_NWEVLOG_H__)*/