#ifndef __NATLIB_NWLOCK_H__
#define __NATLIB_NWLOCK_H__
/****************************************************************************/
/** nwlock.h                                                               **/
/** ---------------------------------------------------------------------- **/
/** Natlib multithread syncronization facility for MS Win32                **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** June 24, 1997 by Oleg Kagan                                            **/
/****************************************************************************/

//============================================================================
#ifndef WIN32 /*MS Win32*/
	#error nwlock.h can be used only for Microsoft Win32 applications
#endif /*!defined(WIN23)*/

//============================================================================
#include <nllock.h>
#include <nwerror.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

//============================================================================
class _NATLIB_DECLSPEC_ TWin32Lock : public TLock {
public:
	//------------------------------------------------------------------------
	typedef TLock TParent;

	//------------------------------------------------------------------------
	NLEXCEPT_DECLARE(TException, TLock::TException, _NATLIB_DECLSPEC_);
	NLEXCEPT_DECLARE(TXUnableToCreateMutex, TException, _NATLIB_DECLSPEC_);
	NLEXCEPT_DECLARE(TXUnableToReleaseMutex, TException, _NATLIB_DECLSPEC_);
	NLEXCEPT_DECLARE(TXUnableToGetMemberAccess, TException,_NATLIB_DECLSPEC_);
	NLEXCEPT_DECLARE(TXOperationFailed, TException, _NATLIB_DECLSPEC_);

	//------------------------------------------------------------------------
	TWin32Lock();
	//------------------------------------------------------------------------
	~TWin32Lock();

	//------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE(TWin32Lock);

	//------------------------------------------------------------------------
    void Lock(const void* pc_LockKey, nl_ulong_t uWaitTimeout = 0);
    //------------------------------------------------------------------------
    void Unlock(const void* pc_LockKey);

    //------------------------------------------------------------------------
    const void* Key() const;

	// Default security attributes uset to create Win32 mutexes
	//------------------------------------------------------------------------
	static const SECURITY_ATTRIBUTES c_SecurityAttributes;

protected:
	//------------------------------------------------------------------------
	HANDLE hMutex;
	mutable HANDLE hMemberAccessMutex;

	//------------------------------------------------------------------------
	void LockMemberAccess() const;
	//------------------------------------------------------------------------
	void UnlockMemberAccess() const;

private:
	//------------------------------------------------------------------------
	TWin32Lock(const TWin32Lock&) : hMutex(null), hMemberAccessMutex(null)
	{
		throw TXOperationIsNotAllowed(THIS_CLASS);
	} /*TWin32Lock(const TWin32Lock&)*/
	//------------------------------------------------------------------------
	TWin32Lock& operator=(const TWin32Lock&) 
	{
		throw TXOperationIsNotAllowed(THIS_CLASS); return *this;
	} /*operator=(const TWin32Lock&) */
} /*TWin32Lock*/;

//============================================================================
NATLIB_NAMESPACE_END

#endif /*!defined(__NATLIB_NWLOCK_H__)*/