#ifndef __NATLIB_NWREG_H__
#define __NATLIB_NWREG_H__
/****************************************************************************/
/** nwreg.h                                                                **/
/** ---------------------------------------------------------------------- **/
/** C++ envelope for Win32 Registry operation                              **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** August 13, 1998 by Oleg Kagan                                          **/
/****************************************************************************/

// Natlib Basics includes
//============================================================================
#include <nlautopt.h>

// Natlib Win32 includes
//============================================================================
#include <nwerror.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TWin32Registry
//============================================================================
class _NATLIB_DECLSPEC_ TWin32Registry : public virtual TNatlibObject {
public:
	//------------------------------------------------------------------------
	typedef TNatlibObject TParent;

	//------------------------------------------------------------------------
	static const char c_cPathDelimiter;

	//------------------------------------------------------------------------
	NLEXCEPT_DECLARE(
		TException, TNatlibObject::TException, _NATLIB_DECLSPEC_
	) /*NLEXCEPT_DECLARE*/;
	NLEXCEPT_DECLARE(TXInvalidPath, TException, _NATLIB_DECLSPEC_);
	NLEXCEPT_DECLARE_DATA_HOLDER(
		TXRegOperationFailed, LONG, TException, _NATLIB_DECLSPEC_
	) /*NLEXCEPT_DECLARE_DATA_HOLDER*/;
	NLEXCEPT_DECLARE(TXNoParentExists, TException, _NATLIB_DECLSPEC_);

	//------------------------------------------------------------------------
	static HKEY RegCreateKey(HKEY hKey, const wchar_t* c_wszSubkey);
	//------------------------------------------------------------------------
	static HKEY RegCreateKey(HKEY hKey, const char* c_szSubkey = null);

	//------------------------------------------------------------------------
	static HKEY RegOpenKey(HKEY hKey, const wchar_t* c_wszSubkey);
	//------------------------------------------------------------------------
	static HKEY RegOpenKey(HKEY hKey, const char* c_szSubkey = null);

	//------------------------------------------------------------------------
	static void RegCloseKey(HKEY);

	//------------------------------------------------------------------------
	static DWORD RegQueryValue(
		HKEY hKey, const wchar_t* c_wszName, 
		void* pValue, DWORD nValueSize, DWORD& r_dwType
	) /*RegQueryValue(HKEY, const wchar_t*, void*, DWORD, DWORD&)*/;
	//------------------------------------------------------------------------
	static DWORD RegQueryValue(
		HKEY hKey, const char* c_szName, 
		void* pValue, DWORD nValueSize, DWORD& r_dwType
	) /*RegQueryValue(HKEY, const char*, void*, DWORD, DWORD&)*/;

	//------------------------------------------------------------------------
	static void RegSetValue(
		HKEY hKey, const wchar_t* c_wszName, 
		const void* pc_Value, DWORD nValueSize, DWORD dwType
	) /*RegSetValue(HKEY, const wchar_t*, const void*, DWORD, DWORD)*/;
	//------------------------------------------------------------------------
	static void RegSetValue(
		HKEY hKey, const char* c_szName, 
		const void* pc_Value, DWORD nValueSize, DWORD dwType
	) /*RegSetValue(HKEY, const char*, const void*, DWORD, DWORD)*/;

	//------------------------------------------------------------------------
	static void RegDeleteKey(HKEY hKey, const wchar_t* c_wszSubkey);
	//------------------------------------------------------------------------
	static void RegDeleteKey(HKEY hKey, const char* c_szSubkey);

	//------------------------------------------------------------------------
	static void RegDeleteValue(HKEY hKey, const wchar_t* c_wszName);
	//------------------------------------------------------------------------
	static void RegDeleteValue(HKEY hKey, const char* c_szName);

	//------------------------------------------------------------------------
	static HKEY ResolvePath(
		HKEY hRootKey, const wchar_t* c_szPath, bool bCreate = false
	) /*ResolvePath(HKEY, const wchar_t*, bool= false*/;

	//------------------------------------------------------------------------
	static HKEY ResolvePath(
		HKEY hRootKey, const char* c_szPath, bool bCreate = false
	) /*ResolvePath(HKEY, const char*, bool= false*/;

	//------------------------------------------------------------------------
	static HKEY CreateKey(HKEY hKey, const wchar_t* c_wszPath);
	//------------------------------------------------------------------------
	static HKEY CreateKey(HKEY hKey, const char* c_szPath = null);

	//------------------------------------------------------------------------
	static HKEY OpenKey(HKEY hKey, const wchar_t* c_wszPath);
	//------------------------------------------------------------------------
	static HKEY OpenKey(HKEY hKey, const char* c_szPath = null);

	//------------------------------------------------------------------------
	TWin32Registry(const TWin32Registry&);

	//------------------------------------------------------------------------
	TWin32Registry(
		HKEY hRootKey, const wchar_t* c_wszRootPath = null, 
		bool bCreate = false
	) /*TWin32Registry(HKEY, const wchar_t* = null, bool = false)*/;
	//------------------------------------------------------------------------
	TWin32Registry(HKEY, const char*, bool = false);
#ifndef _NATLIB_NO_WIDE_STRINGS_
	//------------------------------------------------------------------------
	TWin32Registry(HKEY, const TWideString&, bool = false);
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/
	//------------------------------------------------------------------------
	TWin32Registry(HKEY, const TStdString&, bool = false);

	//------------------------------------------------------------------------
	~TWin32Registry();

	//------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE(TWin32Registry);

	//------------------------------------------------------------------------
	void Assign(HKEY, const wchar_t* c_wszPath = null, bool bCreate = false);
	//------------------------------------------------------------------------
	void Assign(HKEY, const char*, bool bCreate = false);
#ifndef _NATLIB_NO_WIDE_STRINGS_
	//------------------------------------------------------------------------
	void Assign(
		HKEY hNewKey, const TWideString& c_wsNewPath, bool bCreate = false
	){
		Assign(hNewKey, c_wsNewPath.c_str(), bCreate);
	} /*Assign(HKEY, const TWideString&, bool = false)*/
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/
	//------------------------------------------------------------------------
	void Assign(
		HKEY hNewKey, const TStdString& c_sNewPath, bool bCreate = false
	){
		Assign(hNewKey, c_sNewPath.c_str(), bCreate);
	} /*Assign(HKEY, const TStdString&, bool = false)*/

	//------------------------------------------------------------------------
	void Unassign();

	//------------------------------------------------------------------------
	DWORD QueryValue(
		const wchar_t* c_szName, 
		void* pValue, DWORD nValueSize, DWORD& r_dwType
	) /*QueryValue(const wchar_t*, void*, DWORD, DWORD&)*/;
	//------------------------------------------------------------------------
	DWORD QueryValue(const char*, void*, DWORD, DWORD&);
#ifndef _NATLIB_NO_WIDE_STRINGS_
	//------------------------------------------------------------------------
	DWORD QueryValue(
		const TWideString& c_wsName, 
		void* pValue, DWORD nValueSize, DWORD& r_dwType)
	{
		return QueryValue(c_wsName.c_str(), pValue, nValueSize, r_dwType);
	} /*QueryValue(const TWideString&, void*, DWORD, DWORD&)*/
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/
	//------------------------------------------------------------------------
	DWORD QueryValue(
		const TStdString& c_sName, 
		void* pValue, DWORD nValueSize, DWORD& r_dwType
	){
		return QueryValue(c_sName.c_str(), pValue, nValueSize, r_dwType);
	} /*QueryValue(const TStdString&, void*, DWORD, DWORD&)*/

#ifndef _NATLIB_NO_WIDE_STRINGS_
	//------------------------------------------------------------------------
	TWideString QueryStringValue(
		const wchar_t* c_wszName, DWORD* p_dwType = null
	) /*QueryStringValue(const wchar_t*, DWORD* = null)*/;
	//------------------------------------------------------------------------
	TWideString QueryStringValue(
		const TWideString& c_wsName, DWORD* p_dwType = null
	){
		return QueryStringValue(c_wsName.c_str(), p_dwType);
	} /*QueryStringValueW(const TWideString&)*/
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/
	//------------------------------------------------------------------------
	TStdString QueryStringValue(const char*, DWORD* = null);
	//------------------------------------------------------------------------
	TStdString QueryStringValue(
		const TStdString& c_sName, DWORD* p_dwType = null
	){
		return QueryStringValue(c_sName.c_str(), p_dwType);
	} /*QueryStringValue(const TStdString&)*/

	//------------------------------------------------------------------------
	void SetValue(
		const wchar_t* c_wszName, 
		const void* pc_Value, DWORD nValueSize, DWORD dwType
	) /*SetValue(const wchar_t*, const void*, DWORD, DWORD*/;
	//------------------------------------------------------------------------
	void SetValue(const char*, const void*, DWORD, DWORD);
#ifndef _NATLIB_NO_WIDE_STRINGS_
	//------------------------------------------------------------------------
	void SetValue(
		const TWideString& c_wsName, 
		const void* pc_Value, DWORD nValueSize, DWORD dwType
	){
		SetValue(c_wsName.c_str(), pc_Value, nValueSize, dwType);
	} /*SetValue(const TWideString&, const void*, DWORD, DWORD)*/
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/
	//------------------------------------------------------------------------
	void SetValue(
		const TStdString& c_sName, 
		const void* pc_Value, DWORD nValueSize, DWORD dwType
	){
		SetValue(c_sName.c_str(), pc_Value, nValueSize, dwType);
	} /*SetValue(const TStdString&, const void*, DWORD, DWORD)*/

	//------------------------------------------------------------------------
	void SetStringValue(
		const wchar_t* c_wszName, const wchar_t* c_wszValue, 
		DWORD dwType = REG_SZ
	) /*SetStringValue(const wchar_t*, const wchar_t*, DWORD = REG_SZ)*/;
	//------------------------------------------------------------------------
	void SetStringValue(const char*, const char*, DWORD = REG_SZ);
#ifndef _NATLIB_NO_WIDE_STRINGS_
	//------------------------------------------------------------------------
	void SetStringValue(
		const TWideString& c_wsName, const wchar_t* c_wszValue,
		DWORD dwType = REG_SZ
	){
		SetStringValue(c_wsName.c_str(), c_wszValue, dwType);
	} /*SetStringValue(const TWideString&, const wchar_t*, DWORD = REG_SZ)*/
	//------------------------------------------------------------------------
	void SetStringValue(
		const TWideString& c_wsName, const TWideString& c_wsValue,
		DWORD dwType = REG_SZ
	) /*SetStringValue(const TWideString&, const TWideString&,DWORD=REG_SZ)*/;
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/
	//------------------------------------------------------------------------
	void SetStringValue(
		const TStdString& c_sName, const char* c_szValue, 
		DWORD dwType = REG_SZ
	){
		SetStringValue(c_sName.c_str(), c_szValue, dwType);
	} /*SetStringValue(const TStdString&, const char*, DWORD = REG_SZ)*/
	//------------------------------------------------------------------------
	void SetStringValue(
		const TStdString& c_sName, const TStdString& c_sValue,
		DWORD dwType = REG_SZ
	) /*SetStringValue(const TStdString&, const TStdString&, DWORD*=REG_SZ)*/;

	//------------------------------------------------------------------------
	void Delete(const wchar_t* c_wszSubkey);
	//------------------------------------------------------------------------
	void Delete(const char* c_szSubkey = null);
#ifndef _NATLIB_NO_WIDE_STRINGS_
	//------------------------------------------------------------------------
	void Delete(const TWideString& c_wsSubkey) {Delete(c_wsSubkey.c_str());}
#endif /*_NATLIB_NO_WIDE_STRINGS_*/
	//------------------------------------------------------------------------
	void Delete(const TStdString& c_sSubkey) {Delete(c_sSubkey.c_str());}

	//------------------------------------------------------------------------
	void DeleteValue(const wchar_t* c_wszName);
	//------------------------------------------------------------------------
	void DeleteValue(const char* c_szName = null);
#ifndef _NATLIB_NO_WIDE_STRINGS_
	//------------------------------------------------------------------------
	void DeleteValue(const TWideString& c_wsName) 
	{
		DeleteValue(c_wsName.c_str());
	} /*DeleteValue(const TWideString&)*/
#endif /*_NATLIB_NO_WIDE_STRINGS_*/
	//------------------------------------------------------------------------
	void DeleteValue(const TStdString& c_sName) 
	{
		DeleteValue(c_sName.c_str());
	} /*DeleteValue(const TStdString&)*/

	//------------------------------------------------------------------------
	TWin32Registry GetParent();

	//------------------------------------------------------------------------
	const wchar_t* GetName();

	//------------------------------------------------------------------------
	operator HKEY() {return hTargetKey;}

protected:
	//------------------------------------------------------------------------
	HKEY hRootKey;
	HKEY hTargetKey;
	TAutoPointerTo<wchar_t> h_wszPath;

	//------------------------------------------------------------------------
	static wchar_t* GetLastDelimiter(wchar_t*);
} /*TWin32Registry*/;

//============================================================================
NATLIB_NAMESPACE_END

#endif /*!defined(__NATLIB_NWREG_H__)*/