#ifndef __NATLIB_NWSHLLNK_H__
#define __NATLIB_NWSHLLNK_H__
/****************************************************************************/
/** nwshllnk.h                                                             **/
/** ---------------------------------------------------------------------- **/
/** C++ envelope for Win32 Shell Links                                     **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** July 10, 1998 by Oleg Kagan                                            **/
/****************************************************************************/

//============================================================================
#ifndef WIN32
	#error nwshllnk.h can be used only for Microsoft Win32 applications
#endif /*!defined(WIN23)*/

// Natlib Basics includes
//============================================================================
#include <nlobject.h>
#include <nlstring.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

//============================================================================
#ifdef _NATLIB_NO_WIDE_STRINGS_
	#define NWSHLLNK_STRING_TYPE TStdString
	#define NWSHLLNK_STRING_TYPE_CAST(String) ToStdString(String)
#else
	#define NWSHLLNK_STRING_TYPE TWideString
	#define NWSHLLNK_STRING_TYPE_CAST(String) ToWideString(String)
#endif /*_NATLIB_NO_WIDE_STRINGS_*/

// TWin32ShellLink
//============================================================================
class _NATLIB_DECLSPEC_ TWin32ShellLink : public TNatlibObject {
public:
	//------------------------------------------------------------------------
    typedef TNatlibObject TParent;

	//------------------------------------------------------------------------
	NLEXCEPT_DECLARE(
		TException, TNatlibObject::TException, _NATLIB_DECLSPEC_
    ) /*NLEXCEPT_DECLARE*/;
	NLEXCEPT_DECLARE(TXFileNotFound, TException, _NATLIB_DECLSPEC_);
	NLEXCEPT_DECLARE(TXUnableToResolve, TException, _NATLIB_DECLSPEC_);

	//------------------------------------------------------------------------
	NWSHLLNK_STRING_TYPE sTarget;
	NWSHLLNK_STRING_TYPE sArguments;
	NWSHLLNK_STRING_TYPE sWorkingDirectory;
	NWSHLLNK_STRING_TYPE sIconPath;
	nl_int_t iIconIndex;
	NWSHLLNK_STRING_TYPE sDescription;
	WORD wHotKey;
	nl_int_t iShowCommand;

	//------------------------------------------------------------------------
	TWin32ShellLink(
		const TStdString& c_sInitTarget = c_sNull,
		const TStdString& c_sInitArguments = c_sNull,
		const TStdString& c_sInitWorkingDirectory = c_sNull,
		const TStdString& c_sInitIconPath = c_sNull,
		nl_int_t c_iInitIconIndex = nl_int_t(0),
		const TStdString& c_sInitDescription = c_sNull,
		WORD c_wInitHotKey = WORD(0),
		nl_int_t c_iInitShowCommand = nl_int_t(0)
	) /*TWin32ShellLink(const TStdString&, const TStdString&, ...)*/;

#ifndef _NATLIB_NO_WIDE_STRINGS_
	//------------------------------------------------------------------------
	TWin32ShellLink(
		const TWideString& c_wsInitTarget,
		const TWideString& c_wsInitArguments = c_wsNull,
		const TWideString& c_wsInitWorkingDirectory = c_wsNull,
		const TWideString& c_wsInitIconPath = c_wsNull,
		nl_int_t c_iInitIconIndex = nl_int_t(0),
		const TWideString& c_wsInitDescription = c_wsNull,
		WORD c_wInitHotKey = WORD(0),
		nl_int_t c_iInitShowCommand = nl_int_t(0)
	) /*TWin32ShellLink(const TStdString&, const TStdString&, ...)*/;
#endif /*!defined (_NATLIB_NO_WIDE_STRINGS_)*/

	//------------------------------------------------------------------------
	TWin32ShellLink(const TWin32ShellLink&);

	//------------------------------------------------------------------------
	NLOBJECT_DECLARE_CLONE(TWin32ShellLink);

#ifndef _NATLIB_NO_WIDE_STRINGS_
	//------------------------------------------------------------------------
	bool LoadFrom(
		const TWideString& c_sPath, 
		bool bResolve = false, DWORD uFlag = 0, HWND hParentWindow = null
	) /*LoadFrom(const TWideString&, bool, DWORD, HWND)*/;
	//------------------------------------------------------------------------
	void LoadFromEx(
		const TWideString& c_sPath,
		bool bResolve = false, DWORD uFlag = 0, HWND hParentWindow = null
	) /*LoadFromEx(const TWideString&, bool, DWORD, HWND)*/;
	//------------------------------------------------------------------------
	bool SaveAs(const TWideString& c_sPath);
	//------------------------------------------------------------------------
	void SaveAsEx(const TWideString& c_sPath);
#endif /*!defined (_NATLIB_NO_WIDE_STRINGS_)*/

	//------------------------------------------------------------------------
	bool LoadFrom(
		const TStdString& c_wsPath,
		bool bResolve = false, DWORD uFlags = 0, HWND hParentWindow = null
	) /*LoadFrom(const TStdString&, boll, DWORD, HWND)*/;
	//------------------------------------------------------------------------
	void LoadFromEx(
		const TStdString& c_wsPath,
		bool bResolve = false, DWORD uFlags = 0, HWND hParentWindow = null
	) /*LoadFromEx(const TStdString&, boll, DWORD, HWND)*/;
	//------------------------------------------------------------------------
	bool SaveAs(const TStdString& c_wsPath);
	//------------------------------------------------------------------------
	void SaveAsEx(const TStdString& c_wsPath);

protected:
	//------------------------------------------------------------------------
	void Release(IUnknown*&);
} /*TWin32ShellLink*/;

//============================================================================
NATLIB_NAMESPACE_END

#endif /*!defined(__NATLIB_NWSHLLNK_H__)*/