/****************************************************************************/
/** nlalloc.cpp                                                            **/
/** ---------------------------------------------------------------------- **/
/** User defined memory allocator definitions                              **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** October 20, 1996 by Oleg Kagan                                         **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>
#pragma hdrstop

//============================================================================
#include <nlcmptex.h>
#include <nlalloc.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TAllocationAware::TLastDynamicAllocation implementation
//============================================================================
TAllocationAware::TLastDynamicAllocation::TLastDynamicAllocation() :
	pLock(null), pLast(null)
{
	pLock = &TNatlibCompatibilityCore::GetCore().CreateLock();
} /*TAllocationAware::TLastDynamicAllocation()*/

//----------------------------------------------------------------------------
TAllocationAware::TLastDynamicAllocation::~TLastDynamicAllocation() 
{
	delete pLock; pLock = null;
} /*TAllocationAware::~TLastDynamicAllocation()*/

// TAllocationAware implementation
//============================================================================
void* TAllocationAware::operator new(

	size_t nSize

){
	void* pThis = ::operator new(nSize);
    g_LastDynamicAllocation.Set(pThis);
    return pThis;
} /*TAllocationAware::operator new(size_t)*/

//----------------------------------------------------------------------------
TAllocationAware::TAllocationAware(

	void* pThis

):
	bDynamic(g_LastDynamicAllocation.Get() == pThis)
{
	if (bDynamic) g_LastDynamicAllocation.Clear();
} /*TAllocationAware(void*)*/

//----------------------------------------------------------------------------
TAllocationAware::TLastDynamicAllocation _NATLIB_DECLSPEC_
	TAllocationAware::g_LastDynamicAllocation;

// TAllocator implementation
//============================================================================
TAllocator::TAllocator()
{} /*TAllocator()*/

//----------------------------------------------------------------------------
TAllocator::~TAllocator()
{} /*~TAllocator()*/

// TStdAllocator implementation
//============================================================================
TStdAllocator::TStdAllocator() : nAllocations(0)
{} /*TStdAllocator()*/

//----------------------------------------------------------------------------
TStdAllocator::~TStdAllocator()
{} /*~TStdAllocator()*/

//----------------------------------------------------------------------------
NLOBJECT_CLONE_RESULT(TStdAllocator) TStdAllocator::Clone() const 
{
	TStdAllocator* pCopy; NLEXCEPT_CLASS_NEW(TStdAllocator(), pCopy);
	return pCopy;
} /*TStdAllocator::Clone() const*/

//----------------------------------------------------------------------------
void* TStdAllocator::Allocate(

	size_t nRequested, 
	size_t& r_nAllocated
	
){
	r_nAllocated = null;
	char* pAllocated; NLEXCEPT_CLASS_NEW(char[nRequested], pAllocated);
	r_nAllocated = nRequested; 
	++nAllocations;
	return pAllocated;
} /*TStdAllocator::Allocate(size_t, size_t&)*/

//----------------------------------------------------------------------------
void TStdAllocator::Deallocate(

	void* pMemory
	
){
	if (pMemory) delete[] (char*)pMemory;
	--nAllocations;
} /*TStdAllocator::Deallocate(void*)*/

// TStdFixedAllocator implementation
//============================================================================
TStdFixedAllocator::TStdFixedAllocator(

	size_t nAllocationSize /*= 0*/
	
): 
	nAllowedSize(nAllocationSize)
{} /*TStdFixedAllocator(size_t)*/

//----------------------------------------------------------------------------
TStdFixedAllocator::~TStdFixedAllocator()
{} /*~TStdFixedAllocator()*/

//----------------------------------------------------------------------------
NLOBJECT_CLONE_RESULT(TStdFixedAllocator) TStdFixedAllocator::Clone() const 
{
	TStdFixedAllocator* pCopy; 
	NLEXCEPT_CLASS_NEW(TStdFixedAllocator(), pCopy);
	return pCopy;
} /*TStdFixedAllocator::Clone() const*/

//----------------------------------------------------------------------------
void* TStdFixedAllocator::Allocate(

	size_t nRequested,
	size_t& r_nAllocated

){
	if (nRequested != nAllowedSize) 
		throw TXInvalidAllocationSize(THIS_CLASS);
	return TParent::Allocate(nRequested, r_nAllocated);
} /*TStdFixedAllocator::Allocate(size_t, size_t&)*/

//============================================================================
NATLIB_NAMESPACE_END


