/****************************************************************************/
/** nlavltre.cpp                                                           **/
/** ---------------------------------------------------------------------- **/
/** Binary tree implementation based on AVL tree data structure            **/
/** (See N.Wirth "Algorithms and Data Structures")                         **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** October 22, 1996 by Oleg Kagan                                         **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>
#pragma hdrstop

//============================================================================
#include <nlavltre.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TAvlTreeSubdelete implementation
//============================================================================
bool TAvlTreeSubdelete::BalanceL(

	TBinaryTreeNode*& r_pNode

){
	// left part smaller
	//-------------------
	bool bRet = true;
	TNode* p1;
    TNode* p2;

	switch (r_pNode->Balance) {
		case TNode::balanceLeft:
        	r_pNode->Balance = TNode::balanceEqual;
            break;

		case TNode::balanceEqual:
			r_pNode->Balance = TNode::balanceRight;
            bRet = false;
            break;

		case TNode::balanceRight:
			p1 = r_pNode->pRight;
			TNode::TBalance b1 = p1->Balance;
			if (b1 >= TNode::balanceEqual) {
				// Single RR-Rotate
				//-----------------
				r_pNode->pRight = p1->pLeft; p1->pLeft = r_pNode;
				if (b1 == TNode::balanceEqual) {
					r_pNode->Balance = TNode::balanceRight;
                    p1->Balance = TNode::balanceLeft;
					bRet = false;
				}
				else {
					r_pNode->Balance = p1->Balance = TNode::balanceEqual;
				} /*if*/
				r_pNode = p1;
			}
			else {
				// Double RL-Rotate
				//------------------
				p2 = p1->pLeft;
				TNode::TBalance b2 = p2->Balance;
				p1->pLeft = p2->pRight; p2->pRight = p1;
				r_pNode->pRight = p2->pLeft; p2->pLeft = r_pNode;
				r_pNode->Balance = (
					(b2 == TNode::balanceRight) ?
						TNode::balanceLeft : TNode::balanceEqual
                ) /*r_pNode->Balance*/;
				p1->Balance = (
					(b2 == TNode::balanceLeft) ?
						TNode::balanceRight : TNode::balanceEqual
                ) /*p1->Balance*/;
				r_pNode = p2;
				p2->Balance = TNode::balanceEqual;
			} /*if*/
	} /*switch (r_pNode->Balance)*/
	return bRet;
} /*TAvlTreeSubdelete::BalanceL(TNode*&)*/

//----------------------------------------------------------------------------
bool TAvlTreeSubdelete::BalanceR(

	TBinaryTreeNode*& r_pNode

){
	// Right part smaller
	//--------------------
	bool bRet = true;
	TNode* p1;
    TNode* p2;

	switch (r_pNode->Balance) {
		case TNode::balanceRight :
			r_pNode->Balance = TNode::balanceEqual;
            break;

		case TNode::balanceEqual :
			r_pNode->Balance = TNode::balanceLeft;
            bRet = false;
            break;

		case TNode::balanceLeft :
			p1 = r_pNode->pLeft;
			TNode::TBalance b1 = p1->Balance;
			if (b1 <= TNode::balanceEqual) {
				// Single LL-Rotate
				//-----------------
				r_pNode->pLeft = p1->pRight; p1->pRight = r_pNode;
				if (b1 == TNode::balanceEqual) {
					r_pNode->Balance = TNode::balanceLeft;
                    p1->Balance = TNode::balanceRight;
					bRet = false;
				}
				else {
					r_pNode->Balance = p1->Balance = TNode::balanceEqual;
				} /*if*/
				r_pNode = p1;
			}
			else {
				// Double LR-Rotate
				//------------------
				p2 = p1->pRight;
				TNode::TBalance b2 = p2->Balance;
				p1->pRight = p2->pLeft; p2->pLeft = p1;
				r_pNode->pLeft = p2->pRight; p2->pRight = r_pNode;
				r_pNode->Balance = (
					(b2 == TNode::balanceLeft) ?
						TNode::balanceRight : TNode::balanceEqual
                ) /*r_pNode->Balance*/;
				p1->Balance = (
					(b2 == TNode::balanceRight) ?
						TNode::balanceLeft : TNode::balanceEqual
                ) /*p1->Balance*/;
				r_pNode = p2; p2->Balance = TNode::balanceEqual;
			} /*if*/
	} /*switch (p->nBalance)*/

	return bRet;
} /*TAvlTreeSubdelete::BalanceR(TNode*&)*/

//----------------------------------------------------------------------------
bool TAvlTreeSubdelete::operator()(

	TBinaryTreeNode*& r_pr

){
	bool bBalanceChanged;
	if (r_pr->pRight) {
		if ((bBalanceChanged = (*this)(r_pr->pRight)) != 0) {
			bBalanceChanged = BalanceR(r_pr);
		} /*if*/
	}
	else {
		// Save pointer to the moving node
		//---------------------------------
		TNode* pNodeToMove = r_pr;

		// Left subtree to this place
		//----------------------------
		r_pr = r_pr->pLeft;

		// Move to place of the deleting node
		//------------------------------------
		pNodeToMove->pLeft = r_pNode->pLeft;
		pNodeToMove->pRight = r_pNode->pRight;
		pNodeToMove->Balance = r_pNode->Balance;
		r_pNode = pNodeToMove;

		bBalanceChanged = true;
	} /*if*/
	return bBalanceChanged;
} /*TAvlTreeSubdelete::operator()(TNode*&)*/

//============================================================================
NATLIB_NAMESPACE_END
