/****************************************************************************/
/** nlcmptex.h                                                             **/
/** ---------------------------------------------------------------------- **/
/** Compatibility enhanced tools                                           **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** August 15, 1998 by Oleg Kagan                                          **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>
#pragma hdrstop

#include <windows.h>

// Natlib Basics includes
//============================================================================
#include <nlcmptex.h>
#include <nlstring.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN;

// TNatlibCompatibilityCore implementation
//============================================================================
TNatlibCompatibilityCore& TNatlibCompatibilityCore::GetCore()
{
	extern TNatlibCompatibilityCore& NATLIB_GET_COMPATIBILITY_CORE ();	
	return NATLIB_GET_COMPATIBILITY_CORE ();	
} /*TNatlibCompatibilityCore::GetCore()*/

// TStdCompatibilityCore
//============================================================================
class _NATLIB_DECLSPEC_ TStdCompatibilityCore : 
	public TNatlibCompatibilityCore 
{
public:
	//------------------------------------------------------------------------
	size_t WideToStd(const wchar_t*, char*, size_t);

	//------------------------------------------------------------------------
	size_t StdToWide(const char*, wchar_t*, size_t);

	//------------------------------------------------------------------------
	TLock& CreateLock();
} /*TStdCompatibilityCore*/;

// TStdCompatibilityCore
//============================================================================
size_t TStdCompatibilityCore::WideToStd(
	
	const wchar_t* c_wszSource,  
	char* szDestination, 
	size_t nToConvert 
	
){
	if (!c_wszSource || (szDestination && !nToConvert)) 
		throw TXInvalidParameter(THIS_CLASS);

	size_t nCopied = 0;
	char a_cMbResult[4];

	if ((MB_CUR_MAX + 1) > sizeof(a_cMbResult))
		throw basic_TNatlibException(THIS_PLACE);

	for (size_t i = 0; i < nToConvert; ++i) {
		if (c_wszSource[i]) {
			int iResult = wctomb(a_cMbResult, c_wszSource[i]);
			if (iResult == -1) return size_t(-1);
			if (szDestination) {
				for (size_t j = 0; j < size_t(iResult); ++j) {
					*(szDestination++) = a_cMbResult[j];
				} /*for (size_t j)*/
			} /*if*/
			nCopied += iResult;
		}
		else {
			if (szDestination) {
				*(szDestination++) = char(0);
			} /*if*/
			++nCopied;
		} /*iff*/
	} /*for (size_t i)*/

	return nCopied;
} /*TStdCompatibilityCore::WideToStd(const wchar_t*, size_t, char*, size_t)*/

//----------------------------------------------------------------------------
size_t TStdCompatibilityCore::StdToWide(

	const char* c_szSource,
	wchar_t* wszDestination,
	size_t nToConvert

){
	if (!c_szSource || (wszDestination && !nToConvert)) 
		throw TXInvalidParameter(THIS_CLASS);

	size_t i = 0; size_t iLast = nToConvert - 1;
	while (i < nToConvert) {
		wchar_t wcResult;
		int iResult;

		if (c_szSource[i]) {
			iResult = mbtowc(	
				&wcResult, c_szSource + i, (i != iLast) ? MB_CUR_MAX : 1
			) /*mbtowc*/;
			if (iResult == -1) return -1;
		} 
		else {
			wcResult = wchar_t(0); iResult = 1;
		} /*iff*/

		if (wszDestination) *(wszDestination++) = wcResult;
		i += iResult;
	} /*while*/
	return nToConvert;
} /*TStdCompatibilityCore::StdToWide(const char*, size_t, wchar_t*, size_t)*/

//----------------------------------------------------------------------------
TLock& TStdCompatibilityCore::CreateLock()
{
	TLock* pLock;
	NLEXCEPT_CLASS_NEW(TLock(), pLock);
	return *pLock;
} /*TStdCompatibilityCore::CreateLock()*/

//============================================================================
NATLIB_NAMESPACE_END

//============================================================================
NATLIB_NAMESPACE_MEMBER(TNatlibCompatibilityCore)& 
NLCMPTEX_StdGetCompatibilityCore()
{
	static NATLIB_NAMESPACE_MEMBER(TStdCompatibilityCore) 
		StdCompatibilityCore;
	return StdCompatibilityCore;
} /*NLCMPTEX_StdGetCompatibilityCore*/


