/****************************************************************************/
/** nlexcept.cpp                                                           **/
/** ---------------------------------------------------------------------- **/
/** Definition of base classes for Errors Handling                         **/
/** and Emergency Information Passing                                      **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** June 16, 1996 by Oleg Kagan                                            **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>
#pragma hdrstop

//============================================================================
#include <nlexcept.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// Used while converting unsigned long to TStdString
//============================================================================
#define NLEXCEPT_ULONG_DEC_PLACES 128

// TNatlibException implementation
//============================================================================
const char* const TNatlibException::c_szWhyWhereSeparator = " at ";
const char* const TNatlibException::c_szStartDelimiter    = "(";
const char* const TNatlibException::c_szEndDelimiter      = ")";
const char* const TNatlibException::c_szSeparator         = ", ";

//----------------------------------------------------------------------------
TNatlibException::TNatlibException(

	const TSourceLocation& c_Location,
	const TStdString& c_sExplanation /*= TStdString()*/

) _NATLIB_THROW_NONE_ :
	TSourceLocation(c_Location), sExplanation(c_sExplanation)
{} /*TNatlibException(const TSourceLocaion&)*/

//----------------------------------------------------------------------------
TNatlibException::TNatlibException(

	const TNatlibException& c_Exception

) _NATLIB_THROW_NONE_ :
	TSourceLocation(c_Exception), sExplanation(c_Exception.Explanation())
{} /*TNatlibException(const TNatlibException&)*/

//----------------------------------------------------------------------------
void TNatlibException::Rethrow() _NATLIB_THROW1_(TNatlibException)
{
	throw /**this*/;  // *this is commented to assure that throwing exception
					  // will be the same type that catched
} /*TNatlibException::Re_NATLIB_THROW_NONE_*/

//----------------------------------------------------------------------------
void TNatlibException::Rethrow(

	const TSourceLocation& c_Location

) _NATLIB_THROW1_(TNatlibException) {
	TSourceLocation::operator=(c_Location);
	throw /**this*/;  // *this is commented to assure that throwing exception
					  // will be the same type that catched
} /*TNatlibException::Rethrow(const TSourceLocation& = c_None)*/

//----------------------------------------------------------------------------
TStdString TNatlibException::Explanation() const _NATLIB_THROW_NONE_
{
	return sExplanation;
} /*TNatlibException::Explanation() const*/

//----------------------------------------------------------------------------
TStdString TNatlibException::Why() const _NATLIB_THROW_NONE_
{
	TStdString sException(typeid(*this).name());
	TStdString sWhy(sException);
	TStdString sCompleteExplanation = Explanation();
	if (sCompleteExplanation.length()) {
		sWhy += c_szStartDelimiter;
		sWhy += sCompleteExplanation;
		sWhy += c_szEndDelimiter;
	} /*if*/
    sWhy += c_szWhyWhereSeparator;
	sWhy += OptimizedWhere();

	return sWhy;
} /*TNatlibException::Why()*/

//----------------------------------------------------------------------------
TStdString TNatlibException::OptimizedWhere() const _NATLIB_THROW_NONE_
{
	TStdString sException(typeid(*this).name());

	TStdString sWhere;
	if (sException.find(TSourceLocation::sClass) == 0) {
		sWhere = TSourceLocation(sSource, nLine).Where();
	}
	else {
		sWhere = TSourceLocation::Where();
	} /*iff*/

	return sWhere;
} /*TNatlibException::OptimizedWhere() const _NATLIB_THROW_NONE_*/

//----------------------------------------------------------------------------
TNatlibException& TNatlibException::operator=(

	const TNatlibException& c_Exception

) _NATLIB_THROW_NONE_ {
	TSourceLocation::operator=(c_Exception);
	sExplanation = c_Exception.Explanation();
	return *this;
} /*TNatlibException::operator=(const TNatlibException&)*/

//----------------------------------------------------------------------------
TNatlibException::operator TStdString() const _NATLIB_THROW_NONE_
{
	return Why();
} /*TNatlibException::operator TStdString() const*/

// TXLowMemory implementation
//============================================================================
TXLowMemory::TXLowMemory(

	const TSourceLocation& c_Location,
	const size_t c_nRequested,
	const TStdString& c_sExplanation

) _NATLIB_THROW_NONE_ :
	TNatlibException(c_Location, c_sExplanation), nRequested(c_nRequested)
{} /*TXLowMemory(const TSourceLocation&, size_t = 0)*/

//----------------------------------------------------------------------------
TXLowMemory::TXLowMemory(

	const TXLowMemory& c_XLowMemory

) _NATLIB_THROW_NONE_ :
	TNatlibException(c_XLowMemory), nRequested(c_XLowMemory.Requested())
{} /*TXLowMemory(const TXLowMemory&)*/

//----------------------------------------------------------------------------
TStdString TXLowMemory::Explanation() const _NATLIB_THROW_NONE_
{
	TStdString sCompleteExplanation = TNatlibException::Explanation();
	if (Requested()) {
		if (sCompleteExplanation.length()) 
			sCompleteExplanation += c_szSeparator;

		// Add requested memory size
		//---------------------------
		{
			char szRequested[NLEXCEPT_ULONG_DEC_PLACES + 1];
			sprintf(szRequested, "%lu", (unsigned long)nRequested);
			sCompleteExplanation += szRequested;
		} /*Add requested memory size*/
	} /*if*/

	return sCompleteExplanation;
} /*TXLowMemory::Explanation() const*/

//----------------------------------------------------------------------------
size_t TXLowMemory::Requested() const _NATLIB_THROW_NONE_
{
	return nRequested;
} /*TXLowMemory::Requested() const*/

// TXBadCast implementation
//============================================================================
TXBadCast::TXBadCast(

	const TSourceLocation& c_Location,
    const type_info& c_Source,
    const type_info& c_Target,
	const TStdString& c_sExplanation

) _NATLIB_THROW_NONE_ :
	TNatlibException(c_Location, c_sExplanation),
	pc_Source(&c_Source), pc_Target(&c_Target)
{} /*TXBadCast(const TSourceLocation&, const type_info&,...)*/

//----------------------------------------------------------------------------
TXBadCast::TXBadCast(

	const TXBadCast& c_XBadCast

) _NATLIB_THROW_NONE_ :
	TNatlibException(c_XBadCast),
	pc_Source(&c_XBadCast.SourceTypeId()),
    pc_Target(&c_XBadCast.TargetTypeId())
{} /*TXBadCast(const TXBadCast&)*/

//----------------------------------------------------------------------------
TStdString TXBadCast::Explanation() const _NATLIB_THROW_NONE_
{
	TStdString sCompleteExplanation = TNatlibException::Explanation();
	if (sCompleteExplanation.length()) sCompleteExplanation += c_szSeparator;
    sCompleteExplanation += SourceTypeId().name();
    sCompleteExplanation += c_szSeparator;
    sCompleteExplanation += TargetTypeId().name();
	return sCompleteExplanation;
} /*TXBadCast::Explanation() const*/

//----------------------------------------------------------------------------
const type_info& TXBadCast::SourceTypeId() const _NATLIB_THROW_NONE_
{
	return *pc_Source;
} /*TXBadCast::SourceTypeId()*/

//----------------------------------------------------------------------------
const type_info& TXBadCast::TargetTypeId() const _NATLIB_THROW_NONE_
{
	return *pc_Target;
} /*TXBadCast::TargetTypeId()*/

//============================================================================
NATLIB_NAMESPACE_END
