/****************************************************************************/
/** nllstack.cpp                                                           **/
/** ---------------------------------------------------------------------- **/
/** Natlib Lite Stack (unsafe but efficient stack implementation)          **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** June 11, 1998 by Oleg Kagan                                            **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>
#pragma hdrstop

//============================================================================
#include <nllstack.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TLiteStack implementation
//============================================================================
TLiteStack::TLiteStack(

	const TAllocator& c_Allocator /*= TStdAllocator()*/

):
	pHead(null), nSize(0), pAllocator(null)
{
	pAllocator = CloneObject(c_Allocator);
} /*TLiteStack(const TAllocator& = TStdAllocator())*/

//----------------------------------------------------------------------------
TLiteStack::~TLiteStack()
{
	while (nSize) Pop();
	delete pAllocator; pAllocator = null;
} /*~TLiteStack()*/

//----------------------------------------------------------------------------
void TLiteStack::AssertValid() const
{
	TParent::AssertValid();
	if (!nSize ^ !pHead) 
		throw TXInvalidSize(THIS_CLASS);
} /*TLiteStack::AssertValid() const*/

//----------------------------------------------------------------------------
void* TLiteStack::Push(

	const void* pc_Buffer,
	size_t nBufferSize

){
	TRecord* pRecord = (TRecord*)(
		pAllocator->Allocate(sizeof(TRecord) - 1 + nBufferSize)
	) /*pRecord*/;
	if (!pRecord) 
		throw TXUnableToAllocateRecord(THIS_CLASS);
	
	pRecord->pNext = pHead;
	pHead = pRecord;
	++nSize;

	if (pc_Buffer) {
		memcpy(pRecord->a_cBuffer, pc_Buffer, nBufferSize);
	} /*if*/
	return pRecord->a_cBuffer;
} /*TLiteStack::Push(size_t, void* = null)*/

//----------------------------------------------------------------------------
void TLiteStack::Pop()
{
	if (!pHead) 
		throw TXEndOfStack(THIS_CLASS);
	TRecord* pRecord = pHead;
	pHead = pRecord->pNext;
	--nSize;
	pAllocator->Deallocate(pRecord);
} /*TLiteStack::Pop()*/

//============================================================================
NATLIB_NAMESPACE_END
