/****************************************************************************/
/** nlsafeh.h                                                              **/
/** ---------------------------------------------------------------------- **/
/** Safe object handling implementation                                    **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** Match 21, 1997 by Oleg Kagan                                           **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>
#pragma hdrstop

//============================================================================
#include <nlsafeh.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TSafeHandleAllowed implementation
//============================================================================
TSafeHandleAllowed::~TSafeHandleAllowed()
{
	InvalidateHandles();
} /*~TSafeHandleAllowed()*/

//----------------------------------------------------------------------------
void TSafeHandleAllowed::InvalidateHandles()
{
	if (pDescriptor) {
    	pDescriptor->pObject = null;
        pDescriptor = null;
		EvHandleDeleted();
    } /*if*/
} /*TSafeHandleAllowed::InvalidateHandles()*/

//----------------------------------------------------------------------------
TSafeHandleAllowed::TDescriptor& TSafeHandleAllowed::GetDescriptor() const
{
	if (!pDescriptor) {
    	const_cast<TSafeHandleAllowed*>(this)->pDescriptor = 
			AllocateDescriptor();
	    if (!pDescriptor) 
			throw TXUnableToCreateDescriptor(THIS_CLASS);
    } /*if*/
    return *pDescriptor;
} /*TSafeHandleAllowed::GetDescriptor() const*/

//----------------------------------------------------------------------------
TSafeHandleAllowed::TDescriptor*
TSafeHandleAllowed::AllocateDescriptor() const
{
	TDescriptor* pNewDescriptor;
    NLEXCEPT_CLASS_NEW(TDescriptor(*this), pNewDescriptor);
    return pNewDescriptor;
} /*TSafeHandleAllowed::AllocateDescriptor() const*/

// TSafeHandleBase implementation
//============================================================================
TSafeHandleBase::~TSafeHandleBase()
{
	Unassign();
} /*TSafeHandleBase::~TSafeHandleBase()*/

//----------------------------------------------------------------------------
void TSafeHandleBase::Assign(const TSafeHandleAllowed& c_Target)
{
   	Unassign();
   	TDescriptor* pTargetDescriptor = &c_Target.GetDescriptor();
	pTargetDescriptor->RegisterReference();
    pDescriptor = pTargetDescriptor;
} /*TSafeHandleBase::Assign(const TSafeHandleAllowed&)*/

//----------------------------------------------------------------------------
void TSafeHandleBase::Unassign()
{
    if (pDescriptor) {
        TDescriptor* pDescriptorToUnassign = pDescriptor;
        pDescriptor = null;
        pDescriptorToUnassign->UnregisterReference();
    } /*if*/
} /*TSafeHandleBase::Unassign()*/

//----------------------------------------------------------------------------
void TSafeHandleBase::Let(

	const TSafeHandleBase& c_hTarget

){
	Unassign();
	if (c_hTarget.pDescriptor) {
    	c_hTarget.pDescriptor->RegisterReference();
    	pDescriptor = c_hTarget.pDescriptor;
    } /*if*/
} /*TSafeHandleBase::Let(const TSafeHandleBase&)*/

//============================================================================
NATLIB_NAMESPACE_END

