/****************************************************************************/
/** nlsrcloc.cpp                                                           **/
/** -----------------------------------------------------------------------**/
/** Provides basic ability to represent source locations                   **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** June 14, 1996 by Oleg Kagan                                            **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>
#pragma hdrstop

// Natlib Basics includes
//============================================================================
#include <nlsrcloc.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TSourceLocation implementation
//============================================================================
const char* const TSourceLocation::c_szUnknownLocation = "Unknown location";
const char* const TSourceLocation::c_szStartDelimiter  = "[";
const char* const TSourceLocation::c_szEndDelimiter    = "]";
const char* const TSourceLocation::c_szSeparator       = ":";
const unsigned   TSourceLocation::c_nLineCharSize      = 10;
const char* const TSourceLocation::c_szNoSource        = "";
const unsigned   TSourceLocation::c_nNoLine            = 0;
const char* const TSourceLocation::c_szNoClass         = "";

//----------------------------------------------------------------------------
const TSourceLocation TSourceLocation::c_None = (
	TSourceLocation(
		TSourceLocation::c_szNoSource, TSourceLocation::c_nNoLine, 
		TSourceLocation::c_szNoClass
	) /*TSourceLocation*/
) /*TSourceLocation::c_None*/;

//----------------------------------------------------------------------------
TSourceLocation::TSourceLocation(

	const TStdString& c_sSource,
	const unsigned c_nLine,
	const TStdString& c_sClass

):
	sSource(c_sSource),
	nLine(c_nLine),
	sClass(c_sClass)
{} /*TSourceLocation(const TStdString&, const unsigned, const TStdString&)*/

//----------------------------------------------------------------------------
TSourceLocation::TSourceLocation(

	const TSourceLocation& c_Location

):
	sSource(c_Location.sSource),
	nLine(c_Location.nLine),
	sClass(c_Location.sClass)
{} /*TSourceLocation(const TSourceLocation&)*/

//----------------------------------------------------------------------------
TStdString TSourceLocation::Where() const
{
	TStdString sWhere; 
	sWhere.reserve(sSource.length() + sClass.length() + 10);

	// Is source name available
	//--------------------------
	if (sSource == c_szNoSource) {
    	sWhere += c_szStartDelimiter;
        sWhere += c_szUnknownLocation;
    	sWhere += c_szEndDelimiter;
    }
    else {
    	sWhere += c_szStartDelimiter;
        sWhere += sSource;

		if (nLine != c_nNoLine) { 	// Is Line number available
			char szLine[c_nLineCharSize];
			sprintf(szLine, "%u", nLine); 
			sWhere += c_szSeparator;
            sWhere += szLine;
		} /*if*/

		if (sClass.length()) { // Is class info available
			sWhere += c_szSeparator;
			sWhere += sClass;
		} /*if*/

		sWhere += c_szEndDelimiter;
    } /*iff*/
	return sWhere;
} /*TSourceLocation::Where()*/

//----------------------------------------------------------------------------
TSourceLocation& TSourceLocation::operator=(

	const TSourceLocation& c_Location

){
	sSource = c_Location.sSource;
	nLine = c_Location.nLine;
	sClass = c_Location.sClass;
	return *this;
} /*TSourceLocation::operator=(const TSourceLocation&)*/

//----------------------------------------------------------------------------
TSourceLocation::operator TStdString() const
{
	return Where();
} /*TSourceLocation::operator TStdString*/

//============================================================================
NATLIB_NAMESPACE_END
