/****************************************************************************/
/** nlstring.cpp                                                           **/
/** -----------------------------------------------------------------------**/
/** Some utilities for string management                                   **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** August 20, 1997 by Oleg Kagan                                          **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>
#pragma hdrstop

// Standard includes
//============================================================================
#include <stdarg.h>

// Natlib Basics includes
//============================================================================
#include <nlcmptex.h>
#include <nlstring.h>
#include <nlautopt.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

//============================================================================
size_t _NATLIB_DECLSPEC_ nFormatDefaultBufferSize = 4096;

//============================================================================
TStdString _NATLIB_DECLSPEC_ Format(

    const char* c_szFormat,
    ...

){
	TAutoPointerTo<char> p_cBuffer((char*)null, true); {
		NLEXCEPT_NEW(char[nFormatDefaultBufferSize + 1], p_cBuffer);
	} /*p_cBuffer*/;

 	va_list ArgList; va_start(ArgList, c_szFormat);
	vsprintf((char*)p_cBuffer, c_szFormat, ArgList);
	va_end(ArgList);

	return TStdString((char*)p_cBuffer);
} /*Format(const char*, ...)*/

//============================================================================
TStdString _NATLIB_DECLSPEC_ Format(

    size_t nBufferSize,
    const char* c_szFormat,
    ...

){
	TAutoPointerTo<char> p_cBuffer((char*)null, true); {
	    NLEXCEPT_NEW(char[nBufferSize + 1], p_cBuffer);
    } /*p_cBuffer*/;

	va_list ArgList; va_start(ArgList, c_szFormat);
 	vsprintf((char*)p_cBuffer, c_szFormat, ArgList);
	va_end(ArgList);

	return TStdString((char*)p_cBuffer);
} /*Format(size_t, const char*, ...)*/

//============================================================================
size_t _NATLIB_DECLSPEC_ SetFormatBufferSize(

	size_t nNewSize

){
	size_t nPreviousSize = nFormatDefaultBufferSize;
    nFormatDefaultBufferSize = nNewSize;
    return nPreviousSize;
} /*SetFormatBufferSize(nNewSize)*/

//============================================================================
size_t _NATLIB_DECLSPEC_ GetFormatBufferSize()
{
	return nFormatDefaultBufferSize;
} /*GetFormatBufferSize()*/

//============================================================================
const char const c_szNull[] = {char(0), char(0)};
const wchar_t const c_wszNull[] = {wchar_t(0), wchar_t(0)};

//----------------------------------------------------------------------------
const TStdString c_sNull;

//============================================================================
TStdString ToStdString(

	const char* c_szSource,
	size_t nSize /*= 0*/,
	bool /*bMultibyte = true*/

){
	if (!nSize) nSize = SzLength(c_szSource);
	return TStdString(c_szSource, nSize);
} /*ToStdString(const char*, size_t, bool)*/

//----------------------------------------------------------------------------
TStdString ToStdString(

	const wchar_t* c_wszSource,
	size_t nSize /*= 0*/,
	bool bMultibyte /*= true*/

){
	TStdString sResult;
	if (!nSize) nSize = SzLength(c_wszSource);
	if (!nSize) return sResult;

	if (bMultibyte) {
		TNatlibCompatibilityCore& r_Core = (
			TNatlibCompatibilityCore::GetCore()
		) /*r_Core*/;
		size_t nResultSize = r_Core.WideToStd(c_wszSource, null, nSize);
		if (nResultSize == size_t(-1))
			throw TXInvalidParameter(THIS_PLACE);

		// Borland's ClassLib string class doesn't support some standard 
		// features of STL string like data() method
		//---------------------------------------------------------------
#if defined(__BCPLUSPLUS__) && !defined(_NATLIB_USE_STL_STRING_CLASS_)
		TAutoPointerTo<char> ha_cResult(null, true);
		NLEXCEPT_NEW(char[nResultSize + 1], ha_cResult);
		r_Core.WideToStd(c_wszSource, (char*)ha_cResult, nSize);
		sResult = TStdString((const char*)ha_cResult, nResultSize);
#else
		sResult.reserve(nResultSize + 1);
		sResult.resize(nResultSize);
		r_Core.WideToStd(
			c_wszSource, const_cast<char*>(sResult.data()), nSize
		) /*r_Core.WideToStd*/;
#endif /*defined(__BCPLUSPLUS__) && !defined(_NATLIB_USE_STL_STRING_CLASS_)*/
	}
	else {
		sResult.reserve(nSize + 1);
		sResult.resize(nSize);
		for (size_t i = 0; i < nSize; ++i) 
			sResult[i] += char(c_wszSource[i]);
	} /*iff*/
	return sResult;
} /*ToStdString(const wchar_t, size_t, bool)*/

//----------------------------------------------------------------------------
TStdString ToStdString(

	const TStdString& c_sSource,
	bool /*bMultibyte = true*/

){
	return c_sSource;
} /*ToStdString(const TStdString&, bool)*/

// Wide string tools
//============================================================================
#if !defined(_NATLIB_NO_WIDE_STRINGS_)

//============================================================================
TWideString _NATLIB_DECLSPEC_ Format(

    const wchar_t* c_wszFormat,
    ...

){
	TAutoPointerTo<wchar_t> p_wcBuffer((wchar_t*)null, true); {
	    NLEXCEPT_NEW(wchar_t[nFormatDefaultBufferSize + 1], p_wcBuffer);
    } /*p_cBuffer*/;

	va_list ArgList; va_start(ArgList, c_wszFormat);
	vswprintf((wchar_t*)p_wcBuffer, c_wszFormat, ArgList);
	va_end(ArgList);

	return TWideString((wchar_t*)p_wcBuffer);
} /*Format(const wchar_t*, ...)*/

//============================================================================
TWideString _NATLIB_DECLSPEC_ Format(

    size_t nBufferSize,
    const wchar_t* c_wszFormat,
    ...

){
	TAutoPointerTo<wchar_t> p_wcBuffer((wchar_t*)null, true); {
	    NLEXCEPT_NEW(wchar_t[nBufferSize + 1], p_wcBuffer);
    } /*p_cBuffer*/;

	va_list ArgList; va_start(ArgList, c_wszFormat);
	vswprintf((wchar_t*)p_wcBuffer, c_wszFormat, ArgList);
	va_end(ArgList);

	return TWideString((wchar_t*)p_wcBuffer);
} /*Format(size_t, const wchar_t*, ...)*/

//============================================================================
TWideString c_wsNull;

//============================================================================
TStdString ToStdString(

	const TWideString& c_wsSource,
	bool bMultibyte /*= true*/

){
	return ToStdString(c_wsSource.data(), c_wsSource.length());
} /*ToStdString(const TStdWideString&, bool)*/

//----------------------------------------------------------------------------
TWideString ToWideString(

	const wchar_t* c_wszSource,
	size_t nSize /*= 0*/,
	bool /*bMultibyte *= true*/

){
	if (!nSize) nSize = SzLength(c_wszSource);
	return TWideString(c_wszSource, nSize);
} /*ToStdString(const wchar_t*, size_t, bool)*/

//----------------------------------------------------------------------------
TWideString ToWideString(

	const char* c_szSource,
	size_t nSize /*= 0*/,
	bool bMultibyte /*= true*/

){
	TWideString wsResult;
	if (!nSize) nSize = SzLength(c_szSource);
	if (!nSize) return wsResult;
	
	if (bMultibyte) {
		TNatlibCompatibilityCore& r_Core = (
			TNatlibCompatibilityCore::GetCore()
		) /*r_Core*/;
		size_t nResultSize = r_Core.StdToWide(c_szSource, null, nSize);
		if (nResultSize == size_t(-1))
			throw TXInvalidParameter(THIS_PLACE, c_szSource);

		// Borland's ClassLib string class doesn't support some standard 
		// features of STL string like data() method
		//---------------------------------------------------------------
#if defined(__BCPLUSPLUS__) && !defined(_NATLIB_USE_STL_STRING_CLASS_)
		TAutoPointerTo<wchar_t> ha_wcResult(null, true);
		NLEXCEPT_NEW(wchar_t[nResultSize + 1], ha_wcResult);
		r_Core.StdToWide(c_szSource, (wchar_t*)ha_wcResult, nSize);
		wsResult = TWideString((const wchar_t*)ha_wcResult, nResultSize);
#else
		wsResult.reserve(nResultSize + 1);
		wsResult.resize(nResultSize);
		r_Core.StdToWide(c_szSource, (wchar_t*)wsResult.data(), nSize);
#endif /*defined(__BCPLUSPLUS__) && !defined(_NATLIB_USE_STL_STRING_CLASS_)*/
	}
	else {
		wsResult.reserve(nSize + 1);
		wsResult.resize(nSize);
		for (size_t i = 0; i < nSize; ++i) 
			wsResult[i] += wchar_t(c_szSource[i]);
	} /*iff*/

	return wsResult;
} /*ToWideString(const char*, size_t, bool)*/

//----------------------------------------------------------------------------
TWideString ToWideString(

	const TStdString& c_sSource, 
	bool bMultibyte /*= true*/

){
	return ToWideString(c_sSource.data(), c_sSource.length(), bMultibyte);
} /*ToWideString(const TStdString&, bool)*/

//----------------------------------------------------------------------------
TWideString ToWideString(

	const TWideString& c_wsSource,
	bool /*bMultibyte = true*/

){
	return c_wsSource;
} /*ToWideString(const TWideString&, bool)*/

//============================================================================
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

//============================================================================
wchar_t* SzToWide(

	const char* c_szText,
	wchar_t* wszText /*= null*/,
	bool bMultibyte /*= true*/

){
	if (!c_szText) c_szText = c_szNull;

	if (!*c_szText) {
		if (!wszText) {NLEXCEPT_NEW(wchar_t[2], wszText);}
		*wszText = wchar_t(0);
	}
	else {
		TNatlibCompatibilityCore& r_Core = (
			TNatlibCompatibilityCore::GetCore()
		) /*r_Core*/;

		if (!wszText) {
			size_t nResultLength = (
				bMultibyte ? 
					r_Core.StdToWide(c_szText, null, SzLength(c_szText)) : 
					SzLength(c_szText)
			) /*nResultLength*/; 
			if (nResultLength == size_t(-1)) 
				throw TXInvalidParameter(THIS_PLACE, c_szText);
			NLEXCEPT_NEW(wchar_t[nResultLength + 1], wszText);
		} /*if*/

		if (bMultibyte) {
			r_Core.StdToWide(c_szText, wszText, SzLength(c_szText) + 1);
		}
		else {
			register wchar_t* wszCopy = wszText;
			while (*c_szText) *(wszCopy++) = wchar_t(*(c_szText++));
			*wszCopy = wchar_t(0);
		} /*iff*/
	} /*iff*/

	return wszText;
} /*SzToWide(const char*, wchar_t*, bool)*/

//----------------------------------------------------------------------------
wchar_t* SzToWide(

	const wchar_t* c_wszText,
	wchar_t* wszText /*= null*/,
	bool /*bMultibyte = true*/

){
	if (!c_wszText) c_wszText = c_wszNull;
	size_t nLength = SzLength(c_wszText);
	if (!wszText) NLEXCEPT_NEW(wchar_t[nLength + 1], wszText);
	memcpy(wszText, c_wszText, nLength * sizeof(wchar_t)); 
	wszText[nLength] = wchar_t(0);
	return wszText;
} /*SzToWide(const wchar_t*, wchar_t*, bool)*/

//----------------------------------------------------------------------------
char* SzToStd(

	const wchar_t* c_wszText, 
	char* szText /*= null*/,
	bool bMultibyte /*= true*/
	
){
	if (!c_wszText) c_wszText = c_wszNull;

	if (!*c_wszText) {
		if (!szText) {NLEXCEPT_NEW(char[2], szText);} 
		*szText = char(0);
	}
	else {
		TNatlibCompatibilityCore& r_Core = (
			TNatlibCompatibilityCore::GetCore()
		) /*r_Core*/;

		if (!szText) {
			size_t nResultLength = (
				bMultibyte ? 
					r_Core.WideToStd(c_wszText, null, SzLength(c_wszText)) : 
					SzLength(c_wszText)
			) /*nResultLength*/; 
			if (nResultLength == size_t(-1)) 
				throw TXInvalidParameter(THIS_PLACE);
			NLEXCEPT_NEW(char[nResultLength + 1], szText);
		} /*if*/

		if (bMultibyte) {
			r_Core.WideToStd(c_wszText, szText, SzLength(c_wszText) + 1);
		}
		else {
			register char* szCopy = szText;
			while (*c_wszText) *(szCopy++) = char(*(c_wszText++));
			*szCopy = char(0);
		} /*iff*/
	} /*iff*/

	return szText;
} /*SzToStd(const wchar_t*, char*, bool)*/

//----------------------------------------------------------------------------
char* SzToStd(

	const char* c_szText, 
	char* szText /*= null*/,
	bool /*bMultibyte = true*/

){
	if (!c_szText) c_szText = c_szNull;
	size_t nLength = SzLength(c_szText);
	if (!szText) NLEXCEPT_NEW(char[nLength + 1], szText);
	memcpy(szText, c_szText, nLength); 
	szText[nLength] = char(0);
	return szText;
} /*SzToStd(const char*, char*, bool)*/

//----------------------------------------------------------------------------
size_t SzLength(

	const wchar_t* c_wszText
	
){
	if (!c_wszText) c_wszText = c_wszNull;  
	register const wchar_t* c_wszEnd = c_wszText;
	while (*c_wszEnd) ++c_wszEnd;
	return size_t(c_wszEnd - c_wszText);
} /*SzLength(const wchar_t*)*/

//----------------------------------------------------------------------------
size_t SzLength(

	const char* c_szText

){
	if (!c_szText) c_szText = c_szNull;
	return strlen(c_szText);
} /*SzLength(const wchar_t*)*/

//----------------------------------------------------------------------------
int SzCmp(

	const char* c_szSrc,
    const char* c_szDest

){
	if (c_szSrc == c_szDest) return 0;

	if (!c_szSrc) c_szSrc = c_szNull;
    if (!c_szDest) c_szDest = c_szNull;

    if (!*c_szSrc) {
    	if (*c_szDest) return -1;
        else return 0;
    } /*if*/

    if (!*c_szDest) return 1;

    int iRet = strcmp(c_szSrc, c_szDest);
	if (iRet < 0) iRet = -1;
    else if (iRet > 0) iRet = 1;
    return iRet;
} /*SzCmp(const wchar_t*, const wchar_t*)*/

//----------------------------------------------------------------------------
int SzCmp(

	const wchar_t* c_wszSrc,
    const wchar_t* c_wszDest

){
	if (c_wszSrc == c_wszDest) return 0;

	if (!c_wszSrc) c_wszSrc = c_wszNull;
    if (!c_wszDest) c_wszDest = c_wszNull;

	register long iRet;
    while (!(iRet = long(*c_wszSrc) - long(*c_wszDest)) && *c_wszDest) {
    	++c_wszSrc;
        ++c_wszDest;
    } /*while*/

	if (iRet < 0) iRet = -1;
    else if (iRet > 0) iRet = 1;
    return int(iRet);
} /*SzCmp(const wchar_t*, const wchar_t*)*/

//----------------------------------------------------------------------------
int SzCmp(

	const char* c_szSrc,
    const wchar_t* c_wszDest

){
	if (!c_szSrc) c_szSrc = c_szNull;
    if (!c_wszDest) c_wszDest = c_wszNull;

	register long iRet;
    while (!(iRet = long(*c_szSrc) - long(*c_wszDest)) && *c_wszDest) {
    	++c_szSrc;
        ++c_wszDest;
    } /*while*/

	if (iRet < 0) iRet = -1;
    else if (iRet > 0) iRet = 1;
    return int(iRet);
} /*SzCmp(const char*, const wchar_t*)*/

//----------------------------------------------------------------------------
int SzCmp(

	const wchar_t* c_wszSrc,
    const char* c_szDest

){
	return -SzCmp(c_szDest, c_wszSrc);
} /*SzCmp(const wchar_t*, const char*)*/

//----------------------------------------------------------------------------
int SzICmp(

	const char* c_szSrc,
    const char* c_szDest

){
	if (c_szSrc == c_szDest) return 0;

	if (!c_szSrc) c_szSrc = c_szNull;
    if (!c_szDest) c_szDest = c_szNull;

    if (!*c_szSrc) {
    	if (*c_szDest) return -1;
        else return 0;
    } /*if*/

    if (!*c_szDest) return 1;

    int iRet = stricmp(c_szSrc, c_szDest);
	if (iRet < 0) iRet = -1;
    else if (iRet > 0) iRet = 1;
    return iRet;
} /*SzICmp(const wchar_t*, const wchar_t*)*/

#ifndef _NATLIB_NO_WIDE_STRINGS_
//----------------------------------------------------------------------------
int SzICmp(

	const wchar_t* c_wszSrc,
    const wchar_t* c_wszDest

){
	if (c_wszSrc == c_wszDest) return 0;

	if (!c_wszSrc) c_wszSrc = c_wszNull;
    if (!c_wszDest) c_wszDest = c_wszNull;

	register long iRet;
    while (
		!(iRet = long(towlower(*c_wszSrc)) - long(towlower(*c_wszDest))) && 
		*c_wszDest
	){
    	++c_wszSrc;
        ++c_wszDest;
    } /*while*/

	if (iRet < 0) iRet = -1;
    else if (iRet > 0) iRet = 1;
    return int(iRet);
} /*SzICmp(const wchar_t*, const wchar_t*)*/

//----------------------------------------------------------------------------
int SzICmp(

	const char* c_szSrc,
    const wchar_t* c_wszDest

){
	if (!c_szSrc) c_szSrc = c_szNull;
    if (!c_wszDest) c_wszDest = c_wszNull;

	register long iRet;
    while(
		!(iRet = 
			long(towlower(wchar_t(*c_szSrc))) - long(towlower(*c_wszDest))
		) /*iRet*/ && 
		*c_wszDest
	){
    	++c_szSrc;
        ++c_wszDest;
    } /*while*/

	if (iRet < 0) iRet = -1;
    else if (iRet > 0) iRet = 1;
    return int(iRet);
} /*SzICmp(const char*, const wchar_t*)*/

//----------------------------------------------------------------------------
int SzICmp(

	const wchar_t* c_wszSrc,
    const char* c_szDest

){
	return -SzCmp(c_szDest, c_wszSrc);
} /*SzICmp(const wchar_t*, const char*)*/
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

//============================================================================
NATLIB_NAMESPACE_END
