/****************************************************************************/
/** nltime.cpp                                                             **/
/** ---------------------------------------------------------------------- **/
/** Date & Time routines implementation                                    **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** May 10, 1997 by Ilya Marchevsky                                        **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>
#pragma hdrstop

//============================================================================
#include <nltime.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

//============================================================================
template <class T> inline bool NLTIME_InRangeByValue(

	T Value,
    T Low,
    T High

){
	return Low <= Value && Value <= High;
} /*NLTIME_InRangeByValue<T>(T, T, T)*/

//============================================================================
#define NLTIME_DAY_OF_WEEK_GUARD(Day) \
{ \
	if (!NLTIME_InRangeByValue((Day), 1L, c_nDaysInWeek)) \
		throw TXWeekDayOutOfBounds(THIS_PLACE); \
} /*NLTIME_DAY_OF_WEEK_GUARD(Day)*/

//----------------------------------------------------------------------------
#define NLTIME_MONTH_GUARD(Month) \
{ \
	if (!NLTIME_InRangeByValue((Month), 1L, c_nMonthsInYear)) \
		throw TXMonthOutOfBounds(THIS_PLACE); \
} /*NLTIME_MONTH_GUARD(Month)*/

//----------------------------------------------------------------------------
#define NLTIME_DAYS_CALENDAR_GUARD(Days) \
	{if ((Days) <= DaysOnEpoch) throw TXDateBeyondCalculations(THIS_CLASS);}

//----------------------------------------------------------------------------
#define NLTIME_YEAR_CALENDAR_GUARD(Year) \
{ \
	if (!NLTIME_InRangeByValue(nl_long_t(Year), 1753L, 100000L)) \
		throw TXDateBeyondCalculations(THIS_PLACE); \
} /*NLTIME_MONTH_GUARD(Month)*/

// TTime implementation
//============================================================================
const nl_long_t TTime::c_nDaysOnEpoch     = 2440588L;
const nl_long_t TTime::c_nSecondsInDay    = 86400L;
const nl_long_t TTime::c_nMSecondsInDay   = 86400000L;
const nl_long_t TTime::c_nDaysInWeek      = 7;
const nl_long_t TTime::c_nMonthsInYear    = 12;
const nl_long_t TTime::c_nHoursInDay      = 24;
const nl_long_t TTime::c_nMinutesInHour   = 60;
const nl_long_t TTime::c_nSecondsInMinute = 60;

//----------------------------------------------------------------------------
const nl_int_t TTime::cac_nDaysInMonth[TTime::c_nMonthsInYear] = {
	31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
} /*TTime::cac_DaysInMonth*/;
//----------------------------------------------------------------------------
const nl_int_t TTime::cac_nStartOfEachMonth[TTime::c_nMonthsInYear] = {
	1, 32, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335
} /*TTime::cac_nStartOfEachMonth*/;

//----------------------------------------------------------------------------
const char* const TTime::cac_szMonthNames[TTime::c_nMonthsInYear] = {
	"January", "February", "March", "April", "May", "June", "July",
    "August", "September", "October", "November", "December"
} /*TTime::cac_szMonthNames*/;
//----------------------------------------------------------------------------
const char* const TTime::cac_szWeekDayNames[TTime::c_nDaysInWeek] = {
	"Monday", "Tuesday", "Wednesday", "Thursday",
    "Friday", "Saturday", "Sunday"
} /*TTime::cac_WeekDayNames*/;

//----------------------------------------------------------------------------
TTime TTime::FromTime(

	const TCalendarForm& c_CalendarForm

){
	NLTIME_YEAR_CALENDAR_GUARD(c_CalendarForm.nYear);
    nl_long_t nMonth = c_CalendarForm.nMonth;
    nl_long_t nYear = c_CalendarForm.nYear;

    if (nMonth > 2) nMonth -= 3;
    else {
        nMonth += 9;
        --nYear;
    } /*iff*/

    nl_ulong_t nCentury = nYear / 100;
    nl_ulong_t nYearOfAge = nYear - 100 * nCentury;
    nl_long_t nDays = (
    	((146097L * nCentury) >> 2) + ((1461 * nYearOfAge) >> 2) +
		(153 * nMonth + 2) / 5 + c_CalendarForm.nDay + 1721119L
    ) /*nDays*/;
    nl_long_t nSec = (
    	(c_CalendarForm.nHour * c_nMinutesInHour +
    	c_CalendarForm.nMinute) * c_nSecondsInMinute + c_CalendarForm.nSecond
    ) /*nSec*/;
    TTime Result(nDays, nSec);
    Result.nMSeconds += c_CalendarForm.nMSecond;
    Result.ToGreenwich();
    return Result;
} /*TTime::FromTime(const TCalendarForm&)*/

//----------------------------------------------------------------------------
const char* TTime::DayOfWeekName(

	nl_int_t nDayOfWeek

){
	NLTIME_DAY_OF_WEEK_GUARD(nl_long_t(nDayOfWeek))
	return cac_szWeekDayNames[nDayOfWeek - 1];
} /*TTime::DayOfWeekName(nl_int_t)*/

//----------------------------------------------------------------------------
nl_int_t TTime::DayOfWeekFromName(

	const char* c_szDayName

){
    nl_long_t nDayOfWeek = (
    	PatternLookup(c_szDayName, cac_szWeekDayNames, c_nDaysInWeek) + 1
    ) /*nDayOfWeek*/;
	NLTIME_DAY_OF_WEEK_GUARD(nDayOfWeek)
    return nDayOfWeek;
} /*TTime::DayOfWeekFromName(const char*)*/

//----------------------------------------------------------------------------
nl_int_t TTime::MonthFromName(

	const char* c_szMonthName

){
    nl_long_t nMonth =
    	PatternLookup(c_szMonthName, cac_szMonthNames, c_nMonthsInYear) + 1;
	NLTIME_MONTH_GUARD(nMonth)
    return nMonth;
} /*TTime::MonthFromName(const char*)*/

//----------------------------------------------------------------------------
const char* TTime::MonthName(

	nl_int_t nMonth

){
	NLTIME_MONTH_GUARD(nl_long_t(nMonth))
	return cac_szMonthNames[nMonth - 1];
} /*TTime::MonthName(nl_int_t)*/

//----------------------------------------------------------------------------
TTime::TTime(

	nl_long_t nYear,
    nl_int_t nMonth,
    nl_int_t nDay,
    nl_int_t nHour,
    nl_int_t nMinute,
    nl_int_t nSec,
    nl_int_t nMSec

){
    TCalendarForm CalendarForm;
    CalendarForm.nYear = nYear;
    CalendarForm.nMonth = nMonth;
    CalendarForm.nDay = nDay;
    CalendarForm.nHour = nHour;
    CalendarForm.nMinute = nMinute;
    CalendarForm.nSecond = nSec;
    CalendarForm.nMSecond = nMSec;
	SetDateTime(CalendarForm);
} /*TTime(nl_int_t, nl_int_t, nl_int_t, nl_int_t = 0, nl_int_t = 0, ...)*/

//----------------------------------------------------------------------------
TTime::TTime(

	const tm& c_Tm

){
    TCalendarForm CalendarForm;
    CalendarForm.nYear =
    	c_Tm.tm_year < 1900 ? c_Tm.tm_year + 1900 : c_Tm.tm_year;
    CalendarForm.nMonth = c_Tm.tm_mon;
    CalendarForm.nDay = c_Tm.tm_mday;
    CalendarForm.nHour = c_Tm.tm_hour;
    CalendarForm.nMinute = c_Tm.tm_min;
    CalendarForm.nSecond = c_Tm.tm_sec;
    CalendarForm.nMSecond = 0;
	SetDateTime(CalendarForm);
} /*TTime(const tm&)*/

//----------------------------------------------------------------------------
TStdString TTime::Dump() const
{
	TStdString sDump(TParent::Dump());
    char szDump[sizeof(nDays) * 8 + sizeof(nMSeconds) * 8];
    sprintf(szDump, " [%D,%U]", nDays, nMSeconds);
    sDump += szDump;
    return sDump;
} /*TTime::Dump() const*/

//----------------------------------------------------------------------------
NLOBJECT_IMPLEMENT_CLONE_DEFAULT(TTime)

//----------------------------------------------------------------------------
TStdString TTime::AsString(

    const char* c_szCustomFormat

) const {
	TStdString sTime;
    if (!c_szCustomFormat) c_szCustomFormat = c_szDefaultFormat;
    tm Tm; ToTm(Tm);
   	char szTmpBuf[200];
    strftime(szTmpBuf, sizeof(szTmpBuf), c_szCustomFormat, &Tm);
   	sTime += szTmpBuf;
    return sTime;
} /*TTime::AsString(const char*) const*/

//----------------------------------------------------------------------------
TTime::TCalendarForm& TTime::Time(

	TCalendarForm& r_CalendarForm

) const {
	TTime Local(*this);
    Local.ToLocal();
    return Local.TimeGMT(r_CalendarForm);
} /*TTime::Time(TCalendarForm&) const*/

//----------------------------------------------------------------------------
TTime::TCalendarForm& TTime::TimeGMT(

	TCalendarForm& r_CalendarForm

) const {
	nl_ulong_t nJulianDays = nDays - 1721119L;
    nl_ulong_t nYear = nl_ulong_t(((nJulianDays << 2) - 1) / 146097L);
    nJulianDays = (nJulianDays << 2) - 1 - 146097L * nYear;
    nl_ulong_t nDayTemp = (nJulianDays >> 2);
    nJulianDays = ((nDayTemp << 2) + 3) / 1461;
    nDayTemp = (nDayTemp << 2) + 3 - 1461 * nJulianDays;
    nDayTemp = (nDayTemp + 4) >> 2;
    nl_ulong_t nMonth = nl_ulong_t((5 * nDayTemp - 3) / 153);
    nDayTemp = 5 * nDayTemp - 3 - 153 * nMonth;
    nYear = nl_ulong_t(100 * nYear + nJulianDays);

    if (nMonth < 10) nMonth += 3;
    else {
    	nMonth -= 9;
        ++nYear;
    } /*iff*/

    r_CalendarForm.nYear = nYear;
    r_CalendarForm.nMonth = nMonth;
    r_CalendarForm.nDay = (nDayTemp + 5) / 5;
    r_CalendarForm.nHour = nMSeconds / 1000 / 60 / 60;
    r_CalendarForm.nMinute = (nMSeconds / 1000 / 60) % 60;
    r_CalendarForm.nSecond = nMSeconds / 1000 % 60;
    r_CalendarForm.nMSecond = nMSeconds % 1000;

    return r_CalendarForm;
} /*TTime::TimeGMT(TCalendarForm&) const*/

//----------------------------------------------------------------------------
nl_int_t TTime::DayOfWeek() const
{
	TTime Local(*this);
    Local.ToLocal();
    return (Local.nDays-340584L)%c_nDaysInWeek;
} /*TTime::DayOfWeek() const*/

//----------------------------------------------------------------------------
void TTime::SetDate(

	nl_long_t nYear,
    nl_int_t nMonth,
    nl_int_t nDay

){
	TCalendarForm CalendarForm;
    Time(CalendarForm);
    CalendarForm.nYear = nYear;
    CalendarForm.nMonth = nMonth;
    CalendarForm.nDay = nDay;
    SetDateTime(CalendarForm);
} /*TTime::SetDate(nl_long_t, nl_int_t, nl_int_t)*/

//----------------------------------------------------------------------------
void TTime::SetTime(

	nl_int_t nHour,
    nl_int_t nMinute,
	nl_int_t nSecond,
    nl_int_t nMSecond

){
    TCalendarForm CalendarForm;
    Time(CalendarForm);
    CalendarForm.nHour = nHour;
    CalendarForm.nMinute = nMinute;
    CalendarForm.nSecond = nSecond;
    CalendarForm.nMSecond = nMSecond;
    SetDateTime(CalendarForm);
} /*TTime::SetTime(nl_int_t, nl_int_t, nl_int_t, nl_int_t)*/

//----------------------------------------------------------------------------
void TTime::AddYears(

	nl_long_t nYearsToAdd

){
	TCalendarForm CalendarForm;
    Time(CalendarForm);
    CalendarForm.nYear += nYearsToAdd;
    *this = FromTime(CalendarForm);
} /*TTime::AddYears(nl_long_t)*/

//----------------------------------------------------------------------------
void TTime::AddMonths(

	nl_long_t nMonthToAdd

){
    TCalendarForm CalendarForm;
    Time(CalendarForm);
    CalendarForm.nMonth += nMonthToAdd - 1;
    nl_long_t nYearsToAdd = CalendarForm.nMonth / c_nMonthsInYear;
    CalendarForm.nMonth -= nYearsToAdd * c_nMonthsInYear;
    if (CalendarForm.nMonth < 0) {
        CalendarForm.nYear--;
        CalendarForm.nMonth += c_nMonthsInYear;
    } /*if*/
    *this = FromTime(CalendarForm);
} /*TTime::AddMonths(nl_long_t)*/

//----------------------------------------------------------------------------
tm& TTime::ToTm(

	tm& r_Tm

) const {
    TCalendarForm CalendarForm;
    Time(CalendarForm);
    r_Tm.tm_sec = CalendarForm.nSecond;
    r_Tm.tm_min = CalendarForm.nMinute;
    r_Tm.tm_hour = CalendarForm.nHour;
    r_Tm.tm_mday = CalendarForm.nDay;
    r_Tm.tm_mon = CalendarForm.nMonth - 1;
    r_Tm.tm_year = nl_int_t(CalendarForm.nYear - 1900);
    r_Tm.tm_wday = DayOfWeek()-1;
    r_Tm.tm_yday = DayOfYear()-1;
    r_Tm.tm_isdst = 0;
    return r_Tm;
} /*TTime::ToTm(tm&) const*/

//----------------------------------------------------------------------------
TTime TTime::operator+(

	const TTime& c_Add

) const {
	TTime Result(*this);
    Result.AddTime(c_Add);
    return Result;
} /*TTime::operator+(const TTime&)*/

//----------------------------------------------------------------------------
TTime TTime::operator-(

	const TTime& c_Sub

) const {
	TTime Result(*this);
    Result.DiffTime(c_Sub);
	return Result;
} /*TTime::operator-(const TTime&)*/

//----------------------------------------------------------------------------
TTime TTime::operator-() const
{
	TTime Result(0, 0);
    Result.DiffTime(*this);
    return Result;
} /*TTime::operator-()*/

//----------------------------------------------------------------------------
nl_long_t TTime::SecondsDiffLocalFromGMT = 0;
TTime TTime::DiffLocalFromGMT(0, 0);
char TTime::c_szDefaultFormat[100];

//----------------------------------------------------------------------------
nl_int_t TTime::PatternLookup(

	const char* c_szSearch,
    const char* const cac_szPattern[],
    size_t nPatterns

){
    const size_t nSearchLen = strlen(c_szSearch);

    while (nPatterns--) {
        if (!strnicmp(c_szSearch, cac_szPattern[nPatterns], nSearchLen))
            return nPatterns;
    } /*while*/
    return -1; // not found
} /*TTime::PatternLookup(const char*, const char* const*, nl_int_t)*/

//----------------------------------------------------------------------------
void TTime::Initialize()
{
	tzset();
	time_t now = time(0);
    TTime Local(*localtime(&now));
    TTime GMT(*gmtime(&now));
    DiffLocalFromGMT = Local - GMT;
    SecondsDiffLocalFromGMT = DiffLocalFromGMT.nDays*c_nSecondsInDay +
    	DiffLocalFromGMT.nMSeconds/1000L;
    SetDefaultFormat("%d.%m.%Y %H:%M:%S");  // ToDo: support locale
} /*TTime::Initialize()*/

//----------------------------------------------------------------------------
const TTime::TInitializer TTime::Initializer;

//============================================================================
NATLIB_NAMESPACE_END

