/****************************************************************************/
/** ntasks.cpp                                                            **/
/** ---------------------------------------------------------------------- **/
/** Tasks primitives                                                       **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** September 01, 1997 by Oleg Kagan                                       **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>
#pragma hdrstop

//============================================================================
#include <ntasks.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TTask implementation
//============================================================================
TTask::TTask(

	const TStdString& c_sName

):
    TAllocationAware(this), sName(c_sName)
{
} /*TTask(const TStdString&)*/

//----------------------------------------------------------------------------
TTask::~TTask()
{} /*TTask::~TTask()*/

//----------------------------------------------------------------------------
void TTask::EvStartup() 
{} /*TTask::EvStartup()*/

//----------------------------------------------------------------------------
void TTask::EvTransaction(const TSignal*) 
{
	Wait();
} /*TTask::EvTransaction(const TSignal*)*/

//----------------------------------------------------------------------------
void TTask::EvTerminate() 
{} /*TTask::EvTerminate()*/

//----------------------------------------------------------------------------
bool TTask::EvCanTerminate() 
{
	return true;
} /*TTask::EvCanTerminate()*/

//----------------------------------------------------------------------------
void TTask::EvException(
    	
	TPhase ExceptionPhase, 
	const TNatlibException* pc_Exception
    
){
   	if (pc_Exception) ShowException(*pc_Exception, ExceptionPhase);
	else ShowException(ExceptionPhase);
	Terminate();
} /*TTask::EvException(TPhase, const TNatlibException*)*/

//----------------------------------------------------------------------------
void TTask::Send(

	TSignal& r_Signal,
	const THandle& c_hTarget

){
	Scheduler().Send(r_Signal, c_hTarget);
} /*TTask::Send(TSignal&, const THandle&)*/

//----------------------------------------------------------------------------
void TTask::Send(

	TSignal& r_Signal,
	const TContainerOf<THandle>& c_Targets

){
	Scheduler().Send(r_Signal, c_Targets);
} /*TTask::Send(TSignal&, const TContainerOf<THandle>&)*/

//----------------------------------------------------------------------------
void TTask::Wait()
{
	throw TSchedulerInterface::TXWaitRequest(THIS_CLASS);
} /*TTask::Wait()*/

//----------------------------------------------------------------------------
void TTask::Start(

	TTask& r_Task,
	bool bAutodelete /*= true*/

){
	Scheduler().Start(r_Task, bAutodelete);
} /*TTask::Start(const THandle&, bool)*/

//----------------------------------------------------------------------------
void TTask::Terminate(

	const THandle& c_hTaskToTerminate,
    bool bUnconditional /*= false*/

){
	if (
    	c_hTaskToTerminate.IsNull() ||
        (c_hTaskToTerminate == THandle(*this))
    ){
    	Terminate(bUnconditional);
    }
    else if (c_hTaskToTerminate.IsTargetValid()) {
    	Scheduler().Terminate(c_hTaskToTerminate);
    }
    else {
		throw TXInvalidParameter(THIS_CLASS);
    } /*iff*/
} /*TTask::Terminate(const THandle&, bool = false)*/

//----------------------------------------------------------------------------
void TTask::Terminate(

	bool bUnconditional /*= false*/

){
	throw TSchedulerInterface::TXTerminateRequest(bUnconditional, THIS_CLASS);
} /*TTask::Terminate(bool = false)*/

//----------------------------------------------------------------------------
void TTask::GlobalTerminate()
{
	throw TSchedulerInterface::TXGlobalTerminateRequest(THIS_CLASS);
} /*TTask::GlobalTerminate()*/

//----------------------------------------------------------------------------
void TTask::ShowException(

	TTask::TPhase ExceptionPhase

){
    Scheduler().ShowTaskException(*this, ExceptionPhase);
} /*TTask::ShowException(TTask::TPhase)*/

//----------------------------------------------------------------------------
void TTask::ShowException(

    const TNatlibException& r_Exception,
    TTask::TPhase ExceptionPhase

){
    Scheduler().ShowTaskException(*this, ExceptionPhase, &r_Exception);
} /*TTask::ShowException(const TNatlibException&, TTask::TPhase)*/

//----------------------------------------------------------------------------
TSchedulerInterface& TTask::Scheduler() 
{
	if (!hScheduler.IsTargetValid())
		throw TXInvalidSchedulerHandle(THIS_CLASS);
	return dynamic_cast<TSchedulerInterface&>(hScheduler.Dereference());
} /*TTask::Scheduler()*/

// TSignal implementation
//============================================================================
TSignal::TSignal() : 
	TAllocationAware(this), Destination(TTask::THandle(), 0)
{} /*TSignal()*/

//----------------------------------------------------------------------------
TSignal::~TSignal()
{} /*TSignal::~TSignal()*/

// TSchedulerInterface implementation
//============================================================================
TSchedulerInterface::TSchedulerInterface() :
	TSafeHandleAllowed(),
	TAllocationAware(this)
{} /*TSchedulerInterface()*/

//----------------------------------------------------------------------------
TSchedulerInterface::~TSchedulerInterface()
{} /*TSchedulerInterface::~TSchedulerInterface()*/

//----------------------------------------------------------------------------
TSchedulerInterface* TSchedulerInterface::GetParentPtr()
{
	TSchedulerInterface* pParent = null;

	if (hParent.IsTargetValid()) {
		pParent = dynamic_cast<TSchedulerInterface*>(&hParent.Dereference());
		if (!pParent) 
			throw TXBadSchedulerChain(THIS_CLASS);
	} /*if*/

    return pParent;
} /*TSchedulerInterface::GetParentPtr()*/
//----------------------------------------------------------------------------
TSchedulerInterface* TSchedulerInterface::GetMasterPtr()
{
	TSchedulerInterface* pMaster;
    TSchedulerInterface* pParent = null;
	do {
		pMaster = pParent;
    	pParent = GetParentPtr();
    } while (pParent);
	return pMaster;
} /*TSchedulerInterface::GetMasterPtr()*/

//============================================================================
NATLIB_NAMESPACE_END
