/****************************************************************************/
/** nwconsol.cpp                                                           **/
/** ---------------------------------------------------------------------- **/
/** C++ envelope for Win32 Console API                                     **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** January 18, 1998 by Oleg Kagan                                         **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>
#pragma hdrstop

//============================================================================
#include <conio.h>

//============================================================================
#include <nwconsol.h>
#include <nwerror.h>
#include <nllstack.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

//============================================================================
static TLiteStack NWCONSOL_ConsoleStateStack;
static TLiteStack NWCONSOL_CursorStateStack;

// TWin32Console implementation
//============================================================================
const char* const TWin32Console::c_szNewLine = "\n";
const char* const TWin32Console::c_szPauseDefaultPrompt = (
	"Press any key to continue..."
) /*TWin32Console::c_szPauseDefaultPrompt*/;
 
//----------------------------------------------------------------------------
const 
unsigned long const TWin32Console::cac_ulOctave[5][TWin32Console::noteAll] = {
	{ 131,  139,  147,  156,  165,  175,  185,  196,  208,  220,  233,  248},
	{ 262,  277,  294,  311,  330,  349,  370,  392,  415,  440,  466,  494},
	{ 523,  554,  587,  622,  659,  698,  740,  784,  831,  880,  932,  988},
	{1046, 1108, 1175, 1244, 1318, 1397, 1480, 1568, 1662, 1760, 1866, 1973},
	{2093, 2217, 2349, 2489, 2637, 2794, 2960, 3136, 3322, 3520, 3729, 3951}
} /*TWin32Console::cac_ulOctave*/;

//----------------------------------------------------------------------------
TWin32Console::TColor TWin32Console::GetNextColor(

	TColor Color
	
){
	if (Color >= colorAll) Color = colorAll;
	else Color = TColor(nl_ushort_t(Color) + 1);
	return Color;
} /*TWin32Console::GetNextColor(TColor)*/

//----------------------------------------------------------------------------
TWin32Console::TColor TWin32Console::GetPreviousColor(

	TColor Color
	
){
	if (Color <= colorFirst) Color = colorFirst;
	else Color = TColor(nl_ushort_t(Color) - 1);
	return Color;
} /*TWin32Console::GetPreviousColor(TColor)*/

//----------------------------------------------------------------------------
void TWin32Console::Initialize()
{
	HANDLE hDummy;

	if (!hInput || (hInput == INVALID_HANDLE_VALUE)) {
		hInput = hDummy = null;
		hDummy = GetStdHandle(STD_INPUT_HANDLE);
		if (hDummy && (hDummy != INVALID_HANDLE_VALUE)) hInput = hDummy;
	} /*if*/
		
	if (!hOutput || (hOutput == INVALID_HANDLE_VALUE)) {
		hOutput = hDummy = null;
		hDummy = GetStdHandle(STD_OUTPUT_HANDLE);
		if (hDummy && (hDummy != INVALID_HANDLE_VALUE)) hOutput = hDummy;
	} /*if*/

	if (!hError || (hError == INVALID_HANDLE_VALUE)) {
		hError = hDummy = null;
		hDummy = GetStdHandle(STD_ERROR_HANDLE);
		if (hDummy && (hDummy != INVALID_HANDLE_VALUE)) hError = hDummy;
	} /*if*/
} /*TWin32Console::Initialize()*/

//----------------------------------------------------------------------------
void TWin32Console::Close()
{
	if (hInput) {
//		HANDLE hDummy = hInput;
        hInput = null;
//		if (hDummy != INVALID_HANDLE_VALUE) CloseHandle(hDummy);
	} /*if*/
	if (hOutput) {
//		HANDLE hDummy = hOutput;
        hOutput = null;
//		if (hDummy != INVALID_HANDLE_VALUE) CloseHandle(hDummy);
	} /*if*/
	if (hError) {
//		HANDLE hDummy = hError;
        hError = null;
//		if (hDummy != INVALID_HANDLE_VALUE) CloseHandle(hDummy);
	} /*if*/
} /*TWin32Console::Close()*/

//----------------------------------------------------------------------------
void TWin32Console::SetTitle(

	const char* c_szTitle

){
	if (!c_szTitle) c_szTitle = c_szNull;
	if (!SetConsoleTitleA(c_szTitle)) {
		DWORD uError = GetLastError();
		throw TXOperationFailed(
			THIS_PLACE, Win32GetErrorExplanationA(uError)
		) /*throw TXOperationFailed*/;
	} /*if*/
} /*TWin32Console::SetTitle(const char*)*/

//----------------------------------------------------------------------------
void TWin32Console::SetTitle(

	const wchar_t* c_wszTitle

){
	if (Win32IsNT()) {
		if (!c_wszTitle) c_wszTitle = c_wszNull;
		if (!SetConsoleTitleW(c_wszTitle)) {
			DWORD uError = GetLastError();
			throw TXOperationFailed(
				THIS_PLACE, Win32GetErrorExplanationA(uError)
			) /*throw TXOperationFailed*/;
		} /*if*/
	}
	else {
		TStdString sTitle = ToStdString(c_wszTitle);
		SetTitle(sTitle.c_str());
	} /*if*/
} /*TWin32Console::SetTitle(const wchar_t*)*/

//----------------------------------------------------------------------------
void TWin32Console::SetTitle(

	const TStdString& c_sTitle

){
	SetTitle(c_sTitle.c_str());
} /*TWin32Console::SetTitle(const TStdString&)*/

#ifndef _NATLIB_NO_WIDE_STRINGS_
//----------------------------------------------------------------------------
void TWin32Console::SetTitle(

	const TWideString& c_wsTitle

){
	SetTitle(c_wsTitle.c_str());
} /*TWin32Console::SetTitle(const TStdString&)*/
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

//----------------------------------------------------------------------------
TStdString TWin32Console::GetTitleA()
{
	TStdString sTitle; {
		char szTitle[1024];
		if (!GetConsoleTitleA(szTitle, sizeof(szTitle) - 1)) {
			DWORD uError = GetLastError();
			throw TXOperationFailed(
				THIS_PLACE, Win32GetErrorExplanationA(uError)
			) /*throw TXOperationFailed*/;
		} /*if*/
		sTitle = szTitle;
	} /*sTitle*/
	return sTitle;
} /*TWin32Console::GetTitleA()*/

#ifndef _NATLIB_NO_WIDE_STRINGS_
//----------------------------------------------------------------------------
TWideString TWin32Console::GetTitleW()
{
	TWideString wsTitle; 
	if (Win32IsNT()) {
		wchar_t wszTitle[1024];
		if (!GetConsoleTitleW(wszTitle, sizeof(wszTitle) - 1)) {
			DWORD uError = GetLastError();
			throw TXOperationFailed(
				THIS_PLACE, Win32GetErrorExplanationA(uError)
			) /*throw TXOperationFailed*/;
		} /*if*/
		wsTitle = wszTitle;
	}
	else {
		wsTitle = ToWideString(GetTitleA());
	} /*iff*/
	return wsTitle;
} /*TWin32Console::GetTitleW()*/
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

//----------------------------------------------------------------------------
void TWin32Console::Fill(

	char cFiller

){
	Fill(wchar_t(cFiller));
} /*TWin32Console::Fill(char)*/

//----------------------------------------------------------------------------
void TWin32Console::Fill(

	wchar_t wcFiller

){
	Initialize();

	if (!hOutput)
		throw TXInvalidHandle(THIS_PLACE);

	DWORD nWritten;
	COORD StartPoint = {0, 0};
	if (
		!(
			Win32IsNT() ?
				FillConsoleOutputCharacterW(
					hOutput, wcFiller,
					GetWidth() * GetHeight(), StartPoint, &nWritten
				) /*!FillConsoleOutputCharacterW*/ :
				FillConsoleOutputCharacterA(
					hOutput, char(wcFiller),
					GetWidth() * GetHeight(), StartPoint, &nWritten
				) /*!FillConsoleOutputCharacterA*/ 
		)
	){
		DWORD uError = GetLastError();
		throw TXOperationFailed(
			THIS_PLACE, Win32GetErrorExplanationA(uError)
		) /*throw TXOperationFailed*/;
	} /*if*/

	if (
		!FillConsoleOutputAttribute(
			hOutput, GetAttributes(),
			GetWidth() * GetHeight(), StartPoint, &nWritten
		) /*!FillConsoleOutputAttribute*/
	){
		DWORD uError = GetLastError();
		throw TXOperationFailed(
			THIS_PLACE, Win32GetErrorExplanationA(uError)
		) /*throw TXOperationFailed*/;
	} /*if*/
} /*TWin32Console::Fill(wchar_t)*/

//----------------------------------------------------------------------------
bool TWin32Console::ShowCursor(

	bool bShow

){
	CONSOLE_CURSOR_INFO CursorInfo; GetCursorInfo(CursorInfo);
   	bool bOrgState = CursorInfo.bVisible;
	CursorInfo.bVisible = bShow;
	SetCursorInfo(CursorInfo);
    return bOrgState;
} /*TWin32Console::ShowCursor()*/

//----------------------------------------------------------------------------
bool TWin32Console::IsCursorVisible()
{
	CONSOLE_CURSOR_INFO CursorInfo; GetCursorInfo(CursorInfo);
	return CursorInfo.bVisible != false;
} /*TWin32Console::IsCursorVisible()*/

//----------------------------------------------------------------------------
short TWin32Console::SetCursorSize(

	short nSize

){
	if ((nSize < 1) || (nSize > 100))
		throw TXOperationFailed(THIS_PLACE);

	CONSOLE_CURSOR_INFO CursorInfo; GetCursorInfo(CursorInfo);
    bool nOrgSize = CursorInfo.dwSize;
	CursorInfo.dwSize = nSize;

	SetCursorInfo(CursorInfo);

    return short(nOrgSize);
} /*TWin32Console::SetCursorSize()*/

//----------------------------------------------------------------------------
short TWin32Console::GetCursorSize()
{
	CONSOLE_CURSOR_INFO CursorInfo; GetCursorInfo(CursorInfo);
	return short(CursorInfo.dwSize);
} /*TWin32Console::GetCursorSize()*/

//----------------------------------------------------------------------------
void TWin32Console::GotoXY(

	short nX, 
	short nY
	
){
	Initialize();

	if (!hOutput)
		throw TXInvalidHandle(THIS_PLACE);

	COORD Position = {nX, nY};
	if (!SetConsoleCursorPosition(hOutput, Position)) {
		DWORD uError = GetLastError();
		throw TXOperationFailed(
			THIS_PLACE, Win32GetErrorExplanationA(uError)
		) /*throw TXOperationFailed*/;
	} /*if*/
} /*TWin32Console::GotoXY(short, short)*/

//----------------------------------------------------------------------------
short TWin32Console::WhereX()
{
	short X, Y; WhereXY(X, Y);
	return X;
} /*TWin32Console::WhereX()*/

//----------------------------------------------------------------------------
short TWin32Console::WhereY()
{
	short X, Y; WhereXY(X, Y);
	return Y;
} /*TWin32Console::WhereY()*/

//----------------------------------------------------------------------------
void TWin32Console::WhereXY(

	short& r_nX, 
	short& r_nY

){
	CONSOLE_SCREEN_BUFFER_INFO ConsoleInfo; GetConsoleInfo(ConsoleInfo);

	r_nX = ConsoleInfo.dwCursorPosition.X;
	r_nY = ConsoleInfo.dwCursorPosition.Y;
} /*TWin32Console::WhereXY(short&, short&)*/

//----------------------------------------------------------------------------
short TWin32Console::GetWidth()
{
	CONSOLE_SCREEN_BUFFER_INFO ConsoleInfo; GetConsoleInfo(ConsoleInfo);
	return ConsoleInfo.dwSize.X;
} /*TWin32Console::GetWidth()*/

//----------------------------------------------------------------------------
short TWin32Console::GetHeight()
{
	CONSOLE_SCREEN_BUFFER_INFO ConsoleInfo; GetConsoleInfo(ConsoleInfo);
	return ConsoleInfo.dwSize.Y;
} /*TWin32Console::GetHeight()*/

//----------------------------------------------------------------------------
size_t TWin32Console::Write(

	const char* c_szText

){
	return Write(hOutput, c_szText);
} /*TWin32Console::Write(const char*)*/

//----------------------------------------------------------------------------
size_t TWin32Console::Write(

	const wchar_t* c_wszText

){
	return Write(hOutput, c_wszText);
} /*TWin32Console::Write(const wchar_t*)*/

//----------------------------------------------------------------------------
size_t TWin32Console::Write(

	const TStdString& c_sText

){
	return Write(c_sText.c_str());
} /*TWin32Console::Write(const TStdString&)*/

#ifndef _NATLIB_NO_WIDE_STRINGS_
//----------------------------------------------------------------------------
size_t TWin32Console::Write(

	const TWideString& c_wsText

){
	return Write(c_wsText.c_str());
} /*TWin32Console::Write(const TWideString&)*/
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

//----------------------------------------------------------------------------
size_t TWin32Console::WriteLn(

	const char* c_szText /*= null*/

){
	return Write(c_szText) + Write(c_szNewLine);
} /*TWin32Console::WriteLn(const char*)*/

//----------------------------------------------------------------------------
size_t TWin32Console::WriteLn(

	const wchar_t* c_wszText

){
	return Write(c_wszText) + WriteLn();
} /*TWin32Console::WriteLn(const wchar_t*)*/

//----------------------------------------------------------------------------
size_t TWin32Console::WriteLn(

	const TStdString& c_sText

){
	return WriteLn(c_sText.c_str());
} /*TWin32Console::WriteLn(const TStdString&)*/

#ifndef _NATLIB_NO_WIDE_STRINGS_
//----------------------------------------------------------------------------
size_t TWin32Console::WriteLn(

	const TWideString& c_wsText

){
	return WriteLn(c_wsText.c_str());
} /*TWin32Console::WriteLn(const TWideString&)*/
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

//----------------------------------------------------------------------------
size_t TWin32Console::WriteError(

	const char* c_szText

){
	return Write(hError, c_szText);
} /*TWin32Console::WriteError(const char*)*/

//----------------------------------------------------------------------------
size_t TWin32Console::WriteError(

	const wchar_t* c_wszText

){
	return Write(hError, c_wszText);
} /*TWin32Console::WriteError(const wchar_t*)*/

//----------------------------------------------------------------------------
size_t TWin32Console::WriteError(

	const TStdString& c_sText

){
	return Write(hError, c_sText);
} /*TWin32Console::WriteError(const TStdString&)*/

#ifndef _NATLIB_NO_WIDE_STRINGS_
//----------------------------------------------------------------------------
size_t TWin32Console::WriteError(

	const TWideString& c_wsText

){
	return Write(hError, c_wsText);
} /*TWin32Console::WriteError(const TWideString&)*/
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

//----------------------------------------------------------------------------
size_t TWin32Console::WriteErrorLn(

	const char* c_szText /*= null*/

){
	return WriteError(c_szText) + WriteError(c_szNewLine);
} /*TWin32Console::WriteErrorLn(const char*)*/

//----------------------------------------------------------------------------
size_t TWin32Console::WriteErrorLn(

	const wchar_t* c_wszText /*= null*/

){
	return WriteError(c_wszText) + WriteErrorLn();
} /*TWin32Console::WriteErrorLn(const wchar_t*)*/

//----------------------------------------------------------------------------
size_t TWin32Console::WriteErrorLn(

	const TStdString& c_sText

){
	return WriteErrorLn(c_sText.c_str());
} /*TWin32Console::WriteErrorLn(const TStdString&)*/

#ifndef _NATLIB_NO_WIDE_STRINGS_
//----------------------------------------------------------------------------
size_t TWin32Console::WriteErrorLn(

	const TWideString& c_wsText

){
	return WriteErrorLn(c_wsText.c_str());
} /*TWin32Console::WriteErrorLn(const TWideString&)*/
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

//----------------------------------------------------------------------------
void TWin32Console::Pause(

	const char* c_szPrompt /*= c_szPauseDefaultPrompt*/

){
	Write(c_szPrompt);
	while (!kbhit());
	if (!getch()) {
    	getch();
        putch('\n');
    } /*if*/
} /*TWin32Console::Pause(const char*)*/;

//----------------------------------------------------------------------------
void TWin32Console::Pause(

	const wchar_t* c_wszPrompt

){
	Write(c_wszPrompt);
    Pause((const char*)null);
} /*TWin32Console::Pause(const wchar_t*)*/;

//----------------------------------------------------------------------------
void TWin32Console::Pause(

	const TStdString& c_sPrompt

){
	Pause(c_sPrompt.c_str());
} /*TWin32Console::Pause(const TStdString&)*/;

#ifndef _NATLIB_NO_WIDE_STRINGS_
//----------------------------------------------------------------------------
void TWin32Console::Pause(

	const TWideString& c_wsPrompt

){
	Pause(c_wsPrompt.c_str());
} /*TWin32Console::Pause(const TWideString&)*/;
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

//----------------------------------------------------------------------------
TWin32Console::TColor TWin32Console::SetColor(

	TColor Color

){
    TColor OrgColor = GetColor();

	WORD wAttr = GetAttributes();
	TColorStruct ColorStruct(Color);

	ChangeAttributeFlag(wAttr, FOREGROUND_RED, ColorStruct.Rgb.uRed);
	ChangeAttributeFlag(wAttr, FOREGROUND_GREEN, ColorStruct.Rgb.uGreen);
	ChangeAttributeFlag(wAttr, FOREGROUND_BLUE, ColorStruct.Rgb.uBlue);
	ChangeAttributeFlag(wAttr, FOREGROUND_INTENSITY, ColorStruct.Rgb.uLight);

	SetAttributes(wAttr);

    return OrgColor;
} /*TWin32Console::SetColor(TColor)*/

//----------------------------------------------------------------------------
TWin32Console::TColor TWin32Console::GetColor()
{
	WORD wAttr = GetAttributes();

	TColorStruct ColorStruct(
		wAttr & FOREGROUND_RED,
		wAttr & FOREGROUND_GREEN,
		wAttr & FOREGROUND_BLUE,
		wAttr & FOREGROUND_INTENSITY
	) /*ColorStruct*/;

	return ColorStruct.Color;
} /*TWin32Console::GetColor()*/

//----------------------------------------------------------------------------
bool TWin32Console::SetHighlight(

	bool bHighlight
	
){
	WORD wAttr = GetAttributes();
    bool bOrgState = wAttr & FOREGROUND_INTENSITY;

	ChangeAttributeFlag(wAttr, FOREGROUND_INTENSITY, bHighlight);

    return bOrgState;
} /*TWin32Console::SetHighlight(bool)*/

//----------------------------------------------------------------------------
bool TWin32Console::GetHighlight()
{
	return (GetAttributes() & FOREGROUND_INTENSITY) != null;
} /*TWin32Console::GetHighlight()*/

//----------------------------------------------------------------------------
TWin32Console::TColor TWin32Console::SetBackgroundColor(

	TColor Color

){
	TColor OrgColor = GetColor();

	WORD wAttr = GetAttributes();
	TColorStruct ColorStruct(Color);

	ChangeAttributeFlag(wAttr, BACKGROUND_RED, ColorStruct.Rgb.uRed);
	ChangeAttributeFlag(wAttr, BACKGROUND_GREEN, ColorStruct.Rgb.uGreen);
	ChangeAttributeFlag(wAttr, BACKGROUND_BLUE, ColorStruct.Rgb.uBlue);
	ChangeAttributeFlag(wAttr, BACKGROUND_INTENSITY, ColorStruct.Rgb.uLight);

	SetAttributes(wAttr);

    return OrgColor;
} /*TWin32Console::SetBackgroundColor(TColor)*/

//----------------------------------------------------------------------------
TWin32Console::TColor TWin32Console::GetBackgroundColor()
{
	WORD wAttr = GetAttributes();

	TColorStruct ColorStruct(
		wAttr & BACKGROUND_RED, 
		wAttr & BACKGROUND_GREEN, 
		wAttr & BACKGROUND_BLUE,
		wAttr & BACKGROUND_INTENSITY
	) /*ColorStruct*/;

	return ColorStruct.Color;
} /*TWin32Console::GetBackgroundColor()*/

//----------------------------------------------------------------------------
bool TWin32Console::SetBackgroundHighlight(

	bool bHighlight

){
	WORD wAttr = GetAttributes();
    bool bOrgState = wAttr & BACKGROUND_INTENSITY;
	ChangeAttributeFlag(wAttr, BACKGROUND_INTENSITY, bHighlight);
    return bOrgState;
} /*TWin32Console::SetBackgroundHighlight(bool)*/

//----------------------------------------------------------------------------
bool TWin32Console::GetBackgroundHighlight()
{
	return (GetAttributes() & BACKGROUND_INTENSITY) != null;
} /*TWin32Console::GetBackgroundHighlight()*/

//----------------------------------------------------------------------------
void TWin32Console::Delay(

	DWORD ulDuration
	
){
	Sleep(ulDuration);
} /*TWin32Console::Delay(DWORD)*/

//----------------------------------------------------------------------------
unsigned long TWin32Console::Octave(

	unsigned long ulOctave, 
	TNote Note
	
){
	if (ulOctave >= sizeof(cac_ulOctave)) 
		ulOctave = sizeof(cac_ulOctave) - 1;
	return cac_ulOctave[ulOctave][Note];
} /*TWin32Console::Octave(unsigned long, TNote)*/

//----------------------------------------------------------------------------
void TWin32Console::Sound(

	unsigned long ulFrequency, 
	unsigned long ulDuration
	
){
	Beep(DWORD(ulFrequency), DWORD(ulDuration));
} /*TWin32Console::Sound(unsigned long, unsigned long)*/

//----------------------------------------------------------------------------
void TWin32Console::Sound(

	unsigned long ulOctave, 
	TNote Note, 
	unsigned long ulDuration

){
	Sound(Octave(ulOctave, Note), ulDuration);
} /*TWin32Console::Sound(unsigned long, TNote, unsigned long)*/;


//----------------------------------------------------------------------------
void TWin32Console::SaveState()
{
	CONSOLE_SCREEN_BUFFER_INFO ConsoleInfo; GetConsoleInfo(ConsoleInfo);
	NWCONSOL_ConsoleStateStack.Push(&ConsoleInfo, sizeof(ConsoleInfo));

	CONSOLE_CURSOR_INFO CursorInfo; GetCursorInfo(CursorInfo);
	NWCONSOL_CursorStateStack.Push(&CursorInfo, sizeof(CursorInfo));
} /*TWin32Console::SaveState()*/

//----------------------------------------------------------------------------
void TWin32Console::RestoreState()
{
	CONSOLE_SCREEN_BUFFER_INFO ConsoleInfo;
	CONSOLE_CURSOR_INFO CursorInfo;
	{
		if (
			!NWCONSOL_ConsoleStateStack.Current() ||
			!NWCONSOL_CursorStateStack.Current()
		) throw TXOperationFailed(THIS_PLACE);

		memcpy(	
			&ConsoleInfo, NWCONSOL_ConsoleStateStack.Current(), 
			sizeof(ConsoleInfo)
		) /*memcpy*/;
		memcpy(
			&CursorInfo, NWCONSOL_CursorStateStack.Current(),
			sizeof(CursorInfo)
		) /*memcpy*/;

		NWCONSOL_ConsoleStateStack.Pop();
		NWCONSOL_CursorStateStack.Pop();
	} /*ConsoleInfo, CursorInfo*/

	SetAttributes(ConsoleInfo.wAttributes);
	GotoXY(ConsoleInfo.dwCursorPosition.X, ConsoleInfo.dwCursorPosition.Y);

	SetCursorInfo(CursorInfo);
} /*TWin32Console::RestoreState()*/

//----------------------------------------------------------------------------
char TWin32Console::ReadChar(

	bool bEcho /*= true*/
	
){
	if (bEcho) return char(getche());
	else return char(getch());
} /*TWin32Console::ReadChar(bool)*/

//----------------------------------------------------------------------------
bool TWin32Console::KeyboardHit()
{
	return kbhit() != 0;
} /*TWin32Console::KeyboardHit()*/

//----------------------------------------------------------------------------
TWin32Console::TInitializer TWin32Console::Initializer;

//----------------------------------------------------------------------------
HANDLE TWin32Console::hInput = null; 
HANDLE TWin32Console::hOutput = null;
HANDLE TWin32Console::hError = null;

//----------------------------------------------------------------------------
size_t TWin32Console::Write(

	HANDLE hDestination,
	const char* c_szText

){
	Initialize();

	if (!hDestination)
		throw TXInvalidHandle(THIS_PLACE);

	if (!c_szText) return 0;

	DWORD uWritten = 0;
	if (
		!WriteConsoleA(
        	hDestination, c_szText, strlen(c_szText), &uWritten, null
		) /*!WriteConsoleA*/
	){
		DWORD uError = GetLastError();
		throw TXOperationFailed(
			THIS_PLACE, Win32GetErrorExplanationA(uError)
		) /*throw TXOperationFailed*/;
	} /*if*/

	return uWritten;
} /*TWin32Console::Write(HANDLE, const char*)*/

//----------------------------------------------------------------------------
size_t TWin32Console::Write(

	HANDLE hDestination,
	const wchar_t* c_wszText

){
	if (Win32IsNT()) {
		Initialize();
	
		if (!hDestination)
			throw TXInvalidHandle(THIS_PLACE);

		if (!c_wszText) return 0;

		DWORD uWritten = 0;
		if (
			!WriteConsoleW(
        		hDestination, c_wszText, SzLength(c_wszText), &uWritten, null
			) /*!WriteConsoleW*/
		){
			DWORD uError = GetLastError();
			throw TXOperationFailed(
				THIS_PLACE, Win32GetErrorExplanationA(uError)
			) /*throw TXOperationFailed*/;
		} /*if*/

		return uWritten;
	}
	else {
		TStdString sText = ToStdString(c_wszText);
		return Write(hDestination, sText.c_str());
	} /*iff*/
} /*TWin32Console::Write(HANDLE, const wchar_t*)*/

//----------------------------------------------------------------------------
size_t TWin32Console::Write(

	HANDLE hDestination,
	const TStdString& c_sText

){
	return Write(hDestination, c_sText.c_str());
} /*TWin32Console::Write(HANDLE, const TStdString&)*/

#ifndef _NATLIB_NO_WIDE_STRINGS_
//----------------------------------------------------------------------------
size_t TWin32Console::Write(

	HANDLE hDestination,
	const TWideString& c_wsText

){
	return Write(hDestination, c_wsText.c_str());
} /*TWin32Console::Write(HANDLE, const TWideString&)*/
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

//----------------------------------------------------------------------------
WORD TWin32Console::GetAttributes()
{
	CONSOLE_SCREEN_BUFFER_INFO ConsoleInfo; GetConsoleInfo(ConsoleInfo);
	return ConsoleInfo.wAttributes;
} /*TWin32Console::GetAttributes()*/

//----------------------------------------------------------------------------
void TWin32Console::SetAttributes(

	WORD wAttributes

){
	Initialize();

	if (!hOutput)
		throw TXInvalidHandle(THIS_PLACE);

	if (!SetConsoleTextAttribute(hOutput, wAttributes)) {	
		DWORD uError = GetLastError();
		throw TXOperationFailed(
			THIS_PLACE, Win32GetErrorExplanationA(uError)
		) /*throw TXOperationFailed*/;
	} /*if*/
} /*TWin32Console::SetAttributes(WORD)*/

//----------------------------------------------------------------------------
void TWin32Console::SetCursorInfo(

	const CONSOLE_CURSOR_INFO& c_CursorInfo

){
	Initialize();

	if (!hOutput)
		throw TXInvalidHandle(THIS_PLACE);

	if (!SetConsoleCursorInfo(hOutput, &c_CursorInfo)) {
		DWORD uError = GetLastError();
		throw TXOperationFailed(
			THIS_PLACE, Win32GetErrorExplanationA(uError)
		) /*throw TXOperationFailed*/;
	} /*if*/
} /*TWin32Console::SetCursorInfo(const CONSOLE_CURSOR_INFO&*/

//----------------------------------------------------------------------------
void TWin32Console::ChangeAttributeFlag(

	WORD& r_wAttributes, 
	WORD wMask,
	bool bValue

){
	if (bValue) r_wAttributes |= wMask;
	else r_wAttributes &= WORD(~wMask);
} /*TWin32Console::ChangeAttributeFlag(WORD&, WORD, bool)*/

//----------------------------------------------------------------------------
void TWin32Console::GetConsoleInfo(

	CONSOLE_SCREEN_BUFFER_INFO& r_ConsoleInfo

){
	if (!hOutput)
		throw TXInvalidHandle(THIS_PLACE);

	if (!GetConsoleScreenBufferInfo(hOutput, &r_ConsoleInfo)) {
		DWORD uError = GetLastError();
		throw TXOperationFailed(
			THIS_PLACE, Win32GetErrorExplanationA(uError)
		) /*throw TXOperationFailed*/;
	} /*if*/
} /*TWin32Console::GetConsoleInfo(CONSOLE_SCREEN_BUFFER_INFO&)*/

//----------------------------------------------------------------------------
void TWin32Console::GetCursorInfo(

	CONSOLE_CURSOR_INFO& r_CursorInfo

){
	if (!hOutput)
		throw TXInvalidHandle(THIS_PLACE);

	if (!GetConsoleCursorInfo(hOutput, &r_CursorInfo)) {
		DWORD uError = GetLastError();
		throw TXOperationFailed(
			THIS_PLACE, Win32GetErrorExplanationA(uError)
		) /*throw TXOperationFailed*/;
	} /*if*/
} /*TWin32Console::GetCursorInfo(CONSOLE_CURSOR_INFO&)*/

//----------------------------------------------------------------------------
TWin32Console::TWin32Console() {}

//----------------------------------------------------------------------------
TWin32Console::~TWin32Console() {}

// TWin32Console::TInitializer implementation
//============================================================================
TWin32Console::TInitializer::TInitializer() 
{
	TWin32Console::hInput = TWin32Console::hOutput = 
		TWin32Console::hOutput = null;
	TWin32Console::Initialize();
} /*TWin32Console::TInitializer()*/

//----------------------------------------------------------------------------
TWin32Console::TInitializer::~TInitializer()
{
	TWin32Console::Close();
} /*TWin32Console::~TInitializer()*/

//============================================================================
NATLIB_NAMESPACE_END
