/***************************************************************************/
/** nwerror.cpp                                                            **/
/** ---------------------------------------------------------------------- **/
/** Some usefull C++ functions for error processing under MS Win32         **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** January 18, 1998 by Oleg Kagan                                         **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>
#pragma hdrstop

// Natlib Basics includes
//============================================================================
#include <nlstring.h>

// Natlib Win32 includes
//============================================================================
#include <nwerror.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

//============================================================================
bool Win32IsNT()
{
	static unsigned uIsNT = 2;
	if (uIsNT > 1) {
		// Check NT or Win95/98
		//----------------------
		OSVERSIONINFO VersionInfo; {
			VersionInfo.dwOSVersionInfoSize = sizeof(VersionInfo);
			GetVersionEx(&VersionInfo);
		} /*VersionInfo*/

		uIsNT = (VersionInfo.dwPlatformId == VER_PLATFORM_WIN32_NT);
	} /*if*/
	return bool(uIsNT);
} /*Win32IsNT()*/

//============================================================================
TStdString Win32GetErrorExplanation(

	DWORD uError /*= null*/

){
	return Win32GetErrorExplanationA(uError);
} /*Win32GetErrorExplanation(DWORD = null)*/

//----------------------------------------------------------------------------
TStdString Win32GetErrorExplanationA(

	DWORD uError /*= null*/

){
	if (!uError) uError = GetLastError();
	TStdString sExplanation; {
		char* szExplanation;
		FormatMessageA(
			FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM |
				FORMAT_MESSAGE_IGNORE_INSERTS,
			null,
			uError,
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
			(char*)(&szExplanation), 0,
			null
		) /*FormatMessageA*/;
		if (szExplanation) {
			sExplanation = szExplanation;
			LocalFree(szExplanation); szExplanation = null;

			// Delete tail characters
			//------------------------
			{
				int iLast = sExplanation.length() - 1;
				while (
					(iLast >= 0) && 
					(
						(sExplanation[iLast] == '\n') || 
						(sExplanation[iLast] == '\r')
					)
				) --iLast;
				sExplanation.resize((iLast >= 0) ? iLast + 1 : 0);
			} /*Delete tail characters*/
		} /*if*/
	} /*sExplanation*/
	return sExplanation;
} /*Win32GetErrorExplanationA(DWORD)*/

#ifndef _NATLIB_NO_WIDE_STRINGS_
//----------------------------------------------------------------------------
TWideString Win32GetErrorExplanationW(

	DWORD uError /*= null*/

){
	if (!uError) uError = GetLastError();

	TWideString wsExplanation;

	if (Win32IsNT()) {
		wchar_t* wszExplanation;
		FormatMessageW(
			FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM |
				FORMAT_MESSAGE_IGNORE_INSERTS,
			null,
			uError,
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
			(wchar_t*)(&wszExplanation), 0,
			null
		) /*FormatMessageW*/;
		if (wszExplanation) {
			wsExplanation = wszExplanation;
			LocalFree(wszExplanation); wszExplanation = null;

			// Delete tail characters
			//------------------------
			{
				int iLast = wsExplanation.length() - 1;
				while (
					(iLast >= 0) && 
					(
						(wsExplanation[iLast] == '\n') || 
						(wsExplanation[iLast] == '\r')
					)
				) --iLast;
				wsExplanation.resize((iLast >= 0) ? iLast + 1 : 0);
			} /*Delete tail characters*/
		} /*if*/
	}
	else {
		wsExplanation = ToWideString(Win32GetErrorExplanationA(uError));
	} /*if*/
	return wsExplanation;
} /*Win32GetErrorExplanationW(DWORD)*/
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

//============================================================================
NATLIB_NAMESPACE_END
