/****************************************************************************/
/** nwevlog.h                                                              **/
/** ---------------------------------------------------------------------- **/
/** Event logging utilities for Win32 (Windows NT)                         **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** January 18, 1998 by Oleg Kagan                                         **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>
#pragma hdrstop

// Natlib Basics includes
//============================================================================
#include <nlautopt.h>

// Natlib Win32 includes
//============================================================================
#include <nwevlog.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TWin32EventLog implementation
//============================================================================
const char* const TWin32EventLog::c_szRegistryPath = (
	"SYSTEM\\CurrentControlSet\\Services\\EventLog\\Application\\"
) /*c_szRegistryPath*/;

//----------------------------------------------------------------------------
const char* const TWin32EventLog::c_szEventMessageFileValue = (
	"EventMessageFile"
) /*TWin32EventLog::c_szEventMessageFileValue*/;
//----------------------------------------------------------------------------
const char* const TWin32EventLog::c_szTypesSupportedValue = (
	"TypesSupported"
) /*TWin32EventLog::c_szTypesSupportedValue*/;

//----------------------------------------------------------------------------
WORD TWin32EventLog::ToStdEventType(

	TWin32EventLog::TEventType Type

){
	WORD uType;

	switch (Type) {
		case eventError: 
			uType = EVENTLOG_ERROR_TYPE; break;
		case eventWarning: 
			uType = EVENTLOG_WARNING_TYPE; break;
		case eventInformation: 
			uType = EVENTLOG_INFORMATION_TYPE; break;
		case eventAuditSuccess: 
			uType = EVENTLOG_AUDIT_SUCCESS; break;
		case eventAuditFailure: 
			uType = EVENTLOG_AUDIT_FAILURE; break;
		default: 
			throw TXInvalidEventType(THIS_PLACE);
	} /*switch (Type)*/

	return uType;
} /*TWin32EventLog::ToStdEventType(TWin32EventLog::TEventType)*/

//----------------------------------------------------------------------------
TWin32EventLog::TEventType TWin32EventLog::FromStdEventType(

	WORD uType

){
	TEventType Type;

	switch (uType) {
		case EVENTLOG_ERROR_TYPE: 
			Type = eventError; break;
		case EVENTLOG_WARNING_TYPE: 
			Type = eventWarning; break;
		case EVENTLOG_INFORMATION_TYPE: 
			Type = eventInformation; break;
		case EVENTLOG_AUDIT_SUCCESS: 
			Type = eventAuditSuccess; break;
		case EVENTLOG_AUDIT_FAILURE: 
			Type = eventAuditFailure; break;
		default: 
			throw TXInvalidEventType(THIS_PLACE);
	} /*switch (uType)*/

	return Type;
} /*TWin32EventLog::FromStdEventType(WORD)*/

//----------------------------------------------------------------------------
TWin32EventLog::TWin32EventLog(

	const TStdString& c_sApplication,
	const TStdString& c_sMessagesDll

):
	wsApplication(NWEVLOG_STRING_TYPE_CAST(c_sApplication)),
    wsMessagesDll(NWEVLOG_STRING_TYPE_CAST(c_sMessagesDll))
{
	Init();
} /*TWin32EventLog(const TStdString&, const TStdString&)*/

#ifndef _NATLIB_NO_WIDE_STRINGS_
//----------------------------------------------------------------------------
TWin32EventLog::TWin32EventLog(

	const TWideString& c_sApplication,
	const TWideString& c_sMessagesDll

):
	wsApplication(c_sApplication),
	wsMessagesDll(c_sMessagesDll)
{
	Init();
} /*TWin32EventLog(const TWideString&, const TWideString&)*/;
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

//----------------------------------------------------------------------------
TWin32EventLog::~TWin32EventLog()
{
	if (hEventSource) {
		HANDLE hToDeregister = hEventSource; hEventSource = null;
		DeregisterEventSource(hToDeregister); 
	} /*if*/
} /*~TWin32EventLog()*/

//----------------------------------------------------------------------------
bool TWin32EventLog::ReportEvent(

	TWin32EventLog::TEventType Type,
	WORD uCategory,
	DWORD uEventId,
	const TStdString& c_sParam0,
	const TStdString& c_sParam1 /*= c_sNull*/,
	const TStdString& c_sParam2 /*= c_sNull*/,
	const TStdString& c_sParam3 /*= c_sNull*/,
	const TStdString& c_sParam4 /*= c_sNull*/,
	const TStdString& c_sParam5 /*= c_sNull*/,
	const TStdString& c_sParam6 /*= c_sNull*/,
	const TStdString& c_sParam7 /*= c_sNull*/,
	const TStdString& c_sParam8 /*= c_sNull*/

){
	// Parameters index & number
	//---------------------------
	const char* apc_szParams[10];
	apc_szParams[0] = c_sParam0.c_str();
	apc_szParams[1] = c_sParam1.c_str();
	apc_szParams[2] = c_sParam2.c_str();
	apc_szParams[3] = c_sParam3.c_str();
	apc_szParams[4] = c_sParam4.c_str();
	apc_szParams[5] = c_sParam5.c_str();
	apc_szParams[6] = c_sParam6.c_str();
	apc_szParams[7] = c_sParam7.c_str();
	apc_szParams[8] = c_sParam8.c_str();
	apc_szParams[9] = c_sNull.c_str();

	size_t nParams = 0; while (*(apc_szParams[nParams])) ++nParams;

	return (
		ReportEvent(
			DWORD(uEventId), nParams, apc_szParams,
			ToStdEventType(Type), uCategory
		) /*ReportEvent*/
	) /*return*/;
} /*TWin32EventLog::ReportEvent(TWin32EventLog::TEventType, WORD, ...)*/;

#ifndef _NATLIB_NO_WIDE_STRINGS_
//----------------------------------------------------------------------------
bool TWin32EventLog::ReportEvent(

	TWin32EventLog::TEventType Type,
	WORD uCategory,
	DWORD uEventId,
	const TWideString& c_wsParam0,
	const TWideString& c_wsParam1 /*= c_wsNull*/,
	const TWideString& c_wsParam2 /*= c_wsNull*/,
	const TWideString& c_wsParam3 /*= c_wsNull*/,
	const TWideString& c_wsParam4 /*= c_wsNull*/,
	const TWideString& c_wsParam5 /*= c_wsNull*/,
	const TWideString& c_wsParam6 /*= c_wsNull*/,
	const TWideString& c_wsParam7 /*= c_wsNull*/,
	const TWideString& c_wsParam8 /*= c_wsNull*/

){
	// Parameters index & number
	//---------------------------
	const wchar_t* apc_wszParams[10];
	apc_wszParams[0] = c_wsParam0.c_str();
	apc_wszParams[1] = c_wsParam1.c_str();
	apc_wszParams[2] = c_wsParam2.c_str();
	apc_wszParams[3] = c_wsParam3.c_str();
	apc_wszParams[4] = c_wsParam4.c_str();
	apc_wszParams[5] = c_wsParam5.c_str();
	apc_wszParams[6] = c_wsParam6.c_str();
	apc_wszParams[7] = c_wsParam7.c_str();
	apc_wszParams[8] = c_wsParam8.c_str();
	apc_wszParams[9] = c_wsNull.c_str();

	size_t nParams = 0; while (*(apc_wszParams[nParams])) ++nParams;

	return (
		ReportEvent(
			DWORD(uEventId), nParams, apc_wszParams,
			ToStdEventType(Type), uCategory
		) /*ReportEvent*/
	) /*return*/;
} /*TWin32EventLog::ReportEvent(TWin32EventLog::TEventType, WORD, ...)*/;
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

//----------------------------------------------------------------------------
bool TWin32EventLog::ReportEvent(

	DWORD uEventId,
	size_t nParams,
	const wchar_t* ac_wszParams[],
	WORD uType /*= EVENTLOG_INFORMATION_TYPE*/,
	WORD uCategory /*= 0*/

){
	BOOL bSuccess;

	if (!hEventSource) Init();

	// Now report the event, which will add this event to the event log
	//------------------------------------------------------------------
	bSuccess = ReportEventW(
		hEventSource,  // event-log handle
		uType,  // event type
		uCategory, // category zero
		uEventId,  // event ID
		null,  // no user SID
		nParams,  // number of substitution strings
		0,  // no binary data
		ac_wszParams,  // string array
		null  // address of data
	) /*ReportEventW*/;

	if (!bSuccess) uLastError = GetLastError();

	return bSuccess;
} /*TWin32EventLog::ReportEvent(DWORD, size_t, const wchar_t*, WORD, WORD)*/

//----------------------------------------------------------------------------
bool TWin32EventLog::ReportEvent(

	DWORD uEventId, 
	size_t nParams,
	const char* a_szParams[],
	WORD uType /*= EVENTLOG_INFORMATION_TYPE*/,
	WORD uCategory /*= 0*/
	
){
	BOOL bSuccess;

	if (!hEventSource) Init();

	// Now report the event, which will add this event to the event log 
	//------------------------------------------------------------------
	bSuccess = ::ReportEventA(
		hEventSource,  // event-log handle 
		uType,  // event type
		uCategory, // category zero
		uEventId,  // event ID
		null,  // no user SID
		nParams,  // number of substitution strings
		0,  // no binary data
		a_szParams,  // string array
		null  // address of data
	) /*ReportEventA*/;

	if (!bSuccess) uLastError = GetLastError();

	return bSuccess;
} /*TWin32EventLog::ReportEvent(DWORD, size_t, const wchar_t*, WORD, WORD)*/

//----------------------------------------------------------------------------
void TWin32EventLog::Init()
{
	// Works only under Windows NT
	//-----------------------------
	if (!Win32IsNT()) throw TXInvalidOSVersion(THIS_CLASS);

	// When an application uses the RegisterEventSource or OpenEventLog
	// function to get a handle of an event log, the event loggging service
	// searches for the specified source name in the registry. You can add a
	// new source name to the registry by opening a new registry subkey
	// under the Application key and adding registry values to the new subkey.
	//------------------------------------------------------------------------
	
	// Add source to the registry
	//----------------------------
	{
		// Create a new key for the application
		//--------------------------------------
		HKEY hKey; {
            NWEVLOG_STRING_TYPE wsKeyName(
            	NWEVLOG_STRING_TYPE_CAST(c_szRegistryPath)
            ) /*wsKeyName*/;
			wsKeyName += NWEVLOG_STRING_TYPE_CAST(wsApplication);
            TAutoPointerTo<wchar_t> h_wszKeyName(
            	SzToWide(wsKeyName.c_str()), true
            ) /*h_wszKeyName*/;

			LONG uError = RegCreateKeyW(
				HKEY_LOCAL_MACHINE, (const wchar_t*)h_wszKeyName, &hKey
			) /*RegCreateKeyW*/;

			if (uError != ERROR_SUCCESS) {
				uLastError = uError;
				throw TXUnableToCreateRegistryKey(
					THIS_PLACE, Win32GetErrorExplanationA(uError)
				) /*TXUnableToCreateRegistryKey*/;
			} /*if*/
		} /*hKey*/

		// Add the Event-ID message-file name to the subkey
		//--------------------------------------------------
		{
        	TAutoPointerTo<wchar_t> h_wszEventMessageFileValue(
            	SzToWide(c_szEventMessageFileValue), true
            ) /*h_wszEventMessageFileValue*/;
            TAutoPointerTo<wchar_t> h_wszMessagesDll(
	            SzToWide(wsMessagesDll.c_str()), true
            ) /*h_wszMessagesDll*/;
			LONG uError = RegSetValueExW(
				hKey,  // subkey handle
				(const wchar_t*)h_wszEventMessageFileValue, // value name
				0,  // must be zero
				REG_EXPAND_SZ, // value type
				(CONST BYTE*)(const wchar_t*)h_wszMessagesDll,
				(wsMessagesDll.length() + 1) * sizeof(wchar_t)
			) /*RegSetValueExW*/;   

			if (uError != ERROR_SUCCESS) {
				uLastError = uError;
				throw TXUnableToSetRegistryValue(
					THIS_PLACE, Win32GetErrorExplanationA(uError)
				) /*TXUnableToSetRegistryValue*/;
			} /*if*/
		} /*Add the Event-ID message-file name to the subkey*/

		// Set the supported types flags and addit to the subkey
		//-------------------------------------------------------
		{
			DWORD dwData = (
				EVENTLOG_ERROR_TYPE | EVENTLOG_WARNING_TYPE | 
				EVENTLOG_INFORMATION_TYPE
			) /*wdData*/;

			TAutoPointerTo<wchar_t> h_wszTypesSupportedValue(
            	SzToWide(c_szTypesSupportedValue), true
            ) /*h_wszTypesSupportedValue*/;

			LONG uError = RegSetValueExW(
				hKey,  // subkey handle
				(const wchar_t*)h_wszTypesSupportedValue, // value name
				0,  // must be zero
				REG_DWORD,  // value type
				(CONST BYTE*)&dwData,  // address of value data
				sizeof(DWORD)  // length of value data
			) /*RegSetValueExW*/;

			if (uError != ERROR_SUCCESS) {
				uLastError = uError;
				throw TXUnableToSetRegistryValue(
					THIS_PLACE, Win32GetErrorExplanation(uError)
				) /*TXUnableToSetRegistryValue*/;
			} /*if*/
		} /*Set the supported types flags and addit to the subkey*/

		LONG uError = RegCloseKey(hKey);
		if (uError != ERROR_SUCCESS) uLastError = uError;
	} /*Add source to the registry*/

	// Get a handle to the Application event log 
	//-------------------------------------------
	{
    	TAutoPointerTo<wchar_t> h_wszApplication(
	        SzToWide(wsApplication.c_str()), true
        ) /*h_wszApplication*/;

		hEventSource = RegisterEventSourceW(
        	null, (const wchar_t*)h_wszApplication
        ) /*RegisterEventSourceW*/;

		if (!hEventSource) {
			uLastError = GetLastError();
			throw TXUnableToRegisterEventSource(
				THIS_CLASS, Win32GetErrorExplanation(uLastError)
			) /*throw TXCanRegisterEventSource*/;
		} /*if*/
	} /*Get a handle to the Application event log */
} /*TWin32EventLog::Init()*/

//============================================================================
NATLIB_NAMESPACE_END
 