/****************************************************************************/
/** nwlock.cpp                                                             **/
/** ---------------------------------------------------------------------- **/
/** Natlib (for Win32) syncronization facility base                        **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** June 24, 1997 by Oleg Kagan                                            **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>
#pragma hdrstop

//============================================================================
#include <nwlock.h>
#include <nwerror.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TWin32Lock implementation (for Microsoft Win32 Operating Systems)
//============================================================================
const SECURITY_ATTRIBUTES TWin32Lock::c_SecurityAttributes = {
	sizeof(SECURITY_ATTRIBUTES), null, true
} /*TWin32Lock::c_SecurityAttributes*/;

//----------------------------------------------------------------------------
TWin32Lock::TWin32Lock() :
	hMemberAccessMutex(null), hMutex(null)
{
	SECURITY_ATTRIBUTES LocalSecurityAttributes = c_SecurityAttributes;

	hMemberAccessMutex = CreateMutex(&LocalSecurityAttributes, false, null);
	if (!hMemberAccessMutex) {
		DWORD uError = GetLastError();
		throw TXUnableToCreateMutex(
			THIS_CLASS, Win32GetErrorExplanationA(uError)
		) /*throw TXUnableToCreateMutex*/;
	} /*if*/

    hMutex = CreateMutex(&LocalSecurityAttributes, false, null);
	if (!hMutex) {
		DWORD uError = GetLastError();
		throw TXUnableToCreateMutex(
			THIS_CLASS, Win32GetErrorExplanation(uError)
		) /*throw TXUnableToCreateMutex*/;
	} /*if*/		
} /*TWin32Lock()*/

//----------------------------------------------------------------------------
TWin32Lock::~TWin32Lock()
{
	if (hMemberAccessMutex) {
    	CloseHandle(hMemberAccessMutex); hMemberAccessMutex = null;
	} /*if*/

	if (hMutex) {
    	CloseHandle(hMutex); hMutex = null;
    } /*if*/
} /*~TWin32Lock();*/

//----------------------------------------------------------------------------
NLOBJECT_CLONE_RESULT(TWin32Lock) TWin32Lock::Clone() const 
{
	TWin32Lock* pCopy; 
	NLEXCEPT_CLASS_NEW(TWin32Lock(), pCopy); 
	return pCopy; 
} /*TWin32Lock::Clone() const*/

//----------------------------------------------------------------------------
void TWin32Lock::Lock(

	const void* pc_LockKey,
	nl_ulong_t uWaitTimeout /*= 0*/

){
	if (
		WaitForSingleObject(
			hMutex,
			(uWaitTimeout != c_uWaitInfinite) ?
				DWORD(uWaitTimeout) : DWORD(INFINITE)
		) /*WaitForSingleObject*/ == WAIT_FAILED
	){
		DWORD uError = GetLastError();
		if (uError == WAIT_TIMEOUT) {
			throw TXBusy(THIS_CLASS);
		}
		else {
			throw TXOperationFailed(
				THIS_CLASS, Win32GetErrorExplanationA(uError)
			) /*throw TXOperationFailed*/;
		} /*iff*/
	} /*if*/

	LockMemberAccess();	
	try {
		TParent::Lock(pc_LockKey, uWaitTimeout);
	} /*try*/
	catch (...) {UnlockMemberAccess(); throw;}
	UnlockMemberAccess();
} /*TWin32Lock::Lock(const void*, nl_ulong_t = 0)*/

//----------------------------------------------------------------------------
void TWin32Lock::Unlock(

	const void* pc_LockKey
	
){
	LockMemberAccess();
	try {
		if (pc_Key) {
			if (!ReleaseMutex(hMutex)) {
				DWORD uError = GetLastError();
				throw TXUnableToReleaseMutex(
                	THIS_CLASS, Win32GetErrorExplanationA(uError)
                ) /*throw TXUnableToReleaseMutex*/;
			} /*if*/
            TParent::Unlock(pc_LockKey);
		} /*if*/
	} /*try*/
	catch (...) {UnlockMemberAccess(); throw;}
	UnlockMemberAccess();
} /*TWin32Lock::Unlock(const void*) throw(TXBadKey)*/

//----------------------------------------------------------------------------
const void* TWin32Lock::Key() const
{
	LockMemberAccess();
	const void* c_pKey = null;
	try {c_pKey = TParent::Key();} catch (...) {UnlockMemberAccess(); throw;}
	UnlockMemberAccess();
    return c_pKey;
} /*TWin32Lock::Key() const*/

//----------------------------------------------------------------------------
void TWin32Lock::LockMemberAccess() const
{
	if (
		WaitForSingleObject(hMemberAccessMutex, INFINITE) == WAIT_FAILED
	){
		DWORD uError = GetLastError();
		throw TXUnableToGetMemberAccess(
			THIS_CLASS, Win32GetErrorExplanationA(uError)
		) /*throw TXUnableToGetMemberAccess(*/;
	} /*if*/;
} /*TWin32Lock::LockMemberAccess() const*/

//----------------------------------------------------------------------------
void TWin32Lock::UnlockMemberAccess() const
{
	if (!ReleaseMutex(hMemberAccessMutex)) {
		DWORD uError = GetLastError();
		throw TXUnableToReleaseMutex(
			THIS_CLASS, Win32GetErrorExplanationA(uError)
		) /*throw TXUnableToReleaseMutex*/;
	} /*if*/
} /*TWin32Lock::UnlockMemberAccess() const*/

//============================================================================
NATLIB_NAMESPACE_END

