/****************************************************************************/
/** nwreg.cpp                                                              **/
/** ---------------------------------------------------------------------- **/
/** C++ envelope for Win32 Registry operation                              **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** August 13, 1998 by Oleg Kagan                                          **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>
#pragma hdrstop

// Batlib Basics includes
//============================================================================
#include <nlstring.h>

// Natlib Win32 includes
//============================================================================
#include <nwreg.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TWin32Registry implementation
//============================================================================
const char TWin32Registry::c_cPathDelimiter = '\\';

//----------------------------------------------------------------------------
HKEY TWin32Registry::RegCreateKey(

	HKEY hKey,      
	const wchar_t* c_wszSubkey /*= null*/

){
	if (!Win32IsNT()) {
		if (!c_wszSubkey) return RegCreateKey(hKey, (const char*)null);
		return RegCreateKey(hKey, ToStdString(c_wszSubkey).c_str());
	} /*if*/

	HKEY hResultKey;
	LONG lError = RegCreateKeyW(hKey, c_wszSubkey, &hResultKey);
	if (lError != ERROR_SUCCESS) {
		throw TXRegOperationFailed(
			lError, THIS_PLACE, Win32GetErrorExplanation(lError)
		) /*throw TXRegOperationFailed*/;
	} /*if*/
	return hResultKey;
} /*TWin32Registry::RegCreateKey(HKEY, const wchar_t*)*/

//----------------------------------------------------------------------------
HKEY TWin32Registry::RegCreateKey(

	HKEY hKey,      
	const char* c_szSubkey

){
	HKEY hResultKey;
	LONG lError = ::RegCreateKeyA(hKey, c_szSubkey, &hResultKey);
	if (lError != ERROR_SUCCESS) {
		throw TXRegOperationFailed(
			lError, THIS_PLACE, Win32GetErrorExplanation(lError)
		) /*throw TXRegOperationFailed*/;
	} /*if*/
	return hResultKey;
} /*TWin32Registry::RegCreateKey(HKEY, const char*)*/

//----------------------------------------------------------------------------
HKEY TWin32Registry::RegOpenKey(

	HKEY hKey,      
	const wchar_t* c_wszSubkey /*= null*/

){
	if (!Win32IsNT()) {
		if (!c_wszSubkey) return RegOpenKey(hKey, (const char*)null);
		return RegOpenKey(hKey, ToStdString(c_wszSubkey).c_str());
	} /*if*/

	HKEY hResultKey;
	LONG lError = ::RegOpenKeyExW(
		hKey, c_wszSubkey, NULL, KEY_ALL_ACCESS, &hResultKey
	) /*::RegOpenKeyExW*/;
	if (lError != ERROR_SUCCESS) {
		throw TXRegOperationFailed(
			lError, THIS_PLACE, Win32GetErrorExplanation(lError)
		) /*throw TXRegOperationFailed*/;
	} /*if*/
	return hResultKey;
} /*TWin32Registry::RegOpenKey(HKEY, const wchar_t*)*/

//----------------------------------------------------------------------------
HKEY TWin32Registry::RegOpenKey(

	HKEY hKey,      
	const char* c_szSubkey

){
	HKEY hResultKey;
	LONG lError = ::RegOpenKeyExA(
		hKey, c_szSubkey, null, KEY_ALL_ACCESS, &hResultKey
	) /*::RegOpenKeyExA*/;
	if (lError != ERROR_SUCCESS) {
		throw TXRegOperationFailed(
			lError, THIS_PLACE, Win32GetErrorExplanation(lError)
		) /*throw TXRegOperationFailed*/;
	} /*if*/
	return hResultKey;
} /*TWin32Registry::RegOpenKey(HKEY, const char*)*/

//----------------------------------------------------------------------------
void TWin32Registry::RegCloseKey(

	HKEY hKey
	
){
	LONG lError = ERROR_SUCCESS;

	try {::RegCloseKey(hKey);}
	catch (...) {
		throw TXRegOperationFailed(0, THIS_PLACE);
	} /*catch (...)*/

	if (lError != ERROR_SUCCESS) {
		throw TXRegOperationFailed(
			lError, THIS_PLACE, Win32GetErrorExplanation(lError)
		) /*throw TXRegOperationFailed*/;
	} /*if*/
} /*TWin32Registry::RegCloseKey(HKEY)*/

//----------------------------------------------------------------------------
DWORD TWin32Registry::RegQueryValue(

	HKEY hKey, 
	const wchar_t* c_wszName, 
	void* pValue, 
	DWORD nValueSize, 
	DWORD& r_dwType

){
	if (!Win32IsNT()) {
		return RegQueryValue(
			hKey, ToStdString(c_wszName).c_str(), pValue, nValueSize, r_dwType
		) /*return RegQueryValue*/;
	} /*if*/

	LONG lError = ::RegQueryValueExW(
		hKey, c_wszName, null, &r_dwType, (LPBYTE)pValue, &nValueSize
	) /*::RegQueryValueExW*/;

	if (lError != ERROR_SUCCESS) {
		throw TXRegOperationFailed(
			lError, THIS_PLACE, Win32GetErrorExplanation(lError)
		) /*throw TXRegOperationFailed*/;
	} /*if*/

	return nValueSize;
} /*TWin32Registry::RegQueryValue(HKEY, const wchar_t*, void*,DWORD, DWORD&)*/

//----------------------------------------------------------------------------
DWORD TWin32Registry::RegQueryValue(

	HKEY hKey, 
	const char* c_szName, 
	void* pValue, 
	DWORD nValueSize, 
	DWORD& r_dwType

){
	LONG lError = ::RegQueryValueExA(
		hKey, c_szName, null, &r_dwType, (LPBYTE)pValue, &nValueSize
	) /*::RegQueryValueExA*/;

	if (lError != ERROR_SUCCESS) {
		throw TXRegOperationFailed(
			lError, THIS_PLACE, Win32GetErrorExplanation(lError)
		) /*throw TXRegOperationFailed*/;
	} /*if*/

	return nValueSize;
} /*TWin32Registry::RegQueryValue(HKEY, const char*, void*, DWORD, DWORD&)*/

//----------------------------------------------------------------------------
void TWin32Registry::RegSetValue(

	HKEY hKey, 
	const wchar_t* c_wszName, 
	const void* pc_Value, 
	DWORD nValueSize, 
	DWORD dwType
	
){
	if (!Win32IsNT()) {
		RegSetValue(
			hKey, ToStdString(c_wszName).c_str(), pc_Value, nValueSize, dwType
		) /*return RegSetValue*/;
	}
	else {
		LONG lError = ::RegSetValueExW(
			hKey, c_wszName, null, dwType, (CONST BYTE*)pc_Value, nValueSize
		) /*::RegSetValueExW*/;

		if (lError != ERROR_SUCCESS) {
			throw TXRegOperationFailed(
				lError, THIS_PLACE, Win32GetErrorExplanation(lError)
			) /*throw TXRegOperationFailed*/;
		} /*if*/
	} /*iff*/
} /*TWin32Registry::RegSetValue(HKEY, const wchar_t*, const void*, ...)*/

//----------------------------------------------------------------------------
void TWin32Registry::RegSetValue(

	HKEY hKey, 
	const char* c_szName, 
	const void* pc_Value, 
	DWORD nValueSize, 
	DWORD dwType

){
	LONG lError = ::RegSetValueExA(
		hKey, c_szName, null, dwType, (CONST BYTE*)pc_Value, nValueSize
	) /*::RegSetValueExA*/;

	if (lError != ERROR_SUCCESS) {
		throw TXRegOperationFailed(
			lError, THIS_PLACE, Win32GetErrorExplanation(lError)
		) /*throw TXRegOperationFailed*/;
	} /*if*/
} /*TWin32Registry::RegSetValue(HKEY, const char*, const void*, ...)*/

//----------------------------------------------------------------------------
void TWin32Registry::RegDeleteKey(

	HKEY hKey, 
	const wchar_t* c_wszSubkey

){
	if (!c_wszSubkey || !*c_wszSubkey)
		throw TXInvalidParameter(THIS_PLACE);

	if (!Win32IsNT()) {
		RegDeleteKey(hKey, ToStdString(c_wszSubkey).c_str()); 
	}
	else {
		LONG lError = ::RegDeleteKeyW(hKey, c_wszSubkey);
		if (lError != ERROR_SUCCESS) {
			throw TXRegOperationFailed(
				lError, THIS_PLACE, Win32GetErrorExplanation(lError)
			) /*throw TXRegOperationFailed*/;
		} /*if*/
	} /*iff*/
} /*TWin32Registry::RegDeleteKey(HKEY, const wchar_t*)*/;

//----------------------------------------------------------------------------
void TWin32Registry::RegDeleteKey(

	HKEY hKey, 
	const char* c_szSubkey

){
	if (!c_szSubkey || !*c_szSubkey)
		throw TXInvalidParameter(THIS_PLACE);

	LONG lError = ::RegDeleteKeyA(hKey, c_szSubkey);
	if (lError != ERROR_SUCCESS) {
		throw TXRegOperationFailed(
			lError, THIS_PLACE, Win32GetErrorExplanation(lError)
		) /*throw TXRegOperationFailed*/;
	} /*if*/
} /*TWin32Registry::RegDeleteKey(HKEY, const char*)*/;

//----------------------------------------------------------------------------
void TWin32Registry::RegDeleteValue(

	HKEY hKey, 
	const wchar_t* c_wszName
	
){
	if (!Win32IsNT()) {
		RegDeleteValue(hKey, ToStdString(c_wszName).c_str());
	}
	else {
		LONG lError = ::RegDeleteValueW(hKey, c_wszName);
		if (lError != ERROR_SUCCESS) {
			throw TXRegOperationFailed(
				lError, THIS_PLACE, Win32GetErrorExplanation(lError)
			) /*throw TXRegOperationFailed*/;
		} /*if*/
	} /*iff*/
} /*TWin32Registry::RegDeleteValue(HKEY, const wchar_t*)*/

//----------------------------------------------------------------------------
void TWin32Registry::RegDeleteValue(

	HKEY hKey, 
	const char* c_szName
	
){
	LONG lError = ::RegDeleteValueA(hKey, c_szName);
	if (lError != ERROR_SUCCESS) {
		throw TXRegOperationFailed(
			lError, THIS_PLACE, Win32GetErrorExplanation(lError)
		) /*throw TXRegOperationFailed*/;
	} /*if*/
} /*TWin32Registry::RegDeleteValue(HKEY, const char*)*/

//----------------------------------------------------------------------------
HKEY TWin32Registry::ResolvePath(

	HKEY hRootKey,
	const wchar_t* c_wszPath,
	bool bCreate /* = false*/

){
	if (!Win32IsNT()) 
		return ResolvePath(hRootKey, ToStdString(c_wszPath).c_str(), bCreate);

	if (!hRootKey) 
		throw TXInvalidParameter(THIS_PLACE);

	if (!c_wszPath || !*c_wszPath) return RegOpenKey(hRootKey);

	HKEY hCurrentKey = RegOpenKey(hRootKey);
	HKEY hNextKey = null;
	try {
		TAutoPointerTo<wchar_t> h_wszName(null, true);
		NLEXCEPT_NEW(wchar_t[SzLength(c_wszPath) + 1], h_wszName);

		while (*c_wszPath) {
			if (*c_wszPath == wchar_t(c_cPathDelimiter)) ++c_wszPath;

			wchar_t* p_wcCopyTo = (wchar_t*)h_wszName;
			while (*c_wszPath && (*c_wszPath != wchar_t(c_cPathDelimiter))) 
				*(p_wcCopyTo++) = *(c_wszPath++);
			*p_wcCopyTo = wchar_t(0);

			if (p_wcCopyTo == (wchar_t*)h_wszName) 
				throw TXInvalidPath(THIS_PLACE);

			hNextKey = (bCreate) ? 
				RegCreateKey(hCurrentKey, (const wchar_t*)h_wszName) :
				RegOpenKey(hCurrentKey, (const wchar_t*)h_wszName);

			RegCloseKey(hCurrentKey); hCurrentKey = hNextKey; 
			hNextKey = null;
		} /*while*/
	} /*try*/
	catch (...) {
		try {if (hCurrentKey != null) RegCloseKey(hCurrentKey);} 
		catch (...) {}
		try {if (hNextKey != null) RegCloseKey(hNextKey);}
		catch (...) {}
		throw;
	} /*catch (...)*/

	return hCurrentKey;	
} /*TWin32Registry::ResolvePath(HKEY, const wchar_t*, bool)*/

//----------------------------------------------------------------------------
HKEY TWin32Registry::ResolvePath(

	HKEY hRootKey,
	const char* c_szPath,
	bool bCreate /* = false*/

){
	if (!hRootKey) 
		throw TXInvalidParameter(THIS_PLACE);

	if (!c_szPath || !*c_szPath) return RegOpenKey(hRootKey);

	HKEY hCurrentKey = RegOpenKey(hRootKey);
	HKEY hNextKey = null;
	try {
		TAutoPointerTo<char> h_szName(null, true);
		NLEXCEPT_NEW(char[SzLength(c_szPath) + 1], h_szName);

		while (*c_szPath) {
			if (*c_szPath == c_cPathDelimiter) ++c_szPath;

			char* p_cCopyTo = (char*)h_szName;
			while (*c_szPath && (*c_szPath != c_cPathDelimiter)) 
				*(p_cCopyTo++) = *(c_szPath++);
			*p_cCopyTo = char(0);

			if (p_cCopyTo == (char*)h_szName) 
				throw TXInvalidPath(THIS_PLACE);

			hNextKey = (bCreate) ? 
				RegCreateKey(hCurrentKey, (const char*)h_szName) :
				RegOpenKey(hCurrentKey, (const char*)h_szName);

			RegCloseKey(hCurrentKey); hCurrentKey = hNextKey; 
			hNextKey = null;
		} /*while*/
	} /*try*/
	catch (...) {
		try {if (hCurrentKey != null) RegCloseKey(hCurrentKey);} 
		catch (...) {}
		try {if (hNextKey != null) RegCloseKey(hNextKey);}
		catch (...) {}
		throw;
	} /*catch (...)*/

	return hCurrentKey;	
} /*TWin32Registry::ResolvePath(HKEY, const char*, bool)*/

//----------------------------------------------------------------------------
HKEY TWin32Registry::CreateKey(

	HKEY hKey, 
	const wchar_t* c_wszPath
	
){
	return ResolvePath(hKey, c_wszPath, true); 
} /*TWin32Registry::CreateKey(HKEY, const wchar_t* = null)*/

//----------------------------------------------------------------------------
HKEY TWin32Registry::CreateKey(

	HKEY hKey, 
	const char* c_szPath /*= null*/
	
){
	return ResolvePath(hKey, c_szPath, true); 
} /*TWin32Registry::CreateKey(HKEY, const char*)*/

//----------------------------------------------------------------------------
HKEY TWin32Registry::OpenKey(

	HKEY hKey, 
	const wchar_t* c_wszPath /*= null*/
	
){
	return ResolvePath(hKey, c_wszPath, false); 
} /*TWin32Registry::OpenKey(HKEY, const wchar_t* = null)*/

//----------------------------------------------------------------------------
HKEY TWin32Registry::OpenKey(

	HKEY hKey, 
	const char* c_szPath
	
){
	return ResolvePath(hKey, c_szPath, false); 
} /*TWin32Registry::OpenKey(HKEY, const char*)*/

//----------------------------------------------------------------------------
TWin32Registry::TWin32Registry(

	const TWin32Registry& c_Source
	
):
	hRootKey(null), hTargetKey(null), h_wszPath(null, true)
{
	Assign(c_Source.hRootKey, (const wchar_t*)c_Source.h_wszPath, false);
} /*TWin32Registry(const TWin32Registry&)*/

//----------------------------------------------------------------------------
TWin32Registry::TWin32Registry(

	HKEY hRootKey, 
	const wchar_t* c_wszRootPath /*= null*/, 
	bool bCreate /*= false*/

):
	hRootKey(null), hTargetKey(null), h_wszPath(null, true)
{
	Assign(hRootKey, c_wszRootPath, bCreate);
} /*TWin32Registry(HKEY, const wchar_t*, bool)*/

//----------------------------------------------------------------------------
TWin32Registry::TWin32Registry(

	HKEY hRootKey, 
	const char* c_szRootPath, 
	bool bCreate /*= false*/

):
	hRootKey(null), hTargetKey(null), h_wszPath(null, true)
{
	Assign(hRootKey, c_szRootPath, bCreate);
} /*TWin32Registry(HKEY, const char*, bool)*/

#ifndef _NATLIB_NO_WIDE_STRINGS_
//----------------------------------------------------------------------------
TWin32Registry::TWin32Registry(

	HKEY hRootKey, 
	const TWideString& c_wsRootPath, 
	bool bCreate /*= false*/

):
	hRootKey(null), hTargetKey(null), h_wszPath(null, true)
{
	Assign(hRootKey, c_wsRootPath, bCreate);
} /*TWin32Registry(HKEY, const TWideString&, bool)*/
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

//----------------------------------------------------------------------------
TWin32Registry::TWin32Registry(

	HKEY hRootKey, 
	const TStdString& c_sRootPath, 
	bool bCreate /*= false*/

):
	hRootKey(null), hTargetKey(null), h_wszPath(null, true)
{
	Assign(hRootKey, c_sRootPath, bCreate);
} /*TWin32Registry::TWin32Registry(HKEY, const TStdString&, bool)*/

//----------------------------------------------------------------------------
TWin32Registry::~TWin32Registry()
{
	Unassign();
} /*~TWin32Registry()*/

//----------------------------------------------------------------------------
NLOBJECT_IMPLEMENT_CLONE_DEFAULT(TWin32Registry)

//----------------------------------------------------------------------------
void TWin32Registry::Assign(

	HKEY hNewKey, 
	const wchar_t* c_wszNewPath /*= null*/,
	bool bCreate /*= false*/
	
){
	if (!hNewKey) 
		throw TXInvalidParameter(THIS_CLASS);

	Unassign();

	if (c_wszNewPath && !*c_wszNewPath) c_wszNewPath = null;

	HKEY hNewTargetKey = ResolvePath(hNewKey, c_wszNewPath, bCreate);

	try {hRootKey = RegOpenKey(hNewKey);}
	catch (...) {try {RegCloseKey(hNewTargetKey);} catch (...) {} throw;}

	hTargetKey = hNewTargetKey;
	if (c_wszNewPath && *c_wszNewPath) h_wszPath = SzToWide(c_wszNewPath);
} /*TWin32Registry::Assign(HKEY, const wchar_t*, bool)*/

//----------------------------------------------------------------------------
void TWin32Registry::Assign(

	HKEY hNewKey, 
	const char* c_szNewPath,
	bool bCreate /*= false*/
	
){
	TAutoPointerTo<wchar_t> h_wszNewPath(SzToWide(c_szNewPath), true);
	Assign(hNewKey, (const wchar_t*)h_wszNewPath, bCreate);
} /*TWin32Registry::Assign(HKEY, const char*)*/

//----------------------------------------------------------------------------
void TWin32Registry::Unassign()
{
	HKEY hRootKeyToClose = hRootKey; hRootKey = null;
	HKEY hTargetKeyToClose = hTargetKey; hTargetKey = null;
	try {if (hRootKeyToClose) RegCloseKey(hRootKeyToClose);} 
	catch (...) {
		if (hTargetKeyToClose) RegCloseKey(hTargetKeyToClose); 
		throw;
	} /*catch (...)*/
	if (hTargetKeyToClose) RegCloseKey(hTargetKeyToClose);

	h_wszPath.Unassign(true);
} /*TWin32Registry::Unassign()*/

//----------------------------------------------------------------------------
DWORD TWin32Registry::QueryValue(

	const wchar_t* c_wszName, 
	void* pValue, 
	DWORD nValueSize, 
	DWORD& r_dwType

){
	return RegQueryValue(hTargetKey, c_wszName, pValue, nValueSize, r_dwType);
} /*TWin32Registry::QueryValue(const wchar_t*, void*, DWORD, DWORD&)*/

//----------------------------------------------------------------------------
DWORD TWin32Registry::QueryValue(

	const char* c_szName, 
	void* pValue, 
	DWORD nValueSize, 
	DWORD& r_dwType
	
){
	return RegQueryValue(hTargetKey, c_szName, pValue, nValueSize, r_dwType);
} /*TWin32Registry::QueryValue(const char*, void*, DWORD, DWORD&)*/

#ifndef _NATLIB_NO_WIDE_STRINGS_
//----------------------------------------------------------------------------
TWideString TWin32Registry::QueryStringValue(

	const wchar_t* c_wszName,
	DWORD* p_dwType /*= null*/
	
){
	DWORD dwType = 0;
	DWORD nValueLength = QueryValue(c_wszName, null, 0, dwType);

	TAutoPointerTo<char> ha_cValue(null, true);
	NLEXCEPT_CLASS_NEW(char[nValueLength + 2], ha_cValue);

	nValueLength = QueryValue(
		c_wszName, (LPBYTE)(char*)ha_cValue, nValueLength + 1, dwType
	) /*QueryValue*/;

	TWideString wsResult(	
		(const wchar_t*)(char*)ha_cValue, nValueLength / sizeof(wchar_t)
	) /*wsResult*/;

	if ((dwType == REG_EXPAND_SZ) || (dwType == REG_SZ)) 
		wsResult.resize(SzLength(wsResult.c_str()));

	if (p_dwType) *p_dwType = dwType;
	return wsResult;
} /*TWin32Registry::QueryStringValue(const wchar_t*, DWORD*)*/
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

//----------------------------------------------------------------------------
TStdString TWin32Registry::QueryStringValue(

	const char* c_szName,
	DWORD* p_dwType /*= null*/
	
){
	DWORD dwType = 0;
	DWORD nValueLength = QueryValue(c_szName, null, 0, dwType);

	TAutoPointerTo<char> ha_cValue(null, true);
	NLEXCEPT_CLASS_NEW(char[nValueLength + 2], ha_cValue);

	nValueLength = QueryValue(
		c_szName, (LPBYTE)(char*)ha_cValue, nValueLength + 1, dwType
	) /*QueryValue*/;

	TStdString sResult((char*)ha_cValue, nValueLength);

	if ((dwType == REG_EXPAND_SZ) || (dwType == REG_SZ)) 
		sResult.resize(SzLength(sResult.c_str()));

	if (p_dwType) *p_dwType = dwType;
	return sResult;
} /*TWin32Registry::QueryStringValue(const char*, DWORD*)*/

//----------------------------------------------------------------------------
void TWin32Registry::SetValue(

	const wchar_t* c_wszName, 
	const void* pc_Value, 
	DWORD nValueSize, 
	DWORD dwType
	
){
	RegSetValue(hTargetKey, c_wszName, pc_Value, nValueSize, dwType);
} /*TWin32Registry::SetValue(const wchar_t*, const void*, size_t, DWORD)*/;

//----------------------------------------------------------------------------
void TWin32Registry::SetValue(

	const char* c_szName, 
	const void* pc_Value, 
	DWORD nValueSize, 
	DWORD dwType
	
){
	RegSetValue(hTargetKey, c_szName, pc_Value, nValueSize, dwType);
} /*TWin32Registry::SetValue(const char*, const void*, size_t, DWORD)*/

//----------------------------------------------------------------------------
void TWin32Registry::SetStringValue(

	const wchar_t* c_wszName, 
	const wchar_t* c_wszValue,
	DWORD dwType /*= REG_SZ*/
	
){
	SetValue(
		c_wszName, c_wszValue, 
		SzLength(c_wszValue) * sizeof(wchar_t), dwType
	) /*SetValue*/;
} /*TWin32Registry::SetStringValue(const wchar_t*, const wchar_t*, DWORD)*/

//----------------------------------------------------------------------------
void TWin32Registry::SetStringValue(

	const char* c_szName, 
	const char* c_szValue,
	DWORD dwType /*= REG_SZ*/

){
	SetValue(c_szName, c_szValue, SzLength(c_szValue), dwType);
} /*TWin32Registry::SetStringValue(const char*, const char*, DWORD)*/

#ifndef _NATLIB_NO_WIDE_STRINGS_
//----------------------------------------------------------------------------
void TWin32Registry::SetStringValue(

	const TWideString& c_wsName, 
	const TWideString& c_wsValue,
	DWORD dwType /*= REG_SZ*/

){
	SetValue(
		c_wsName.c_str(), 
		c_wsValue.data(), c_wsValue.length() * sizeof(wchar_t), dwType
	) /*SetValue*/;
} /*TWin32Registry::SetStringValue(const TWideString&, ...)*/
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

//------------------------------------------------------------------------
void TWin32Registry::SetStringValue(

	const TStdString& c_sName, 
	const TStdString& c_sValue,
	DWORD dwType /*= REG_SZ*/

){
	SetValue(
		c_sName.c_str(),
#ifdef __BORLANDC__
		c_sValue.c_str(), c_sValue.length(),
#else
		c_sValue.data(), c_sValue.length(),
#endif /*__BORLANDC__*/
        dwType
	) /*SetValue*/;
} /*TWin32Registry::SetStringValue(const TStdString&, ...)*/

//----------------------------------------------------------------------------
void TWin32Registry::Delete(

	const wchar_t* c_wszSubkey
	
){
	if (c_wszSubkey && *c_wszSubkey) {
		RegDeleteKey(hTargetKey, c_wszSubkey);
	} 
	else {
		GetParent().Delete(GetName());
	} /*if*/
} /*TWin32Registry::Delete(const wchar_t*)*/

//----------------------------------------------------------------------------
void TWin32Registry::Delete(

	const char* c_szSubkey /*= null*/
	
){
	if (c_szSubkey && *c_szSubkey) {
		RegDeleteKey(hTargetKey, c_szSubkey);
	} 
	else {
		GetParent().Delete(GetName());
	} /*if*/
} /*TWin32Registry::Delete(const char*)*/

//----------------------------------------------------------------------------
void TWin32Registry::DeleteValue(

	const wchar_t* c_wszName
){
	RegDeleteValue(hTargetKey, c_wszName);
} /*TWin32Registry::DeleteValue(const wchar_t*)*/

//----------------------------------------------------------------------------
void TWin32Registry::DeleteValue(

	const char* c_szName /*= null*/
	
){
	RegDeleteValue(hTargetKey, c_szName);
} /*TWin32Registry::DeleteValue(const char*)*/

//----------------------------------------------------------------------------
TWin32Registry TWin32Registry::GetParent()
{
	TAutoPointerTo<wchar_t> h_wszParentPath(
		SzToWide((wchar_t*)h_wszPath), true
	) /*h_wszParentPath*/;
	wchar_t* p_wcLastDelimiter = GetLastDelimiter((wchar_t*)h_wszParentPath);
	if (!p_wcLastDelimiter || !*p_wcLastDelimiter)
		throw TXNoParentExists(THIS_CLASS);

	*p_wcLastDelimiter = wchar_t(0);

	return TWin32Registry(hRootKey, (wchar_t*)h_wszPath);
} /*TWin32Registry::GetParent() const*/

//----------------------------------------------------------------------------
const wchar_t* TWin32Registry::GetName()
{
	const wchar_t* pc_wcLastDelimiter = GetLastDelimiter((wchar_t*)h_wszPath);
	return pc_wcLastDelimiter ? pc_wcLastDelimiter + 1 : c_wszNull;
} /*TWin32Registry::GetName()*/

//----------------------------------------------------------------------------
wchar_t* TWin32Registry::GetLastDelimiter(

	wchar_t* wszPath
	
){
	if (!wszPath || !*wszPath) return null;

	wchar_t* p_wcIter = wszPath;

	wchar_t* p_wcLastDelimiter = p_wcIter;
	while (*p_wcIter) {
		if (*p_wcIter == wchar_t(c_cPathDelimiter)) 
			p_wcLastDelimiter = p_wcIter;
		++p_wcIter;
	} /*while*/

	return p_wcLastDelimiter;
} /*TWin32Registry::GetLastDelimiter(wchar_t*)*/

//============================================================================
NATLIB_NAMESPACE_END
