/****************************************************************************/
/** nwshllnk.cpp                                                           **/
/** ---------------------------------------------------------------------- **/
/** C++ envelope for Win32 Shell Links                                     **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** July 10, 1998 by Oleg Kagan                                            **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>
#pragma hdrstop

// Standard includes
//============================================================================
#include <shlobj.h>

// Natlib Basics includes
//============================================================================
#include <nlautopt.h>
 
// Natlib Win32 includes
//============================================================================
#include <nwutils.h>
#include <nwshllnk.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TWin32ShellLink implementation
//============================================================================
TWin32ShellLink::TWin32ShellLink(

	const TStdString& c_sInitTarget /*= c_sNull*/,
	const TStdString& c_sInitArguments /*= c_sNull*/,
	const TStdString& c_sInitWorkingDirectory /*= c_sNull*/,
	const TStdString& c_sInitIconPath /*= c_sNull*/,
	nl_int_t c_iInitIconIndex /*= nl_int_t(0)*/,
	const TStdString& c_sInitDescription /*= c_sNull*/,
	WORD c_wInitHotKey /*= WORD(0)*/,
	nl_int_t c_iInitShowCommand /*= nl_int_t(0)*/

):
	sTarget(NWSHLLNK_STRING_TYPE_CAST(c_sInitTarget)),
	sArguments(NWSHLLNK_STRING_TYPE_CAST(c_sInitArguments)),
	sWorkingDirectory(NWSHLLNK_STRING_TYPE_CAST(c_sInitWorkingDirectory)),
	sIconPath(NWSHLLNK_STRING_TYPE_CAST(c_sInitIconPath)),
	iIconIndex(c_iInitIconIndex),
	sDescription(NWSHLLNK_STRING_TYPE_CAST(c_sInitDescription)),
	wHotKey(c_wInitHotKey),
	iShowCommand(c_iInitShowCommand)
{} /*TWin32ShellLink(const NWSHLLNK_STRING_TYPE&, ...)*/

#ifndef _NATLIB_NO_WIDE_STRINGS_
//----------------------------------------------------------------------------
TWin32ShellLink::TWin32ShellLink(

	const TWideString& c_wsInitTarget /*= c_wsNull*/,
	const TWideString& c_wsInitArguments /*= c_wsNull*/,
	const TWideString& c_wsInitWorkingDirectory /*= c_wsNull*/,
	const TWideString& c_wsInitIconPath /*= c_wsNull*/,
	nl_int_t c_iInitIconIndex /*= nl_int_t(0)*/,
	const TWideString& c_wsInitDescription /*= c_wsNull*/,
	WORD c_wInitHotKey /*= WORD(0)*/,
	nl_int_t c_iInitShowCommand /*= nl_int_t(0)*/

):
	sTarget(c_wsInitTarget),
	sArguments(c_wsInitArguments),
	sWorkingDirectory(c_wsInitWorkingDirectory),
	sIconPath(c_wsInitIconPath),
	iIconIndex(c_iInitIconIndex),
	sDescription(c_wsInitDescription),
	wHotKey(c_wInitHotKey),
	iShowCommand(c_iInitShowCommand)
{} /*TWin32ShellLink(const TWideString&, const TWidetring&, ...)*/
#endif /*!defined (_NATLIB_NO_WIDE_STRINGS_)*/

//----------------------------------------------------------------------------
TWin32ShellLink::TWin32ShellLink(

	const TWin32ShellLink& c_Source

):
	sTarget(c_Source.sTarget),
	sArguments(c_Source.sArguments),
	sWorkingDirectory(c_Source.sWorkingDirectory),
	sIconPath(c_Source.sIconPath),
	iIconIndex(c_Source.iIconIndex),
	sDescription(c_Source.sDescription),
	wHotKey(c_Source.wHotKey),
	iShowCommand(c_Source.iShowCommand)
{} /*TWin32ShellLink(const TWin32ShellLink&)*/

//----------------------------------------------------------------------------
NLOBJECT_IMPLEMENT_CLONE_DEFAULT(TWin32ShellLink)

#ifndef _NATLIB_NO_WIDE_STRINGS_
//----------------------------------------------------------------------------
bool TWin32ShellLink::LoadFrom(

	const TWideString& c_wsPath,
	bool bResolve /*= false*/,
	DWORD uFlags /*= 0*/,
	HWND hParentWindow /*= null*/

){
	try {LoadFromEx(c_wsPath, bResolve, uFlags, hParentWindow);}
	catch (TException&) {return false;}
	catch (...) {throw;}
	return true;
} /*TWin32ShellLink::LoadFrom(const TWideString&)*/

//----------------------------------------------------------------------------
void TWin32ShellLink::LoadFromEx(

	const TWideString& c_wsPath,
	bool bResolve /*= false*/,
	DWORD uFlags /*= 0*/,
	HWND hParentWindow /*= null*/

){
	IShellLinkW* pShellLinkW = null;
	IPersistFile* pPersistFile = null;

	try {
		if (!Win32IsFileExists(c_wsPath))
			throw TXFileNotFound(THIS_CLASS, ToStdString(c_wsPath));

		// Create an IShellLink object and get a pointer to the IShellLink
		// interface (returned from CoCreateInstance)
		//-------------------------------------------------------------------
		if (
			FAILED(
				CoCreateInstance(
					CLSID_ShellLink, null, CLSCTX_INPROC_SERVER,
					IID_IShellLinkW, (void**)&pShellLinkW
				) /*CoCreateInstance*/
			) /*FAILED*/
		){
			pShellLinkW = null; throw TException(THIS_CLASS);
		} /*if*/

		// Query IShellLinkW for the IPersistFile interface for
		// saving the shortcut in persistent storage
		//-----------------------------------------------------
		if (
			FAILED(
				pShellLinkW->QueryInterface(
					IID_IPersistFile, (void**)&pPersistFile
				) /*pShellLinkW->QueryInterface*/
			) /*FAILED*/
		){
			pPersistFile = null; throw TException(THIS_CLASS);
		} /*if*/

		// Load shortcut file
		//--------------------
		{
			if (FAILED(pPersistFile->Load(c_wsPath.c_str(), STGM_READ)))
				throw TException(THIS_CLASS);
		} /*Load shortcut file*/

		// Target
		//--------
		{
			wchar_t wszTarget[MAX_PATH + 1];
			WIN32_FIND_DATAW FindData;
			if (
				FAILED(
					pShellLinkW->GetPath(
						wszTarget, sizeof(wszTarget) - 1, &FindData,
						SLGP_UNCPRIORITY
					) /*GetPath*/
				) /*FAILED*/
			) throw TException(THIS_CLASS);
			sTarget = wszTarget;
		} /*Target*/

		// Arguments
		//-----------
		{
			wchar_t wszArguments[MAX_PATH + 1];
			if (
				FAILED(
					pShellLinkW->GetArguments(
						wszArguments, sizeof(wszArguments) - 1
					) /*GetArguments*/
				) /*FAILED*/
			) throw TException(THIS_CLASS);
			sArguments = wszArguments;
		} /*Arguments*/

		// WorkingDirectory
		//------------------
		{
			wchar_t wszWorkingDir[MAX_PATH + 1];
			if (
				FAILED(
					pShellLinkW->GetWorkingDirectory(
						wszWorkingDir, sizeof(wszWorkingDir) - 1
					) /*GetWorkingDirectory*/
				) /*FAILED*/
			) throw TException(THIS_CLASS);
			sWorkingDirectory = wszWorkingDir;
		} /*WorkingDirectory*/

		// Icon
		//------
		{
			wchar_t wszIconPath[MAX_PATH + 1];
			if (
				FAILED(
					pShellLinkW->GetIconLocation(
						wszIconPath, sizeof(wszIconPath) - 1, &iIconIndex
					) /*GetIconLocation*/
				) /*FAILED*/
			) throw TException(THIS_CLASS);
			sIconPath = wszIconPath;
		} /*Icon*/

		// Description
		//-------------
		{
			wchar_t wszDescription[MAX_PATH + 1];
			if (
				FAILED(
					pShellLinkW->GetDescription(
						wszDescription, sizeof(wszDescription) - 1
					) /*GetDescription*/
				) /*FAILED*/
			) throw TException(THIS_CLASS);
			sDescription = wszDescription;
		} /*Description*/

		// HotKey
		//--------
		if (FAILED(pShellLinkW->GetHotkey(&wHotKey)))
			throw TException(THIS_CLASS);

		if (FAILED(pShellLinkW->GetShowCmd(&iShowCommand)))
			throw TException(THIS_CLASS);

		// Try to resolve the shortcut
		//-----------------------------
		if (bResolve) {
			if (!hParentWindow && !IsWindow(hParentWindow))
				hParentWindow = null;

			if (FAILED(pShellLinkW->Resolve(hParentWindow, uFlags)))
				throw TXUnableToResolve(THIS_CLASS, ToStdString(sTarget));

			LoadFrom(c_wsPath, false);
		} /*if*/
	} /*try*/
	catch (...) {
		Release((IUnknown*&)pPersistFile); Release((IUnknown*&)pShellLinkW);
		throw;
	} /*catch (...)*/

	Release((IUnknown*&)pPersistFile); Release((IUnknown*&)pShellLinkW);
} /*TWin32ShellLink::LoadFromEx(const TWideString&, bool, DWORD, HWND)*/

//----------------------------------------------------------------------------
bool TWin32ShellLink::SaveAs(

	const TWideString& c_wsPath

){
	try {SaveAsEx(c_wsPath);}
	catch (TException&) {return false;}
	catch (...) {throw;}
	return true;
} /*TWin32ShellLink::SaveAs(const TWideString&)*/

//----------------------------------------------------------------------------
void TWin32ShellLink::SaveAsEx(

	const TWideString& c_wsPath

){
	IShellLinkW* pShellLinkW = null;
	IPersistFile* pPersistFile = null;

	try {
		// Create an IShellLink object and get a pointer to the IShellLink
		// interface (returned from CoCreateInstance)
		//------------------------------------------------------------------
		if (
			FAILED(
				CoCreateInstance(
					CLSID_ShellLink, null, CLSCTX_INPROC_SERVER,
					IID_IShellLinkW, (void**)&pShellLinkW
				) /*CoCreateInstance*/
			) /*FAILED*/
		){
			pShellLinkW = null; throw TException(THIS_CLASS);
		} /*if*/

		// Query IShellLinkW for the IPersistFile interface for
		// saving the shortcut in persistent storage
		//-----------------------------------------------------
		if (
			FAILED(
				pShellLinkW->QueryInterface(
					IID_IPersistFile, (void**)&pPersistFile
				) /*pShellLinkW->QueryInterface*/
			) /*FAILED*/
		){
			pPersistFile = null; throw TException(THIS_CLASS);
		} /*if*/

		if (FAILED(pShellLinkW->SetPath(sTarget.c_str())))
			throw TException(THIS_CLASS);

		if (FAILED(pShellLinkW->SetArguments(sArguments.c_str())))
			throw TException(THIS_CLASS);

		if (
			FAILED(
				pShellLinkW->SetWorkingDirectory(sWorkingDirectory.c_str())
			) /*FAILED*/
		) throw TException(THIS_CLASS);

		if (
			FAILED(
				pShellLinkW->SetIconLocation(sIconPath.c_str(), iIconIndex)
			) /*FAILED*/
		) throw TException(THIS_CLASS);

		if (FAILED(pShellLinkW->SetDescription(sDescription.c_str())))
			throw TException(THIS_CLASS);

		if (FAILED(pShellLinkW->SetHotkey(wHotKey)))
			throw TException(THIS_CLASS);

		if (FAILED(pShellLinkW->SetShowCmd(iShowCommand)))
			throw TException(THIS_CLASS);

		// Save shortcut file
		//--------------------
		{
			// Save the shortcut via the IPersistFile::Save
			// member function
			//----------------------------------------------
			if (FAILED(pPersistFile->Save(c_wsPath.c_str(), true)))
				throw TException(THIS_CLASS);
 		} /*Save shortcut file*/
	} /*try*/
	catch (...) {
		Release((IUnknown*&)pPersistFile); Release((IUnknown*&)pShellLinkW);
		throw;
	} /*catch (...)*/

	Release((IUnknown*&)pPersistFile); Release((IUnknown*&)pShellLinkW);
} /*TWin32ShellLink::SaveAsEx(const TWideString&)*/
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

//----------------------------------------------------------------------------
bool TWin32ShellLink::LoadFrom(

	const TStdString& c_sPath,
	bool bResolve /*= false*/,
	DWORD uFlags /*= 0*/,
	HWND hParentWindow /*= null*/

){
	try {LoadFromEx(c_sPath, bResolve, uFlags, hParentWindow);}
	catch (TException&) {return false;}
	catch (...) {throw;}
	return true;
} /*TWin32ShellLink::LoadFrom(const TStdString&, HWND)*/

//----------------------------------------------------------------------------
void TWin32ShellLink::LoadFromEx(

	const TStdString& c_sPath,
	bool bResolve /*= false*/,
	DWORD uFlags /*= 0*/,
	HWND hParentWindow /*= null*/

){
	IShellLink* pShellLink = null;
	IPersistFile* pPersistFile = null;

	try {
		// Create an IShellLink object and get a pointer to the IShellLink
		// interface (returned from CoCreateInstance)
		//-------------------------------------------------------------------
		if (
			FAILED(
				CoCreateInstance(
					CLSID_ShellLink, null, CLSCTX_INPROC_SERVER,
					IID_IShellLink, (void**)&pShellLink
				) /*CoCreateInstance*/
			) /*FAILED*/
		){
			pShellLink = null; throw TException(THIS_CLASS);
		} /*if*/

		// Query IShellLink for the IPersistFile interface for
		// saving the shortcut in persistent storage
		//-----------------------------------------------------
		if (
			FAILED(
				pShellLink->QueryInterface(
					IID_IPersistFile, (void**)&pPersistFile
				) /*pShellLink->QueryInterface*/
			) /*FAILED*/
		){
			pPersistFile = null; throw TException(THIS_CLASS);
		} /*if*/

		// Load shortcut file
		//--------------------
		{
			TAutoPointerTo<wchar_t> h_wszPath(
				SzToWide(c_sPath.c_str()), true
			) /*h_wszPath*/;
			if (
            	FAILED(
                	pPersistFile->Load((const wchar_t*)h_wszPath, STGM_READ)
                ) /*FAILED*/
            ) throw TException(THIS_CLASS);
		} /*Load shortcut file*/

		// Target
		//--------
		{
			char szTarget[MAX_PATH + 1];
			WIN32_FIND_DATA FindData;
			if (
				FAILED(
					pShellLink->GetPath(
						szTarget, sizeof(szTarget) - 1, &FindData,
						SLGP_UNCPRIORITY
					) /*pShellLink->GetPath*/
				) /*FAILED*/
			) throw TException(THIS_CLASS);
			sTarget = NWSHLLNK_STRING_TYPE_CAST(szTarget);
		} /*Target*/

		// Arguments
		//-----------
		{
			char szArguments[MAX_PATH + 1];
			if (
				FAILED(
					pShellLink->GetArguments(
						szArguments, sizeof(szArguments) - 1
					) /*pShellLink->GetArguments*/
				) /*FAILED*/
			) throw TException(THIS_CLASS);
			sArguments = NWSHLLNK_STRING_TYPE_CAST(szArguments);
		} /*Arguments*/

		// WorkingDirectory
		//------------------
		{
			char szWorkingDir[MAX_PATH + 1];
			if (
				FAILED(
					pShellLink->GetWorkingDirectory(
						szWorkingDir, sizeof(szWorkingDir) - 1
					) /*pShellLink->GetWorkingDirectory*/
				) /*FAILED*/
			) throw TException(THIS_CLASS);
			sWorkingDirectory = NWSHLLNK_STRING_TYPE_CAST(szWorkingDir);
		} /*WorkingDirectory*/

		// Icon
		//------
		{
			char szIconPath[MAX_PATH + 1];
			if (
				FAILED(
					pShellLink->GetIconLocation(
						szIconPath, sizeof(szIconPath) - 1, &iIconIndex
					) /*pShellLink->GetIconLocation*/
				) /*FAILED*/
			) throw TException(THIS_CLASS);
			sIconPath = NWSHLLNK_STRING_TYPE_CAST(szIconPath);
		} /*Icon*/

		// Description
		//-------------
		{
			char szDescription[MAX_PATH + 1];
			if (
				FAILED(
					pShellLink->GetDescription(
						szDescription, sizeof(szDescription) - 1
					) /*pShellLink->GetDescription*/
				) /*FAILED*/
			) throw TException(THIS_CLASS);
			sDescription = NWSHLLNK_STRING_TYPE_CAST(szDescription);
		} /*Description*/

		// HotKey
		//--------
		if (FAILED(pShellLink->GetHotkey(&wHotKey)))
			throw TException(THIS_CLASS);

		if (FAILED(pShellLink->GetShowCmd(&iShowCommand)))
			throw TException(THIS_CLASS);

		// Try to resolve the shortcut
		//-----------------------------
		if (bResolve) {
			if (!hParentWindow && !IsWindow(hParentWindow))
				hParentWindow = null;

			if (FAILED(pShellLink->Resolve(hParentWindow, uFlags)))
				throw TXUnableToResolve(THIS_CLASS, ToStdString(sTarget));

			LoadFrom(c_sPath, false);
		} /*if*/
	} /*try*/
	catch (...) {
		Release((IUnknown*&)pPersistFile); Release((IUnknown*&)pShellLink);
		throw;
	} /*catch (...)*/

	Release((IUnknown*&)pPersistFile); Release((IUnknown*&)pShellLink);
} /*TWin32ShellLink::LoadFromEx(const TStdString&, bool, DWORD, HWND)*/

//------------------------------------------------------------------------
bool TWin32ShellLink::SaveAs(

	const TStdString& c_sPath

){
	try {SaveAsEx(c_sPath);}
	catch (TException&) {return false;}
	catch (...) {throw;}
	return true;
} /*TWin32ShellLink::SaveAs(const TStdString&)*/

//------------------------------------------------------------------------
void TWin32ShellLink::SaveAsEx(

	const TStdString& c_sPath

){
	IShellLink* pShellLink = null;
	IPersistFile* pPersistFile = null;

	try {
		// Create an IShellLink object and get a pointer to the IShellLink
		// interface (returned from CoCreateInstance)
		//------------------------------------------------------------------
		if (
			FAILED(
				CoCreateInstance(
					CLSID_ShellLink, null, CLSCTX_INPROC_SERVER,
					IID_IShellLink, (void**)&pShellLink
				) /*CoCreateInstance*/
			) /*FAILED*/
		){
			pShellLink = null; throw TException(THIS_CLASS);
		} /*if*/

		// Query IShellLink for the IPersistFile interface for 
		// saving the shortcut in persistent storage
		//-----------------------------------------------------
		if (
			FAILED(
				pShellLink->QueryInterface(
					IID_IPersistFile, (void**)&pPersistFile
				) /*pShellLink->QueryInterface*/
			) /*FAILED*/
		){
			pPersistFile = null; throw TException(THIS_CLASS);
		} /*if*/

		if (FAILED(pShellLink->SetPath(ToStdString(sTarget).c_str())))
			throw TException(THIS_CLASS);

		if (FAILED(pShellLink->SetArguments(ToStdString(sArguments).c_str())))
			throw TException(THIS_CLASS);

		if (
			FAILED(
				pShellLink->SetWorkingDirectory(
					ToStdString(sWorkingDirectory).c_str()
				) /*pShellLink->SetWorkingDirectory*/
			) /*FAILED*/
		) throw TException(THIS_CLASS);

		if (
			FAILED(
				pShellLink->SetIconLocation(
					ToStdString(sIconPath).c_str(), iIconIndex
				) /*pShellLink->SetIconLocation*/
			) /*FAILED*/
		) throw TException(THIS_CLASS);

		if (
			FAILED(
				pShellLink->SetDescription(ToStdString(sDescription).c_str())
			) /*FAILED*/
		) throw TException(THIS_CLASS);

		if (FAILED(pShellLink->SetHotkey(wHotKey)))
			throw TException(THIS_CLASS);

		if (FAILED(pShellLink->SetShowCmd(iShowCommand)))
			throw TException(THIS_CLASS);

		// Save shortcut file
		//--------------------
		{
			// Save the shortcut via the IPersistFile::Save
			// member function
			//----------------------------------------------
			TAutoPointerTo<wchar_t> h_wszPath(
				SzToWide(c_sPath.c_str()), true
			) /*h_wszPath*/;
			if (FAILED(pPersistFile->Save((const wchar_t*)h_wszPath, true)))
				throw TException(THIS_CLASS);
 		} /*Save shortcut file*/
	} /*try*/
	catch (...) {
		Release((IUnknown*&)pPersistFile); Release((IUnknown*&)pShellLink);
		throw;
	} /*catch (...)*/

	Release((IUnknown*&)pPersistFile); Release((IUnknown*&)pShellLink);
} /*TWin32ShellLink::SaveAsEx(const TStdString&)*/

//----------------------------------------------------------------------------
void TWin32ShellLink::Release(

	IUnknown*& r_pInterface

){
	if (r_pInterface) {r_pInterface->Release(); r_pInterface = null;}
} /*TWin32ShellLink::Release(IUnknown*&)*/

//============================================================================
NATLIB_NAMESPACE_END
