/****************************************************************************/
/** nwthread.cpp                                                           **/
/** ---------------------------------------------------------------------- **/
/** The C++ envelope for MS Win32 threads                                  **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** January 25, 1998 by Oleg Kagan                                         **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>
#pragma hdrstop

//============================================================================
#include <nwthread.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

// TWin32Thread implementation
//============================================================================
TWin32Thread::TWin32Thread(

	TCurrentThread
	
):
	uThreadId(::GetCurrentThreadId()),
	hThread(0),
	State(stateRunning),
	bTerminationRequested(false),
	bAttached(true)
{
	if (
		::DuplicateHandle(
			::GetCurrentProcess(), ::GetCurrentThread(), 
			::GetCurrentProcess(), &hThread, 0, false, DUPLICATE_SAME_ACCESS
		) /*::DuplicateHandle*/
	){
		DWORD uError = GetLastError();
		throw TXCreationFailure(
			THIS_CLASS, Win32GetErrorExplanationA(uError)
		) /*throw TXCreationFailure*/;
	} /*if*/
} /*TWin32Thread(TCurrent)*/

//----------------------------------------------------------------------------
TWin32Thread::~TWin32Thread()
{
	if (
		!bAttached && 
		(GetState() == stateRunning || GetState() == stateSuspended)
	){
		if (hThread) {::CloseHandle(hThread); hThread = null;}
		throw TXDestroyBeforeExit(THIS_CLASS);
	} /*if*/
	if (hThread) {::CloseHandle(hThread); hThread = null;}
} /*~TWin32Thread()*/

//----------------------------------------------------------------------------
TWin32Thread::THandle TWin32Thread::Start()
{
	// If Start() has already been called for this thread, release the
	// previously created system thread object before launching a new one
	//--------------------------------------------------------------------
	if ((GetState() != stateCreated) && hThread) {
		::CloseHandle(hThread); hThread = null;
	} /*if*/

	hThread = ::CreateThread(
		null, 0, &TWin32Thread::Execute, 
		dynamic_cast<TWin32Thread*>(this), 0, &uThreadId
	) /*CreateThread*/;

	if (!hThread) {
		DWORD uError = GetLastError();
		State = stateInvalid;
		throw TXCreationFailure(
			THIS_CLASS, Win32GetErrorExplanationA(uError)
		) /*throw TXCreationFailure*/;
	} /*if*/

	State = stateRunning;
	return hThread;
} /*TWin32Thread::Start()*/

//----------------------------------------------------------------------------
DWORD TWin32Thread::Suspend()
{
	switch (GetState()) {
		case stateCreated:
			throw TXSuspendBeforeRun(THIS_CLASS);

		case stateFinished:
			throw TXSuspendAfterExit(THIS_CLASS);

		default: {
			DWORD uResult = ::SuspendThread(hThread);
			if (uResult == ~DWORD(0)) {
				DWORD uError = GetLastError();
				throw TException(
					THIS_CLASS, Win32GetErrorExplanationA(uError)
				) /*throw TException*/;
			} /*if*/
			State = stateSuspended;
			return uResult;
		} /*default*/
	} /*switch (GetStatus())*/
} /*TWin32Thread::Suspend()*/

//----------------------------------------------------------------------------
DWORD TWin32Thread::Resume()
{
	switch (GetState()) {
		case stateCreated:
			throw TXResumeBeforeRun(THIS_CLASS);

		case stateRunning:
			throw TXResumeDuringRun(THIS_CLASS);
			
		case stateFinished:
			throw TXResumeAfterExit(THIS_CLASS);
			
		default: {
			DWORD uResult = ::ResumeThread(hThread);
			if (uResult == ~DWORD(0)) {
				DWORD uError = GetLastError();
				throw TException(
					THIS_CLASS, Win32GetErrorExplanationA(uError)
				) /*throw TException*/;
			} /*if*/
			State = stateRunning;
			return uResult;
		} /*default*/
    } /*switch (GetStatus())*/
} /*TWin32Thread::Resume()*/

//----------------------------------------------------------------------------
bool TWin32Thread::Sleep(

	DWORD uTimeout, 
	bool bAlertable /*= false*/
	
){
	return ::SleepEx(uTimeout, bAlertable) == WAIT_IO_COMPLETION;
} /*TWin32Thread::Sleep(DWORD, bool)*/

//----------------------------------------------------------------------------
void TWin32Thread::Terminate()
{
	bTerminationRequested = true;
} /*TWin32Thread::Terminate()*/

//----------------------------------------------------------------------------
void TWin32Thread::Kill(

	DWORD uExitCode

){
	if (!TerminateThread(hThread, uExitCode)) {
		DWORD uError = GetLastError();
		throw TXUnableToKillThread(
			THIS_CLASS, Win32GetErrorExplanationA(uError)
		) /*throw TXUnableToKillThread*/;
	} /*if*/
	uThreadId = 0; hThread = null;
	State = stateCreated; 
	bTerminationRequested = false; bAttached = false;
} /*TWin32Thread::Kill(DWORD)*/

//----------------------------------------------------------------------------
DWORD TWin32Thread::WaitForExit(

	nl_ulong_t uTimeout /*= TLock::c_uWaitInfinite*/
	
){
	if (State == stateRunning) {
		DWORD uResult = ::WaitForSingleObject(
			hThread, 
			(uTimeout == TLock::c_uWaitInfinite) ? INFINITE : DWORD(uTimeout)
		) /*::WaitForSingleObject*/;
		if (uResult == WAIT_FAILED) {
			DWORD uError = GetLastError();
			throw TException(THIS_CLASS, Win32GetErrorExplanationA(uError));
		} /*if*/
		return uResult;
	}
	else return ~DWORD(0);
} /*TWin32Thread::WaitForExit(nl_ulong_t)*/

//----------------------------------------------------------------------------
DWORD TWin32Thread::GetExitCode() const
{
	DWORD uCode; 
	if (!::GetExitCodeThread(hThread, &uCode)) {
		DWORD uError = GetLastError();
		throw TException(THIS_CLASS, Win32GetErrorExplanationA(uError));
	} /*if*/
	return uCode;
} /*TWin32Thread::GetExitCode() const*/

//----------------------------------------------------------------------------
nl_int_t TWin32Thread::GetPriority() const
{
	nl_int_t iPriority = ::GetThreadPriority(hThread);
	if (iPriority == THREAD_PRIORITY_ERROR_RETURN) {
		DWORD uError = GetLastError();
		throw TException(THIS_CLASS, Win32GetErrorExplanationA(uError));
	} /*if*/
	return iPriority;
} /*TWin32Thread::GetPriority() const*/

//----------------------------------------------------------------------------
void TWin32Thread::SetPriority(

	nl_int_t iPriority
	
){
	BOOL bResult = ::SetThreadPriority(hThread, iPriority);
	if (!bResult) {
		DWORD uError = GetLastError();
		throw TException(THIS_CLASS, Win32GetErrorExplanationA(uError));
	} /*if*/
} /*TWin32Thread::SetPriority(nl_int_t)*/

//------------------------------------------------------------------------
void TWin32Thread::EvExecutionCompleted()
{} /*TWin32Thread::EvExecutionCompleted()*/

//----------------------------------------------------------------------------
const TWin32Thread& TWin32Thread::operator=(

	const TWin32Thread& c_Thread
	
){
	switch (GetState()) {
		case stateCreated:
		case stateSuspended:
		case stateFinished: {
			if (this != &c_Thread) {
				hThread = null;
				uThreadId = 0;
				State = stateCreated;
				bTerminationRequested = bAttached = false;
			} /*if*/
		} /*case stateCreated, stateSuspended, stateFinished*/ break;
		default:
			throw TXAssignError(THIS_CLASS);
	} /*switch (GetStatus())*/
	return *this;
} /*TWin32Thread::operator=(const TWin32Thread&)*/

//----------------------------------------------------------------------------
DWORD WINAPI TWin32Thread::Execute(

	void* pThread
	
){
	TWin32Thread* pWin32Thread = static_cast<TWin32Thread*>(pThread); 
	nl_int_t iCode = pWin32Thread->Run();
	pWin32Thread->State = stateFinished;
	pWin32Thread->EvExecutionCompleted();
	return iCode;
} /*TWin32Thread::Execute(void*)*/

//----------------------------------------------------------------------------
nl_int_t TWin32Thread::Run()
{
	return -1;
} /*TWin32Thread::Run()*/

//----------------------------------------------------------------------------
TWin32Thread::TState TWin32Thread::CheckState() const
{
	DWORD uCode;
	if (!::GetExitCodeThread(hThread, &uCode)) {
		DWORD uError = GetLastError();
		throw TException(THIS_CLASS, Win32GetErrorExplanationA(uError));
	} /*if*/
	if (uCode == STILL_ACTIVE) return stateRunning;
	else return stateFinished;
} /*TWin32Thread::CheckState() const*/

//============================================================================
NATLIB_NAMESPACE_END
