/****************************************************************************/
/** nwutils.cpp                                                            **/
/** ---------------------------------------------------------------------- **/
/** Usefull C++ envelopes for Win32 functions                              **/
/** ---------------------------------------------------------------------- **/
/** Copyright (C) 1996-98 by OK Thinking Software, All Rights Reserved     **/
/** ---------------------------------------------------------------------- **/
/** May 10, 1998 by Oleg Kagan                                             **/
/****************************************************************************/

//============================================================================
#include <nlcompat.h>
#pragma hdrstop

// Natlib Basics includes
//============================================================================
#include <nlcmptex.h>
#include <nlstring.h>
#include <nlautopt.h>

// Natlib Win32 includes
//============================================================================
#include <nwutils.h>
#include <nwreg.h>
#include <nwlock.h>

//============================================================================
NATLIB_NAMESPACE_BEGIN

//============================================================================
TStdString Win32GetCurrentUserA()
{
	TStdString sUser, sDomain;
	Win32GetCurrentUserDomain(sUser, sDomain);
	return sUser;
} /*Win32GetCurrentUserA()*/

//----------------------------------------------------------------------------
TStdString Win32GetCurrentDomainA()
{
	TStdString sUser, sDomain;
	Win32GetCurrentUserDomain(sUser, sDomain);
	return sDomain;
} /*Win32GetCurrentUserA()*/

//----------------------------------------------------------------------------
void Win32GetCurrentUserDomain(

	TStdString& r_sUser,
	TStdString& r_sDomain

){
	wchar_t wszUser[MAX_PATH], wszDomain[MAX_PATH];
	Win32GetCurrentUserDomain(wszUser, MAX_PATH - 1, wszDomain, MAX_PATH - 1);
	r_sUser = ToStdString(wszUser);
	r_sDomain = ToStdString(wszDomain);
} /*Win32GetCurrentUserDomain(TStdString&, TStdString&)*/

#ifndef _NATLIB_NO_WIDE_STRINGS_
//----------------------------------------------------------------------------
TWideString Win32GetCurrentUserW()
{
	TWideString wsUser, wsDomain;
	Win32GetCurrentUserDomain(wsUser, wsDomain);
	return wsUser;
} /*Win32GetCurrentUserW()*/

//----------------------------------------------------------------------------
TWideString Win32GetCurrentDomainW()
{
	TWideString wsUser, wsDomain;
	Win32GetCurrentUserDomain(wsUser, wsDomain);
	return wsDomain;
} /*Win32GetCurrentUserW()*/

//----------------------------------------------------------------------------
void Win32GetCurrentUserDomain(

	TWideString& r_wsUser,
	TWideString& r_wsDomain

){
	wchar_t wszUser[MAX_PATH], wszDomain[MAX_PATH];
	Win32GetCurrentUserDomain(wszUser, MAX_PATH - 1, wszDomain, MAX_PATH - 1);
	r_wsUser = wszUser;
	r_wsDomain = wszDomain;
} /*Win32GetCurrentUserDomain(TStdString&, TStdString&)*/
#endif /*!defined(_NATLIB_NO_WIDE_STRINGS_)*/

//----------------------------------------------------------------------------
void Win32GetCurrentUserDomain(

	wchar_t* wszUser, 
	DWORD nUserSize,
	wchar_t* wszDomain,
	DWORD nDomainSize
	
){
	Win32GetProcessUserDomain(
		GetCurrentProcess(), wszUser, nUserSize, wszDomain, nDomainSize
	) /*Win32GetProcessUserDomain*/;
} /*Win32GetCurrentUserDomain(wchar_t*, size_t, wchar_t, size_t)*/

//----------------------------------------------------------------------------
void Win32GetProcessUserDomain(

	HANDLE hProcess,
	wchar_t* wszUser, 
	DWORD nUserSize,
	wchar_t* wszDomain,
	DWORD nDomainSize
	
){
	if (!Win32IsNT()) 
		throw TXOperationIsNotAllowed(THIS_PLACE);

	HANDLE hAccessToken; 
	if (!OpenProcessToken(hProcess, TOKEN_READ, &hAccessToken)) {
		DWORD uError = GetLastError();
		throw basic_TNatlibException(
			THIS_PLACE, Win32GetErrorExplanation(uError)
		) /*throw basic_TNatlibException*/;
	} /*if*/

	char a_cInfoBuffer[1000]; 
	DWORD nInfoBufferSize = sizeof(a_cInfoBuffer) - 1;
	GetTokenInformation(
		hAccessToken, TokenUser, 
		a_cInfoBuffer, sizeof(a_cInfoBuffer) - 1, &nInfoBufferSize
	) /*GetTokenInformation*/;

	PTOKEN_USER pTokenUser = (PTOKEN_USER)a_cInfoBuffer;
	SID_NAME_USE Snu;

	LookupAccountSidW(
		null, pTokenUser->User.Sid, 
		wszUser, &nUserSize, wszDomain, &nDomainSize, &Snu
	) /*LookupAccountSidW*/;

	wszUser[nUserSize] = wszDomain[nDomainSize] = wchar_t(0);
} /*Win32GetProcessUserDomain(wchar_t*, size_t, wchar_t, size_t)*/

//============================================================================
bool Win32IsFileExists(

	const char* c_szMask

){
	if (!c_szMask || !*c_szMask) return false;
	WIN32_FIND_DATA FindData;
	HANDLE hSearch = FindFirstFile(c_szMask, &FindData);
	bool bExists = (hSearch != INVALID_HANDLE_VALUE);
	FindClose(hSearch);
	return bExists;
} /*Win32IsFileExists(const char*)*/

//----------------------------------------------------------------------------
bool Win32IsFileExists(

	const wchar_t* c_wszMask

){
	if (!c_wszMask || !*c_wszMask) return false;

	if (!Win32IsNT()) {
		return Win32IsFileExists(ToStdString(c_wszMask).c_str());
	} /*if*/

	WIN32_FIND_DATAW FindData;
	HANDLE hSearch = FindFirstFileW(c_wszMask, &FindData);
	bool bExists = (hSearch != INVALID_HANDLE_VALUE);
	FindClose(hSearch);
	return bExists;
} /*Win32IsFileExists(const wchar_t*)*/

//============================================================================
void Win32RegisterShellFileType(

	const char* c_szDescription, 
	const char* c_szExtension, 
	const char* c_szDefaultIcon,
	const char* c_szOperation, 
	const char* c_szCommandLine,
	bool bDefaultOperation /*= false*/

){
	TAutoPointerTo<wchar_t> h_szDescription(SzToWide(c_szDescription), true);
	TAutoPointerTo<wchar_t> h_szExtension(SzToWide(c_szExtension), true);
	TAutoPointerTo<wchar_t> h_szDefaultIcon(SzToWide(c_szDefaultIcon), true);
	TAutoPointerTo<wchar_t> h_szOperation(SzToWide(c_szOperation), true);
	TAutoPointerTo<wchar_t> h_szCommandLine(SzToWide(c_szCommandLine), true);

	Win32RegisterShellFileType(
		(const wchar_t*)h_szDescription,
		(const wchar_t*)h_szExtension,
		(const wchar_t*)h_szDefaultIcon,
		(const wchar_t*)h_szOperation,
		(const wchar_t*)h_szCommandLine,
		bDefaultOperation
	) /*Win32RegisterShellFileType*/;
} /*Win32RegisterShellFileType(const char*, const char*, const char*, ...)*/

//----------------------------------------------------------------------------
void Win32RegisterShellFileType(

	const wchar_t* c_wszDescription, 
	const wchar_t* c_wszExtension, 
	const wchar_t* c_wszDefaultIcon,
	const wchar_t* c_wszOperation, 
	const wchar_t* c_wszCommandLine,
	bool bDefaultOperation /*= false*/

){
	if (
		!c_wszDescription || !*c_wszDescription || 
		!c_wszExtension || !*c_wszExtension 
	) throw basic_TXInvalidParameter(THIS_PLACE);

	// Validate the extension
	//------------------------
	TAutoPointerTo<wchar_t> h_wszExtension(null, true);
    NLEXCEPT_NEW(wchar_t[SzLength(c_wszExtension) + 3], h_wszExtension);

    if (*c_wszExtension != L'.') {
   		(*(wchar_t*)h_wszExtension) = L'.';
        SzToWide(c_wszExtension, ((wchar_t*)h_wszExtension) + 1);
    }
    else {
    	SzToWide(c_wszExtension, (wchar_t*)h_wszExtension);
	} /*iff*/

	// Create extension->type record in the Registry
	//-----------------------------------------------
	{
		TWin32Registry ExtensionKey(
			HKEY_CLASSES_ROOT, (const wchar_t*)h_wszExtension, true
		) /*ExtensionKey*/;
		ExtensionKey.SetStringValue(null, c_wszDescription);
	} /*Create extension->type record in the Registry*/

	// Create type definition in the Registry
	//----------------------------------------
	{
		TWin32Registry TypeKey(HKEY_CLASSES_ROOT, c_wszDescription, true);
		TypeKey.SetStringValue(null, c_wszDescription);

		// default icon subkey
		//---------------------
		if (c_wszDefaultIcon && *c_wszDefaultIcon) {
			TWin32Registry DefaultIconKey(TypeKey, "DefaultIcon", true);
			DefaultIconKey.SetStringValue(
				null, c_wszDefaultIcon, REG_EXPAND_SZ
			) /*DefaultIconKey.SetStringValue*/;
		} /*if*/

		// shell subkey 
		//-------------- 
		TWin32Registry ShellKey(TypeKey, "shell", true);
		if (bDefaultOperation) 
			ShellKey.SetStringValue(null, c_wszOperation);

		// command subkey
		//----------------
		if (c_wszCommandLine && *c_wszCommandLine) {
			TWin32Registry ActionKey(ShellKey, c_wszOperation, true);
			TWin32Registry CommandKey(ActionKey, "command", true);
			CommandKey.SetStringValue(null, c_wszCommandLine, REG_EXPAND_SZ);
		} /*if*/
	} /*Create type definition in the Registry*/
} /*Win32RegisterShellFileType(const wchar_t*, const wchar_t*, ...)*/

//============================================================================
class _NATLIB_DECLSPEC_ TWin32CompatibilityCore :
	public TNatlibCompatibilityCore 
{
public:
	//------------------------------------------------------------------------
	size_t WideToStd(const wchar_t*, char*, size_t);

	//------------------------------------------------------------------------
	size_t StdToWide(const char*, wchar_t*, size_t);

	//------------------------------------------------------------------------
	TLock& CreateLock();
} /*TWin32CompatibilityCore*/;

// TWin32CompatibilityCore implementation
//============================================================================
size_t TWin32CompatibilityCore::WideToStd(

	const wchar_t* c_wszSource, 
	char* szDestination, 
	size_t nToConvert
	
){
	if (!c_wszSource || (szDestination && !nToConvert))
		throw TXInvalidParameter(THIS_CLASS);

	int iResult = WideCharToMultiByte(
		CP_ACP, 0,
		c_wszSource, nToConvert, 
		szDestination, szDestination ? (nToConvert * 2) : 0,
		null, null
	) /*WideCharToMultiByte*/;

	return iResult ? size_t(iResult) : size_t(-1);
} /*TWin32CompatibilityCore::WideToStd(const wchar_t*, char*, size_t)*/

//----------------------------------------------------------------------------
size_t TWin32CompatibilityCore::StdToWide(

	const char* c_szSource, 
	wchar_t* wszDestination, 
	size_t nToConvert
	
){
	if (!c_szSource || (wszDestination && !nToConvert))
		throw TXInvalidParameter(THIS_CLASS);

	int iResult = MultiByteToWideChar(
		CP_ACP, 0,
		c_szSource, nToConvert,
		wszDestination, wszDestination ? nToConvert : 0 
	) /*MultiByteToWideChar*/;

	return iResult ? size_t(iResult) : size_t(-1);
} /*TWin32CompatibilityCore::StdToWide(const char*, wchar_t*, size_t)*/

//----------------------------------------------------------------------------
TLock& TWin32CompatibilityCore::CreateLock()
{
	TWin32Lock* pLock;
	NLEXCEPT_CLASS_NEW(TWin32Lock(), pLock);
	return *pLock;
} /*TWin32CompatibilityCore::CreateLock()*/

//============================================================================
NATLIB_NAMESPACE_END

//============================================================================
NATLIB_NAMESPACE_MEMBER(TNatlibCompatibilityCore)& 
NWUTILS_Win32GetCompatibilityCore()
{
	static NATLIB_NAMESPACE_MEMBER(TWin32CompatibilityCore) 
		Win32CompatibilityCore;
	return Win32CompatibilityCore;
} /*NWUTILS_Win32GetCompatibilityCore()*/